@@ -30,67 +30,50 @@
 #ifndef MOZC_BASE_CLOCK_MOCK_H_
 #define MOZC_BASE_CLOCK_MOCK_H_
 
-#include <cstdint>
-
 #include "base/clock.h"
-#include "base/port.h"
+#include "absl/strings/string_view.h"
+#include "absl/time/time.h"
 
 namespace mozc {
 
+// Parses the given time in RFC3399 format (see `absl::RFC3399_full`), or
+// terminates the program on failure.
+absl::Time ParseTimeOrDie(absl::string_view time);
+
 // Standard mock clock implementation.
 // This mock behaves in UTC
 class ClockMock : public ClockInterface {
  public:
-  ClockMock(uint64_t sec, uint32_t usec);
+  explicit ClockMock(absl::Time time) : time_(time) {}
 
   ClockMock(const ClockMock &) = delete;
   ClockMock &operator=(const ClockMock &) = delete;
 
-  ~ClockMock() override;
+  ~ClockMock() override = default;
 
-  void GetTimeOfDay(uint64_t *sec, uint32_t *usec) override;
-  uint64_t GetTime() override;
   absl::Time GetAbslTime() override;
-  uint64_t GetFrequency() override;
-  uint64_t GetTicks() override;
 
-  const absl::TimeZone& GetTimeZone() override;
-  void SetTimeZoneOffset(int32_t timezone_offset_sec) override;
+  absl::TimeZone GetTimeZone() override;
 
-  // Puts this clock forward.
-  // It has no impact on ticks.
-  void PutClockForward(uint64_t delta_sec, uint32_t delta_usec);
+  // Advances the clock.
+  void Advance(absl::Duration delta) { time_ += delta; }
 
-  // Puts this clock forward by ticks
-  // It has no impact on seconds and micro seconds.
-  void PutClockForwardByTicks(uint64_t ticks);
+  // Automatically advances the clock every time it returns time.
+  void AutoAdvance(absl::Duration delta) { auto_delta_ = delta; }
 
-  // Automatically puts this clock forward on every time after it returns time.
-  // It has no impact on ticks.
-  void SetAutoPutClockForward(uint64_t delta_sec, uint32_t delta_usec);
-
-  void SetTime(uint64_t sec, uint32_t usec);
-  void SetFrequency(uint64_t frequency);
-  void SetTicks(uint64_t ticks);
+  void SetTime(absl::Time time) { time_ = time; }
 
  private:
-  uint64_t seconds_;
-  uint64_t frequency_;
-  uint64_t ticks_;
-  absl::TimeZone timezone_;
-  int32_t timezone_offset_sec_;
-  // To optimize the padding, micro_seconds_ is after another int32_t value.
-  uint32_t micro_seconds_;
-  // Everytime user requests time clock, following time is added to the
-  // internal clock.
-  uint64_t delta_seconds_;
-  uint32_t delta_micro_seconds_;
+  absl::Time time_ = absl::InfinitePast();
+  absl::TimeZone timezone_ = absl::UTCTimeZone();
+  // Every time user requests the time, auto_delta_ is added to time_.
+  absl::Duration auto_delta_ = absl::ZeroDuration();
 };
 
 // Changes the global clock with a mock during the life time of this object.
 class ScopedClockMock {
  public:
-  ScopedClockMock(uint64_t sec, uint32_t usec) : mock_(sec, usec) {
+  explicit ScopedClockMock(absl::Time time) : mock_(time) {
     Clock::SetClockForUnitTest(&mock_);
   }
 
@@ -99,8 +82,8 @@ class ScopedClockMock {
 
   ~ScopedClockMock() { Clock::SetClockForUnitTest(nullptr); }
 
-  ClockMock *operator->() { return &mock_; }
-  const ClockMock *operator->() const { return &mock_; }
+  constexpr ClockMock *operator->() { return &mock_; }
+  constexpr const ClockMock *operator->() const { return &mock_; }
 
  private:
   ClockMock mock_;
