@@ -30,20 +30,23 @@
 #ifndef MOZC_BASE_LOGGING_H_
 #define MOZC_BASE_LOGGING_H_
 
-
-#include <iostream>
 #include <string>
 
-#include "base/port.h"
+
+#include <iostream>
+#include <sstream>
+#include <type_traits>
+
 #include "absl/flags/declare.h"
 #include "absl/flags/flag.h"
+#include "absl/strings/string_view.h"
 
 ABSL_DECLARE_FLAG(bool, logtostderr);
 
 namespace mozc {
 
 enum LogSeverity {
-#ifdef OS_ANDROID
+#ifdef __ANDROID__
   // Defined in <android/log.h>
   LOG_UNKNOWN = 0,  // ANDROID_LOG_UNKNOWN
   LOG_DEFAULT = 1,  // ANDROID_LOG_DEFAULT
@@ -55,30 +58,34 @@ enum LogSeverity {
   LOG_FATAL = 7,    // ANDROID_LOG_FATAL
   LOG_SILENT = 8,   // ANDROID_LOG_SILENT
   LOG_SEVERITY_SIZE = 9,
-#else   // OS_ANDROID
+#else  // __ANDROID__
   LOG_INFO = 0,
   LOG_WARNING = 1,
   LOG_ERROR = 2,
 // Special hack for Windows build, where ERROR is defined as 0 in wingdi.h.
-#ifdef OS_WIN
+#ifdef _WIN32
   LOG_0 = LOG_ERROR,
-#endif  // OS_WIN
+#endif  // _WIN32
   LOG_FATAL = 3,
   LOG_SEVERITY_SIZE = 4,
-#endif  // OS_ANDROID
+#endif  // !__ANDROID__
 };
 
 // DFATAL is FATAL in debug mode, ERROR in normal mode
 #ifdef DEBUG
 #define LOG_DFATAL LOG_FATAL
-#else   // DEBUG
+#else  // DEBUG
 #define LOG_DFATAL LOG_ERROR
-#endif  // DEBUG
+#endif  // !DEBUG
 
+class WorkingLogStream;
 class NullLogStream;
 
 class Logging {
  public:
+  Logging() = delete;
+  Logging(const Logging &) = delete;
+  Logging &operator=(const Logging &) = delete;
   // Initializes log stream with the output file path and --logtostderr.
   static void InitLogStream(const std::string &log_file_path);
 
@@ -87,14 +94,14 @@ class Logging {
 
   // Gets working log stream. The log message can be written to the stream.
   // The stream must be finalized by FinalizeWorkingLogStream().
-  static std::ostream &GetWorkingLogStream();
+  static WorkingLogStream &GetWorkingLogStream();
 
   // Finalizes the working stream.
   // - Appends std::endl to working stream.
   // - Writes the content to real backing logging stream, which is initialized
   //     by InitLogStream().
   // - Deletes the working stream object.
-  static void FinalizeWorkingLogStream(LogSeverity, std::ostream *);
+  static void FinalizeWorkingLogStream(LogSeverity, WorkingLogStream *);
 
   // Gets NullLogStream for MOZC_NO_LOGGING mode
   static NullLogStream &GetNullLogStream();
@@ -122,9 +129,61 @@ class Logging {
   static const char *GetEndColorEscapeSequence();
 
   static void SetLogToStderr(bool log_to_stderr);
+};
+
+namespace logging_internal {
+
+// Based on
+// https://github.com/abseil/abseil-cpp/blob/master/absl/strings/internal/has_absl_stringify.h
+//
+// This can be removed once Abseil LTS supports `absl::HasAbslStringify`.
+template <typename T, typename = void>
+struct HasAbslStringify : std::false_type {};
+
+template <typename T>
+struct HasAbslStringify<
+    T, std::void_t<decltype(AbslStringify(std::declval<WorkingLogStream &>(),
+                                          std::declval<const T &>()))>>
+    : std::true_type {};
+
+}  // namespace logging_internal
+
+// WorkingLogStream is a std::ostringstream that also implements Abseil's Sink.
+class WorkingLogStream {
+ public:
+  WorkingLogStream() = default;
+
+  // Sink methods.
+  // https://github.com/abseil/abseil-cpp/blob/master/absl/strings/internal/stringify_sink.h
+  void Append(size_t count, char ch);
+  void Append(absl::string_view v);
+  friend void AbslFormatFlush(WorkingLogStream *sink, absl::string_view v) {
+    sink->Append(v);
+  }
+
+  template <typename T,
+            std::enable_if_t<logging_internal::HasAbslStringify<T>::value,
+                             std::nullptr_t> = nullptr>
+  WorkingLogStream &operator<<(const T &v) {
+    AbslStringify(*this, v);
+    return *this;
+  }
+  template <typename T,
+            std::enable_if_t<!logging_internal::HasAbslStringify<T>::value,
+                             std::nullptr_t> = nullptr>
+  WorkingLogStream &operator<<(const T &v) {
+    os_ << v;
+    return *this;
+  }
+  WorkingLogStream &operator<<(std::ostream &(*func)(std::ostream &)) {
+    os_ << func;
+    return *this;
+  }
+
+  std::string str() const { return os_.str(); }
 
  private:
-  DISALLOW_IMPLICIT_CONSTRUCTORS(Logging);
+  std::ostringstream os_;
 };
 
 // Finalizer to flush/delete working log stream.
@@ -135,13 +194,11 @@ class LogFinalizer {
   explicit LogFinalizer(LogSeverity severity);
   ~LogFinalizer();
 
-  // Google's C++ style guide requires reference argument to be const.
-  // Here we need non-const reference in order to delete working stream object.
-  void operator&(std::ostream &);
+  void operator&(WorkingLogStream &);
 
  private:
   const LogSeverity severity_;
-  std::ostream *working_stream_;
+  WorkingLogStream *working_stream_;
 };
 
 // When using NullLogStream, all debug message will be stripped
@@ -236,7 +293,7 @@ class NullLogFinalizer {
                 << mozc::Logging::GetBeginColorEscapeSequence(mozc::LOG_FATAL) \
                 << "CHECK" << mozc::Logging::GetEndColorEscapeSequence()       \
                 << " [" << #condition << "] "
-#endif  // end MOZC_NO_LOGGING
+#endif  // !MOZC_NO_LOGGING
 
 #define VLOG_IS_ON(verboselevel) \
   (mozc::Logging::GetVerboseLevel() >= verboselevel)
@@ -292,14 +349,17 @@ class NullLogFinalizer {
 #define DCHECK_LT(a, b) \
   while (false) CHECK_LT(a, b)
 
-#endif  // DEBUG
+#endif  // opt build
 
 #define DVLOG(verboselevel) DLOG_IF(INFO, VLOG_IS_ON(verboselevel))
 
+#ifndef MOZC_LOG_PROTOBUF
+#define MOZC_LOG_PROTOBUF(message) ((message).DebugString())
+#endif  // MOZC_LOG_PROTOBUF
 
-#ifndef DVLOG_IF
-#define DVLOG_IF(verboselevel, condition) \
-  DLOG_IF(INFO, (condition) && VLOG_IS_ON(verboselevel))
-#endif  // DVLOG_IF
+
+#ifndef MOZC_LOG_PROTOBUF
+#define MOZC_LOG_PROTOBUF(message) (message)
+#endif  // !MOZC_LOG_PROTOBUF
 
 #endif  // MOZC_BASE_LOGGING_H_
