@@ -29,30 +29,7 @@
 
 #include "base/process.h"
 
-#ifdef OS_WIN
-#include <Windows.h>
-#else  // OS_WIN
-#include <string.h>
-#include <sys/stat.h>
-
-#include <cerrno>
-#endif  // OS_WIN
-
-#ifdef __APPLE__
-#include <fcntl.h>
-#include <signal.h>
-#include <spawn.h>  // for posix_spawn().
-
-#include "base/mac_process.h"
-#endif  // __APPLE__
-
-#if defined(OS_LINUX) || defined(OS_ANDROID)
-#include <fcntl.h>
-#include <signal.h>
-#include <spawn.h>  // for posix_spawn().
-#include <sys/types.h>
-#endif  // OS_LINUX || OS_ANDROID
-
+#include <cstddef>
 #include <cstdlib>
 #include <memory>
 #include <string>
@@ -62,28 +39,50 @@
 #include "base/file_util.h"
 #include "base/logging.h"
 #include "base/system_util.h"
-#include "base/util.h"
 #include "absl/strings/match.h"
 #include "absl/strings/str_split.h"
+#include "absl/strings/string_view.h"
+#include "absl/time/clock.h"
+#include "absl/time/time.h"
 
-#ifdef OS_WIN
-#include "base/scoped_handle.h"
-#include "base/win_util.h"
-#endif  // OS_WIN
+#ifdef _WIN32
+#include <wil/resource.h>
+#include <windows.h>
+
+#include "base/win32/wide_char.h"
+#include "base/win32/win_util.h"
+#else  // _WIN32
+#include <fcntl.h>
+#include <signal.h>
+#include <spawn.h>  // for posix_spawn().
+#include <string.h>
+#include <sys/stat.h>
+
+#include <cerrno>
+#endif  // !_WIN32
+
+#ifdef __linux
+#include <sys/types.h>
+#endif  // __linux__
+
+#ifdef __APPLE__
+#include <crt_externs.h>
+
+#include "base/mac/mac_process.h"
+#endif  // __APPLE__
 
 #ifdef __APPLE__
 // We do not use the global environ variable because it is unavailable
 // in Mac Framework/dynamic libraries.  Instead call _NSGetEnviron().
 // See the "PROGRAMMING" section of http://goo.gl/4Hq0D for the
 // detailed information.
-#include <crt_externs.h>
 static char **environ = *_NSGetEnviron();
-#elif !defined(OS_WIN)
+#elif !defined(_WIN32)
 // Defined somewhere in libc. We can't pass nullptr as the 6th argument of
 // posix_spawn() since Qt applications use (at least) DISPLAY and QT_IM_MODULE
 // environment variables.
 extern char **environ;
-#endif  // __APPLE__
+#endif  // !__APPLE__ && !_WIN32
 
 namespace mozc {
 
@@ -94,13 +93,12 @@ bool Process::OpenBrowser(const std::string &url) {
     return false;
   }
 
-#ifdef OS_WIN
-  std::wstring wurl;
-  Util::Utf8ToWide(url, &wurl);
-  return WinUtil::ShellExecuteInSystemDir(L"open", wurl.c_str(), nullptr);
-#endif  // OS_WIN
+#ifdef _WIN32
+  return WinUtil::ShellExecuteInSystemDir(
+      L"open", win32::Utf8ToWide(url).c_str(), nullptr);
+#endif  // _WIN32
 
-#if defined(OS_LINUX) || defined(OS_ANDROID)
+#ifdef __linux__
 
 #ifndef MOZC_BROWSER_COMMAND
   // xdg-open which uses kfmclient or gnome-open internally works both on KDE
@@ -109,7 +107,7 @@ bool Process::OpenBrowser(const std::string &url) {
 #endif  // MOZC_BROWSER_COMMAND
 
   return SpawnProcess(MOZC_BROWSER_COMMAND, url);
-#endif  // OS_LINUX || OS_ANDROID
+#endif  // __linux__
 
 #ifdef __APPLE__
   return MacProcess::OpenBrowserForMac(url);
@@ -119,13 +117,11 @@ bool Process::OpenBrowser(const std::string &url) {
 
 bool Process::SpawnProcess(const std::string &path, const std::string &arg,
                            size_t *pid) {
-#ifdef OS_WIN
-  std::wstring wpath;
-  Util::Utf8ToWide(path, &wpath);
+#ifdef _WIN32
+  std::wstring wpath = win32::Utf8ToWide(path);
   wpath = L"\"" + wpath + L"\"";
   if (!arg.empty()) {
-    std::wstring warg;
-    Util::Utf8ToWide(arg, &warg);
+    std::wstring warg = win32::Utf8ToWide(arg);
     wpath += L" ";
     wpath += warg;
   }
@@ -162,10 +158,10 @@ bool Process::SpawnProcess(const std::string &path, const std::string &arg,
   }
 
   return create_process_succeeded;
-#elif defined(OS_WASM)
-  // Spawning processes is not supported in WASM.
+#elif defined(__wasm__) || defined(__ANDROID__)
+  // Spawning processes is not supported in WASM or Android.
   return false;
-#else  // OS_WASM
+#else  // __wasm__ || __ANDROID__
 
   const std::vector<std::string> arg_tmp =
       absl::StrSplit(arg, ' ', absl::SkipEmpty());
@@ -191,7 +187,7 @@ bool Process::SpawnProcess(const std::string &path, const std::string &arg,
   }
 #endif  // __APPLE__
 
-#if defined(OS_LINUX) || defined(OS_ANDROID)
+#ifdef __linux__
   // Do not call posix_spawn() for obviously bad path.
   if (!S_ISREG(statbuf.st_mode)) {
     LOG(ERROR) << "Not a regular file: " << path;
@@ -214,7 +210,7 @@ bool Process::SpawnProcess(const std::string &path, const std::string &arg,
   // (www.gnu.org/software/libc/manual/html_node/Heap-Consistency-Checking.html)
   constexpr int kOverwrite = 0;  // Do not overwrite.
   ::setenv("MALLOC_CHECK_", "2", kOverwrite);
-#endif  // OS_LINUX || OS_ANDROID
+#endif  // __linux__
   pid_t tmp_pid = 0;
 
   // Spawn new process.
@@ -235,7 +231,7 @@ bool Process::SpawnProcess(const std::string &path, const std::string &arg,
     *pid = tmp_pid;
   }
   return result == 0;
-#endif  // OS_WIN
+#endif  // !_WIN32
 }
 
 bool Process::SpawnMozcProcess(const std::string &filename,
@@ -255,10 +251,11 @@ bool Process::WaitProcess(size_t pid, int timeout) {
     return false;
   }
 
-#ifdef OS_WIN
+#ifdef _WIN32
   DWORD processe_id = static_cast<DWORD>(pid);
-  ScopedHandle handle(::OpenProcess(SYNCHRONIZE, FALSE, processe_id));
-  if (handle.get() == nullptr) {
+  wil::unique_process_handle handle(
+      ::OpenProcess(SYNCHRONIZE, FALSE, processe_id));
+  if (!handle) {
     LOG(ERROR) << "Cannot get handle id";
     return true;
   }
@@ -274,15 +271,15 @@ bool Process::WaitProcess(size_t pid, int timeout) {
   }
 
   return true;
-#elif defined(OS_WASM)
+#elif defined(__wasm__)
   // Process handling is not supported in WASM.
   return false;
-#else   // OS_WASM
+#else   // __wasm__
   pid_t processe_id = static_cast<pid_t>(pid);
   constexpr int kPollingDuration = 250;
   int left_time = timeout < 0 ? 1 : timeout;
   while (left_time > 0) {
-    Util::Sleep(kPollingDuration);
+    absl::SleepFor(absl::Milliseconds(kPollingDuration));
     if (::kill(processe_id, 0) != 0) {
       if (errno == EPERM) {
         return false;  // access defined
@@ -296,7 +293,7 @@ bool Process::WaitProcess(size_t pid, int timeout) {
 
   LOG(ERROR) << pid << " didn't terminate within " << timeout << " msec";
   return false;
-#endif  // OS_WASM
+#endif  // __wasm__
 }
 
 bool Process::IsProcessAlive(size_t pid, bool default_result) {
@@ -304,11 +301,11 @@ bool Process::IsProcessAlive(size_t pid, bool default_result) {
     return default_result;
   }
 
-#ifdef OS_WIN
+#ifdef _WIN32
   {
-    ScopedHandle handle(
+    wil::unique_process_handle handle(
         ::OpenProcess(SYNCHRONIZE, FALSE, static_cast<DWORD>(pid)));
-    if (nullptr == handle.get()) {
+    if (!handle) {
       const DWORD error = ::GetLastError();
       if (error == ERROR_ACCESS_DENIED) {
         LOG(ERROR) << "OpenProcess failed: " << error;
@@ -323,10 +320,10 @@ bool Process::IsProcessAlive(size_t pid, bool default_result) {
   }  // release handle
 
   return default_result;  // unknown
-#elif defined(OS_WASM)
+#elif defined(__wasm__)
   // Process handling is not supported in WASM.
   return false;
-#else   // OS_WIN
+#else   // _WIN32
   constexpr int kSig = 0;
   if (::kill(static_cast<pid_t>(pid), kSig) == -1) {
     if (errno == EPERM || errno == EINVAL) {
@@ -337,19 +334,19 @@ bool Process::IsProcessAlive(size_t pid, bool default_result) {
   }
 
   return true;
-#endif  // OS_WIN
+#endif  // _WIN32
 }
 
 bool Process::IsThreadAlive(size_t thread_id, bool default_result) {
-#ifdef OS_WIN
+#ifdef _WIN32
   if (thread_id == 0) {
     return default_result;
   }
 
   {
-    ScopedHandle handle(
+    wil::unique_process_handle handle(
         ::OpenThread(SYNCHRONIZE, FALSE, static_cast<DWORD>(thread_id)));
-    if (nullptr == handle.get()) {
+    if (!handle) {
       const DWORD error = ::GetLastError();
       if (error == ERROR_ACCESS_DENIED) {
         LOG(ERROR) << "OpenThread failed: " << error;
@@ -365,11 +362,11 @@ bool Process::IsThreadAlive(size_t thread_id, bool default_result) {
 
   return default_result;  // unknown
 
-#else   // OS_WIN
+#else   // _WIN32
   // On Linux/Mac, there is no way to check the status of thread id from
   // other process.
   return default_result;
-#endif  // OS_WNIDOWS
+#endif  // _WIN32
 }
 
 bool Process::LaunchErrorMessageDialog(const std::string &error_type) {
@@ -380,7 +377,7 @@ bool Process::LaunchErrorMessageDialog(const std::string &error_type) {
   }
 #endif  // __APPLE__
 
-#ifdef OS_WIN
+#ifdef _WIN32
   const std::string arg =
       "--mode=error_message_dialog --error_type=" + error_type;
   size_t pid = 0;
@@ -388,9 +385,9 @@ bool Process::LaunchErrorMessageDialog(const std::string &error_type) {
     LOG(ERROR) << "cannot launch " << kMozcTool;
     return false;
   }
-#endif  // OS_WIN
+#endif  // _WIN32
 
-#if defined(OS_LINUX) || defined(OS_ANDROID)
+#if defined(__linux__) && !defined(__ANDROID__)
   constexpr char kMozcTool[] = "mozc_tool";
   const std::string arg =
       "--mode=error_message_dialog --error_type=" + error_type;
@@ -399,7 +396,7 @@ bool Process::LaunchErrorMessageDialog(const std::string &error_type) {
     LOG(ERROR) << "cannot launch " << kMozcTool;
     return false;
   }
-#endif  // OS_LINUX || OS_ANDROID
+#endif  // __linux__ && !__ANDROID__
 
   return true;
 }
