@@ -28,10 +28,18 @@
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include <algorithm>
+#include <climits>
 #include <cmath>
+#include <cstddef>
 #include <cstdint>
-#include <iostream>  // NOLINT
+#include <fstream>
+#include <functional>
+#include <iostream>
 #include <iterator>
+#include <limits>
+#include <memory>
+#include <numeric>
+#include <ostream>
 #include <string>
 #include <vector>
 
@@ -39,16 +47,20 @@
 #include "base/init_mozc.h"
 #include "base/japanese_util.h"
 #include "base/logging.h"
-#include "base/port.h"
 #include "base/singleton.h"
 #include "base/stopwatch.h"
 #include "base/util.h"
-#include "client/client.h"
 #include "config/config_handler.h"
 #include "protocol/commands.pb.h"
+#include "protocol/config.pb.h"
 #include "session/random_keyevents_generator.h"
+#include "absl/algorithm/container.h"
 #include "absl/flags/flag.h"
 #include "absl/strings/str_format.h"
+#include "absl/strings/string_view.h"
+#include "absl/time/time.h"
+#include "absl/types/span.h"
+#include "client/client.h"
 
 ABSL_FLAG(std::string, server_path, "", "specify server path");
 ABSL_FLAG(std::string, log_path, "", "specify log output file path");
@@ -58,30 +70,28 @@ namespace {
 
 struct Result {
   std::string test_name;
-  std::vector<uint32_t> operations_times;
+  std::vector<absl::Duration> operations_times;
 };
 
 class TestSentenceGenerator {
  public:
-  const std::vector<std::vector<commands::KeyEvent> > &GetTestKeys() const {
+  const std::vector<std::vector<commands::KeyEvent>> &GetTestKeys() const {
     return keys_;
   }
 
   TestSentenceGenerator() {
-    size_t size = 0;
-    const char **sentences =
-        session::RandomKeyEventsGenerator::GetTestSentences(&size);
-    CHECK_GT(size, 0);
-    size = std::min<size_t>(200, size);
+    const absl::Span<const char *> sentences =
+        session::RandomKeyEventsGenerator::GetTestSentences();
+    CHECK(!sentences.empty());
+    const size_t size = std::min<size_t>(200, sentences.size());
 
     for (size_t i = 0; i < size; ++i) {
       std::string output;
       japanese_util::HiraganaToRomanji(sentences[i], &output);
       std::vector<commands::KeyEvent> tmp;
       for (ConstChar32Iterator iter(output); !iter.Done(); iter.Next()) {
-        const char32 ucs4 = iter.Get();
-        if (ucs4 >= static_cast<char32>('a') &&
-            ucs4 <= static_cast<char32>('z')) {
+        const char32_t ucs4 = iter.Get();
+        if (ucs4 >= 'a' && ucs4 <= 'z') {
           commands::KeyEvent key;
           key.set_key_code(static_cast<int>(ucs4));
           tmp.push_back(key);
@@ -94,12 +104,12 @@ class TestSentenceGenerator {
   }
 
  private:
-  std::vector<std::vector<commands::KeyEvent> > keys_;
+  std::vector<std::vector<commands::KeyEvent>> keys_;
 };
 
 class TestScenarioInterface {
  public:
-  virtual void Run(Result *result) = 0;
+  virtual Result Run() = 0;
 
   TestScenarioInterface() {
     if (!absl::GetFlag(FLAGS_server_path).empty()) {
@@ -110,7 +120,7 @@ class TestScenarioInterface {
     CHECK(client_.NoOperation()) << "Server is not responding";
   }
 
-  virtual ~TestScenarioInterface() {}
+  virtual ~TestScenarioInterface() = default;
 
  protected:
   virtual void IMEOn() {
@@ -151,48 +161,39 @@ class TestScenarioInterface {
   commands::Output output_;
 };
 
-std::string GetBasicStats(const std::vector<uint32_t> times) {
-  uint32_t total_time = 0;
-  uint32_t avg_time = 0;
-  uint32_t max_time = 0;
-  uint32_t min_time = 0;
-  uint32_t sd_time = 0;  // Standard Deviation
-  uint32_t med_time = 0;
-
-  min_time = INT_MAX;
-  max_time = 0;
-  for (size_t i = 0; i < times.size(); ++i) {
-    total_time += times[i];
-    min_time = std::min(times[i], min_time);
-    max_time = std::max(times[i], max_time);
+std::string GetBasicStats(const std::vector<absl::Duration> &times) {
+  std::vector<uint64_t> temp;
+  temp.resize(times.size());
+  absl::c_transform(times, temp.begin(), absl::ToInt64Microseconds);
+  absl::c_sort(temp);
+  const uint64_t total = absl::c_accumulate(temp, 0);
+  uint64_t mean = 0;
+  uint64_t max = 0;
+  uint64_t min = std::numeric_limits<uint64_t>::max();
+  uint64_t median = 0;
+  if (!temp.empty()) {
+    min = temp.front();
+    max = temp.back();
+    mean = total / times.size();
+    median = temp[temp.size() / 2];
   }
 
-  avg_time = static_cast<uint32_t>(1.0 * total_time / times.size());
-
+  uint64_t stddev = 0;
   if (times.size() >= 2) {
-    double dsd_time = 0;
-    for (size_t i = 0; i < times.size(); ++i) {
-      dsd_time += (avg_time - times[i]) * (avg_time - times[i]);
-    }
-    dsd_time = sqrt(dsd_time / (times.size() - 1));
-    sd_time = static_cast<uint32_t>(dsd_time);
-  }
-
-  if (!times.empty()) {
-    std::vector<uint32_t> tmp(times);
-    std::sort(tmp.begin(), tmp.end());
-    med_time = tmp[tmp.size() / 2];
+    double dsd = std::transform_reduce(
+        temp.begin(), temp.end(), 0.0, std::plus<>(),
+        [mean](uint64_t t) -> double { return (mean - t) * (mean - t); });
+    stddev = static_cast<uint64_t>(sqrt(dsd / (temp.size() - 1)));
   }
 
   return absl::StrFormat("size=%d total=%d avg=%d max=%d min=%d st=%d med=%d",
-                         static_cast<int>(times.size()), total_time, avg_time,
-                         max_time, min_time, sd_time, med_time);
+                         times.size(), total, mean, max, min, stddev, median);
 }
 
 class PreeditCommon : public TestScenarioInterface {
  protected:
   virtual void RunTest(Result *result) {
-    const std::vector<std::vector<commands::KeyEvent> > &keys =
+    const std::vector<std::vector<commands::KeyEvent>> &keys =
         Singleton<TestSentenceGenerator>::get()->GetTestKeys();
     for (size_t i = 0; i < keys.size(); ++i) {
       for (int j = 0; j < keys[i].size(); ++j) {
@@ -200,7 +201,7 @@ class PreeditCommon : public TestScenarioInterface {
         stopwatch.Start();
         client_.SendKey(keys[i][j], &output_);
         stopwatch.Stop();
-        result->operations_times.push_back(stopwatch.GetElapsedMicroseconds());
+        result->operations_times.push_back(stopwatch.GetElapsed());
       }
       commands::SessionCommand command;
       command.set_type(commands::SessionCommand::REVERT);
@@ -211,27 +212,31 @@ class PreeditCommon : public TestScenarioInterface {
 
 class PreeditWithoutSuggestion : public PreeditCommon {
  public:
-  void Run(Result *result) override {
-    result->test_name = "preedit_without_suggestion";
+  Result Run() override {
+    Result result;
+    result.test_name = "preedit_without_suggestion";
     ResetConfig();
     IMEOn();
     DisableSuggestion();
-    RunTest(result);
+    RunTest(&result);
     IMEOff();
     ResetConfig();
+    return result;
   }
 };
 
 class PreeditWithSuggestion : public PreeditCommon {
  public:
-  void Run(Result *result) override {
-    result->test_name = "preedit_with_suggestion";
+  Result Run() override {
+    Result result;
+    result.test_name = "preedit_with_suggestion";
     ResetConfig();
     IMEOn();
     EnableSuggestion();
-    RunTest(result);
+    RunTest(&result);
     IMEOff();
     ResetConfig();
+    return result;
   }
 };
 
@@ -286,7 +291,7 @@ class PredictionCommon : public TestScenarioInterface {
     std::vector<std::string> request_keys;
     CreatePredictionKeys(type, &request_keys);
     for (size_t i = 0; i < request_keys.size(); ++i) {
-      const std::string &keys = request_keys[i];
+      const absl::string_view keys = request_keys[i];
       for (size_t j = 0; j < keys.size(); ++j) {
         commands::KeyEvent key;
         key.set_key_code(static_cast<int>(keys[j]));
@@ -298,7 +303,7 @@ class PredictionCommon : public TestScenarioInterface {
       stopwatch.Start();
       client_.SendKey(key, &output_);
       stopwatch.Stop();
-      result->operations_times.push_back(stopwatch.GetElapsedMicroseconds());
+      result->operations_times.push_back(stopwatch.GetElapsed());
 
       commands::SessionCommand command;
       command.set_type(commands::SessionCommand::REVERT);
@@ -310,29 +315,34 @@ class PredictionCommon : public TestScenarioInterface {
 
 class PredictionWithOneChar : public PredictionCommon {
  public:
-  void Run(Result *result) override {
-    result->test_name = "prediction_one_char";
-    RunTest(ONE_CHAR, result);
+  Result Run() override {
+    Result result;
+    result.test_name = "prediction_one_char";
+    RunTest(ONE_CHAR, &result);
+    return result;
   }
 };
 
 class PredictionWithTwoChars : public PredictionCommon {
  public:
-  void Run(Result *result) override {
-    result->test_name = "prediction_two_chars";
-    RunTest(TWO_CHARS, result);
+  Result Run() override {
+    Result result;
+    result.test_name = "prediction_two_chars";
+    RunTest(TWO_CHARS, &result);
+    return result;
   }
 };
 
 class Conversion : public TestScenarioInterface {
  public:
-  void Run(Result *result) override {
-    result->test_name = "conversion";
+  Result Run() override {
+    Result result;
+    result.test_name = "conversion";
     ResetConfig();
     DisableSuggestion();
     IMEOn();
 
-    const std::vector<std::vector<commands::KeyEvent> > &keys =
+    const std::vector<std::vector<commands::KeyEvent>> &keys =
         Singleton<TestSentenceGenerator>::get()->GetTestKeys();
     for (size_t i = 0; i < keys.size(); ++i) {
       for (int j = 0; j < keys[i].size(); ++j) {
@@ -344,7 +354,7 @@ class Conversion : public TestScenarioInterface {
       stopwatch.Start();
       client_.SendKey(key, &output_);
       stopwatch.Stop();
-      result->operations_times.push_back(stopwatch.GetElapsedMicroseconds());
+      result.operations_times.push_back(stopwatch.GetElapsed());
 
       commands::SessionCommand command;
       command.set_type(commands::SessionCommand::REVERT);
@@ -353,45 +363,44 @@ class Conversion : public TestScenarioInterface {
 
     IMEOff();
     ResetConfig();
+    return result;
   }
 };
+
+void Run(std::ostream &os) {
+  std::vector<std::unique_ptr<TestScenarioInterface>> tests;
+  tests.push_back(std::make_unique<PreeditWithoutSuggestion>());
+  tests.push_back(std::make_unique<PreeditWithSuggestion>());
+  tests.push_back(std::make_unique<Conversion>());
+  tests.push_back(std::make_unique<PredictionWithOneChar>());
+  tests.push_back(std::make_unique<PredictionWithTwoChars>());
+
+  std::vector<Result> results;
+  results.reserve(tests.size());
+  for (const auto &test : tests) {
+    results.push_back(test->Run());
+  }
+
+  CHECK_EQ(results.size(), tests.size());
+
+  // TODO(taku): generate histogram with ChartAPI
+  for (const Result &result : results) {
+    os << result.test_name << ": "
+       << mozc::GetBasicStats(result.operations_times) << std::endl;
+  }
+}
+
 }  // namespace
 }  // namespace mozc
 
 int main(int argc, char **argv) {
   mozc::InitMozc(argv[0], &argc, &argv);
 
-  std::vector<mozc::TestScenarioInterface *> tests;
-  std::vector<mozc::Result *> results;
-
-  tests.push_back(new mozc::PreeditWithoutSuggestion);
-  tests.push_back(new mozc::PreeditWithSuggestion);
-  tests.push_back(new mozc::Conversion);
-  tests.push_back(new mozc::PredictionWithOneChar);
-  tests.push_back(new mozc::PredictionWithTwoChars);
-
-  for (size_t i = 0; i < tests.size(); ++i) {
-    mozc::Result *result = new mozc::Result;
-    tests[i]->Run(result);
-    results.push_back(result);
-  }
-
-  CHECK_EQ(results.size(), tests.size());
-
-  std::ostream *ofs = &std::cout;
   if (!absl::GetFlag(FLAGS_log_path).empty()) {
-    ofs = new mozc::OutputFileStream(absl::GetFlag(FLAGS_log_path).c_str());
-  }
-
-  // TODO(taku): generate histogram with ChartAPI
-  for (size_t i = 0; i < tests.size(); ++i) {
-    (*ofs) << results[i]->test_name << ": "
-           << mozc::GetBasicStats(results[i]->operations_times) << std::endl;
-    delete tests[i];
-    delete results[i];
-  }
-  if (ofs != &std::cout) {
-    delete ofs;
+    std::ofstream ofs = mozc::OutputFileStream(absl::GetFlag(FLAGS_log_path));
+    mozc::Run(ofs);
+  } else {
+    mozc::Run(std::cout);
   }
 
   return 0;
