@@ -30,21 +30,28 @@
 #include "client/client.h"
 
 #include <cstdint>
-#include <map>
 #include <memory>
 #include <string>
+#include <utility>
+#include <vector>
 
 #include "base/logging.h"
 #include "base/number_util.h"
-#include "base/port.h"
+#include "base/strings/assign.h"
 #include "base/version.h"
+#include "composer/key_parser.h"
+#include "config/config_handler.h"
+#include "ipc/ipc.h"
 #include "ipc/ipc_mock.h"
 #include "protocol/commands.pb.h"
 #include "protocol/config.pb.h"
-#include "testing/base/public/gunit.h"
-#include "absl/strings/str_format.h"
+#include "testing/gunit.h"
+#include "absl/container/flat_hash_map.h"
+#include "absl/strings/str_cat.h"
 #include "absl/strings/str_join.h"
 #include "absl/strings/str_split.h"
+#include "absl/strings/string_view.h"
+#include "client/client_interface.h"
 
 namespace mozc {
 namespace client {
@@ -54,14 +61,11 @@ constexpr char kPrecedingText[] = "preceding_text";
 constexpr char kFollowingText[] = "following_text";
 constexpr bool kSuppressSuggestion = true;
 
-const std::string UpdateVersion(int diff) {
+std::string UpdateVersion(int diff) {
   std::vector<std::string> tokens =
       absl::StrSplit(Version::GetMozcVersion(), '.', absl::SkipEmpty());
   EXPECT_EQ(tokens.size(), 4);
-  char buf[64];
-  absl::SNPrintF(buf, sizeof(buf), "%d",
-                 NumberUtil::SimpleAtoi(tokens[3]) + diff);
-  tokens[3] = buf;
+  tokens[3] = absl::StrCat(NumberUtil::SimpleAtoi(tokens[3]) + diff);
   return absl::StrJoin(tokens, ".");
 }
 
@@ -93,7 +97,7 @@ class TestServerLauncher : public ServerLauncherInterface {
     return start_server_result_;
   }
 
-  bool ForceTerminateServer(const std::string &name) override {
+  bool ForceTerminateServer(const absl::string_view name) override {
     force_terminate_server_called_ = true;
     return force_terminate_server_result_;
   }
@@ -123,7 +127,7 @@ class TestServerLauncher : public ServerLauncherInterface {
     force_terminate_server_called_ = force_terminate_server_called;
   }
 
-  void set_server_program(const std::string &server_path) override {}
+  void set_server_program(const absl::string_view server_path) override {}
 
   const std::string &server_program() const override {
     return placeholder_server_program_path_;
@@ -149,8 +153,8 @@ class TestServerLauncher : public ServerLauncherInterface {
     mock_output.SerializeToString(&response_);
   }
 
-  void set_product_version_after_start_server(const std::string &version) {
-    product_version_after_start_server_ = version;
+  void set_product_version_after_start_server(const absl::string_view version) {
+    strings::Assign(product_version_after_start_server_, version);
   }
 
  private:
@@ -163,20 +167,24 @@ class TestServerLauncher : public ServerLauncherInterface {
   uint32_t server_protocol_version_;
   std::string response_;
   std::string product_version_after_start_server_;
-  std::map<int, int> error_map_;
+  absl::flat_hash_map<int, int> error_map_;
 };
 
 class ClientTest : public testing::Test {
  protected:
   ClientTest() : version_diff_(0) {}
+  ClientTest(const ClientTest &) = delete;
+  ClientTest &operator=(const ClientTest &) = delete;
 
   void SetUp() override {
     client_factory_ = std::make_unique<IPCClientFactoryMock>();
     client_ = std::make_unique<Client>();
     client_->SetIPCClientFactory(client_factory_.get());
 
-    server_launcher_ = new TestServerLauncher(client_factory_.get());
-    client_->SetServerLauncher(server_launcher_);
+    auto server_launcher =
+        std::make_unique<TestServerLauncher>(client_factory_.get());
+    server_launcher_ = server_launcher.get();
+    client_->SetServerLauncher(std::move(server_launcher));
   }
 
   void TearDown() override {
@@ -222,9 +230,6 @@ class ClientTest : public testing::Test {
   std::unique_ptr<Client> client_;
   TestServerLauncher *server_launcher_;
   int version_diff_;
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(ClientTest);
 };
 
 TEST_F(ClientTest, ConnectionError) {
@@ -259,12 +264,12 @@ TEST_F(ClientTest, SendKey) {
 
   commands::Output output;
   EXPECT_TRUE(client_->SendKey(key_event, &output));
-  EXPECT_EQ(mock_output.consumed(), output.consumed());
+  EXPECT_EQ(output.consumed(), mock_output.consumed());
 
   commands::Input input;
   GetGeneratedInput(&input);
-  EXPECT_EQ(mock_id, input.id());
-  EXPECT_EQ(commands::Input::SEND_KEY, input.type());
+  EXPECT_EQ(input.id(), mock_id);
+  EXPECT_EQ(input.type(), commands::Input::SEND_KEY);
 }
 
 TEST_F(ClientTest, SendKeyWithContext) {
@@ -286,15 +291,15 @@ TEST_F(ClientTest, SendKeyWithContext) {
 
   commands::Output output;
   EXPECT_TRUE(client_->SendKeyWithContext(key_event, context, &output));
-  EXPECT_EQ(mock_output.consumed(), output.consumed());
+  EXPECT_EQ(output.consumed(), mock_output.consumed());
 
   commands::Input input;
   GetGeneratedInput(&input);
-  EXPECT_EQ(mock_id, input.id());
-  EXPECT_EQ(commands::Input::SEND_KEY, input.type());
-  EXPECT_EQ(kPrecedingText, input.context().preceding_text());
-  EXPECT_EQ(kFollowingText, input.context().following_text());
-  EXPECT_EQ(kSuppressSuggestion, input.context().suppress_suggestion());
+  EXPECT_EQ(input.id(), mock_id);
+  EXPECT_EQ(input.type(), commands::Input::SEND_KEY);
+  EXPECT_EQ(input.context().preceding_text(), kPrecedingText);
+  EXPECT_EQ(input.context().following_text(), kFollowingText);
+  EXPECT_EQ(input.context().suppress_suggestion(), kSuppressSuggestion);
 }
 
 TEST_F(ClientTest, TestSendKey) {
@@ -312,12 +317,12 @@ TEST_F(ClientTest, TestSendKey) {
 
   commands::Output output;
   EXPECT_TRUE(client_->TestSendKey(key_event, &output));
-  EXPECT_EQ(mock_output.consumed(), output.consumed());
+  EXPECT_EQ(output.consumed(), mock_output.consumed());
 
   commands::Input input;
   GetGeneratedInput(&input);
-  EXPECT_EQ(mock_id, input.id());
-  EXPECT_EQ(commands::Input::TEST_SEND_KEY, input.type());
+  EXPECT_EQ(input.id(), mock_id);
+  EXPECT_EQ(input.type(), commands::Input::TEST_SEND_KEY);
 }
 
 TEST_F(ClientTest, TestSendKeyWithContext) {
@@ -340,15 +345,15 @@ TEST_F(ClientTest, TestSendKeyWithContext) {
 
   commands::Output output;
   EXPECT_TRUE(client_->TestSendKeyWithContext(key_event, context, &output));
-  EXPECT_EQ(mock_output.consumed(), output.consumed());
+  EXPECT_EQ(output.consumed(), mock_output.consumed());
 
   commands::Input input;
   GetGeneratedInput(&input);
-  EXPECT_EQ(mock_id, input.id());
-  EXPECT_EQ(commands::Input::TEST_SEND_KEY, input.type());
-  EXPECT_EQ(kPrecedingText, input.context().preceding_text());
-  EXPECT_EQ(kFollowingText, input.context().following_text());
-  EXPECT_EQ(kSuppressSuggestion, input.context().suppress_suggestion());
+  EXPECT_EQ(input.id(), mock_id);
+  EXPECT_EQ(input.type(), commands::Input::TEST_SEND_KEY);
+  EXPECT_EQ(input.context().preceding_text(), kPrecedingText);
+  EXPECT_EQ(input.context().following_text(), kFollowingText);
+  EXPECT_EQ(input.context().suppress_suggestion(), kSuppressSuggestion);
 }
 
 TEST_F(ClientTest, SendCommand) {
@@ -368,8 +373,8 @@ TEST_F(ClientTest, SendCommand) {
 
   commands::Input input;
   GetGeneratedInput(&input);
-  EXPECT_EQ(mock_id, input.id());
-  EXPECT_EQ(commands::Input::SEND_COMMAND, input.type());
+  EXPECT_EQ(input.id(), mock_id);
+  EXPECT_EQ(input.type(), commands::Input::SEND_COMMAND);
 }
 
 TEST_F(ClientTest, SendCommandWithContext) {
@@ -395,11 +400,207 @@ TEST_F(ClientTest, SendCommandWithContext) {
 
   commands::Input input;
   GetGeneratedInput(&input);
-  EXPECT_EQ(mock_id, input.id());
-  EXPECT_EQ(commands::Input::SEND_COMMAND, input.type());
-  EXPECT_EQ(kPrecedingText, input.context().preceding_text());
-  EXPECT_EQ(kFollowingText, input.context().following_text());
-  EXPECT_EQ(kSuppressSuggestion, input.context().suppress_suggestion());
+  EXPECT_EQ(input.id(), mock_id);
+  EXPECT_EQ(input.type(), commands::Input::SEND_COMMAND);
+  EXPECT_EQ(input.context().preceding_text(), kPrecedingText);
+  EXPECT_EQ(input.context().following_text(), kFollowingText);
+  EXPECT_EQ(input.context().suppress_suggestion(), kSuppressSuggestion);
+}
+
+TEST_F(ClientTest, IsDirectModeCommandPresetTest) {
+  const int mock_id = 123;
+  EXPECT_TRUE(SetupConnection(mock_id));
+
+  commands::Output mock_output;
+  mock_output.set_id(mock_id);
+  mock_output.set_consumed(true);
+  SetMockOutput(mock_output);
+
+  config::Config config = config::ConfigHandler::DefaultConfig();
+  config.set_session_keymap(config::Config::ATOK);
+  client_->SetConfig(config);
+  {
+    commands::KeyEvent key;
+    KeyParser::ParseKey("HENKAN", &key);
+    EXPECT_TRUE(client_->IsDirectModeCommand(key));
+  }
+  {
+    commands::KeyEvent key;
+    KeyParser::ParseKey("EISU", &key);
+    EXPECT_FALSE(client_->IsDirectModeCommand(key));
+  }
+  {
+    commands::KeyEvent key;
+    KeyParser::ParseKey("ON", &key);
+    EXPECT_TRUE(client_->IsDirectModeCommand(key));
+  }
+  {
+    // Reconcersion
+    commands::KeyEvent key;
+    KeyParser::ParseKey("Shift HENKAN", &key);
+    EXPECT_TRUE(client_->IsDirectModeCommand(key));
+  }
+
+  config.set_session_keymap(config::Config::MSIME);
+  client_->SetConfig(config);
+  {
+    commands::KeyEvent key;
+    KeyParser::ParseKey("HENKAN", &key);
+    EXPECT_TRUE(client_->IsDirectModeCommand(key));
+  }
+  {
+    commands::KeyEvent key;
+    KeyParser::ParseKey("EISU", &key);
+    EXPECT_TRUE(client_->IsDirectModeCommand(key));
+  }
+  {
+    commands::KeyEvent key;
+    KeyParser::ParseKey("ON", &key);
+    EXPECT_TRUE(client_->IsDirectModeCommand(key));
+  }
+
+  config.set_session_keymap(config::Config::KOTOERI);
+  client_->SetConfig(config);
+  {
+    commands::KeyEvent key;
+    KeyParser::ParseKey("HENKAN", &key);
+    EXPECT_FALSE(client_->IsDirectModeCommand(key));
+  }
+  {
+    commands::KeyEvent key;
+    KeyParser::ParseKey("EISU", &key);
+    EXPECT_FALSE(client_->IsDirectModeCommand(key));
+  }
+  {
+    commands::KeyEvent key;
+    KeyParser::ParseKey("ON", &key);
+    EXPECT_TRUE(client_->IsDirectModeCommand(key));
+  }
+  {
+    // Reconcersion
+    commands::KeyEvent key;
+    KeyParser::ParseKey("Ctrl Shift r", &key);
+    EXPECT_TRUE(client_->IsDirectModeCommand(key));
+  }
+}
+
+TEST_F(ClientTest, IsDirectModeCommandDefaultTest) {
+  const int mock_id = 123;
+  EXPECT_TRUE(SetupConnection(mock_id));
+
+  commands::Output mock_output;
+  mock_output.set_id(mock_id);
+  mock_output.set_consumed(true);
+  SetMockOutput(mock_output);
+
+  config::Config config = config::ConfigHandler::DefaultConfig();
+  config.set_session_keymap(config::Config::NONE);
+  client_->SetConfig(config);
+  {
+    commands::KeyEvent key;
+    KeyParser::ParseKey("HENKAN", &key);
+    // HENKAN key in MSIME is TurnOn key while it's not in KOTOERI.
+    if (config::ConfigHandler::GetDefaultKeyMap() == config::Config::MSIME) {
+      EXPECT_TRUE(client_->IsDirectModeCommand(key));
+    } else {
+      EXPECT_FALSE(client_->IsDirectModeCommand(key));
+    }
+  }
+  {
+    commands::KeyEvent key;
+    KeyParser::ParseKey("EISU", &key);
+    if (config::ConfigHandler::GetDefaultKeyMap() == config::Config::MSIME) {
+      EXPECT_TRUE(client_->IsDirectModeCommand(key));
+    } else {
+      EXPECT_FALSE(client_->IsDirectModeCommand(key));
+    }
+  }
+  {
+    commands::KeyEvent key;
+    KeyParser::ParseKey("ON", &key);
+    key.set_special_key(commands::KeyEvent::ON);
+    if (config::ConfigHandler::GetDefaultKeyMap() == config::Config::CHROMEOS) {
+      EXPECT_FALSE(client_->IsDirectModeCommand(key));
+    } else {
+      EXPECT_TRUE(client_->IsDirectModeCommand(key));
+    }
+  }
+}
+
+TEST_F(ClientTest, IsDirectModeCommandFailureTest) {
+  // As SetupConnection is not called, SetConfig fails to update the config.
+
+  config::Config config = config::ConfigHandler::DefaultConfig();
+  const bool is_kotoeri = (config.session_keymap() == config::Config::KOTOERI);
+  config.set_session_keymap(config::Config::ATOK);
+  // SetConfig should be failed.
+  EXPECT_FALSE(client_->SetConfig(config));
+  {
+    commands::KeyEvent key;
+    KeyParser::ParseKey(is_kotoeri ? "Ctrl Shift r" : "HENKAN", &key);
+    EXPECT_TRUE(client_->IsDirectModeCommand(key));
+  }
+  {
+    // Make sure that the keymap is not updated with no connections.
+    // "Shift HENKAN" is not a direct mode command in the default keymap.
+    commands::KeyEvent key;
+    KeyParser::ParseKey("Shift HENKAN", &key);
+    EXPECT_FALSE(client_->IsDirectModeCommand(key));
+  }
+}
+
+TEST_F(ClientTest, IsDirectModeCommandCustomTest) {
+  const int mock_id = 123;
+  EXPECT_TRUE(SetupConnection(mock_id));
+
+  commands::Output mock_output;
+  mock_output.set_id(mock_id);
+  mock_output.set_consumed(true);
+  SetMockOutput(mock_output);
+
+  config::Config config = config::ConfigHandler::DefaultConfig();
+
+  const std::string custom_keymap_table =
+      "status\tkey\tcommand\n"
+      "DirectInput\tHenkan\tIMEOn\n"
+      "DirectInput\tCtrl j\tIMEOn\n"
+      "DirectInput\tCtrl k\tIMEOff\n"
+      "DirectInput\tCtrl l\tLaunchWordRegisterDialog\n"
+      "Precomposition\tCtrl m\tIMEOn\n";
+
+  config.set_session_keymap(config::Config::CUSTOM);
+  config.set_custom_keymap_table(custom_keymap_table);
+  client_->SetConfig(config);
+  {
+    commands::KeyEvent key;
+    KeyParser::ParseKey("HENKAN", &key);
+    EXPECT_TRUE(client_->IsDirectModeCommand(key));
+  }
+  {
+    commands::KeyEvent key;
+    KeyParser::ParseKey("EISU", &key);
+    EXPECT_FALSE(client_->IsDirectModeCommand(key));
+  }
+  {
+    commands::KeyEvent key;
+    KeyParser::ParseKey("ctrl j", &key);
+    EXPECT_TRUE(client_->IsDirectModeCommand(key));
+  }
+  {
+    commands::KeyEvent key;
+    KeyParser::ParseKey("ctrl k", &key);
+    EXPECT_TRUE(client_->IsDirectModeCommand(key));
+  }
+  {
+    commands::KeyEvent key;
+    KeyParser::ParseKey("ctrl l", &key);
+    EXPECT_TRUE(client_->IsDirectModeCommand(key));
+  }
+  {
+    commands::KeyEvent key;
+    KeyParser::ParseKey("ctrl m", &key);
+    EXPECT_FALSE(client_->IsDirectModeCommand(key));
+  }
 }
 
 TEST_F(ClientTest, SetConfig) {
@@ -423,8 +624,8 @@ TEST_F(ClientTest, GetConfig) {
   config::Config config;
   EXPECT_TRUE(client_->GetConfig(&config));
 
-  EXPECT_EQ(2, config.verbose_level());
-  EXPECT_EQ(true, config.incognito_mode());
+  EXPECT_EQ(config.verbose_level(), 2);
+  EXPECT_EQ(config.incognito_mode(), true);
 }
 
 TEST_F(ClientTest, EnableCascadingWindow) {
@@ -639,24 +840,54 @@ TEST_F(ClientTest, TranslateProtoBufToMozcToolArgTest) {
 
   // If no value is set, we expect to return false
   EXPECT_FALSE(client::Client::TranslateProtoBufToMozcToolArg(output, &mode));
-  EXPECT_EQ("", mode);
+  EXPECT_EQ(mode, "");
 
   // If NO_TOOL is set, we  expect to return false
   output.set_launch_tool_mode(commands::Output::NO_TOOL);
   EXPECT_FALSE(client::Client::TranslateProtoBufToMozcToolArg(output, &mode));
-  EXPECT_EQ("", mode);
+  EXPECT_EQ(mode, "");
 
   output.set_launch_tool_mode(commands::Output::CONFIG_DIALOG);
   EXPECT_TRUE(client::Client::TranslateProtoBufToMozcToolArg(output, &mode));
-  EXPECT_EQ("config_dialog", mode);
+  EXPECT_EQ(mode, "config_dialog");
 
   output.set_launch_tool_mode(commands::Output::DICTIONARY_TOOL);
   EXPECT_TRUE(client::Client::TranslateProtoBufToMozcToolArg(output, &mode));
-  EXPECT_EQ("dictionary_tool", mode);
+  EXPECT_EQ(mode, "dictionary_tool");
 
   output.set_launch_tool_mode(commands::Output::WORD_REGISTER_DIALOG);
   EXPECT_TRUE(client::Client::TranslateProtoBufToMozcToolArg(output, &mode));
-  EXPECT_EQ("word_register_dialog", mode);
+  EXPECT_EQ(mode, "word_register_dialog");
+}
+
+TEST_F(ClientTest, InitRequestForSvsJapaneseTest) {
+  const int mock_id = 1;
+  EXPECT_TRUE(SetupConnection(mock_id));
+
+  client_->InitRequestForSvsJapanese(true);
+  EXPECT_TRUE(client_->EnsureSession());
+
+  commands::Input input;
+  GetGeneratedInput(&input);
+  EXPECT_TRUE(input.has_request());
+  EXPECT_TRUE(
+      input.request().decoder_experiment_params().variation_character_types() &
+      commands::DecoderExperimentParams::SVS_JAPANESE);
+}
+
+TEST_F(ClientTest, NoInitRequestForSvsJapaneseTest) {
+  const int mock_id = 1;
+  EXPECT_TRUE(SetupConnection(mock_id));
+
+  client_->InitRequestForSvsJapanese(false);
+  EXPECT_TRUE(client_->EnsureSession());
+
+  commands::Input input;
+  GetGeneratedInput(&input);
+  EXPECT_TRUE(input.has_request());
+  EXPECT_FALSE(
+      input.request().decoder_experiment_params().variation_character_types() &
+      commands::DecoderExperimentParams::SVS_JAPANESE);
 }
 
 class SessionPlaybackTestServerLauncher : public ServerLauncherInterface {
@@ -685,7 +916,7 @@ class SessionPlaybackTestServerLauncher : public ServerLauncherInterface {
     return start_server_result_;
   }
 
-  bool ForceTerminateServer(const std::string &name) override {
+  bool ForceTerminateServer(const absl::string_view name) override {
     force_terminate_server_called_ = true;
     return force_terminate_server_result_;
   }
@@ -694,7 +925,7 @@ class SessionPlaybackTestServerLauncher : public ServerLauncherInterface {
 
   void OnFatal(ServerLauncherInterface::ServerErrorType type) override {}
 
-  void set_server_program(const std::string &server_path) override {}
+  void set_server_program(const absl::string_view server_path) override {}
 
   void set_restricted(bool restricted) override {}
 
@@ -705,8 +936,8 @@ class SessionPlaybackTestServerLauncher : public ServerLauncherInterface {
   }
 
   const std::string &server_program() const override {
-    static const std::string path;
-    return path;
+    static std::string *path = new std::string();
+    return *path;
   }
 
  private:
@@ -718,23 +949,20 @@ class SessionPlaybackTestServerLauncher : public ServerLauncherInterface {
   uint32_t server_protocol_version_;
   std::string response_;
   std::string product_version_after_start_server_;
-  std::map<int, int> error_map_;
+  absl::flat_hash_map<int, int> error_map_;
 };
 
 class SessionPlaybackTest : public testing::Test {
  protected:
-  SessionPlaybackTest() {}
-  ~SessionPlaybackTest() override {}
-
   void SetUp() override {
     ipc_client_factory_ = std::make_unique<IPCClientFactoryMock>();
-    ipc_client_.reset(
-        reinterpret_cast<IPCClientMock *>(ipc_client_factory_->NewClient("")));
+    ipc_client_ = ipc_client_factory_->NewClient("");
     client_ = std::make_unique<Client>();
     client_->SetIPCClientFactory(ipc_client_factory_.get());
-    server_launcher_ =
-        new SessionPlaybackTestServerLauncher(ipc_client_factory_.get());
-    client_->SetServerLauncher(server_launcher_);
+    auto server_launcher = std::make_unique<SessionPlaybackTestServerLauncher>(
+        ipc_client_factory_.get());
+    server_launcher_ = server_launcher.get();
+    client_->SetServerLauncher(std::move(server_launcher));
   }
 
   void TearDown() override {
@@ -764,12 +992,9 @@ class SessionPlaybackTest : public testing::Test {
   }
 
   std::unique_ptr<IPCClientFactoryMock> ipc_client_factory_;
-  std::unique_ptr<IPCClientMock> ipc_client_;
+  std::unique_ptr<IPCClientInterface> ipc_client_;
   std::unique_ptr<Client> client_;
   SessionPlaybackTestServerLauncher *server_launcher_;
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(SessionPlaybackTest);
 };
 
 // b/2797557
@@ -787,11 +1012,11 @@ TEST_F(SessionPlaybackTest, PushAndResetHistoryWithNoModeTest) {
 
   commands::Output output;
   EXPECT_TRUE(client_->SendKey(key_event, &output));
-  EXPECT_EQ(mock_output.consumed(), output.consumed());
+  EXPECT_EQ(output.consumed(), mock_output.consumed());
 
   std::vector<commands::Input> history;
   client_->GetHistoryInputs(&history);
-  EXPECT_EQ(1, history.size());
+  EXPECT_EQ(history.size(), 1);
 
   mock_output.Clear();
   mock_output.set_id(mock_id);
@@ -801,11 +1026,11 @@ TEST_F(SessionPlaybackTest, PushAndResetHistoryWithNoModeTest) {
   EXPECT_FALSE(mock_output.has_mode());
   SetMockOutput(mock_output);
   EXPECT_TRUE(client_->SendKey(key_event, &output));
-  EXPECT_EQ(mock_output.consumed(), output.consumed());
+  EXPECT_EQ(output.consumed(), mock_output.consumed());
 
   // history should be reset.
   client_->GetHistoryInputs(&history);
-  EXPECT_EQ(0, history.size());
+  EXPECT_EQ(history.size(), 0);
 }
 
 // b/2797557
@@ -826,18 +1051,18 @@ TEST_F(SessionPlaybackTest, PushAndResetHistoryWithModeTest) {
 
   commands::Output output;
   EXPECT_TRUE(client_->SendKey(key_event, &output));
-  EXPECT_EQ(mock_output.consumed(), output.consumed());
+  EXPECT_EQ(output.consumed(), mock_output.consumed());
   EXPECT_TRUE(output.has_mode());
-  EXPECT_EQ(commands::HIRAGANA, output.mode());
+  EXPECT_EQ(output.mode(), commands::HIRAGANA);
 
   EXPECT_TRUE(client_->SendKey(key_event, &output));
-  EXPECT_EQ(mock_output.consumed(), output.consumed());
+  EXPECT_EQ(output.consumed(), mock_output.consumed());
   EXPECT_TRUE(output.has_mode());
-  EXPECT_EQ(commands::HIRAGANA, output.mode());
+  EXPECT_EQ(output.mode(), commands::HIRAGANA);
 
   std::vector<commands::Input> history;
   client_->GetHistoryInputs(&history);
-  EXPECT_EQ(2, history.size());
+  EXPECT_EQ(history.size(), 2);
 
   mock_output.Clear();
   mock_output.set_id(mock_id);
@@ -846,20 +1071,20 @@ TEST_F(SessionPlaybackTest, PushAndResetHistoryWithModeTest) {
   mock_output.mutable_result()->set_value("output");
   SetMockOutput(mock_output);
   EXPECT_TRUE(client_->SendKey(key_event, &output));
-  EXPECT_EQ(mock_output.consumed(), output.consumed());
+  EXPECT_EQ(output.consumed(), mock_output.consumed());
   client_->GetHistoryInputs(&history);
 #ifdef __APPLE__
   // history is reset, but initializer should be added because the last mode
   // is not DIRECT.
   // TODO(team): fix b/10250883 to remove this special treatment.
-  EXPECT_EQ(1, history.size());
+  EXPECT_EQ(history.size(), 1);
   // Implicit IMEOn key must be added. See b/2797557 and b/>10250883.
-  EXPECT_EQ(commands::Input::SEND_KEY, history[0].type());
-  EXPECT_EQ(commands::KeyEvent::ON, history[0].key().special_key());
-  EXPECT_EQ(commands::HIRAGANA, history[0].key().mode());
+  EXPECT_EQ(history[0].type(), commands::Input::SEND_KEY);
+  EXPECT_EQ(history[0].key().special_key(), commands::KeyEvent::ON);
+  EXPECT_EQ(history[0].key().mode(), commands::HIRAGANA);
 #else   // __APPLE__
   // history is reset, but initializer is not required.
-  EXPECT_EQ(0, history.size());
+  EXPECT_EQ(history.size(), 0);
 #endif  // __APPLE__
 }
 
@@ -880,18 +1105,18 @@ TEST_F(SessionPlaybackTest, PushAndResetHistoryWithDirectTest) {
   commands::Output output;
   // Send key twice
   EXPECT_TRUE(client_->SendKey(key_event, &output));
-  EXPECT_EQ(mock_output.consumed(), output.consumed());
+  EXPECT_EQ(output.consumed(), mock_output.consumed());
   EXPECT_TRUE(output.has_mode());
-  EXPECT_EQ(commands::DIRECT, output.mode());
+  EXPECT_EQ(output.mode(), commands::DIRECT);
 
   EXPECT_TRUE(client_->SendKey(key_event, &output));
-  EXPECT_EQ(mock_output.consumed(), output.consumed());
+  EXPECT_EQ(output.consumed(), mock_output.consumed());
   EXPECT_TRUE(output.has_mode());
-  EXPECT_EQ(commands::DIRECT, output.mode());
+  EXPECT_EQ(output.mode(), commands::DIRECT);
 
   std::vector<commands::Input> history;
   client_->GetHistoryInputs(&history);
-  EXPECT_EQ(2, history.size());
+  EXPECT_EQ(history.size(), 2);
 
   mock_output.Clear();
   mock_output.set_id(mock_id);
@@ -900,11 +1125,11 @@ TEST_F(SessionPlaybackTest, PushAndResetHistoryWithDirectTest) {
   mock_output.mutable_result()->set_value("output");
   SetMockOutput(mock_output);
   EXPECT_TRUE(client_->SendKey(key_event, &output));
-  EXPECT_EQ(mock_output.consumed(), output.consumed());
+  EXPECT_EQ(output.consumed(), mock_output.consumed());
 
   // history is reset, and initializer should not be added.
   client_->GetHistoryInputs(&history);
-  EXPECT_EQ(0, history.size());
+  EXPECT_EQ(history.size(), 0);
 }
 
 TEST_F(SessionPlaybackTest, PlaybackHistoryTest) {
@@ -922,14 +1147,14 @@ TEST_F(SessionPlaybackTest, PlaybackHistoryTest) {
 
   commands::Output output;
   EXPECT_TRUE(client_->SendKey(key_event, &output));
-  EXPECT_EQ(mock_output.consumed(), output.consumed());
+  EXPECT_EQ(output.consumed(), mock_output.consumed());
 
   EXPECT_TRUE(client_->SendKey(key_event, &output));
-  EXPECT_EQ(mock_output.consumed(), output.consumed());
+  EXPECT_EQ(output.consumed(), mock_output.consumed());
 
   std::vector<commands::Input> history;
   client_->GetHistoryInputs(&history);
-  EXPECT_EQ(2, history.size());
+  EXPECT_EQ(history.size(), 2);
 
   // Invalid id
   const int new_id = 456;
@@ -940,11 +1165,11 @@ TEST_F(SessionPlaybackTest, PlaybackHistoryTest) {
 #ifndef DEBUG
   // PlaybackHistory and push history
   client_->GetHistoryInputs(&history);
-  EXPECT_EQ(3, history.size());
+  EXPECT_EQ(history.size(), 3);
 #else   // DEBUG
   // PlaybackHistory, dump history(including reset), and add last input
   client_->GetHistoryInputs(&history);
-  EXPECT_EQ(1, history.size());
+  EXPECT_EQ(history.size(), 1);
 #endif  // DEBUG
 }
 
@@ -964,27 +1189,27 @@ TEST_F(SessionPlaybackTest, SetModeInitializerTest) {
 
   commands::Output output;
   EXPECT_TRUE(client_->SendKey(key_event, &output));
-  EXPECT_EQ(mock_output.consumed(), output.consumed());
+  EXPECT_EQ(output.consumed(), mock_output.consumed());
 
   mock_output.set_mode(commands::DIRECT);
   SetMockOutput(mock_output);
 
   EXPECT_TRUE(client_->SendKey(key_event, &output));
-  EXPECT_EQ(mock_output.consumed(), output.consumed());
+  EXPECT_EQ(output.consumed(), mock_output.consumed());
   EXPECT_TRUE(output.has_mode());
-  EXPECT_EQ(commands::DIRECT, output.mode());
+  EXPECT_EQ(output.mode(), commands::DIRECT);
 
   mock_output.set_mode(commands::FULL_KATAKANA);
   SetMockOutput(mock_output);
 
   EXPECT_TRUE(client_->SendKey(key_event, &output));
-  EXPECT_EQ(mock_output.consumed(), output.consumed());
+  EXPECT_EQ(output.consumed(), mock_output.consumed());
   EXPECT_TRUE(output.has_mode());
-  EXPECT_EQ(commands::FULL_KATAKANA, output.mode());
+  EXPECT_EQ(output.mode(), commands::FULL_KATAKANA);
 
   std::vector<commands::Input> history;
   client_->GetHistoryInputs(&history);
-  EXPECT_EQ(3, history.size());
+  EXPECT_EQ(history.size(), 3);
 
   mock_output.Clear();
   mock_output.set_id(mock_id);
@@ -993,18 +1218,18 @@ TEST_F(SessionPlaybackTest, SetModeInitializerTest) {
   mock_output.mutable_result()->set_value("output");
   SetMockOutput(mock_output);
   EXPECT_TRUE(client_->SendKey(key_event, &output));
-  EXPECT_EQ(mock_output.consumed(), output.consumed());
+  EXPECT_EQ(output.consumed(), mock_output.consumed());
   client_->GetHistoryInputs(&history);
 #ifdef __APPLE__
   // history is reset, but initializer should be added.
   // TODO(team): fix b/10250883 to remove this special treatment.
-  EXPECT_EQ(1, history.size());
-  EXPECT_EQ(commands::Input::SEND_KEY, history[0].type());
-  EXPECT_EQ(commands::KeyEvent::ON, history[0].key().special_key());
-  EXPECT_EQ(commands::FULL_KATAKANA, history[0].key().mode());
+  EXPECT_EQ(history.size(), 1);
+  EXPECT_EQ(history[0].type(), commands::Input::SEND_KEY);
+  EXPECT_EQ(history[0].key().special_key(), commands::KeyEvent::ON);
+  EXPECT_EQ(history[0].key().mode(), commands::FULL_KATAKANA);
 #else   // __APPLE__
   // history is reset, but initializer is not required.
-  EXPECT_EQ(0, history.size());
+  EXPECT_EQ(history.size(), 0);
 #endif  // __APPLE__
 }
 
@@ -1022,24 +1247,24 @@ TEST_F(SessionPlaybackTest, ConsumedTest) {
 
   commands::Output output;
   EXPECT_TRUE(client_->SendKey(key_event, &output));
-  EXPECT_EQ(mock_output.consumed(), output.consumed());
+  EXPECT_EQ(output.consumed(), mock_output.consumed());
 
   EXPECT_TRUE(client_->SendKey(key_event, &output));
-  EXPECT_EQ(mock_output.consumed(), output.consumed());
+  EXPECT_EQ(output.consumed(), mock_output.consumed());
 
   std::vector<commands::Input> history;
   client_->GetHistoryInputs(&history);
-  EXPECT_EQ(2, history.size());
+  EXPECT_EQ(history.size(), 2);
 
   mock_output.set_consumed(false);
   SetMockOutput(mock_output);
 
   EXPECT_TRUE(client_->SendKey(key_event, &output));
-  EXPECT_EQ(mock_output.consumed(), output.consumed());
+  EXPECT_EQ(output.consumed(), mock_output.consumed());
 
   // Do not push unconsumed input
   client_->GetHistoryInputs(&history);
-  EXPECT_EQ(2, history.size());
+  EXPECT_EQ(history.size(), 2);
 }
 }  // namespace client
 }  // namespace mozc
