@@ -31,41 +31,45 @@
 
 #include "composer/composer.h"
 
+#include <cstddef>
 #include <cstdint>
+#include <optional>
+#include <set>
+#include <string>
 #include <unordered_map>
+#include <utility>
+#include <vector>
 
 #include "base/clock.h"
 #include "base/japanese_util.h"
 #include "base/logging.h"
+#include "base/strings/assign.h"
+#include "base/strings/unicode.h"
 #include "base/util.h"
 #include "composer/internal/composition.h"
 #include "composer/internal/composition_input.h"
 #include "composer/internal/mode_switching_handler.h"
 #include "composer/internal/transliterators.h"
-#include "composer/internal/typing_corrector.h"
 #include "composer/key_event_util.h"
 #include "composer/table.h"
-#include "composer/type_corrected_query.h"
 #include "config/character_form_manager.h"
 #include "config/config_handler.h"
 #include "protocol/commands.pb.h"
 #include "protocol/config.pb.h"
-#include "absl/flags/flag.h"
+#include "spelling/spellchecker_service_interface.h"
+#include "transliteration/transliteration.h"
+#include "absl/strings/match.h"
+#include "absl/strings/string_view.h"
 #include "absl/time/time.h"
 
-// Use flags instead of constant for performance evaluation.
-ABSL_FLAG(uint64_t, max_typing_correction_query_candidates, 40,
-          "Maximum # of typing correction query temporary candidates.");
-ABSL_FLAG(uint64_t, max_typing_correction_query_results, 8,
-          "Maximum # of typing correction query results.");
-
 namespace mozc {
 namespace composer {
 namespace {
 
 using ::mozc::config::CharacterFormManager;
+using ::mozc::strings::OneCharLen;
 
-const Transliterators::Transliterator GetTransliterator(
+Transliterators::Transliterator GetTransliterator(
     transliteration::TransliterationType comp_mode) {
   switch (comp_mode) {
     case transliteration::HALF_ASCII:
@@ -117,7 +121,7 @@ transliteration::TransliterationType GetTransliterationType(
 }
 
 void Transliterate(const transliteration::TransliterationType mode,
-                   const std::string &input, std::string *output) {
+                   const absl::string_view input, std::string *output) {
   // When the mode is HALF_KATAKANA, Full width ASCII is also
   // transformed.
   if (mode == transliteration::HALF_KATAKANA) {
@@ -164,11 +168,11 @@ void Transliterate(const transliteration::TransliterationType mode,
       japanese_util::HiraganaToKatakana(input, output);
       break;
     case transliteration::HIRAGANA:
-      *output = input;
+      strings::Assign(*output, input);
       break;
     default:
       LOG(ERROR) << "Unknown TransliterationType: " << mode;
-      *output = input;
+      strings::Assign(*output, input);
       break;
   }
 }
@@ -204,7 +208,9 @@ transliteration::TransliterationType GetTransliterationTypeFromCompositionMode(
 // whereby we can suppress prediction from unmodified key when one modified a
 // character explicitly (e.g., we don't want to suggest words starting with
 // "さ" when one typed "ざ" with modified key).
-using ModifierRemovalMap = std::unordered_multimap<std::string, std::string>;
+using ModifierRemovalMap =
+    std::unordered_multimap<absl::string_view, absl::string_view,
+                            absl::Hash<absl::string_view>>;
 
 const ModifierRemovalMap *GetModifierRemovalMap() {
   static const ModifierRemovalMap *const removal_map = new ModifierRemovalMap{
@@ -222,6 +228,21 @@ const ModifierRemovalMap *GetModifierRemovalMap() {
   return removal_map;
 }
 
+void RemoveExpandedCharsForModifier(absl::string_view asis,
+                                    absl::string_view base,
+                                    std::set<std::string> *expanded) {
+  if (!absl::StartsWith(asis, base)) {
+    LOG(DFATAL) << "base is not a prefix of asis.";
+    return;
+  }
+
+  const absl::string_view trailing(asis.substr(base.size()));
+  for (auto [iter, end] = GetModifierRemovalMap()->equal_range(trailing);
+       iter != end; ++iter) {
+    expanded->erase(std::string(iter->second));
+  }
+}
+
 constexpr size_t kMaxPreeditLength = 256;
 
 }  // namespace
@@ -240,32 +261,23 @@ Composer::Composer(const Table *table, const commands::Request *request,
       input_field_type_(commands::Context::NORMAL),
       shifted_sequence_count_(0),
       composition_(table),
-      typing_corrector_(
-          table, absl::GetFlag(FLAGS_max_typing_correction_query_candidates),
-          absl::GetFlag(FLAGS_max_typing_correction_query_results)),
       max_length_(kMaxPreeditLength),
       request_(request),
       config_(config),
+      table_(table),
       is_new_input_(true) {
   SetInputMode(transliteration::HIRAGANA);
   if (config_ == nullptr) {
     config_ = &config::ConfigHandler::DefaultConfig();
   }
-  typing_corrector_.SetConfig(config_);
   Reset();
 }
 
-Composer::Composer(const Composer &) = default;
-Composer &Composer::operator=(const Composer &) = default;
-
-Composer::~Composer() = default;
-
 void Composer::Reset() {
   EditErase();
   ResetInputMode();
   SetOutputMode(transliteration::HIRAGANA);
-  source_text_.assign("");
-  typing_corrector_.Reset();
+  source_text_.clear();
   timeout_threshold_msec_ = config_->composing_timeout_threshold_msec();
 }
 
@@ -278,20 +290,21 @@ void Composer::ReloadConfig() {
 bool Composer::Empty() const { return (GetLength() == 0); }
 
 void Composer::SetTable(const Table *table) {
+  table_ = table;
   composition_.SetTable(table);
-
-  typing_corrector_.SetTable(table);
 }
 
 void Composer::SetRequest(const commands::Request *request) {
   request_ = request;
 }
 
-void Composer::SetConfig(const config::Config *config) {
-  config_ = config;
-  typing_corrector_.SetConfig(config);
+void Composer::SetSpellCheckerService(
+    const spelling::SpellCheckerServiceInterface *spellchecker_service) {
+  spellchecker_service_ = spellchecker_service;
 }
 
+void Composer::SetConfig(const config::Config *config) { config_ = config; }
+
 void Composer::SetInputMode(transliteration::TransliterationType mode) {
   comeback_input_mode_ = mode;
   input_mode_ = mode;
@@ -360,7 +373,7 @@ void Composer::SetOutputMode(transliteration::TransliterationType mode) {
   position_ = composition_.GetLength();
 }
 
-void Composer::ApplyTemporaryInputMode(const std::string &input,
+void Composer::ApplyTemporaryInputMode(const absl::string_view input,
                                        bool caps_locked) {
   DCHECK(!input.empty());
 
@@ -369,7 +382,7 @@ void Composer::ApplyTemporaryInputMode(const std::string &input,
 
   // When input is not an ASCII code, reset the input mode to the one before
   // temporary input mode.
-  if (Util::OneCharLen(input.c_str()) != 1) {
+  if (OneCharLen(input.front()) != 1) {
     // Call SetInputMode() only when the current input mode is temporary, which
     // is detected by the if-condition below.  Without this check,
     // SetInputMode() is called always for multi-byte charactesrs.  This causes
@@ -382,7 +395,7 @@ void Composer::ApplyTemporaryInputMode(const std::string &input,
   }
 
   // Input is an ASCII code.
-  // we use first character to determin temporary input mode.
+  // we use first character to determine temporary input mode.
   const char key = input[0];
   const bool alpha_with_shift = (!caps_locked && ('A' <= key && key <= 'Z')) ||
                                 (caps_locked && ('a' <= key && key <= 'z'));
@@ -425,42 +438,44 @@ void Composer::ApplyTemporaryInputMode(const std::string &input,
   }
 }
 
-bool Composer::ProcessCompositionInput(const CompositionInput &input) {
+bool Composer::ProcessCompositionInput(CompositionInput input) {
   if (!EnableInsert()) {
     return false;
   }
 
-  position_ = composition_.InsertInput(position_, input);
+  position_ = composition_.InsertInput(position_, std::move(input));
   is_new_input_ = false;
-  typing_corrector_.InsertCharacter(input.raw(), input.probable_key_events());
   return true;
 }
 
-void Composer::InsertCharacter(const std::string &key) {
+void Composer::InsertCharacter(std::string key) {
   CompositionInput input;
-  input.InitFromRaw(key, is_new_input_);
-  ProcessCompositionInput(input);
+  input.InitFromRaw(std::move(key), is_new_input_);
+  ProcessCompositionInput(std::move(input));
 }
 
 void Composer::InsertCommandCharacter(const InternalCommand internal_command) {
+  CompositionInput input;
   switch (internal_command) {
     case REWIND:
-      InsertCharacter(Table::ParseSpecialKey("{<}"));
+      input.InitFromRaw(table_->ParseSpecialKey("{<}"), is_new_input_);
+      ProcessCompositionInput(std::move(input));
       break;
     case STOP_KEY_TOGGLING:
-      InsertCharacter(Table::ParseSpecialKey("{!}"));
+      input.InitFromRaw(table_->ParseSpecialKey("{!}"), is_new_input_);
+      ProcessCompositionInput(std::move(input));
       break;
     default:
       LOG(ERROR) << "Unknown command : " << internal_command;
   }
 }
 
-void Composer::InsertCharacterPreedit(const std::string &input) {
+void Composer::InsertCharacterPreedit(const absl::string_view input) {
   size_t begin = 0;
   const size_t end = input.size();
   while (begin < end) {
-    const size_t mblen = Util::OneCharLen(input.c_str() + begin);
-    const std::string character(input, begin, mblen);
+    const size_t mblen = OneCharLen(input[begin]);
+    const absl::string_view character(input.substr(begin, mblen));
     if (!InsertCharacterKeyAndPreedit(character, character)) {
       return;
     }
@@ -470,23 +485,23 @@ void Composer::InsertCharacterPreedit(const std::string &input) {
 }
 
 // Note: This method is only for test.
-void Composer::SetPreeditTextForTestOnly(const std::string &input) {
+void Composer::SetPreeditTextForTestOnly(const absl::string_view input) {
   composition_.SetInputMode(Transliterators::RAW_STRING);
   size_t begin = 0;
   const size_t end = input.size();
   while (begin < end) {
-    const size_t mblen = Util::OneCharLen(input.c_str() + begin);
-    const std::string character(input, begin, mblen);
-    CompositionInput input;
-    input.set_raw(character);
-    input.set_is_new_input(is_new_input_);
-    position_ = composition_.InsertInput(position_, input);
+    const size_t mblen = OneCharLen(input[begin]);
+    CompositionInput composition_input;
+    composition_input.set_raw(input.substr(begin, mblen));
+    composition_input.set_is_new_input(is_new_input_);
+    position_ =
+        composition_.InsertInput(position_, std::move(composition_input));
     is_new_input_ = false;
     begin += mblen;
   }
   DCHECK_EQ(begin, end);
 
-  std::string lower_input = input;
+  std::string lower_input(input);
   Util::LowerString(&lower_input);
   if (Util::IsLowerAscii(lower_input)) {
     // Fake input mode.
@@ -495,11 +510,12 @@ void Composer::SetPreeditTextForTestOnly(const std::string &input) {
   }
 }
 
-bool Composer::InsertCharacterKeyAndPreedit(const std::string &key,
-                                            const std::string &preedit) {
+bool Composer::InsertCharacterKeyAndPreedit(const absl::string_view key,
+                                            const absl::string_view preedit) {
   CompositionInput input;
-  input.InitFromRawAndConv(key, preedit, is_new_input_);
-  return ProcessCompositionInput(input);
+  input.InitFromRawAndConv(std::string(key), std::string(preedit),
+                           is_new_input_);
+  return ProcessCompositionInput(std::move(input));
 }
 
 bool Composer::InsertCharacterKeyEvent(const commands::KeyEvent &key) {
@@ -545,28 +561,28 @@ bool Composer::InsertCharacterKeyEvent(const commands::KeyEvent &key) {
   }
 
   CompositionInput input;
-  if (!input.Init(key, config_->use_typing_correction(), is_new_input_)) {
+  if (!input.Init(*table_, key, is_new_input_)) {
     return false;
   }
 
-  if (key.has_key_string()) {
-    if (key.input_style() == commands::KeyEvent::AS_IS ||
-        key.input_style() == commands::KeyEvent::DIRECT_INPUT) {
+  if (!input.conversion().empty()) {
+    if (input.is_asis()) {
       composition_.SetInputMode(Transliterators::CONVERSION_STRING);
-      ProcessCompositionInput(input);
+      // Disable typing correction mainly for Android. b/258369101
+      ProcessCompositionInput(std::move(input));
       SetInputMode(comeback_input_mode_);
     } else {
-      // Kana input usually has key_string.  Note that, the existence of
+      // Kana input usually has conversion. Note that, the existence of
       // key_string never determine if the input mode is Kana or Romaji.
-      ProcessCompositionInput(input);
+      ProcessCompositionInput(std::move(input));
     }
   } else {
-    // Romaji input usually does not has key_string.  Note that, the
+    // Romaji input usually does not has conversion. Note that, the
     // existence of key_string never determines if the input mode is
     // Kana or Romaji.
     const uint32_t modifiers = KeyEventUtil::GetModifiers(key);
     ApplyTemporaryInputMode(input.raw(), KeyEventUtil::HasCaps(modifiers));
-    ProcessCompositionInput(input);
+    ProcessCompositionInput(std::move(input));
   }
 
   if (comeback_input_mode_ == input_mode_) {
@@ -581,31 +597,23 @@ void Composer::DeleteAt(size_t pos) {
   if (position_ > pos) {
     position_--;
   }
-  // We do not call UpdateInputMode() here.
-  // 1. In composition mode, UpdateInputMode finalizes pending chunk.
-  // 2. In conversion mode, InputMode needs not to change.
-  typing_corrector_.Invalidate();
 }
 
 void Composer::Delete() {
   position_ = composition_.DeleteAt(position_);
   UpdateInputMode();
-
-  typing_corrector_.Invalidate();
 }
 
 void Composer::DeleteRange(size_t pos, size_t length) {
   for (int i = 0; i < length && pos < composition_.GetLength(); ++i) {
     DeleteAt(pos);
   }
-  typing_corrector_.Invalidate();
 }
 
 void Composer::EditErase() {
   composition_.Erase();
   position_ = 0;
   SetInputMode(comeback_input_mode_);
-  typing_corrector_.Reset();
 }
 
 void Composer::Backspace() {
@@ -628,8 +636,6 @@ void Composer::Backspace() {
 
   // Delete 'character to be deleted'
   position_ = composition_.DeleteAt(position_);
-
-  typing_corrector_.Invalidate();
 }
 
 void Composer::MoveCursorLeft() {
@@ -637,8 +643,6 @@ void Composer::MoveCursorLeft() {
     --position_;
   }
   UpdateInputMode();
-
-  typing_corrector_.Invalidate();
 }
 
 void Composer::MoveCursorRight() {
@@ -646,15 +650,11 @@ void Composer::MoveCursorRight() {
     ++position_;
   }
   UpdateInputMode();
-
-  typing_corrector_.Invalidate();
 }
 
 void Composer::MoveCursorToBeginning() {
   position_ = 0;
   SetInputMode(comeback_input_mode_);
-
-  typing_corrector_.Invalidate();
 }
 
 void Composer::MoveCursorToEnd() {
@@ -662,8 +662,6 @@ void Composer::MoveCursorToEnd() {
   // Behavior between MoveCursorToEnd and MoveCursorToRight is different.
   // MoveCursorToEnd always makes current input mode default.
   SetInputMode(comeback_input_mode_);
-
-  typing_corrector_.Invalidate();
 }
 
 void Composer::MoveCursorTo(uint32_t new_position) {
@@ -671,7 +669,6 @@ void Composer::MoveCursorTo(uint32_t new_position) {
     position_ = new_position;
     UpdateInputMode();
   }
-  typing_corrector_.Invalidate();
 }
 
 void Composer::GetPreedit(std::string *left, std::string *focused,
@@ -682,7 +679,7 @@ void Composer::GetPreedit(std::string *left, std::string *focused,
   composition_.GetPreedit(position_, left, focused, right);
 
   // TODO(komatsu): This function can be obsolete.
-  std::string preedit = *left + *focused + *right;
+  std::string preedit = absl::StrCat(*left, *focused, *right);
   if (TransformCharactersForNumbers(&preedit)) {
     const size_t left_size = Util::CharsLen(*left);
     const size_t focused_size = Util::CharsLen(*focused);
@@ -848,18 +845,15 @@ void Composer::GetQueriesForPrediction(std::string *base,
   // following code removes such unnecessary expansion.
   std::string asis;
   composition_.GetStringWithTrimMode(ASIS, &asis);
-  const std::string trailing = asis.substr(base_query.size());
-  for (auto [iter, end] = GetModifierRemovalMap()->equal_range(trailing);
-       iter != end; ++iter) {
-    expanded->erase(iter->second);
-  }
+  RemoveExpandedCharsForModifier(asis, base_query, expanded);
 
   japanese_util::FullWidthAsciiToHalfWidthAscii(base_query, base);
 }
 
-void Composer::GetTypeCorrectedQueriesForPrediction(
-    std::vector<TypeCorrectedQuery> *queries) const {
-  typing_corrector_.GetQueriesForPrediction(queries);
+std::optional<std::vector<spelling::TypeCorrectedQuery>>
+Composer::GetTypeCorrectedQueries(absl::string_view context) const {
+
+  return std::nullopt;
 }
 
 size_t Composer::GetLength() const { return composition_.GetLength(); }
@@ -944,19 +938,12 @@ void Composer::AutoSwitchMode() {
   composition_.GetStringWithTransliterator(
       GetTransliterator(transliteration::HALF_ASCII), &key);
 
-  ModeSwitchingHandler::ModeSwitching display_mode =
-      ModeSwitchingHandler::NO_CHANGE;
-  ModeSwitchingHandler::ModeSwitching input_mode =
-      ModeSwitchingHandler::NO_CHANGE;
-  if (!ModeSwitchingHandler::GetModeSwitchingHandler()->GetModeSwitchingRule(
-          key, &display_mode, &input_mode)) {
-    // If the key is not a pattern of mode switch rule, the procedure
-    // stops here.
-    return;
-  }
+  const ModeSwitchingHandler::Rule mode_switching =
+      ModeSwitchingHandler::GetModeSwitchingHandler()->GetModeSwitchingRule(
+          key);
 
   // |display_mode| affects the existing composition the user typed.
-  switch (display_mode) {
+  switch (mode_switching.display_mode) {
     case ModeSwitchingHandler::NO_CHANGE:
       // Do nothing.
       break;
@@ -979,13 +966,13 @@ void Composer::AutoSwitchMode() {
       SetOutputMode(transliteration::FULL_ASCII);
       break;
     default:
-      LOG(ERROR) << "Unknown value: " << display_mode;
+      LOG(ERROR) << "Unknown value: " << mode_switching.display_mode;
       break;
   }
 
   // |input_mode| affects the current input mode used for the user's
   // new typing.
-  switch (input_mode) {
+  switch (mode_switching.input_mode) {
     case ModeSwitchingHandler::NO_CHANGE:
       // Do nothing.
       break;
@@ -1009,7 +996,7 @@ void Composer::AutoSwitchMode() {
       }
       break;
     default:
-      LOG(ERROR) << "Unknown value: " << display_mode;
+      LOG(ERROR) << "Unknown value: " << mode_switching.input_mode;
       break;
   }
 }
@@ -1069,7 +1056,7 @@ bool Composer::TransformCharactersForNumbers(std::string *query) {
   bool has_symbols = false;
   bool has_alphanumerics = false;
   for (ConstChar32Iterator iter(*query); !iter.Done(); iter.Next()) {
-    const char32 one_char = iter.Get();
+    const char32_t one_char = iter.Get();
     switch (one_char) {
       case 0x30FC:  // "ー"
         has_symbols = true;
@@ -1211,7 +1198,7 @@ bool Composer::TransformCharactersForNumbers(std::string *query) {
   // It is possible that the query's size in byte differs from the
   // orig_query's size in byte.
   DCHECK_EQ(Util::CharsLen(*query), Util::CharsLen(transformed_query));
-  *query = transformed_query;
+  *query = std::move(transformed_query);
   return true;
 }
 
@@ -1231,8 +1218,8 @@ size_t Composer::shifted_sequence_count() const {
 
 const std::string &Composer::source_text() const { return source_text_; }
 std::string *Composer::mutable_source_text() { return &source_text_; }
-void Composer::set_source_text(const std::string &source_text) {
-  source_text_.assign(source_text);
+void Composer::set_source_text(const absl::string_view source_text) {
+  strings::Assign(source_text_, source_text);
 }
 
 size_t Composer::max_length() const { return max_length_; }
