@@ -31,32 +31,33 @@
 
 #include <set>
 #include <string>
+#include <tuple>
+#include <utility>
 #include <vector>
 
 #include "base/logging.h"
+#include "base/strings/unicode.h"
 #include "base/util.h"
 #include "composer/internal/composition_input.h"
+#include "composer/internal/special_key.h"
 #include "composer/internal/transliterators.h"
 #include "composer/table.h"
 #include "absl/container/btree_set.h"
+#include "absl/status/status.h"
+#include "absl/strings/str_cat.h"
+#include "absl/strings/string_view.h"
 
 namespace mozc {
 namespace composer {
-
 namespace {
+
+using ::mozc::composer::internal::DeleteSpecialKeys;
+using ::mozc::composer::internal::TrimLeadingSpecialKey;
+using ::mozc::strings::FrontChar;
+
 // Max recursion count for looking up pending loop.
 constexpr int kMaxRecursion = 4;
 
-// Delete "end" from "target", if "target" ends with the "end".
-bool DeleteEnd(const std::string &end, std::string *target) {
-  const std::string::size_type rindex = target->rfind(end);
-  if (rindex == std::string::npos) {
-    return false;
-  }
-  target->erase(rindex);
-  return true;
-}
-
 // Get from pending rules recursively
 // The recursion will be stopped if recursion_count is 0.
 // When returns false, the caller doesn't append result entries.
@@ -74,7 +75,7 @@ bool DeleteEnd(const std::string &end, std::string *target) {
 // '{*}ぃ' -> '', '{*}い'
 // '{*}い' -> '', '{*}ぃ'
 // Here, we want to get '{*}あ' <-> '{*}ぁ' loop from the input, 'あ'
-bool GetFromPending(const Table *table, const std::string &key,
+bool GetFromPending(const Table *table, const absl::string_view key,
                     int recursion_count, absl::btree_set<std::string> *result) {
   DCHECK(result);
   if (recursion_count == 0) {
@@ -86,12 +87,12 @@ bool GetFromPending(const Table *table, const std::string &key,
     // Return true because we found the loop.
     return true;
   }
-  result->insert(key);
+  result->emplace(key);
 
   std::vector<const Entry *> entries;
   table->LookUpPredictiveAll(key, &entries);
-  for (size_t i = 0; i < entries.size(); ++i) {
-    if (!entries[i]->result().empty()) {
+  for (const Entry *entry : entries) {
+    if (!entry->result().empty()) {
       // skip rules with result, because this causes too many results.
       // for example, if we have
       //  'k' -> 'っ', 'k'
@@ -100,8 +101,7 @@ bool GetFromPending(const Table *table, const std::string &key,
       // So here we stop calling recursion.
       return false;
     }
-    if (!GetFromPending(table, entries[i]->pending(), recursion_count - 1,
-                        result)) {
+    if (!GetFromPending(table, entry->pending(), recursion_count - 1, result)) {
       return false;
     }
   }
@@ -118,9 +118,6 @@ CharChunk::CharChunk(Transliterators::Transliterator transliterator,
   DCHECK_NE(Transliterators::LOCAL, transliterator);
 }
 
-CharChunk::CharChunk(const CharChunk &x) = default;
-CharChunk &CharChunk::operator=(const CharChunk &x) = default;
-
 void CharChunk::Clear() {
   raw_.clear();
   conversion_.clear();
@@ -134,9 +131,8 @@ size_t CharChunk::GetLength(Transliterators::Transliterator t12r) const {
       local_length_cache_ != std::string::npos) {
     return local_length_cache_;
   }
-  const std::string t13n =
-      Transliterate(t12r, Table::DeleteSpecialKey(raw_),
-                    Table::DeleteSpecialKey(conversion_ + pending_));
+  const std::string t13n = Transliterate(
+      t12r, DeleteSpecialKeys(raw_), DeleteSpecialKeys(conversion_ + pending_));
   size_t length = Util::CharsLen(t13n);
   if (t12r == Transliterators::LOCAL) {
     local_length_cache_ = length;
@@ -147,8 +143,8 @@ size_t CharChunk::GetLength(Transliterators::Transliterator t12r) const {
 void CharChunk::AppendResult(Transliterators::Transliterator t12r,
                              std::string *result) const {
   const std::string t13n =
-      Transliterate(t12r, Table::DeleteSpecialKey(raw_),
-                    Table::DeleteSpecialKey(conversion_ + pending_));
+      Transliterate(t12r, DeleteSpecialKeys(raw_),
+                    DeleteSpecialKeys(absl::StrCat(conversion_, pending_)));
   result->append(t13n);
 }
 
@@ -164,27 +160,27 @@ void CharChunk::AppendTrimedResult(Transliterators::Transliterator t12r,
       converted.append(entry->result());
     }
   }
-  result->append(Transliterate(t12r, Table::DeleteSpecialKey(raw_),
-                               Table::DeleteSpecialKey(converted)));
+  result->append(Transliterate(t12r, DeleteSpecialKeys(raw_),
+                               DeleteSpecialKeys(converted)));
 }
 
 void CharChunk::AppendFixedResult(Transliterators::Transliterator t12r,
                                   std::string *result) const {
-  std::string converted = conversion_;
+  std::string converted;
   if (!ambiguous_.empty()) {
     // Add the |ambiguous_| value as a fixed value.  |ambiguous_|
     // contains an undetermined result string like "ん" converted
     // from a single 'n'.
-    converted.append(ambiguous_);
+    converted = absl::StrCat(conversion_, ambiguous_);
   } else {
     // If |pending_| exists but |ambiguous_| does not exist,
     // |pending_| is appended.  If |ambiguous_| exists, the value of
     // |pending_| is usually equal to |ambiguous_| so it is not
     // appended.
-    converted.append(pending_);
+    converted = absl::StrCat(conversion_, pending_);
   }
-  result->append(Transliterate(t12r, Table::DeleteSpecialKey(raw_),
-                               Table::DeleteSpecialKey(converted)));
+  result->append(Transliterate(t12r, DeleteSpecialKeys(raw_),
+                               DeleteSpecialKeys(converted)));
 }
 
 // If we have the rule (roman),
@@ -234,25 +230,24 @@ void CharChunk::GetExpandedResults(std::set<std::string> *results) const {
   }
   // Append current pending string
   if (conversion_.empty()) {
-    results->insert(Table::DeleteSpecialKey(pending_));
+    results->insert(DeleteSpecialKeys(pending_));
   }
   std::vector<const Entry *> entries;
   table_->LookUpPredictiveAll(pending_, &entries);
-  for (size_t i = 0; i < entries.size(); ++i) {
-    if (!entries[i]->result().empty()) {
-      results->insert(Table::DeleteSpecialKey(entries[i]->result()));
+  for (const Entry *entry : entries) {
+    if (!entry->result().empty()) {
+      results->insert(DeleteSpecialKeys(entry->result()));
     }
-    if (entries[i]->pending().empty()) {
+    if (entry->pending().empty()) {
       continue;
     }
     absl::btree_set<std::string> loop_result;
-    if (!GetFromPending(table_, entries[i]->pending(), kMaxRecursion,
+    if (!GetFromPending(table_, entry->pending(), kMaxRecursion,
                         &loop_result)) {
       continue;
     }
-    for (absl::btree_set<std::string>::const_iterator itr = loop_result.begin();
-         itr != loop_result.end(); ++itr) {
-      results->insert(Table::DeleteSpecialKey(*itr));
+    for (const std::string &result : loop_result) {
+      results->insert(DeleteSpecialKeys(result));
     }
   }
 }
@@ -268,14 +263,14 @@ bool CharChunk::IsAppendable(Transliterators::Transliterator t12r,
 
 bool CharChunk::IsConvertible(Transliterators::Transliterator t12r,
                               const Table *table,
-                              const std::string &input) const {
+                              const absl::string_view input) const {
   if (!IsAppendable(t12r, table)) {
     return false;
   }
 
   size_t key_length = 0;
   bool fixed = false;
-  std::string key = pending_ + input;
+  std::string key = absl::StrCat(pending_, input);
   const Entry *entry = table->LookUpPrefix(key, &key_length, &fixed);
 
   return entry && (key.size() == key_length) && fixed;
@@ -299,128 +294,155 @@ void CharChunk::Combine(const CharChunk &left_chunk) {
   pending_ = left_chunk.pending_ + pending_;
 }
 
-bool CharChunk::AddInputInternal(std::string *input) {
+std::pair<bool, absl::string_view> CharChunk::AddInputInternal(
+    absl::string_view input) {
+  constexpr bool kLoop = true;
   constexpr bool kNoLoop = false;
 
-  size_t key_length = 0;
+  size_t used_key_length = 0;
   bool fixed = false;
-  std::string key = pending_ + *input;
-  const Entry *entry = table_->LookUpPrefix(key, &key_length, &fixed);
+  std::string key = absl::StrCat(pending_, input);
+  const Entry *entry = table_->LookUpPrefix(key, &used_key_length, &fixed);
   local_length_cache_ = std::string::npos;
 
   if (entry == nullptr) {
-    if (key_length == 0) {
-      // No prefix character is not contained in the table, fallback
+    if (used_key_length == 0) {
+      // If `input` starts with a special key, erases it and keeps the loop.
+      // For example, if `input` is "{!}ab{?}", `input` becomes "ab{?}".
+      const absl::string_view trimmed = TrimLeadingSpecialKey(input);
+      if (trimmed.size() < input.size()) {
+        return {kLoop, trimmed};
+      }
+
+      // The prefix characters are not contained in the table, fallback
       // operation is performed.
       if (pending_.empty()) {
-        // Conversion data was not found.
-        AddConvertedChar(input);
+        // Conversion data was not found. Add one character to the chunk.
+        const auto [front, rest] = FrontChar(input);
+        absl::StrAppend(&raw_, front);
+        absl::StrAppend(&conversion_, front);
+        input = rest;
       }
-      return kNoLoop;
+      return {kNoLoop, input};
     }
 
-    if (key_length < pending_.size()) {
+    if (used_key_length == pending_.size()) {
+      // If the `input` starts with a special key (e.g. "{!}"),
+      // and it is not used for this chunk and the next chunk,
+      // that special key is removed.
+      size_t used_length = 0;
+      bool next_fixed = false;
+      const Entry *next_entry =
+          table_->LookUpPrefix(input, &used_length, &next_fixed);
+      const bool no_entry = (next_entry == nullptr && used_length == 0);
+      const absl::string_view trimmed = TrimLeadingSpecialKey(input);
+      if (no_entry && trimmed.size() < input.size()) {
+        return {kLoop, trimmed};
+      }
+      return {kNoLoop, input};
+    }
+
+    if (used_key_length < pending_.size()) {
       // Do not modify this char_chunk, all key characters will be used
       // by the next char_chunk.
-      return kNoLoop;
+      return {kNoLoop, input};
     }
 
-    DCHECK_GE(key_length, pending_.size());
     // Some prefix character is contained in the table, but not
     // reached any conversion result (like "t" with "ta->た").
-    key_length -= pending_.size();
-
     // Conversion data had only pending.
-    const std::string new_pending_chars(*input, 0, key_length);
-    raw_.append(new_pending_chars);
-    pending_.append(new_pending_chars);
-    if (!ambiguous_.empty()) {
-      // If ambiguous_ has already a value, ambiguous_ is appended.
-      // ex. "ny" makes ambiguous_ "んy", but "sh" leaves ambiguous_ empty.
-      ambiguous_.append(new_pending_chars);
-    }
-    input->erase(0, key_length);
-    return kNoLoop;
+
+    // Move used input characters to CharChunk data.
+    DCHECK_GT(used_key_length, pending_.size());
+    const size_t used_input_length = used_key_length - pending_.size();
+    const absl::string_view used_input_chars =
+        input.substr(0, used_input_length);
+    absl::StrAppend(&raw_, used_input_chars);
+    absl::StrAppend(&pending_, used_input_chars);
+    ambiguous_.clear();
+    return {kNoLoop, input.substr(used_input_length)};
   }
 
   // The prefix of key reached a conversion result, thus entry is not nullptr.
 
-  if (key.size() == key_length) {
-    const bool is_following_entry =
-        (!conversion_.empty() ||
-         (!raw_.empty() && !pending_.empty() && raw_ != pending_));
-
-    raw_.append(*input);
-    input->clear();
-    if (fixed) {
-      // The whole key has been used, table lookup has reached a fixed
-      // value.  It is a stable world. (like "ka->か", "q@->だ").
-      conversion_.append(entry->result());
-      pending_ = entry->pending();
-      ambiguous_.clear();
-
-      // If this entry is the first entry, the table attributes are
-      // applied to this chunk.
-      if (!is_following_entry) {
-        attributes_ = entry->attributes();
-      }
-    } else {  // !fixed
-      // The whole string of key reached a conversion result, but the
-      // result is ambiguous (like "n" with "n->ん and na->な").
-      pending_ = key;
-      ambiguous_ = entry->result();
-    }
-    return kNoLoop;
+  // Check if this CharChunk does not contain a conversion result before.
+  const bool is_first_entry =
+      (conversion_.empty() &&
+       (raw_.empty() || pending_.empty() || raw_ == pending_));
+  // If this entry is the first entry, the attributes are applied to this chunk.
+  if (is_first_entry) {
+    attributes_ = entry->attributes();
   }
 
-  // Delete pending_ from raw_ if matched.
-  DeleteEnd(pending_, &raw_);
+  // Move used input characters to raw_.
+  const size_t used_input_length = used_key_length - pending_.size();
+  absl::StrAppend(&raw_, input.substr(0, used_input_length));
+  input = input.substr(used_input_length);
 
-  // A result was found without any ambiguity.
-  input->assign(key, key_length, key.size() - key_length);
-  raw_.append(key, 0, key_length);
-  conversion_.append(entry->result());
-  pending_ = entry->pending();
-  ambiguous_.clear();
+  if (fixed || key.size() > used_key_length) {
+    // A result was found. Ambiguity is resolved because `fixed` is true or
+    // the key still remains. e.g. If the key is "nk", "n" is used for "ん"
+    // because the next remaining character "k" is not used with "n".
+    conversion_.append(entry->result());
+    pending_ = entry->pending();
+    ambiguous_.clear();
+  } else {
+    // A result was found, but it is still ambiguous.
+    // e.g. "n" with "n->ん and na->な".
+    pending_ = key;
+    ambiguous_ = entry->result();
+  }
 
-  if (input->empty() || pending_.empty()) {
+  // If the lookup is deterministically done (e.g. fixed is true and input is
+  // empty) but the output is empty and the raw input is not used (e.g.
+  // NO_TRANSLITERATION), raw_ is also cleared to make an empty chunk to be
+  // removed by the caller (see: Composition::InsertInput).
+  if (fixed && input.empty() && conversion_.empty() && pending_.empty() &&
+      (attributes_ & NO_TRANSLITERATION)) {
+    raw_.clear();
+    return {kNoLoop, input};
+  }
+
+  if (input.empty() || pending_.empty()) {
     // If the remaining input character or pending character is empty,
     // there is no reason to continue the looping.
-    return kNoLoop;
+    return {kNoLoop, input};
   }
 
-  constexpr bool kLoop = true;
-  return kLoop;
+  return {kLoop, input};
 }
 
 void CharChunk::AddInput(std::string *input) {
-  while (AddInputInternal(input)) {
+  absl::string_view tmp = *input;
+  bool loop = true;
+  while (loop) {
+    std::tie(loop, tmp) = AddInputInternal(tmp);
   }
+  input->erase(0, input->size() - tmp.size());
 }
 
-void CharChunk::AddConvertedChar(std::string *input) {
-  // TODO(komatsu) Nice to make "string Util::PopOneChar(string *str);".
-  const absl::string_view first_char = Util::Utf8SubString(*input, 0, 1);
-  conversion_.append(first_char.data(), first_char.size());
-  raw_.append(first_char.data(), first_char.size());
-  Util::Utf8SubString(*input, 1, std::string::npos, input);
+void CharChunk::AddInputAndConvertedChar(CompositionInput *input) {
   local_length_cache_ = std::string::npos;
-}
 
-void CharChunk::AddInputAndConvertedChar(std::string *key,
-                                         std::string *converted_char) {
-  local_length_cache_ = std::string::npos;
-  // If this chunk is empty, the key and converted_char are simply
-  // copied.
+  if (input->is_asis()) {
+    if (raw_.empty() && pending_.empty() && conversion_.empty()) {
+      raw_ = input->raw();
+      conversion_ = input->conversion();
+      input->Clear();
+    }
+    return;
+  }
+
+  // If this chunk is empty, the raw and conversion are simply copied.
   if (raw_.empty() && pending_.empty() && conversion_.empty()) {
-    raw_ = *key;
-    key->clear();
-    pending_ = *converted_char;
-    // TODO(komatsu): We should check if the |converted_char| is
+    raw_ = input->raw();
+    input->clear_raw();
+    pending_ = input->conversion();
+    // TODO(komatsu): We should check if the `conversion` is
     // really ambiguous or not, otherwise the last character of the
     // preedit on Kana input is always dropped.
-    ambiguous_ = *converted_char;
-    converted_char->clear();
+    ambiguous_ = input->conversion();
+    input->clear_conversion();
 
     // If this entry is the first entry, the table attributes are
     // applied to this chunk.
@@ -431,19 +453,19 @@ void CharChunk::AddInputAndConvertedChar(std::string *key,
     return;
   }
 
-  const std::string input = pending_ + *converted_char;
+  const std::string key_input = pending_ + input->conversion();
   size_t key_length = 0;
   bool fixed = false;
-  const Entry *entry = table_->LookUpPrefix(input, &key_length, &fixed);
+  const Entry *entry = table_->LookUpPrefix(key_input, &key_length, &fixed);
   if (entry == nullptr) {
-    // Do not modify this char_chunk, all key and converted_char
-    // values will be used by the next char_chunk.
+    // Do not modify this char_chunk, all `raw` and `conversion`
+    // values of `input` will be used by the next char_chunk.
     return;
   }
 
   // The whole input string was used.
-  if (key_length == input.size()) {
-    raw_.append(*key);
+  if (key_length == key_input.size()) {
+    raw_.append(input->raw());
     if (fixed) {
       conversion_.append(entry->result());
       pending_ = entry->pending();
@@ -453,29 +475,28 @@ void CharChunk::AddInputAndConvertedChar(std::string *key,
       pending_ = entry->result();
       ambiguous_ = entry->result();
     }
-    key->clear();
-    converted_char->clear();
+    input->clear_raw();
+    input->clear_conversion();
     return;
   }
 
-  // The following key_length == pending_.size() means the new key and
-  // and converted_char does not affect at all.  Do not any thing here
-  // and a new char_chunk will be made for the new key and
-  // converted_char.
+  // The following key_length == pending_.size() means the new `raw` and
+  // and `conversion` of `input` does not affect at all.  Do not any thing here
+  // and a new char_chunk will be made for `input`.
   if (key_length == pending_.size()) {
     return;
   }
 
-  // The input is partially used.
-  raw_.append(*key);
+  // The `input` is partially used.
+  raw_.append(input->raw());
   conversion_.append(entry->result());
   pending_ = entry->pending();
-  // While the whole key is used in this char_chunk, the
-  // converted_char is separated to this char_chunk and the next
+  // While the whole `raw` is used in this char_chunk, the
+  // `conversion` is separated to this char_chunk and the next
   // char_chunk.  This is not a preferred behavior, but there is no
   // better way to work around this limitation.
-  key->clear();
-  converted_char->assign(input, key_length, input.size() - key_length);
+  input->clear_raw();
+  input->set_conversion(key_input.substr(key_length));
 }
 
 bool CharChunk::ShouldCommit() const {
@@ -500,8 +521,8 @@ bool CharChunk::ShouldInsertNewChunk(const CompositionInput &input) const {
 
 void CharChunk::AddCompositionInput(CompositionInput *input) {
   local_length_cache_ = std::string::npos;
-  if (input->has_conversion()) {
-    AddInputAndConvertedChar(input->mutable_raw(), input->mutable_conversion());
+  if (!input->conversion().empty()) {
+    AddInputAndConvertedChar(input);
     return;
   }
 
@@ -522,61 +543,41 @@ void CharChunk::SetTransliterator(
   transliterator_ = transliterator;
 }
 
-void CharChunk::set_raw(const std::string &raw) {
-  raw_ = raw;
-  local_length_cache_ = std::string::npos;
-}
-
-void CharChunk::set_conversion(const std::string &conversion) {
-  conversion_ = conversion;
-  local_length_cache_ = std::string::npos;
-}
-
-void CharChunk::set_pending(const std::string &pending) {
-  pending_ = pending;
-  local_length_cache_ = std::string::npos;
-}
-
-void CharChunk::set_ambiguous(const std::string &ambiguous) {
-  ambiguous_ = ambiguous;
-  local_length_cache_ = std::string::npos;
-}
-
 void CharChunk::set_attributes(TableAttributes attributes) {
   attributes_ = attributes;
   local_length_cache_ = std::string::npos;
 }
 
-std::unique_ptr<CharChunk> CharChunk::SplitChunk(
+absl::StatusOr<CharChunk> CharChunk::SplitChunk(
     Transliterators::Transliterator t12r, const size_t position) {
   if (position <= 0 || position >= GetLength(t12r)) {
-    LOG(WARNING) << "Invalid position: " << position;
-    return nullptr;
+    return absl::InvalidArgumentError(
+        absl::StrCat("Invalid position: ", position));
   }
 
   local_length_cache_ = std::string::npos;
   std::string raw_lhs, raw_rhs, converted_lhs, converted_rhs;
   Transliterators::GetTransliterator(GetTransliterator(t12r))
-      ->Split(position, Table::DeleteSpecialKey(raw_),
-              Table::DeleteSpecialKey(conversion_ + pending_), &raw_lhs,
-              &raw_rhs, &converted_lhs, &converted_rhs);
+      ->Split(position, DeleteSpecialKeys(raw_),
+              DeleteSpecialKeys(conversion_ + pending_), &raw_lhs, &raw_rhs,
+              &converted_lhs, &converted_rhs);
 
-  auto left_new_chunk = std::make_unique<CharChunk>(transliterator_, table_);
-  left_new_chunk->set_raw(raw_lhs);
-  set_raw(raw_rhs);
+  CharChunk left_new_chunk(transliterator_, table_);
+  left_new_chunk.set_raw(raw_lhs);
+  set_raw(std::move(raw_rhs));
 
   if (converted_lhs.size() > conversion_.size()) {
     // [ conversion | pending ] => [ conv | pend#1 ] [ pend#2 ]
-    const std::string pending_lhs(converted_lhs, conversion_.size());
-    left_new_chunk->set_conversion(conversion_);
-    left_new_chunk->set_pending(pending_lhs);
+    std::string pending_lhs(converted_lhs, conversion_.size());
+    left_new_chunk.set_conversion(conversion_);
+    left_new_chunk.set_pending(std::move(pending_lhs));
 
     conversion_.clear();
-    pending_ = converted_rhs;
+    pending_ = std::move(converted_rhs);
     ambiguous_.clear();
   } else {
     // [ conversion | pending ] => [ conv#1 ] [ conv#2 | pending ]
-    left_new_chunk->set_conversion(converted_lhs);
+    left_new_chunk.set_conversion(std::move(converted_lhs));
     // left_new_chunk->set_pending("");
     const size_t pending_pos = converted_rhs.size() - pending_.size();
     conversion_.assign(converted_rhs, 0, pending_pos);
@@ -602,8 +603,8 @@ Transliterators::Transliterator CharChunk::GetTransliterator(
 }
 
 std::string CharChunk::Transliterate(
-    Transliterators::Transliterator transliterator, const std::string &raw,
-    const std::string &converted) const {
+    Transliterators::Transliterator transliterator, const absl::string_view raw,
+    const absl::string_view converted) const {
   return Transliterators::GetTransliterator(GetTransliterator(transliterator))
       ->Transliterate(raw, converted);
 }
