@@ -30,44 +30,46 @@
 #include "composer/key_event_util.h"
 
 #include <cctype>
+#include <cstddef>
+#include <cstdint>
 
 #include "base/logging.h"
-#include "base/port.h"
 #include "protocol/commands.pb.h"
 
 namespace mozc {
-using commands::KeyEvent;
-
 namespace {
-const uint32 kAltMask =
-    KeyEvent::ALT | KeyEvent::LEFT_ALT | KeyEvent::RIGHT_ALT;
-const uint32 kCtrlMask =
-    KeyEvent::CTRL | KeyEvent::LEFT_CTRL | KeyEvent::RIGHT_CTRL;
-const uint32 kShiftMask =
-    KeyEvent::SHIFT | KeyEvent::LEFT_SHIFT | KeyEvent::RIGHT_SHIFT;
-const uint32 kCapsMask = KeyEvent::CAPS;
 
-uint32 Ignore(uint32 modifiers, uint32 modifiers_to_be_ignored) {
+using ::mozc::commands::KeyEvent;
+
+constexpr uint32_t kAltMask =
+    KeyEvent::ALT | KeyEvent::LEFT_ALT | KeyEvent::RIGHT_ALT;
+constexpr uint32_t kCtrlMask =
+    KeyEvent::CTRL | KeyEvent::LEFT_CTRL | KeyEvent::RIGHT_CTRL;
+constexpr uint32_t kShiftMask =
+    KeyEvent::SHIFT | KeyEvent::LEFT_SHIFT | KeyEvent::RIGHT_SHIFT;
+constexpr uint32_t kCapsMask = KeyEvent::CAPS;
+
+uint32_t Ignore(uint32_t modifiers, uint32_t modifiers_to_be_ignored) {
   return modifiers & ~modifiers_to_be_ignored;
 }
 
-bool Any(uint32 modifiers_to_be_tested, uint32 modifiers_to_be_queried) {
+bool Any(uint32_t modifiers_to_be_tested, uint32_t modifiers_to_be_queried) {
   return (modifiers_to_be_tested & modifiers_to_be_queried) != 0;
 }
 
-bool None(uint32 modifiers_to_be_tested, uint32 modifiers_to_be_queried) {
+bool None(uint32_t modifiers_to_be_tested, uint32_t modifiers_to_be_queried) {
   return !Any(modifiers_to_be_tested, modifiers_to_be_queried);
 }
 
 }  // namespace
 
-uint32 KeyEventUtil::GetModifiers(const KeyEvent &key_event) {
-  uint32 modifiers = 0;
+uint32_t KeyEventUtil::GetModifiers(const KeyEvent &key_event) {
+  uint32_t modifiers = 0;
   if (key_event.has_modifiers()) {
     modifiers = key_event.modifiers();
   } else {
-    for (size_t i = 0; i < key_event.modifier_keys_size(); ++i) {
-      modifiers |= key_event.modifier_keys(i);
+    for (const int key : key_event.modifier_keys()) {
+      modifiers |= key;
     }
   }
   return modifiers;
@@ -77,11 +79,11 @@ bool KeyEventUtil::GetKeyInformation(const KeyEvent &key_event,
                                      KeyInformation *key) {
   DCHECK(key);
 
-  const uint16 modifier_keys = static_cast<uint16>(GetModifiers(key_event));
-  const uint16 special_key = key_event.has_special_key()
-                                 ? key_event.special_key()
-                                 : KeyEvent::NO_SPECIALKEY;
-  const uint32 key_code = key_event.has_key_code() ? key_event.key_code() : 0;
+  const uint16_t modifier_keys = static_cast<uint16_t>(GetModifiers(key_event));
+  const uint16_t special_key = key_event.has_special_key()
+                                   ? key_event.special_key()
+                                   : KeyEvent::NO_SPECIALKEY;
+  const uint32_t key_code = key_event.has_key_code() ? key_event.key_code() : 0;
 
   // Make sure the translation from the obsolete spesification.
   // key_code should no longer contain control characters.
@@ -103,7 +105,7 @@ void KeyEventUtil::NormalizeModifiers(const KeyEvent &key_event,
   // CTRL (or ALT, SHIFT) should be set on modifier_keys when
   // LEFT (or RIGHT) ctrl is set.
   // LEFT_CTRL (or others) is not handled on Japanese, so we remove these.
-  const uint32 kIgnorableModifierMask =
+  constexpr uint32_t kIgnorableModifierMask =
       (KeyEvent::CAPS | KeyEvent::LEFT_ALT | KeyEvent::RIGHT_ALT |
        KeyEvent::LEFT_CTRL | KeyEvent::RIGHT_CTRL | KeyEvent::LEFT_SHIFT |
        KeyEvent::RIGHT_SHIFT);
@@ -111,9 +113,9 @@ void KeyEventUtil::NormalizeModifiers(const KeyEvent &key_event,
   RemoveModifiers(key_event, kIgnorableModifierMask, new_key_event);
 
   // Reverts the flip of alphabetical key events caused by CapsLock.
-  const uint32 original_modifiers = GetModifiers(key_event);
+  const uint32_t original_modifiers = GetModifiers(key_event);
   if ((original_modifiers & KeyEvent::CAPS) && key_event.has_key_code()) {
-    const uint32 key_code = key_event.key_code();
+    const uint32_t key_code = key_event.key_code();
     if ('A' <= key_code && key_code <= 'Z') {
       new_key_event->set_key_code(key_code + ('a' - 'A'));
     } else if ('a' <= key_code && key_code <= 'z') {
@@ -125,7 +127,7 @@ void KeyEventUtil::NormalizeModifiers(const KeyEvent &key_event,
 void KeyEventUtil::NormalizeNumpadKey(const KeyEvent &key_event,
                                       KeyEvent *new_key_event) {
   DCHECK(new_key_event);
-  new_key_event->CopyFrom(key_event);
+  *new_key_event = key_event;
 
   if (!IsNumpadKey(*new_key_event)) {
     return;
@@ -143,7 +145,7 @@ void KeyEventUtil::NormalizeNumpadKey(const KeyEvent &key_event,
   // Handles number keys
   if (KeyEvent::NUMPAD0 <= numpad_key && numpad_key <= KeyEvent::NUMPAD9) {
     new_key_event->set_key_code(
-        static_cast<uint32>('0' + (numpad_key - KeyEvent::NUMPAD0)));
+        static_cast<uint32_t>('0' + (numpad_key - KeyEvent::NUMPAD0)));
     return;
   }
 
@@ -175,14 +177,14 @@ void KeyEventUtil::NormalizeNumpadKey(const KeyEvent &key_event,
       return;
   }
 
-  new_key_event->set_key_code(static_cast<uint32>(new_key_code));
+  new_key_event->set_key_code(static_cast<uint32_t>(new_key_code));
 }
 
 void KeyEventUtil::RemoveModifiers(const KeyEvent &key_event,
-                                   uint32 remove_modifiers,
+                                   uint32_t remove_modifiers,
                                    KeyEvent *new_key_event) {
   DCHECK(new_key_event);
-  new_key_event->CopyFrom(key_event);
+  *new_key_event = key_event;
 
   if (HasAlt(remove_modifiers)) {
     remove_modifiers |= KeyEvent::LEFT_ALT | KeyEvent::RIGHT_ALT;
@@ -232,63 +234,65 @@ bool KeyEventUtil::MaybeGetKeyStub(const KeyEvent &key_event,
   return true;
 }
 
-bool KeyEventUtil::HasAlt(uint32 modifiers) { return Any(modifiers, kAltMask); }
+bool KeyEventUtil::HasAlt(uint32_t modifiers) {
+  return Any(modifiers, kAltMask);
+}
 
-bool KeyEventUtil::HasCtrl(uint32 modifiers) {
+bool KeyEventUtil::HasCtrl(uint32_t modifiers) {
   return Any(modifiers, kCtrlMask);
 }
 
-bool KeyEventUtil::HasShift(uint32 modifiers) {
+bool KeyEventUtil::HasShift(uint32_t modifiers) {
   return Any(modifiers, kShiftMask);
 }
 
-bool KeyEventUtil::HasCaps(uint32 modifiers) {
+bool KeyEventUtil::HasCaps(uint32_t modifiers) {
   return Any(modifiers, kCapsMask);
 }
 
-bool KeyEventUtil::IsAlt(uint32 modifiers) {
+bool KeyEventUtil::IsAlt(uint32_t modifiers) {
   if (!HasAlt(modifiers)) {
     return false;
   }
   return None(Ignore(modifiers, kCapsMask), ~kAltMask);
 }
 
-bool KeyEventUtil::IsCtrl(uint32 modifiers) {
+bool KeyEventUtil::IsCtrl(uint32_t modifiers) {
   if (!HasCtrl(modifiers)) {
     return false;
   }
   return None(Ignore(modifiers, kCapsMask), ~kCtrlMask);
 }
 
-bool KeyEventUtil::IsShift(uint32 modifiers) {
+bool KeyEventUtil::IsShift(uint32_t modifiers) {
   if (!HasShift(modifiers)) {
     return false;
   }
   return None(Ignore(modifiers, kCapsMask), ~kShiftMask);
 }
 
-bool KeyEventUtil::IsAltCtrl(uint32 modifiers) {
+bool KeyEventUtil::IsAltCtrl(uint32_t modifiers) {
   if (!HasAlt(modifiers) || !HasCtrl(modifiers)) {
     return false;
   }
   return None(Ignore(modifiers, kCapsMask), ~(kAltMask | kCtrlMask));
 }
 
-bool KeyEventUtil::IsAltShift(uint32 modifiers) {
+bool KeyEventUtil::IsAltShift(uint32_t modifiers) {
   if (!HasAlt(modifiers) || !HasShift(modifiers)) {
     return false;
   }
   return None(Ignore(modifiers, kCapsMask), ~(kAltMask | kShiftMask));
 }
 
-bool KeyEventUtil::IsCtrlShift(uint32 modifiers) {
+bool KeyEventUtil::IsCtrlShift(uint32_t modifiers) {
   if (!HasCtrl(modifiers) || !HasShift(modifiers)) {
     return false;
   }
   return None(Ignore(modifiers, kCapsMask), ~(kCtrlMask | kShiftMask));
 }
 
-bool KeyEventUtil::IsAltCtrlShift(uint32 modifiers) {
+bool KeyEventUtil::IsAltCtrlShift(uint32_t modifiers) {
   if (!HasAlt(modifiers) || !HasCtrl(modifiers) || !HasShift(modifiers)) {
     return false;
   }
@@ -301,8 +305,8 @@ bool KeyEventUtil::IsLowerAlphabet(const KeyEvent &key_event) {
     return false;
   }
 
-  const uint32 key_code = key_event.key_code();
-  const uint32 modifier_keys = GetModifiers(key_event);
+  const uint32_t key_code = key_event.key_code();
+  const uint32_t modifier_keys = GetModifiers(key_event);
   const bool change_case = (HasShift(modifier_keys) != HasCaps(modifier_keys));
 
   if (change_case) {
@@ -317,8 +321,8 @@ bool KeyEventUtil::IsUpperAlphabet(const KeyEvent &key_event) {
     return false;
   }
 
-  const uint32 key_code = key_event.key_code();
-  const uint32 modifier_keys = GetModifiers(key_event);
+  const uint32_t key_code = key_event.key_code();
+  const uint32_t modifier_keys = GetModifiers(key_event);
   const bool change_case = (HasShift(modifier_keys) != HasCaps(modifier_keys));
 
   if (change_case) {
