@@ -31,62 +31,59 @@
 
 #include "composer/key_parser.h"
 
-#include <map>
-#include <set>
 #include <string>
+#include <utility>
 #include <vector>
 
 #include "base/logging.h"
-#include "base/port.h"
-#include "base/singleton.h"
 #include "base/util.h"
 #include "protocol/commands.pb.h"
+#include "absl/container/btree_set.h"
+#include "absl/container/flat_hash_map.h"
+#include "absl/container/flat_hash_set.h"
 #include "absl/strings/str_split.h"
+#include "absl/strings/string_view.h"
+#include "absl/types/span.h"
 
 namespace mozc {
-using commands::KeyEvent;
-
 namespace {
 
-typedef std::map<std::string, KeyEvent::SpecialKey> SpecialKeysMap;
-typedef std::multimap<std::string, KeyEvent::ModifierKey> ModifiersMap;
+using ::mozc::commands::KeyEvent;
+
+using SpecialKeysMap = absl::flat_hash_map<std::string, KeyEvent::SpecialKey>;
+using ModifiersMap =
+    absl::flat_hash_map<std::string, std::vector<KeyEvent::ModifierKey>>;
 
 class KeyParserData {
  public:
   KeyParserData() { InitData(); }
 
-  const SpecialKeysMap &keycode_map() { return keycode_map_; }
-  const ModifiersMap &modifiers_map() { return modifiers_map_; }
+  const SpecialKeysMap &keycode_map() const { return keycode_map_; }
+  const ModifiersMap &modifiers_map() const { return modifiers_map_; }
 
  private:
   void InitData() {
-    //  CHECK(keymap::KeyType::NUM_KEYTYPES < static_cast<int32>(' '));
+    //  CHECK(keymap::KeyType::NUM_KEYTYPES < static_cast<int32_t>(' '));
     VLOG(1) << "Init KeyParser Data";
 
-    modifiers_map_.insert(std::make_pair("ctrl", KeyEvent::CTRL));
-    modifiers_map_.insert(std::make_pair("control", KeyEvent::CTRL));
-    modifiers_map_.insert(std::make_pair("alt", KeyEvent::ALT));
-    modifiers_map_.insert(std::make_pair("option", KeyEvent::ALT));
-    modifiers_map_.insert(std::make_pair("meta", KeyEvent::ALT));
-    modifiers_map_.insert(std::make_pair("super", KeyEvent::ALT));
-    modifiers_map_.insert(std::make_pair("hyper", KeyEvent::ALT));
-    modifiers_map_.insert(std::make_pair("shift", KeyEvent::SHIFT));
-    modifiers_map_.insert(std::make_pair("caps", KeyEvent::CAPS));
-    modifiers_map_.insert(std::make_pair("keydown", KeyEvent::KEY_DOWN));
-    modifiers_map_.insert(std::make_pair("keyup", KeyEvent::KEY_UP));
+    modifiers_map_["ctrl"] = {KeyEvent::CTRL};
+    modifiers_map_["control"] = {KeyEvent::CTRL};
+    modifiers_map_["alt"] = {KeyEvent::ALT};
+    modifiers_map_["option"] = {KeyEvent::ALT};
+    modifiers_map_["meta"] = {KeyEvent::ALT};
+    modifiers_map_["super"] = {KeyEvent::ALT};
+    modifiers_map_["hyper"] = {KeyEvent::ALT};
+    modifiers_map_["shift"] = {KeyEvent::SHIFT};
+    modifiers_map_["caps"] = {KeyEvent::CAPS};
+    modifiers_map_["keydown"] = {KeyEvent::KEY_DOWN};
+    modifiers_map_["keyup"] = {KeyEvent::KEY_UP};
 
-    modifiers_map_.insert(std::make_pair("leftctrl", KeyEvent::CTRL));
-    modifiers_map_.insert(std::make_pair("leftctrl", KeyEvent::LEFT_CTRL));
-    modifiers_map_.insert(std::make_pair("rightctrl", KeyEvent::CTRL));
-    modifiers_map_.insert(std::make_pair("rightctrl", KeyEvent::RIGHT_CTRL));
-    modifiers_map_.insert(std::make_pair("leftalt", KeyEvent::ALT));
-    modifiers_map_.insert(std::make_pair("leftalt", KeyEvent::LEFT_ALT));
-    modifiers_map_.insert(std::make_pair("rightalt", KeyEvent::ALT));
-    modifiers_map_.insert(std::make_pair("rightalt", KeyEvent::RIGHT_ALT));
-    modifiers_map_.insert(std::make_pair("leftshift", KeyEvent::SHIFT));
-    modifiers_map_.insert(std::make_pair("leftshift", KeyEvent::LEFT_SHIFT));
-    modifiers_map_.insert(std::make_pair("rightshift", KeyEvent::SHIFT));
-    modifiers_map_.insert(std::make_pair("rightshift", KeyEvent::RIGHT_SHIFT));
+    modifiers_map_["leftctrl"] = {KeyEvent::CTRL, KeyEvent::LEFT_CTRL};
+    modifiers_map_["rightctrl"] = {KeyEvent::CTRL, KeyEvent::RIGHT_CTRL};
+    modifiers_map_["leftalt"] = {KeyEvent::ALT, KeyEvent::LEFT_ALT};
+    modifiers_map_["rightalt"] = {KeyEvent::ALT, KeyEvent::RIGHT_ALT};
+    modifiers_map_["leftshift"] = {KeyEvent::SHIFT, KeyEvent::LEFT_SHIFT};
+    modifiers_map_["rightshift"] = {KeyEvent::SHIFT, KeyEvent::RIGHT_SHIFT};
 
     keycode_map_["on"] = KeyEvent::ON;
     keycode_map_["off"] = KeyEvent::OFF;
@@ -183,7 +180,9 @@ class KeyParserData {
 };
 }  // namespace
 
-bool KeyParser::ParseKey(const std::string &key_string, KeyEvent *key_event) {
+// static
+bool KeyParser::ParseKey(const absl::string_view key_string,
+                         KeyEvent *key_event) {
   std::vector<std::string> keys =
       absl::StrSplit(key_string, ' ', absl::SkipEmpty());
   if (keys.empty()) {
@@ -193,51 +192,74 @@ bool KeyParser::ParseKey(const std::string &key_string, KeyEvent *key_event) {
   return KeyParser::ParseKeyVector(keys, key_event);
 }
 
-bool KeyParser::ParseKeyVector(const std::vector<std::string> &keys,
+// static
+bool KeyParser::ParseKeyVector(const absl::Span<const std::string> keys,
                                KeyEvent *key_event) {
   CHECK(key_event);
-
-  const ModifiersMap &modifiers_map =
-      Singleton<KeyParserData>::get()->modifiers_map();
-  const SpecialKeysMap &keycode_map =
-      Singleton<KeyParserData>::get()->keycode_map();
+  static const KeyParserData *parser_data = new KeyParserData();
+  const ModifiersMap &modifiers = parser_data->modifiers_map();
+  const SpecialKeysMap &specials = parser_data->keycode_map();
 
   key_event->Clear();
-  std::set<commands::KeyEvent::ModifierKey> modifiers_set;
+  absl::btree_set<commands::KeyEvent::ModifierKey> modifiers_set;
 
-  for (size_t i = 0; i < keys.size(); ++i) {
-    if (Util::CharsLen(keys[i]) == 1) {
-      char32 key_code = 0;
-      if (Util::SplitFirstChar32(keys[i], &key_code, nullptr)) {
-        key_event->set_key_code(key_code);
-      }
-    } else {
-      std::string key = keys[i];
-      Util::LowerString(&key);
-
-      if (modifiers_map.count(key) > 0) {
-        std::pair<ModifiersMap::const_iterator, ModifiersMap::const_iterator>
-            range = modifiers_map.equal_range(key);
-        for (ModifiersMap::const_iterator iter = range.first;
-             iter != range.second; ++iter) {
-          modifiers_set.insert(iter->second);
-        }
-      } else if (keycode_map.count(key) == 1) {
-        key_event->set_special_key(keycode_map.find(key)->second);
-      } else {
-        LOG(ERROR) << "Unknown key: " << keys[i];
+  for (const std::string &key : keys) {
+    if (Util::CharsLen(key) == 1) {
+      if (key_event->has_key_code()) {
+        // Multiple keys are not supported.
         return false;
       }
+      key_event->set_key_code(Util::Utf8ToUcs4(key));
+      continue;
     }
+
+    std::string lower_key(key);
+    Util::LowerString(&lower_key);
+
+    if (const auto it = modifiers.find(lower_key); it != modifiers.end()) {
+      modifiers_set.insert(it->second.begin(), it->second.end());
+      continue;
+    }
+    if (const auto it = specials.find(lower_key); it != specials.end()) {
+      if (key_event->has_special_key()) {
+        // Multiple special keys are not supported.
+        return false;
+      }
+      key_event->set_special_key(it->second);
+      continue;
+    }
+    LOG(ERROR) << "Unknown key: " << key;
+    return false;
   }
 
-  for (std::set<commands::KeyEvent::ModifierKey>::iterator iter =
-           modifiers_set.begin();
-       iter != modifiers_set.end(); ++iter) {
-    key_event->add_modifier_keys(*iter);
+  for (const commands::KeyEvent::ModifierKey modifier : modifiers_set) {
+    key_event->add_modifier_keys(modifier);
   }
 
   return true;
 }
 
+std::string KeyParser::GetSpecialKeyString(KeyEvent::SpecialKey key) {
+  // Handles exceptional rules that cannot convert from the enum value.
+  switch (key) {
+    case KeyEvent::DEL:
+      return "delete";
+    case KeyEvent::KANA:
+      return "hiragana";
+    case KeyEvent::HANKAKU:
+      return "hankaku/zenkaku";
+    default:
+      // Do nothing.
+      break;
+  }
+
+  // Transforms the enum value in string by removing '_' and using lower case.
+  // e.g. "PAGE_UP" -> "pageup".
+  std::string name = KeyEvent::SpecialKey_Name(key);
+  // Idiom to remove '_' from `name` (e.g. "PAGE_UP" -> "PAGEUP").
+  name.erase(std::remove(name.begin(), name.end(), '_'), name.end());
+  Util::LowerString(&name);
+  return name;
+}
+
 }  // namespace mozc
