@@ -31,41 +31,44 @@
 
 #include "composer/table.h"
 
+#include <cstddef>
 #include <cstdint>
 #include <istream>  // NOLINT
-#include <map>
 #include <memory>
 #include <sstream>
 #include <string>
 #include <utility>
+#include <vector>
 
 #include "base/config_file_stream.h"
-#include "base/file_stream.h"
 #include "base/hash.h"
 #include "base/logging.h"
-#include "base/port.h"
-#include "base/trie.h"
 #include "base/util.h"
-#include "composer/internal/typing_model.h"
-#include "config/config_handler.h"
+#include "composer/internal/special_key.h"
+#include "data_manager/data_manager_interface.h"
 #include "protocol/commands.pb.h"
 #include "protocol/config.pb.h"
 #include "absl/strings/match.h"
+#include "absl/strings/str_cat.h"
 #include "absl/strings/str_split.h"
+#include "absl/strings/string_view.h"
 
 namespace mozc {
 namespace composer {
 namespace {
+
+using ::mozc::composer::internal::DeleteSpecialKeys;
+
 constexpr char kDefaultPreeditTableFile[] = "system://romanji-hiragana.tsv";
 constexpr char kRomajiPreeditTableFile[] = "system://romanji-hiragana.tsv";
 // Table for Kana combinations like "か゛" → "が".
 constexpr char kKanaCombinationTableFile[] = "system://kana.tsv";
 
 // Special tables for 12keys
-const char k12keysHiraganaTableFile[] = "system://12keys-hiragana.tsv";
-const char k12keysHiraganaIntuitiveTableFile[] =
+constexpr char k12keysHiraganaTableFile[] = "system://12keys-hiragana.tsv";
+constexpr char k12keysHiraganaIntuitiveTableFile[] =
     "system://12keys-hiragana_intuitive.tsv";
-const char k12keysHalfwidthasciiTableFile[] =
+constexpr char k12keysHalfwidthasciiTableFile[] =
     "system://12keys-halfwidthascii.tsv";
 constexpr char kFlickHiraganaTableFile[] = "system://flick-hiragana.tsv";
 constexpr char kFlickHalfwidthasciiIosTableFile[] =
@@ -96,17 +99,17 @@ constexpr char kNotouchHiraganaTableFile[] = "system://notouch-hiragana.tsv";
 // Reuse qwerty_mobile-halfwidthascii table
 constexpr char kNotouchHalfwidthasciiTableFile[] =
     "system://qwerty_mobile-halfwidthascii.tsv";
+constexpr char k50KeysHiraganaTableFile[] = "system://50keys-hiragana.tsv";
 
 constexpr char kNewChunkPrefix[] = "\t";
-constexpr char kSpecialKeyOpen[] = "\x0F";   // Shift-In of ASCII
-constexpr char kSpecialKeyClose[] = "\x0E";  // Shift-Out of ASCII
+
 }  // namespace
 
 // ========================================
 // Entry
 // ========================================
-Entry::Entry(const std::string &input, const std::string &result,
-             const std::string &pending, const TableAttributes attributes)
+Entry::Entry(const absl::string_view input, const absl::string_view result,
+             const absl::string_view pending, const TableAttributes attributes)
     : input_(input),
       result_(result),
       pending_(pending),
@@ -115,29 +118,30 @@ Entry::Entry(const std::string &input, const std::string &result,
 // ========================================
 // Table
 // ========================================
-Table::Table() : entries_(new EntryTrie), case_sensitive_(false) {}
+Table::Table() {
+  // Add predefined special keys.
+  special_key_map_.Register("{?}");  // toggle
+  special_key_map_.Register("{*}");  // internal state
+  special_key_map_.Register("{<}");  // rewind
+  special_key_map_.Register("{!}");  // timeout
+}
 
-Table::~Table() { ResetEntrySet(); }
-
-static constexpr char kKuten[] = "、";
-static constexpr char kTouten[] = "。";
-static constexpr char kComma[] = "，";
-static constexpr char kPeriod[] = "．";
-
-static constexpr char kCornerOpen[] = "「";
-static constexpr char kCornerClose[] = "」";
-static constexpr char kSlash[] = "／";
-static constexpr char kSquareOpen[] = "[";
-static constexpr char kSquareClose[] = "]";
-static constexpr char kMiddleDot[] = "・";
+constexpr absl::string_view kKuten = "、";
+constexpr absl::string_view kTouten = "。";
+constexpr absl::string_view kComma = "，";
+constexpr absl::string_view kPeriod = "．";
+constexpr absl::string_view kCornerOpen = "「";
+constexpr absl::string_view kCornerClose = "」";
+constexpr absl::string_view kSlash = "／";
+constexpr absl::string_view kSquareOpen = "[";
+constexpr absl::string_view kSquareClose = "]";
+constexpr absl::string_view kMiddleDot = "・";
 
 bool Table::InitializeWithRequestAndConfig(
     const commands::Request &request, const config::Config &config,
     const DataManagerInterface &data_manager) {
   case_sensitive_ = false;
   bool result = false;
-  typing_model_ = TypingModel::CreateTypingModel(
-      request.special_romanji_table(), data_manager);
   if (request.special_romanji_table() !=
       mozc::commands::Request::DEFAULT_TABLE) {
     const char *table_file_name;
@@ -201,6 +205,9 @@ bool Table::InitializeWithRequestAndConfig(
       case mozc::commands::Request::NOTOUCH_TO_HALFWIDTHASCII:
         table_file_name = kNotouchHalfwidthasciiTableFile;
         break;
+      case mozc::commands::Request::FIFTY_KEYS_TO_HIRAGANA:
+        table_file_name = k50KeysHiraganaTableFile;
+        break;
       default:
         table_file_name = nullptr;
     }
@@ -306,13 +313,13 @@ bool Table::InitializeWithRequestAndConfig(
   return result;
 }
 
-bool Table::IsLoopingEntry(const std::string &input,
-                           const std::string &pending) const {
+bool Table::IsLoopingEntry(const absl::string_view input,
+                           const absl::string_view pending) const {
   if (input.empty() || pending.empty()) {
     return false;
   }
 
-  std::string key = pending;
+  std::string key(pending);
   do {
     // If input is a prefix of key, it should be looping.
     // (ex. input="a", pending="abc").
@@ -327,25 +334,26 @@ bool Table::IsLoopingEntry(const std::string &input,
       return false;
     }
     DCHECK_LE(key_length, key.size());
-    key = entry->pending() + key.substr(key_length);
+    key = absl::StrCat(entry->pending(), key.substr(key_length));
   } while (!key.empty());
 
   return false;
 }
 
-const Entry *Table::AddRule(const std::string &input, const std::string &output,
-                            const std::string &pending) {
+const Entry *Table::AddRule(const absl::string_view input,
+                            const absl::string_view output,
+                            const absl::string_view pending) {
   return AddRuleWithAttributes(input, output, pending, NO_TABLE_ATTRIBUTE);
 }
 
-const Entry *Table::AddRuleWithAttributes(const std::string &escaped_input,
-                                          const std::string &output,
-                                          const std::string &escaped_pending,
-                                          const TableAttributes attributes) {
+const Entry *Table::AddRuleWithAttributes(
+    const absl::string_view escaped_input, const absl::string_view output,
+    const absl::string_view escaped_pending, const TableAttributes attributes) {
   if (attributes & NEW_CHUNK) {
     // TODO(komatsu): Make a new trie tree for checking the new chunk
     // attribute rather than reusing the conversion trie.
-    const std::string additional_input = kNewChunkPrefix + escaped_input;
+    const std::string additional_input =
+        absl::StrCat(kNewChunkPrefix, escaped_input);
     AddRuleWithAttributes(additional_input, output, escaped_pending,
                           NO_TABLE_ATTRIBUTE);
   }
@@ -357,8 +365,8 @@ const Entry *Table::AddRuleWithAttributes(const std::string &escaped_input,
     return nullptr;
   }
 
-  const std::string input = ParseSpecialKey(escaped_input);
-  const std::string pending = ParseSpecialKey(escaped_pending);
+  const std::string input = special_key_map_.Register(escaped_input);
+  const std::string pending = special_key_map_.Register(escaped_pending);
   if (IsLoopingEntry(input, pending)) {
     LOG(WARNING) << "Entry " << input << " " << output << " " << pending
                  << " is removed, since the rule is looping";
@@ -366,30 +374,31 @@ const Entry *Table::AddRuleWithAttributes(const std::string &escaped_input,
   }
 
   const Entry *old_entry = nullptr;
-  if (entries_->LookUp(input, &old_entry)) {
+  if (entries_.LookUp(input, &old_entry)) {
     DeleteEntry(old_entry);
   }
 
-  Entry *entry = new Entry(input, output, pending, attributes);
-  entries_->AddEntry(input, entry);
-  entry_set_.insert(entry);
+  auto entry = std::make_unique<Entry>(input, output, pending, attributes);
+  Entry *entry_ptr = entry.get();
+  entries_.AddEntry(input, entry_ptr);
+  entry_set_.insert(std::move(entry));
 
-  // Check if the input has a large captal character.
+  // Check if the input has a large capital character.
   // Invisible character is exception.
   if (!case_sensitive_) {
-    const std::string trimed_input = DeleteSpecialKey(input);
+    const std::string trimed_input = DeleteSpecialKeys(input);
     for (ConstChar32Iterator iter(trimed_input); !iter.Done(); iter.Next()) {
-      const char32 ucs4 = iter.Get();
+      const char32_t ucs4 = iter.Get();
       if ('A' <= ucs4 && ucs4 <= 'Z') {
         case_sensitive_ = true;
         break;
       }
     }
   }
-  return entry;
+  return entry_ptr;
 }
 
-void Table::DeleteRule(const std::string &input) {
+void Table::DeleteRule(const absl::string_view input) {
   // NOTE : If this method is called and an entry is deleted,
   //     case_sensitive_ turns to be invalid
   //     because it is not updated.
@@ -399,10 +408,10 @@ void Table::DeleteRule(const std::string &input) {
   //     - This method has no tests.
   //     - This method is private scope.
   const Entry *old_entry;
-  if (entries_->LookUp(input, &old_entry)) {
+  if (entries_.LookUp(input, &old_entry)) {
     DeleteEntry(old_entry);
   }
-  entries_->DeleteEntry(input);
+  entries_.DeleteEntry(input);
 }
 
 bool Table::LoadFromString(const std::string &str) {
@@ -418,25 +427,23 @@ bool Table::LoadFromFile(const char *filepath) {
   return LoadFromStream(ifs.get());
 }
 
-const TypingModel *Table::typing_model() const { return typing_model_.get(); }
-
 namespace {
 constexpr char kAttributeDelimiter = ' ';
 
-TableAttributes ParseAttributes(const std::string &input) {
+TableAttributes ParseAttributes(const absl::string_view input) {
   TableAttributes attributes = NO_TABLE_ATTRIBUTE;
 
-  std::vector<std::string> attribute_strings =
+  std::vector<absl::string_view> attribute_strings =
       absl::StrSplit(input, kAttributeDelimiter, absl::AllowEmpty());
 
-  for (size_t i = 0; i < attribute_strings.size(); ++i) {
-    if (attribute_strings[i] == "NewChunk") {
+  for (const absl::string_view attribute_string : attribute_strings) {
+    if (attribute_string == "NewChunk") {
       attributes |= NEW_CHUNK;
-    } else if (attribute_strings[i] == "NoTransliteration") {
+    } else if (attribute_string == "NoTransliteration") {
       attributes |= NO_TRANSLITERATION;
-    } else if (attribute_strings[i] == "DirectInput") {
+    } else if (attribute_string == "DirectInput") {
       attributes |= DIRECT_INPUT;
-    } else if (attribute_strings[i] == "EndChunk") {
+    } else if (attribute_string == "EndChunk") {
       attributes |= END_CHUNK;
     }
   }
@@ -447,7 +454,6 @@ TableAttributes ParseAttributes(const std::string &input) {
 bool Table::LoadFromStream(std::istream *is) {
   DCHECK(is);
   std::string line;
-  const std::string empty_pending("");
   while (!is->eof()) {
     std::getline(*is, line);
     Util::ChopReturns(&line);
@@ -463,7 +469,7 @@ bool Table::LoadFromStream(std::istream *is) {
     } else if (rules.size() == 3) {
       AddRule(rules[0], rules[1], rules[2]);
     } else if (rules.size() == 2) {
-      AddRule(rules[0], rules[1], empty_pending);
+      AddRule(rules[0], rules[1], "");
     } else {
       if (line[0] != '#') {
         LOG(ERROR) << "Format error: " << line;
@@ -475,48 +481,48 @@ bool Table::LoadFromStream(std::istream *is) {
   return true;
 }
 
-const Entry *Table::LookUp(const std::string &input) const {
+const Entry *Table::LookUp(const absl::string_view input) const {
   const Entry *entry = nullptr;
   if (case_sensitive_) {
-    entries_->LookUp(input, &entry);
+    entries_.LookUp(input, &entry);
   } else {
-    std::string normalized_input = input;
+    std::string normalized_input(input);
     Util::LowerString(&normalized_input);
-    entries_->LookUp(normalized_input, &entry);
+    entries_.LookUp(normalized_input, &entry);
   }
   return entry;
 }
 
-const Entry *Table::LookUpPrefix(const std::string &input, size_t *key_length,
-                                 bool *fixed) const {
+const Entry *Table::LookUpPrefix(const absl::string_view input,
+                                 size_t *key_length, bool *fixed) const {
   const Entry *entry = nullptr;
   if (case_sensitive_) {
-    entries_->LookUpPrefix(input, &entry, key_length, fixed);
+    entries_.LookUpPrefix(input, &entry, key_length, fixed);
   } else {
-    std::string normalized_input = input;
+    std::string normalized_input(input);
     Util::LowerString(&normalized_input);
-    entries_->LookUpPrefix(normalized_input, &entry, key_length, fixed);
+    entries_.LookUpPrefix(normalized_input, &entry, key_length, fixed);
   }
   return entry;
 }
 
-void Table::LookUpPredictiveAll(const std::string &input,
+void Table::LookUpPredictiveAll(const absl::string_view input,
                                 std::vector<const Entry *> *results) const {
   if (case_sensitive_) {
-    entries_->LookUpPredictiveAll(input, results);
+    entries_.LookUpPredictiveAll(input, results);
   } else {
-    std::string normalized_input = input;
+    std::string normalized_input(input);
     Util::LowerString(&normalized_input);
-    entries_->LookUpPredictiveAll(normalized_input, results);
+    entries_.LookUpPredictiveAll(normalized_input, results);
   }
 }
 
-bool Table::HasNewChunkEntry(const std::string &input) const {
+bool Table::HasNewChunkEntry(const absl::string_view input) const {
   if (input.empty()) {
     return false;
   }
 
-  const std::string key = kNewChunkPrefix + input;
+  const std::string key = absl::StrCat(kNewChunkPrefix, input);
   size_t key_length = 0;
   bool fixed = false;
   LookUpPrefix(key, &key_length, &fixed);
@@ -527,28 +533,17 @@ bool Table::HasNewChunkEntry(const std::string &input) const {
   return false;
 }
 
-bool Table::HasSubRules(const std::string &input) const {
+bool Table::HasSubRules(const absl::string_view input) const {
   if (case_sensitive_) {
-    return entries_->HasSubTrie(input);
+    return entries_.HasSubTrie(input);
   } else {
-    std::string normalized_input = input;
+    std::string normalized_input(input);
     Util::LowerString(&normalized_input);
-    return entries_->HasSubTrie(normalized_input);
+    return entries_.HasSubTrie(normalized_input);
   }
 }
 
-void Table::DeleteEntry(const Entry *entry) {
-  entry_set_.erase(entry);
-  delete entry;
-}
-
-void Table::ResetEntrySet() {
-  for (EntrySet::iterator it = entry_set_.begin(); it != entry_set_.end();
-       ++it) {
-    delete *it;
-  }
-  entry_set_.clear();
-}
+void Table::DeleteEntry(const Entry *entry) { entry_set_.erase(entry); }
 
 bool Table::case_sensitive() const { return case_sensitive_; }
 
@@ -556,76 +551,6 @@ void Table::set_case_sensitive(const bool case_sensitive) {
   case_sensitive_ = case_sensitive;
 }
 
-namespace {
-bool FindBlock(const std::string &input, const std::string &open,
-               const std::string &close, const size_t offset, size_t *open_pos,
-               size_t *close_pos) {
-  DCHECK(open_pos);
-  DCHECK(close_pos);
-
-  *open_pos = input.find(open, offset);
-  if (*open_pos == std::string::npos) {
-    return false;
-  }
-
-  *close_pos = input.find(close, *open_pos);
-  if (*close_pos == std::string::npos) {
-    return false;
-  }
-
-  return true;
-}
-}  // namespace
-
-// static
-std::string Table::ParseSpecialKey(const std::string &input) {
-  std::string output;
-  size_t open_pos = 0;
-  size_t close_pos = 0;
-  for (size_t cursor = 0; cursor < input.size();) {
-    if (!FindBlock(input, "{", "}", cursor, &open_pos, &close_pos)) {
-      output.append(input, cursor, input.size() - cursor);
-      break;
-    }
-
-    output.append(input, cursor, open_pos - cursor);
-
-    // The both sizes of "{" and "}" is 1.
-    const std::string key(input, open_pos + 1, close_pos - open_pos - 1);
-    if (key == "{") {
-      // "{{}" is treated as "{".
-      output.append("{");
-    } else {
-      // "{abc}" is replaced with "<kSpecialKeyOpen>abc<kSpecialKeyClose>".
-      output.append(kSpecialKeyOpen);
-      output.append(key);
-      output.append(kSpecialKeyClose);
-    }
-
-    cursor = close_pos + 1;
-  }
-  return output;
-}
-
-// static
-std::string Table::DeleteSpecialKey(const std::string &input) {
-  std::string output;
-  size_t open_pos = 0;
-  size_t close_pos = 0;
-  for (size_t cursor = 0; cursor < input.size();) {
-    if (!FindBlock(input, kSpecialKeyOpen, kSpecialKeyClose, cursor, &open_pos,
-                   &close_pos)) {
-      output.append(input, cursor, input.size() - cursor);
-      break;
-    }
-
-    output.append(input, cursor, open_pos - cursor);
-    // The size of kSpecialKeyClose is 1.
-    cursor = close_pos + 1;
-  }
-  return output;
-}
-
 // static
 const Table &Table::GetDefaultTable() {
   static Table *default_table = nullptr;
@@ -639,9 +564,7 @@ const Table &Table::GetDefaultTable() {
 // TableContainer
 // ========================================
 TableManager::TableManager()
-    : custom_roman_table_fingerprint_(Hash::Fingerprint32("")) {}
-
-TableManager::~TableManager() = default;
+    : custom_roman_table_fingerprint_(Fingerprint32("")) {}
 
 const Table *TableManager::GetTable(
     const mozc::commands::Request &request, const mozc::config::Config &config,
@@ -661,7 +584,7 @@ const Table *TableManager::GetTable(
   if ((config.preedit_method() == config::Config::ROMAN) &&
       config.has_custom_roman_table() && !config.custom_roman_table().empty()) {
     const uint32_t custom_roman_table_fingerprint =
-        Hash::Fingerprint32(config.custom_roman_table());
+        Fingerprint32(config.custom_roman_table());
     if (custom_roman_table_fingerprint != custom_roman_table_fingerprint_) {
       update_custom_roman_table = true;
       custom_roman_table_fingerprint_ = custom_roman_table_fingerprint;
@@ -678,7 +601,7 @@ const Table *TableManager::GetTable(
     }
   }
 
-  std::unique_ptr<Table> table(new Table());
+  auto table = std::make_unique<Table>();
   if (!table->InitializeWithRequestAndConfig(request, config, data_manager)) {
     return nullptr;
   }
