@@ -29,84 +29,57 @@
 
 #include "config/config_handler.h"
 
-#include <cstdint>
-
-#ifdef OS_WIN
-#include <windows.h>
-#endif  // OS_WIN
-
-#include <atomic>
+#include <array>
+#include <cstddef>
 #include <memory>
 #include <string>
+#include <vector>
 
+#include "base/clock.h"
+#include "base/clock_mock.h"
+#include "base/file/temp_dir.h"
 #include "base/file_util.h"
-#include "base/logging.h"
-#include "base/port.h"
 #include "base/system_util.h"
 #include "base/thread.h"
-#include "base/util.h"
 #include "protocol/config.pb.h"
-#include "testing/base/public/gmock.h"
-#include "testing/base/public/googletest.h"
-#include "testing/base/public/gunit.h"
-#include "testing/base/public/mozctest.h"
+#include "testing/gmock.h"
+#include "testing/gunit.h"
+#include "testing/mozctest.h"
 #include "absl/container/flat_hash_set.h"
-#include "absl/flags/flag.h"
-#include "absl/strings/str_format.h"
+#include "absl/random/random.h"
+#include "absl/strings/str_cat.h"
+#include "absl/strings/string_view.h"
+#include "absl/synchronization/notification.h"
+#include "absl/time/clock.h"
+#include "absl/time/time.h"
 
 namespace mozc {
 namespace config {
 namespace {
 
-class ConfigHandlerTest : public ::testing::Test {
+class ConfigHandlerTest : public testing::TestWithTempUserProfile {
  protected:
-  void SetUp() override {
-    SystemUtil::SetUserProfileDirectory(absl::GetFlag(FLAGS_test_tmpdir));
-    default_config_filename_ = ConfigHandler::GetConfigFileName();
-    Config default_config;
-    ConfigHandler::GetDefaultConfig(&default_config);
-    ConfigHandler::SetConfig(default_config);
-  }
-
-  void TearDown() override {
-    ConfigHandler::SetConfigFileName(default_config_filename_);
-    Config default_config;
-    ConfigHandler::GetDefaultConfig(&default_config);
-    ConfigHandler::SetConfig(default_config);
-  }
-
- private:
-  std::string default_config_filename_;
-};
-
-class ScopedSetConfigFileName {
- public:
-  explicit ScopedSetConfigFileName(const std::string &new_name)
-      : default_config_filename_(ConfigHandler::GetConfigFileName()) {
-    ConfigHandler::SetConfigFileName(new_name);
-  }
-
-  ~ScopedSetConfigFileName() {
+  ConfigHandlerTest()
+      : default_config_filename_(ConfigHandler::GetConfigFileName()) {}
+  ~ConfigHandlerTest() override {
     ConfigHandler::SetConfigFileName(default_config_filename_);
   }
 
  private:
   std::string default_config_filename_;
-
-  DISALLOW_IMPLICIT_CONSTRUCTORS(ScopedSetConfigFileName);
 };
 
 TEST_F(ConfigHandlerTest, SetConfig) {
   Config input;
   Config output;
 
-  const std::string config_file = FileUtil::JoinPath(
-      absl::GetFlag(FLAGS_test_tmpdir), "mozc_config_test_tmp");
+  TempDirectory temp_dir = testing::MakeTempDirectoryOrDie();
+  const std::string config_file =
+      FileUtil::JoinPath(temp_dir.path(), "mozc_config_test_tmp");
   ASSERT_OK(FileUtil::UnlinkIfExists(config_file));
-  ScopedSetConfigFileName scoped_config_file_name(config_file);
-  EXPECT_EQ(config_file, ConfigHandler::GetConfigFileName());
-  ASSERT_TRUE(ConfigHandler::Reload())
-      << "failed to reload: " << ConfigHandler::GetConfigFileName();
+  ConfigHandler::SetConfigFileName(config_file);
+  EXPECT_EQ(ConfigHandler::GetConfigFileName(), config_file);
+  ConfigHandler::Reload();
 
   ConfigHandler::GetDefaultConfig(&input);
   input.set_incognito_mode(true);
@@ -114,12 +87,15 @@ TEST_F(ConfigHandlerTest, SetConfig) {
   input.set_verbose_level(2);
 #endif  // MOZC_NO_LOGGING
   ConfigHandler::SetMetaData(&input);
-  EXPECT_TRUE(ConfigHandler::SetConfig(input));
+  ConfigHandler::SetConfig(input);
   output.Clear();
-  EXPECT_TRUE(ConfigHandler::GetConfig(&output));
+  ConfigHandler::GetConfig(&output);
+  std::unique_ptr<config::Config> output2 = ConfigHandler::GetConfig();
   input.mutable_general_config()->set_last_modified_time(0);
   output.mutable_general_config()->set_last_modified_time(0);
-  EXPECT_EQ(input.DebugString(), output.DebugString());
+  output2->mutable_general_config()->set_last_modified_time(0);
+  EXPECT_EQ(absl::StrCat(output), absl::StrCat(input));
+  EXPECT_EQ(absl::StrCat(*output2), absl::StrCat(input));
 
   ConfigHandler::GetDefaultConfig(&input);
   input.set_incognito_mode(false);
@@ -127,20 +103,23 @@ TEST_F(ConfigHandlerTest, SetConfig) {
   input.set_verbose_level(0);
 #endif  // MOZC_NO_LOGGING
   ConfigHandler::SetMetaData(&input);
-  EXPECT_TRUE(ConfigHandler::SetConfig(input));
+  ConfigHandler::SetConfig(input);
   output.Clear();
-  EXPECT_TRUE(ConfigHandler::GetConfig(&output));
+  ConfigHandler::GetConfig(&output);
+  output2 = ConfigHandler::GetConfig();
 
   input.mutable_general_config()->set_last_modified_time(0);
   output.mutable_general_config()->set_last_modified_time(0);
-  EXPECT_EQ(input.DebugString(), output.DebugString());
+  output2->mutable_general_config()->set_last_modified_time(0);
+  EXPECT_EQ(absl::StrCat(output), absl::StrCat(input));
+  EXPECT_EQ(absl::StrCat(*output2), absl::StrCat(input));
 
-#if defined(OS_ANDROID) && defined(CHANNEL_DEV)
+#if defined(__ANDROID__) && defined(CHANNEL_DEV)
   input.Clear();
   EXPECT_FALSE(input.general_config().has_upload_usage_stats());
   EXPECT_TRUE(ConfigHandler::SetConfig(input));
   output.Clear();
-  EXPECT_TRUE(ConfigHandler::GetConfig(&output));
+  ConfigHandler::GetConfig(&output);
   EXPECT_TRUE(output.general_config().has_upload_usage_stats());
   EXPECT_TRUE(output.general_config().upload_usage_stats());
 
@@ -150,93 +129,75 @@ TEST_F(ConfigHandlerTest, SetConfig) {
   EXPECT_FALSE(input.general_config().upload_usage_stats());
   EXPECT_TRUE(ConfigHandler::SetConfig(input));
   output.Clear();
-  EXPECT_TRUE(ConfigHandler::GetConfig(&output));
+  ConfigHandler::GetConfig(&output);
   EXPECT_TRUE(output.general_config().has_upload_usage_stats());
   EXPECT_TRUE(output.general_config().upload_usage_stats());
-#endif  // OS_ANDROID && CHANNEL_DEV
+#endif  // __ANDROID__ && CHANNEL_DEV
 }
 
-TEST_F(ConfigHandlerTest, SetImposedConfig) {
+TEST_F(ConfigHandlerTest, SetMetadata) {
+  ClockMock clock1(absl::FromUnixSeconds(1000));
+  Clock::SetClockForUnitTest(&clock1);
+  Config input1;
+  ConfigHandler::SetMetaData(&input1);
+
+  ClockMock clock2(absl::FromUnixSeconds(1000));
+  Clock::SetClockForUnitTest(&clock2);
+  Config input2;
+  ConfigHandler::SetMetaData(&input2);
+
+  ClockMock clock3(absl::FromUnixSeconds(1001));
+  Clock::SetClockForUnitTest(&clock3);
+  Config input3;
+  ConfigHandler::SetMetaData(&input3);
+
+  // input1 and input2 are created at the same time,
+  // but input3 is not.
+  EXPECT_EQ(absl::StrCat(input1), absl::StrCat(input2));
+  EXPECT_NE(absl::StrCat(input2), absl::StrCat(input3));
+  EXPECT_NE(absl::StrCat(input3), absl::StrCat(input1));
+  Clock::SetClockForUnitTest(nullptr);
+}
+
+TEST_F(ConfigHandlerTest, SetConfig_IdentityCheck) {
   Config input;
-  Config output;
 
-  const std::string config_file = FileUtil::JoinPath(
-      absl::GetFlag(FLAGS_test_tmpdir), "mozc_config_test_tmp");
+  TempDirectory temp_dir = testing::MakeTempDirectoryOrDie();
+  const std::string config_file =
+      FileUtil::JoinPath(temp_dir.path(), "mozc_config_test_tmp");
   ASSERT_OK(FileUtil::UnlinkIfExists(config_file));
-  ScopedSetConfigFileName scoped_config_file_name(config_file);
-  ASSERT_TRUE(ConfigHandler::Reload())
-      << "failed to reload: " << ConfigHandler::GetConfigFileName();
+  ConfigHandler::SetConfigFileName(config_file);
+  EXPECT_EQ(ConfigHandler::GetConfigFileName(), config_file);
+  ConfigHandler::Reload();
 
-  struct Testcase {
-    bool stored_config_value;
-    enum { DO_NOT_IMPOSE, IMPOSE_TRUE, IMPOSE_FALSE } imposed_config_value;
-    bool expected_value;
-  };
-  const Testcase kTestcases[] = {
-      {true, Testcase::IMPOSE_TRUE, true},
-      {true, Testcase::IMPOSE_FALSE, false},
-      {false, Testcase::IMPOSE_TRUE, true},
-      {false, Testcase::IMPOSE_FALSE, false},
-      {true, Testcase::DO_NOT_IMPOSE, true},
-      {false, Testcase::DO_NOT_IMPOSE, false},
-  };
+  ConfigHandler::GetDefaultConfig(&input);
+  input.set_incognito_mode(true);
+#ifndef MOZC_NO_LOGGING
+  input.set_verbose_level(2);
+#endif  // MOZC_NO_LOGGING
 
-  for (size_t i = 0; i < std::size(kTestcases); ++i) {
-    const bool stored_config_value = kTestcases[i].stored_config_value;
-    const bool expected = kTestcases[i].expected_value;
+  ClockMock clock1(absl::FromUnixSeconds(1000));
 
-    // Set current config.
-    ConfigHandler::GetDefaultConfig(&input);
-    input.set_incognito_mode(stored_config_value);
-    ConfigHandler::SetMetaData(&input);
-    EXPECT_TRUE(ConfigHandler::SetConfig(input));
-    // Set imposed config.
-    input.Clear();
-    if (kTestcases[i].imposed_config_value != Testcase::DO_NOT_IMPOSE) {
-      input.set_incognito_mode(kTestcases[i].imposed_config_value ==
-                               Testcase::IMPOSE_TRUE);
-    }
-    ConfigHandler::SetImposedConfig(input);
-    // Check post-condition.
-    output.Clear();
-    EXPECT_TRUE(ConfigHandler::GetConfig(&output));
-    EXPECT_EQ(expected, output.incognito_mode());
-    ConfigHandler::GetConfig(&output);
-    EXPECT_EQ(expected, output.incognito_mode());
-    ConfigHandler::GetStoredConfig(&output);
-    EXPECT_EQ(stored_config_value, output.incognito_mode());
+  Clock::SetClockForUnitTest(&clock1);
+  ConfigHandler::SetConfig(input);
+  std::unique_ptr<config::Config> output1 = ConfigHandler::GetConfig();
 
-    // Reload and check.
-    ASSERT_TRUE(ConfigHandler::Reload())
-        << "failed to reload: " << ConfigHandler::GetConfigFileName();
-    output.Clear();
-    EXPECT_TRUE(ConfigHandler::GetConfig(&output));
-    EXPECT_EQ(expected, output.incognito_mode());
-    ConfigHandler::GetConfig(&output);
-    EXPECT_EQ(expected, output.incognito_mode());
-    ConfigHandler::GetStoredConfig(&output);
-    EXPECT_EQ(stored_config_value, output.incognito_mode());
+  ClockMock clock2(absl::FromUnixSeconds(1001));
+  Clock::SetClockForUnitTest(&clock2);
+  ConfigHandler::SetConfig(input);
+  std::unique_ptr<config::Config> output2 = ConfigHandler::GetConfig();
 
-    // Unset imposed config.
-    input.Clear();
-    ConfigHandler::SetImposedConfig(input);
-    // Check post-condition.
-    output.Clear();
-    EXPECT_TRUE(ConfigHandler::GetConfig(&output));
-    EXPECT_EQ(stored_config_value, output.incognito_mode());
-    ConfigHandler::GetConfig(&output);
-    EXPECT_EQ(stored_config_value, output.incognito_mode());
-    ConfigHandler::GetStoredConfig(&output);
-    EXPECT_EQ(stored_config_value, output.incognito_mode());
-  }
+  // As SetConfig() is called twice with the same config,
+  // GetConfig() must return the identical (including metadata!) config.
+  // This also means no actual storage write access happened.
+  EXPECT_EQ(absl::StrCat(*output1), absl::StrCat(*output2));
+  Clock::SetClockForUnitTest(nullptr);
 }
 
 TEST_F(ConfigHandlerTest, ConfigFileNameConfig) {
-  const std::string config_file =
-      std::string("config") + std::to_string(config::CONFIG_VERSION) + ".db";
-
+  const std::string config_file = absl::StrCat("config", kConfigVersion, ".db");
   const std::string filename =
-      FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir), config_file);
+      FileUtil::JoinPath(SystemUtil::GetUserProfileDirectory(), config_file);
   ASSERT_OK(FileUtil::UnlinkIfExists(filename));
   Config input;
   ConfigHandler::SetConfig(input);
@@ -248,59 +209,42 @@ TEST_F(ConfigHandlerTest, SetConfigFileName) {
   const bool default_incognito_mode = mozc_config.incognito_mode();
   mozc_config.set_incognito_mode(!default_incognito_mode);
   ConfigHandler::SetConfig(mozc_config);
-  // ScopedSetConfigFileName internally calls SetConfigFileName.
-  ScopedSetConfigFileName scoped_config_file_name(
-      "memory://set_config_file_name_test.db");
+  ConfigHandler::SetConfigFileName("memory://set_config_file_name_test.db");
   // After SetConfigFileName called, settings are set as default.
   Config updated_config;
   ConfigHandler::GetConfig(&updated_config);
-  EXPECT_EQ(default_incognito_mode, updated_config.incognito_mode());
+  EXPECT_EQ(updated_config.incognito_mode(), default_incognito_mode);
 }
 
-#if !defined(OS_ANDROID)
+#if !defined(__ANDROID__)
 // Temporarily disable this test because FileUtil::CopyFile fails on
 // Android for some reason.
 TEST_F(ConfigHandlerTest, LoadTestConfig) {
   // TODO(yukawa): Generate test data automatically so that we can keep
   //     the compatibility among variety of config files.
   // TODO(yukawa): Enumerate test data in the directory automatically.
-  const char *kDataFiles[] = {
+  constexpr std::array<absl::string_view, 3> kDataFiles = {
       "linux_config1.db",
       "mac_config1.db",
       "win_config1.db",
   };
 
-  for (size_t i = 0; i < std::size(kDataFiles); ++i) {
-    const char *file_name = kDataFiles[i];
-    const std::string &src_path = mozc::testing::GetSourceFileOrDie(
-        {"data", "test", "config", file_name});
-    const std::string &dest_path =
+  for (absl::string_view file_name : kDataFiles) {
+    const std::string src_path = mozc::testing::GetSourceFileOrDie(
+        {MOZC_DICT_DIR_COMPONENTS, "test", "config", file_name});
+    const std::string dest_path =
         FileUtil::JoinPath(SystemUtil::GetUserProfileDirectory(), file_name);
     ASSERT_OK(FileUtil::CopyFile(src_path, dest_path))
         << "Copy failed: " << src_path << " to " << dest_path;
 
-    ScopedSetConfigFileName scoped_config_file_name("user://" +
-                                                    std::string(file_name));
-    ASSERT_TRUE(ConfigHandler::Reload())
-        << "failed to reload: " << ConfigHandler::GetConfigFileName();
+    ConfigHandler::SetConfigFileName(absl::StrCat("user://", file_name));
+    ConfigHandler::Reload();
 
     Config default_config;
-    EXPECT_TRUE(ConfigHandler::GetConfig(&default_config))
-        << "failed to GetConfig from: " << file_name;
-
-#ifdef OS_WIN
-    // Reset the file attributes since it may contain FILE_ATTRIBUTE_READONLY.
-    std::wstring wdest_path;
-    Util::Utf8ToWide(dest_path, &wdest_path);
-    ::SetFileAttributesW(wdest_path.c_str(), FILE_ATTRIBUTE_NORMAL);
-#endif  // OS_WIN
-
-    // Remove test file just in case.
-    ASSERT_OK(FileUtil::Unlink(dest_path));
-    EXPECT_FALSE(FileUtil::FileExists(dest_path).ok());
+    ConfigHandler::GetConfig(&default_config);
   }
 }
-#endif  // !OS_ANDROID
+#endif  // !__ANDROID__
 
 TEST_F(ConfigHandlerTest, GetDefaultConfig) {
   Config output;
@@ -317,11 +261,12 @@ TEST_F(ConfigHandlerTest, GetDefaultConfig) {
   EXPECT_EQ(output.character_form_rules_size(), 13);
 
   struct TestCase {
-    const char *group;
+    absl::string_view group;
     Config::CharacterForm preedit_character_form;
     Config::CharacterForm conversion_character_form;
   };
-  const TestCase testcases[] = {
+
+  constexpr std::array<TestCase, 13> kTestCases = {{
       // "ア"
       {"ア", Config::FULL_WIDTH, Config::FULL_WIDTH},
       {"A", Config::FULL_WIDTH, Config::LAST_FORM},
@@ -338,51 +283,28 @@ TEST_F(ConfigHandlerTest, GetDefaultConfig) {
       {"~", Config::FULL_WIDTH, Config::LAST_FORM},
       {"<>=+-/*", Config::FULL_WIDTH, Config::LAST_FORM},
       {"?!", Config::FULL_WIDTH, Config::LAST_FORM},
-  };
-  EXPECT_EQ(output.character_form_rules_size(), std::size(testcases));
-  for (size_t i = 0; i < std::size(testcases); ++i) {
-    EXPECT_EQ(output.character_form_rules(i).group(), testcases[i].group);
+  }};
+  EXPECT_EQ(output.character_form_rules_size(), kTestCases.size());
+  for (size_t i = 0; i < kTestCases.size(); ++i) {
+    EXPECT_EQ(output.character_form_rules(i).group(), kTestCases[i].group);
     EXPECT_EQ(output.character_form_rules(i).preedit_character_form(),
-              testcases[i].preedit_character_form);
+              kTestCases[i].preedit_character_form);
     EXPECT_EQ(output.character_form_rules(i).conversion_character_form(),
-              testcases[i].conversion_character_form);
+              kTestCases[i].conversion_character_form);
   }
 
-#if defined(OS_ANDROID) && defined(CHANNEL_DEV)
+#if defined(__ANDROID__) && defined(CHANNEL_DEV)
   EXPECT_TRUE(output.general_config().has_upload_usage_stats());
   EXPECT_TRUE(output.general_config().upload_usage_stats());
-#endif  // OS_ANDROID && CHANNEL_DEV
+#endif  // __ANDROID__ && CHANNEL_DEV
 }
 
 TEST_F(ConfigHandlerTest, DefaultConfig) {
   Config config;
   ConfigHandler::GetDefaultConfig(&config);
-  EXPECT_EQ(config.DebugString(), ConfigHandler::DefaultConfig().DebugString());
+  EXPECT_EQ(absl::StrCat(ConfigHandler::DefaultConfig()), absl::StrCat(config));
 }
 
-class SetConfigThread final : public Thread {
- public:
-  explicit SetConfigThread(const std::vector<Config> &configs)
-      : quitting_(false), configs_(configs) {}
-
-  ~SetConfigThread() override {
-    quitting_ = true;
-    Join();
-  }
-
- protected:
-  void Run() override {
-    while (!quitting_) {
-      const size_t next_index = Util::Random(configs_.size());
-      ConfigHandler::SetConfig(configs_.at(next_index));
-    }
-  }
-
- private:
-  std::atomic<bool> quitting_;
-  std::vector<Config> configs_;
-};
-
 // Returns concatenated serialized data of |Config::character_form_rules|.
 std::string ExtractCharacterFormRules(const Config &config) {
   std::string rules;
@@ -392,33 +314,6 @@ std::string ExtractCharacterFormRules(const Config &config) {
   return rules;
 }
 
-class GetConfigThread final : public Thread {
- public:
-  explicit GetConfigThread(
-      const absl::flat_hash_set<std::string> &character_form_rules_set)
-      : quitting_(false), character_form_rules_set_(character_form_rules_set) {}
-
-  ~GetConfigThread() override {
-    quitting_ = true;
-    Join();
-  }
-
- protected:
-  void Run() override {
-    while (!quitting_) {
-      Config config;
-      ConfigHandler::GetConfig(&config);
-      const auto &rules = ExtractCharacterFormRules(config);
-      EXPECT_NE(character_form_rules_set_.end(),
-                character_form_rules_set_.find(rules));
-    }
-  }
-
- private:
-  std::atomic<bool> quitting_;
-  const absl::flat_hash_set<std::string> character_form_rules_set_;
-};
-
 TEST_F(ConfigHandlerTest, ConcurrentAccess) {
   std::vector<Config> configs;
 
@@ -483,43 +378,48 @@ TEST_F(ConfigHandlerTest, ConcurrentAccess) {
     Config returned_config;
     ConfigHandler::GetConfig(&returned_config);
     const auto &rules = ExtractCharacterFormRules(returned_config);
-    ASSERT_NE(character_form_rules_set.end(),
-              character_form_rules_set.find(rules));
+    ASSERT_TRUE(character_form_rules_set.contains(rules));
   }
 
-  // 250 msec is good enough to crash the code if it is not guarded by
-  // the lock, but feel free to change the duration.  It is basically an
-  // arbitrary number.
-  constexpr uint32_t kTestDurationMSec = 250;  // 250 msec
-  constexpr size_t kNumSetThread = 2;
-  constexpr size_t kNumGetThread = 4;
   {
-    // Set up background threads for concurrent access.
-    std::vector<std::unique_ptr<SetConfigThread>> set_threads;
-    for (size_t i = 0; i < kNumSetThread; ++i) {
-      set_threads.emplace_back(std::make_unique<SetConfigThread>(configs));
+    absl::Notification cancel;
+
+    std::vector<Thread> set_threads;
+    for (int i = 0; i < 2; ++i) {
+      set_threads.push_back(Thread([&cancel, &configs] {
+        absl::BitGen gen;
+        while (!cancel.HasBeenNotified()) {
+          const size_t next_index = absl::Uniform(gen, 0u, configs.size());
+          ConfigHandler::SetConfig(configs[next_index]);
+        }
+      }));
     }
-    std::vector<std::unique_ptr<GetConfigThread>> get_threads;
-    for (size_t i = 0; i < kNumGetThread; ++i) {
-      get_threads.emplace_back(
-          std::make_unique<GetConfigThread>(character_form_rules_set));
-    }
-    // Let background threads start accessing ConfigHandler from multiple
-    // background threads.
-    for (size_t i = 0; i < set_threads.size(); ++i) {
-      set_threads[i]->Start(
-          absl::StrFormat("SetConfigThread%d", static_cast<int>(i)));
-    }
-    for (size_t i = 0; i < get_threads.size(); ++i) {
-      get_threads[i]->Start(
-          absl::StrFormat("GetConfigThread%d", static_cast<int>(i)));
+
+    std::vector<Thread> get_threads;
+    for (int i = 0; i < 4; ++i) {
+      get_threads.push_back(Thread([&cancel, &character_form_rules_set] {
+        while (!cancel.HasBeenNotified()) {
+          Config config;
+          ConfigHandler::GetConfig(&config);
+          const auto &rules = ExtractCharacterFormRules(config);
+          EXPECT_TRUE(character_form_rules_set.contains(rules));
+        }
+      }));
     }
+
     // Wait for a while to see if everything goes well.
-    Util::Sleep(kTestDurationMSec);
-    // Destructors of |SetConfigThread| and |GetConfigThread| will take
-    // care of their background threads (in a blocking way).
-    set_threads.clear();
-    get_threads.clear();
+    // 250 msec is good enough to crash the code if it is not guarded by
+    // the lock, but feel free to change the duration.  It is basically an
+    // arbitrary number.
+    absl::SleepFor(absl::Milliseconds(250));
+
+    cancel.Notify();
+    for (auto &set_thread : set_threads) {
+      set_thread.Join();
+    }
+    for (auto &get_thread : get_threads) {
+      get_thread.Join();
+    }
   }
 }
 
