@@ -30,40 +30,42 @@
 #include "converter/connector.h"
 
 #include <algorithm>
-#include <memory>
-#include <random>
+#include <cstdint>
 #include <string>
+#include <utility>
 #include <vector>
 
 #include "base/logging.h"
 #include "base/mmap.h"
 #include "data_manager/connection_file_reader.h"
-#include "testing/base/public/gunit.h"
-#include "testing/base/public/mozctest.h"
-#include "absl/memory/memory.h"
+#include "testing/gmock.h"
+#include "testing/gunit.h"
+#include "testing/mozctest.h"
+#include "absl/random/random.h"
 
 namespace mozc {
 namespace {
 
 struct ConnectionDataEntry {
-  uint16 rid;
-  uint16 lid;
+  uint16_t rid;
+  uint16_t lid;
   int cost;
 };
 
 TEST(ConnectorTest, CompareWithRawData) {
   const std::string path = testing::GetSourceFileOrDie(
       {"data_manager", "testing", "connection.data"});
-  Mmap cmmap;
-  ASSERT_TRUE(cmmap.Open(path.c_str())) << "Failed to open image: " << path;
+  absl::StatusOr<Mmap> cmmap = Mmap::Map(path);
+  ASSERT_OK(cmmap) << cmmap.status();
   auto status_or_connector =
-      Connector::Create(cmmap.begin(), cmmap.size(), 256);
+      Connector::Create(cmmap->begin(), cmmap->size(), 256);
   ASSERT_TRUE(status_or_connector.ok()) << status_or_connector.status();
   auto connector = std::move(status_or_connector).value();
-  ASSERT_EQ(1, connector->GetResolution());
+  ASSERT_EQ(1, connector.GetResolution());
 
   const std::string connection_text_path = testing::GetSourceFileOrDie(
-      {"data_manager", "testing", "connection_single_column.txt"});
+      {MOZC_DICT_DIR_COMPONENTS, "test", "dictionary",
+       "connection_single_column.txt"});
   std::vector<ConnectionDataEntry> data;
   for (ConnectionFileReader reader(connection_text_path); !reader.done();
        reader.Next()) {
@@ -74,18 +76,17 @@ TEST(ConnectorTest, CompareWithRawData) {
     data.push_back(entry);
   }
 
+  absl::BitGen urbg;
   for (int trial = 0; trial < 3; ++trial) {
     // Lookup in random order for a few times.
-    std::random_device rd;
-    std::mt19937 urbg(rd());
     std::shuffle(data.begin(), data.end(), urbg);
     for (size_t i = 0; i < data.size(); ++i) {
-      int actual = connector->GetTransitionCost(data[i].rid, data[i].lid);
-      EXPECT_EQ(data[i].cost, actual);
+      int actual = connector.GetTransitionCost(data[i].rid, data[i].lid);
+      EXPECT_EQ(actual, data[i].cost);
 
       // Cache hit case.
-      actual = connector->GetTransitionCost(data[i].rid, data[i].lid);
-      EXPECT_EQ(data[i].cost, actual);
+      actual = connector.GetTransitionCost(data[i].rid, data[i].lid);
+      EXPECT_EQ(actual, data[i].cost);
     }
   }
 }
@@ -93,15 +94,15 @@ TEST(ConnectorTest, CompareWithRawData) {
 TEST(ConnectorTest, BrokenData) {
   const std::string path = testing::GetSourceFileOrDie(
       {"data_manager", "testing", "connection.data"});
-  Mmap cmmap;
-  ASSERT_TRUE(cmmap.Open(path.c_str())) << "Failed to open image: " << path;
+  absl::StatusOr<Mmap> cmmap = Mmap::Map(path);
+  ASSERT_OK(cmmap) << cmmap.status();
 
   std::string data;
 
   // Invalid magic number.
   {
-    data.assign(cmmap.begin(), cmmap.size());
-    *reinterpret_cast<uint16 *>(&data[0]) = 0;
+    data.assign(cmmap->begin(), cmmap->size());
+    *reinterpret_cast<uint16_t *>(&data[0]) = 0;
     const auto status =
         Connector::Create(data.data(), data.size(), 256).status();
     VLOG(1) << status;
@@ -109,8 +110,8 @@ TEST(ConnectorTest, BrokenData) {
   }
   // Not square.
   {
-    data.assign(cmmap.begin(), cmmap.size());
-    uint16 *array = reinterpret_cast<uint16 *>(&data[0]);
+    data.assign(cmmap->begin(), cmmap->size());
+    uint16_t *array = reinterpret_cast<uint16_t *>(&data[0]);
     array[2] = 100;
     array[3] = 200;
     const auto status =
@@ -120,7 +121,7 @@ TEST(ConnectorTest, BrokenData) {
   }
   // Incomplete data.
   {
-    data.assign(cmmap.begin(), cmmap.size());
+    data.assign(cmmap->begin(), cmmap->size());
     for (size_t divider : {2, 3, 5, 7, 10, 100, 1000}) {
       const auto size = data.size() / divider;
       const auto status = Connector::Create(data.data(), size, 256).status();
@@ -128,12 +129,12 @@ TEST(ConnectorTest, BrokenData) {
       EXPECT_FALSE(status.ok());
     }
   }
-  // Not aligned at 32-bit bounary.
+  // Not aligned at 32-bit boundary.
   {
-    data.resize(cmmap.size() + 2);
-    data.insert(2, cmmap.begin(), cmmap.size());  // Align at 16-bit boundary.
+    data.resize(cmmap->size() + 2);
+    data.insert(2, cmmap->begin(), cmmap->size());  // Align at 16-bit boundary.
     const auto status =
-        Connector::Create(data.data() + 2, cmmap.size(), 256).status();
+        Connector::Create(data.data() + 2, cmmap->size(), 256).status();
     VLOG(1) << status;
     EXPECT_FALSE(status.ok());
   }
