@@ -30,19 +30,20 @@
 #include "converter/immutable_converter.h"
 
 #include <algorithm>
-#include <cctype>
+#include <array>
 #include <climits>
+#include <cstddef>
 #include <cstdint>
+#include <limits>
 #include <memory>
 #include <string>
 #include <utility>
 #include <vector>
 
+#include "base/container/trie.h"
 #include "base/japanese_util.h"
 #include "base/logging.h"
-#include "base/port.h"
 #include "base/util.h"
-#include "config/config_handler.h"
 #include "converter/connector.h"
 #include "converter/key_corrector.h"
 #include "converter/lattice.h"
@@ -53,6 +54,7 @@
 #include "converter/segmenter.h"
 #include "converter/segments.h"
 #include "dictionary/dictionary_interface.h"
+#include "dictionary/dictionary_token.h"
 #include "dictionary/pos_group.h"
 #include "dictionary/pos_matcher.h"
 #include "dictionary/suppression_dictionary.h"
@@ -60,9 +62,11 @@
 #include "protocol/commands.pb.h"
 #include "protocol/config.pb.h"
 #include "request/conversion_request.h"
+#include "absl/algorithm/container.h"
 #include "absl/strings/match.h"
 #include "absl/strings/str_cat.h"
 #include "absl/strings/string_view.h"
+#include "absl/types/span.h"
 
 namespace mozc {
 namespace {
@@ -80,6 +84,10 @@ constexpr int kMaxCost = 32767;
 constexpr int kMinCost = -32767;
 constexpr int kDefaultNumberCost = 3000;
 
+bool IsMobileRequest(const ConversionRequest &request) {
+  return request.request().mixed_conversion();
+}
+
 class KeyCorrectedNodeListBuilder : public BaseNodeListBuilder {
  public:
   KeyCorrectedNodeListBuilder(size_t pos, absl::string_view original_lookup_key,
@@ -222,7 +230,7 @@ void NormalizeHistorySegments(Segments *segments) {
     segment->set_key(key);
 
     // Ad-hoc rewrite for Numbers
-    // Since number candidate is generative, i.e, any number can be
+    // Since number candidate is generative, i.e., any number can be
     // written by users, we normalize the value here. normalzied number
     // is used for the ranking tweaking based on history
     if (key.size() > 1 && key == c->value && key == c->content_value &&
@@ -277,9 +285,9 @@ ImmutableConverterImpl::ImmutableConverterImpl(
     const DictionaryInterface *dictionary,
     const DictionaryInterface *suffix_dictionary,
     const SuppressionDictionary *suppression_dictionary,
-    const Connector *connector, const Segmenter *segmenter,
+    const Connector &connector, const Segmenter *segmenter,
     const PosMatcher *pos_matcher, const PosGroup *pos_group,
-    const SuggestionFilter *suggestion_filter)
+    const SuggestionFilter &suggestion_filter)
     : dictionary_(dictionary),
       suffix_dictionary_(suffix_dictionary),
       suppression_dictionary_(suppression_dictionary),
@@ -293,38 +301,13 @@ ImmutableConverterImpl::ImmutableConverterImpl(
       number_id_(pos_matcher_->GetNumberId()),
       unknown_id_(pos_matcher_->GetUnknownId()),
       last_to_first_name_transition_cost_(
-          connector_->GetTransitionCost(last_name_id_, first_name_id_)) {
+          connector_.GetTransitionCost(last_name_id_, first_name_id_)) {
   DCHECK(dictionary_);
   DCHECK(suffix_dictionary_);
   DCHECK(suppression_dictionary_);
-  DCHECK(connector_);
   DCHECK(segmenter_);
   DCHECK(pos_matcher_);
   DCHECK(pos_group_);
-  DCHECK(suggestion_filter_);
-}
-
-void ImmutableConverterImpl::ExpandCandidates(const ConversionRequest &request,
-                                              const std::string &original_key,
-                                              NBestGenerator *nbest,
-                                              Segment *segment,
-                                              size_t expand_size) const {
-  DCHECK(nbest);
-  DCHECK(segment);
-  CHECK_GT(expand_size, 0);
-
-  while (segment->candidates_size() < expand_size) {
-    Segment::Candidate *candidate = segment->push_back_candidate();
-    DCHECK(candidate);
-    candidate->Init();
-
-    // if NBestGenerator::Next() returns nullptr,
-    // no more entries are generated.
-    if (!nbest->Next(request, original_key, candidate)) {
-      segment->pop_back_candidate();
-      break;
-    }
-  }
 }
 
 void ImmutableConverterImpl::InsertDummyCandidates(Segment *segment,
@@ -377,8 +360,6 @@ void ImmutableConverterImpl::InsertDummyCandidates(Segment *segment,
       *new_candidate = *last_candidate;
       // We cannot copy inner_segment_boundary; see b/8109381.
       new_candidate->inner_segment_boundary.clear();
-    } else {
-      new_candidate->Init();
     }
     new_candidate->key = segment->key();
     new_candidate->value = segment->key();
@@ -408,7 +389,6 @@ void ImmutableConverterImpl::InsertDummyCandidates(Segment *segment,
     Segment::Candidate *new_candidate = segment->add_candidate();
     DCHECK(new_candidate);
     DCHECK(last_candidate);
-    new_candidate->Init();
     new_candidate->key = segment->key();
     new_candidate->value = katakana_value;
     new_candidate->content_key = segment->key();
@@ -687,7 +667,7 @@ bool ImmutableConverterImpl::ResegmentPersonalName(size_t pos,
     //
     // last_name_cost + transition_cost + first_name_cost == compound_cost
     // last_name_cost == first_name_cost
-    // i.e,
+    // i.e.,
     // last_name_cost = first_name_cost =
     // (compound_cost - transition_cost) / 2;
     const int32_t wcost =
@@ -799,7 +779,7 @@ Node *ImmutableConverterImpl::AddCharacterTypeBasedNodes(const char *begin,
                                                          Lattice *lattice,
                                                          Node *nodes) const {
   size_t mblen = 0;
-  const char32 ucs4 = Util::Utf8ToUcs4(begin, end, &mblen);
+  const char32_t ucs4 = Util::Utf8ToUcs4(begin, end, &mblen);
 
   const Util::ScriptType first_script_type = Util::GetScriptType(ucs4);
   const Util::FormType first_form_type = Util::GetFormType(ucs4);
@@ -838,7 +818,7 @@ Node *ImmutableConverterImpl::AddCharacterTypeBasedNodes(const char *begin,
   int num_char = 1;
   const char *p = begin + mblen;
   while (p < end) {
-    const char32 next_ucs4 = Util::Utf8ToUcs4(p, end, &mblen);
+    const char32_t next_ucs4 = Util::Utf8ToUcs4(p, end, &mblen);
     if (first_script_type != Util::GetScriptType(next_ucs4) ||
         first_form_type != Util::GetFormType(next_ucs4)) {
       break;
@@ -906,7 +886,7 @@ class CachingConnector final {
 
   void ResetCacheIfNecessary(uint16_t rnode_lid) {
     if (cache_lid_ != rnode_lid) {
-      std::fill_n(cache_, kCacheSize, -1);
+      absl::c_fill(cache_, -1);
       cache_lid_ = rnode_lid;
     }
   }
@@ -929,7 +909,7 @@ class CachingConnector final {
   constexpr static int kCacheSize = 2048;
 
   const Connector &connector_;
-  int cache_[kCacheSize];
+  std::array<int, kCacheSize> cache_;
   uint16_t cache_lid_ = std::numeric_limits<uint16_t>::max();
 };
 
@@ -1013,7 +993,7 @@ bool ImmutableConverterImpl::Viterbi(const Segments &segments,
 
       rnode->prev = bos_node;
       rnode->cost = bos_node->cost +
-                    connector_->GetTransitionCost(bos_node->rid, rnode->lid) +
+                    connector_.GetTransitionCost(bos_node->rid, rnode->lid) +
                     rnode->wcost;
     }
   }
@@ -1023,12 +1003,12 @@ bool ImmutableConverterImpl::Viterbi(const Segments &segments,
 
   // Specialization for the first segment.
   // Don't run on the left boundary (the connection with BOS node),
-  // beacuse it is already run above.
+  // because it is already run above.
   {
     const size_t right_boundary =
         left_boundary + segments.segment(0).key().size();
     for (size_t pos = left_boundary + 1; pos < right_boundary; ++pos) {
-      ViterbiInternal(*connector_, pos, right_boundary, lattice);
+      ViterbiInternal(connector_, pos, right_boundary, lattice);
     }
     left_boundary = right_boundary;
   }
@@ -1039,7 +1019,7 @@ bool ImmutableConverterImpl::Viterbi(const Segments &segments,
     const size_t right_boundary =
         left_boundary + segments.segment(i).key().size();
     for (size_t pos = left_boundary; pos < right_boundary; ++pos) {
-      ViterbiInternal(*connector_, pos, right_boundary, lattice);
+      ViterbiInternal(connector_, pos, right_boundary, lattice);
     }
     left_boundary = right_boundary;
   }
@@ -1067,8 +1047,8 @@ bool ImmutableConverterImpl::Viterbi(const Segments &segments,
         continue;
       }
 
-      int cost = lnode->cost +
-                 connector_->GetTransitionCost(lnode->rid, eos_node->lid);
+      int cost =
+          lnode->cost + connector_.GetTransitionCost(lnode->rid, eos_node->lid);
       if (cost < best_cost) {
         best_cost = cost;
         best_node = lnode;
@@ -1217,7 +1197,7 @@ void ImmutableConverterImpl::PredictionViterbiInternal(int calc_begin_pos,
          ++liter) {
       for (BestMap::iterator riter = rbest.begin(); riter != rbest.end();
            ++riter) {
-        const int cost = liter->second.first + connector_->GetTransitionCost(
+        const int cost = liter->second.first + connector_.GetTransitionCost(
                                                    liter->first, riter->first);
         if (cost < riter->second.first) {
           riter->second.first = cost;
@@ -1468,8 +1448,11 @@ bool ImmutableConverterImpl::MakeLattice(const ConversionRequest &request,
     dictionary_->ClearReverseLookupCache();
   }
 
-  // Predictive real time conversion
-  if (is_prediction) {
+  // Nodes look up for real time conversion for desktop.
+  // Note:
+  // For mobile, we decided to stop adding predictive nodes based on
+  // experiments.
+  if (is_prediction && !IsMobileRequest(request)) {
     MakeLatticeNodesForPredictiveNodes(*segments, request, lattice);
   }
 
@@ -1561,10 +1544,16 @@ bool ImmutableConverterImpl::MakeLatticeNodesForHistorySegments(
     // Example: history "おいかわ(及川)", conversion: "たくや"
     // Here, try to find "おいかわたくや(及川卓也)" from dictionary
     // and insert "卓也" as a new word node with a modified cost
-    if (s + 1 == history_segments_size) {
-      const bool is_prediction =
-          (request.request_type() == ConversionRequest::SUGGESTION ||
-           request.request_type() == ConversionRequest::PREDICTION);
+    //
+    // Note: The overlapping lookup is disabled for prediction, because it can
+    // produce noisy realtime candidates such as "て配" for the history "に",
+    // which comes from "にて" + "配".
+    // The bigram-like lookup ("卓也" from "及川") is covered in
+    // dictionary_predictor.
+    const bool is_prediction =
+        (request.request_type() == ConversionRequest::SUGGESTION ||
+         request.request_type() == ConversionRequest::PREDICTION);
+    if (!is_prediction && s + 1 == history_segments_size) {
       const Node *node = Lookup(segments_pos, key.size(), request, is_reverse,
                                 is_prediction, lattice);
       for (const Node *compound_node = node; compound_node != nullptr;
@@ -1611,7 +1600,7 @@ bool ImmutableConverterImpl::MakeLatticeNodesForHistorySegments(
         new_node->wcost =
             compound_node->wcost * candidate.value.size() /
                 compound_node->value.size() -
-            connector_->GetTransitionCost(candidate.rid, new_node->lid);
+            connector_.GetTransitionCost(candidate.rid, new_node->lid);
 
         VLOG(2) << " compound_node->lid=" << compound_node->lid
                 << " compound_node->rid=" << compound_node->rid
@@ -1741,13 +1730,12 @@ void ImmutableConverterImpl::Resegment(const Segments &segments,
 // Single segment conversion results should be set to |segments|.
 void ImmutableConverterImpl::InsertFirstSegmentToCandidates(
     const ConversionRequest &request, Segments *segments,
-    const Lattice &lattice, const std::vector<uint16_t> &group,
-    size_t max_candidates_size, FilterType filter_type,
-    bool allow_exact) const {
+    const Lattice &lattice, absl::Span<const uint16_t> group,
+    size_t max_candidates_size, bool allow_exact) const {
   const size_t only_first_segment_candidate_pos =
       segments->conversion_segment(0).candidates_size();
   InsertCandidates(request, segments, lattice, group, max_candidates_size,
-                   ONLY_FIRST_SEGMENT, filter_type);
+                   ONLY_FIRST_SEGMENT);
   // Note that inserted candidates might consume the entire key.
   // e.g. key: "なのは", value: "ナノは"
   // Erase them later.
@@ -1810,10 +1798,11 @@ void ImmutableConverterImpl::InsertFirstSegmentToCandidates(
   }
 }
 
-bool ImmutableConverterImpl::IsSegmentEndNode(
-    const ConversionRequest &request, const Segments &segments,
-    const Node *node, const std::vector<uint16_t> &group,
-    bool is_single_segment) const {
+bool ImmutableConverterImpl::IsSegmentEndNode(const ConversionRequest &request,
+                                              const Segments &segments,
+                                              const Node *node,
+                                              absl::Span<const uint16_t> group,
+                                              bool is_single_segment) const {
   DCHECK(node->next);
   if (node->next->node_type == Node::EOS_NODE) {
     return true;
@@ -1861,11 +1850,10 @@ bool ImmutableConverterImpl::IsSegmentEndNode(
 }
 
 Segment *ImmutableConverterImpl::GetInsertTargetSegment(
-    const Lattice &lattice, const std::vector<uint16_t> &group,
+    const Lattice &lattice, absl::Span<const uint16_t> group,
     InsertCandidatesType type, size_t begin_pos, const Node *node,
     Segments *segments) const {
   if (type != MULTI_SEGMENTS) {
-    DCHECK(type == SINGLE_SEGMENT || type == ONLY_FIRST_SEGMENT);
     // Realtime conversion that produces only one segment.
     return segments->mutable_segment(segments->segments_size() - 1);
   }
@@ -1880,11 +1868,12 @@ Segment *ImmutableConverterImpl::GetInsertTargetSegment(
   return segment;
 }
 
-void ImmutableConverterImpl::InsertCandidates(
-    const ConversionRequest &request, Segments *segments,
-    const Lattice &lattice, const std::vector<uint16_t> &group,
-    size_t max_candidates_size, InsertCandidatesType type,
-    FilterType filter_type) const {
+void ImmutableConverterImpl::InsertCandidates(const ConversionRequest &request,
+                                              Segments *segments,
+                                              const Lattice &lattice,
+                                              absl::Span<const uint16_t> group,
+                                              size_t max_candidates_size,
+                                              InsertCandidatesType type) const {
   // skip HIS_NODE(s)
   Node *prev = lattice.bos_nodes();
   for (Node *node = lattice.bos_nodes()->next;
@@ -1896,10 +1885,11 @@ void ImmutableConverterImpl::InsertCandidates(
   const size_t expand_size =
       std::max<size_t>(1, std::min<size_t>(512, max_candidates_size));
 
-  const bool is_single_segment = (type == SINGLE_SEGMENT);
+  const bool is_single_segment =
+      (type == SINGLE_SEGMENT || type == FIRST_INNER_SEGMENT);
   NBestGenerator nbest_generator(suppression_dictionary_, segmenter_,
                                  connector_, pos_matcher_, &lattice,
-                                 suggestion_filter_, (filter_type == DESKTOP));
+                                 suggestion_filter_);
 
   std::string original_key;
   for (size_t i = 0; i < segments->conversion_segments_size(); ++i) {
@@ -1920,18 +1910,23 @@ void ImmutableConverterImpl::InsertCandidates(
         GetInsertTargetSegment(lattice, group, type, begin_pos, node, segments);
     CHECK(segment);
 
-    NBestGenerator::BoundaryCheckMode mode = NBestGenerator::STRICT;
-    if (type == SINGLE_SEGMENT) {
+    NBestGenerator::Options options;
+    if (type == SINGLE_SEGMENT || type == FIRST_INNER_SEGMENT) {
       // For realtime conversion.
-      mode = NBestGenerator::ONLY_EDGE;
+      options.boundary_mode = NBestGenerator::ONLY_EDGE;
+      options.candidate_mode |= NBestGenerator::FILL_INNER_SEGMENT_INFO;
     } else if (segment->segment_type() == Segment::FIXED_BOUNDARY) {
       // Boundary is specified. Skip boundary check in nbest generator.
-      mode = NBestGenerator::ONLY_MID;
+      options.boundary_mode = NBestGenerator::ONLY_MID;
     }
-    nbest_generator.Reset(prev, node->next, mode);
-
-    ExpandCandidates(request, original_key, &nbest_generator, segment,
-                     expand_size);
+    if (type == FIRST_INNER_SEGMENT) {
+      // Inserts only first segment from realtime conversion path.
+      options.candidate_mode |=
+          NBestGenerator::BUILD_FROM_ONLY_FIRST_INNER_SEGMENT;
+      options.candidate_mode |= NBestGenerator::FILL_INNER_SEGMENT_INFO;
+    }
+    nbest_generator.Reset(prev, node->next, options);
+    nbest_generator.SetCandidates(request, original_key, expand_size, segment);
 
     if (type == MULTI_SEGMENTS || type == SINGLE_SEGMENT) {
       InsertDummyCandidates(segment, expand_size);
@@ -1951,7 +1946,7 @@ void ImmutableConverterImpl::InsertCandidates(
 
 bool ImmutableConverterImpl::MakeSegments(const ConversionRequest &request,
                                           const Lattice &lattice,
-                                          const std::vector<uint16_t> &group,
+                                          absl::Span<const uint16_t> group,
                                           Segments *segments) const {
   if (segments == nullptr) {
     LOG(WARNING) << "Segments is nullptr";
@@ -1959,95 +1954,194 @@ bool ImmutableConverterImpl::MakeSegments(const ConversionRequest &request,
   }
 
   const ConversionRequest::RequestType type = request.request_type();
-  const bool is_prediction = (type == ConversionRequest::PREDICTION ||
-                              type == ConversionRequest::PARTIAL_PREDICTION);
-  const bool is_suggestion = (type == ConversionRequest::SUGGESTION ||
-                              type == ConversionRequest::PARTIAL_SUGGESTION);
-  const FilterType filter_type =
-      request.request().mixed_conversion() ? MOBILE : DESKTOP;
 
-  auto do_suggestion = [this, &request, &lattice, &group, &segments,
-                        &filter_type, &is_prediction]() {
-    const size_t max_candidates_size = request.max_conversion_candidates_size();
-
-    if (request.create_partial_candidates()) {
-      // TODO(toshiyuki): It may be better to change this value
-      // according to the key length.
-      static constexpr size_t kOnlyFirstSegmentCandidateSize = 3;
-      const size_t single_segment_candidates_size =
-          ((max_candidates_size > kOnlyFirstSegmentCandidateSize)
-               ? max_candidates_size - kOnlyFirstSegmentCandidateSize
-               : 1);
-      InsertCandidates(request, segments, lattice, group,
-                       single_segment_candidates_size, SINGLE_SEGMENT,
-                       filter_type);
-
-      // Even if single_segment_candidates_size + kOnlyFirstSegmentCandidateSize
-      // is greater than max_candidates_size, we cannot skip
-      // InsertFirstSegmentToCandidates().
-      // For example:
-      //   the sum: 11
-      //   max_candidates_size: 10
-      //   current candidate size: 8
-      // In this case, the sum > |max_candidates_size|, but we should not
-      // skip calling InsertFirstSegmentToCandidates, as we want to add
-      // two candidates.
-      const size_t only_first_segment_candidates_size =
-          std::min(max_candidates_size, single_segment_candidates_size +
-                                            kOnlyFirstSegmentCandidateSize);
-      InsertFirstSegmentToCandidates(request, segments, lattice, group,
-                                     only_first_segment_candidates_size,
-                                     filter_type, false /* allow_exact */);
-      // TODO(taku): We do not want to refer `is_prediciton` here.
-      // This is a temporal workaround to fill all personal names appeared
-      // as exact partial candidates. Expand candidates as many as possible
-      if (is_prediction) {
-        InsertFirstSegmentToCandidates(request, segments, lattice, group,
-                                       request.max_conversion_candidates_size(),
-                                       filter_type, true /* allow exact */);
-      }
-
-    } else {
-      InsertCandidates(request, segments, lattice, group, max_candidates_size,
-                       SINGLE_SEGMENT, filter_type);
-    }
-  };
-
-  auto do_conversion = [this, &request, &lattice, &group, &segments,
-                        &filter_type]() {
-    DCHECK(!request.create_partial_candidates());
-    // Currently, we assume that REVERSE_CONVERSION only
-    // requires 1 result.
-    // TODO(taku): support to set the size on REVESER_CONVERSION mode.
-    const size_t max_candidates_size =
-        ((request.request_type() == ConversionRequest::REVERSE_CONVERSION)
-             ? 1
-             : request.max_conversion_candidates_size());
-
-    // InsertCandidates inserts new segments after the existing
-    // conversion segments. So we have to erase old conversion segments.
-    // We have to keep old segments for calling InsertCandidates because
-    // we need segment constraints like FIXED_BOUNDARY.
-    // TODO(toshiyuki): We want more beautiful structure.
-    const size_t old_conversion_segments_size =
-        segments->conversion_segments_size();
-    InsertCandidates(request, segments, lattice, group, max_candidates_size,
-                     MULTI_SEGMENTS, filter_type);
-    if (old_conversion_segments_size > 0) {
-      segments->erase_segments(segments->history_segments_size(),
-                               old_conversion_segments_size);
-    }
-  };
-
-  if (is_suggestion || is_prediction) {
-    do_suggestion();
+  if (type == ConversionRequest::CONVERSION ||
+      type == ConversionRequest::REVERSE_CONVERSION) {
+    InsertCandidatesForConversion(request, lattice, group, segments);
   } else {
-    do_conversion();
+    InsertCandidatesForPrediction(request, lattice, group, segments);
   }
 
   return true;
 }
 
+void ImmutableConverterImpl::InsertCandidatesForConversion(
+    const ConversionRequest &request, const Lattice &lattice,
+    absl::Span<const uint16_t> group, Segments *segments) const {
+  DCHECK(!request.create_partial_candidates());
+  // Currently, we assume that REVERSE_CONVERSION only
+  // requires 1 result.
+  // TODO(taku): support to set the size on REVESER_CONVERSION mode.
+  const size_t max_candidates_size =
+      ((request.request_type() == ConversionRequest::REVERSE_CONVERSION)
+           ? 1
+           : request.max_conversion_candidates_size());
+
+  // InsertCandidates inserts new segments after the existing
+  // conversion segments. So we have to erase old conversion segments.
+  // We have to keep old segments for calling InsertCandidates because
+  // we need segment constraints like FIXED_BOUNDARY.
+  // TODO(toshiyuki): We want more beautiful structure.
+  const size_t old_conversion_segments_size =
+      segments->conversion_segments_size();
+  InsertCandidates(request, segments, lattice, group, max_candidates_size,
+                   MULTI_SEGMENTS);
+  if (old_conversion_segments_size > 0) {
+    segments->erase_segments(segments->history_segments_size(),
+                             old_conversion_segments_size);
+  }
+}
+
+void ImmutableConverterImpl::InsertCandidatesForRealtime(
+    const ConversionRequest &request, const Lattice &lattice,
+    absl::Span<const uint16_t> group, Segments *segments) const {
+  Segment *target_segment = segments->mutable_conversion_segment(0);
+
+  Segments tmp_segments = *segments;
+  {
+    // Candidates for the whole path
+    constexpr int kMaxSize = 3;
+    InsertCandidates(request, &tmp_segments, lattice, group, kMaxSize,
+                     SINGLE_SEGMENT);
+
+    auto get_boundary_info = [](const Segment::Candidate &c) {
+      std::vector<absl::string_view> ret;
+      for (Segment::Candidate::InnerSegmentIterator iter(&c); !iter.Done();
+           iter.Next()) {
+        ret.emplace_back(iter.GetKey());
+      }
+      return ret;
+    };
+
+    // InsertCandidates for SINGLE_SEGMENT should insert at least one candidate.
+    DCHECK_GT(tmp_segments.conversion_segment(0).candidates_size(), 0);
+    const auto &top_cand = tmp_segments.conversion_segment(0).candidate(0);
+    const std::vector<absl::string_view> top_boundary =
+        get_boundary_info(top_cand);
+    for (int i = 0; i < tmp_segments.conversion_segment(0).candidates_size();
+         ++i) {
+      const auto &c = tmp_segments.conversion_segment(0).candidate(i);
+      constexpr int kCostDiff = 2302;  // 500*log(100)
+      if (c.cost - top_cand.cost > kCostDiff) {
+        continue;
+      }
+      if (i != 0 && get_boundary_info(c) == top_boundary) {
+        // Skip to add the similar candidates.
+        continue;
+      }
+      Segment::Candidate *candidate = target_segment->add_candidate();
+      *candidate = c;
+    }
+  }
+  tmp_segments.mutable_conversion_segment(0)->clear_candidates();
+
+  {
+    // Candidates for the first segment for each n-best path.
+    InsertCandidates(request, &tmp_segments, lattice, group,
+                     request.max_conversion_candidates_size() -
+                         target_segment->candidates_size(),
+                     FIRST_INNER_SEGMENT);
+    Trie<bool> added;
+    constexpr bool kPlaceholderValue = true;
+    constexpr int kMaxSingleSegmentCandidateSize = 5;
+    int single_segment_candidate_count = 0;
+    for (int i = 0; i < tmp_segments.conversion_segment(0).candidates_size();
+         ++i) {
+      const auto &c = tmp_segments.conversion_segment(0).candidate(i);
+      bool data;
+      size_t key_length;
+      if (added.LongestMatch(c.value, &data, &key_length)) {
+        // Prefix is already added
+        continue;
+      }
+      if (c.key.size() == target_segment->key().size() &&
+          c.key.size() != c.content_key.size() &&
+          single_segment_candidate_count++ >= kMaxSingleSegmentCandidateSize) {
+        // The key is for single segment (ex. きょうの, etc)
+        // Skip to add verbose candidates (凶の, 鏡の, 興の, etc)
+        continue;
+      }
+      Segment::Candidate *candidate = target_segment->add_candidate();
+      *candidate = c;
+      if (c.key.size() != target_segment->key().size()) {
+        // Suffix penalty is not added for the prefix path.
+        const int32_t suffix_penalty =
+            segmenter_->GetSuffixPenalty(candidate->rid);
+        candidate->wcost += suffix_penalty;
+        candidate->cost += suffix_penalty;
+      }
+      added.AddEntry(c.value, kPlaceholderValue);
+    }
+  }
+}
+
+void ImmutableConverterImpl::InsertCandidatesForPrediction(
+    const ConversionRequest &request, const Lattice &lattice,
+    absl::Span<const uint16_t> group, Segments *segments) const {
+  const size_t max_candidates_size = request.max_conversion_candidates_size();
+
+  if (!request.create_partial_candidates()) {
+    // Desktop (or physical keyboard in Mobile)
+    InsertCandidates(request, segments, lattice, group, max_candidates_size,
+                     SINGLE_SEGMENT);
+    return;
+  }
+
+  // Mobile
+  if (request.request()
+          .decoder_experiment_params()
+          .enable_realtime_conversion_v2()) {
+    InsertCandidatesForRealtime(request, lattice, group, segments);
+    return;
+  }
+
+  // TODO(toshiyuki): It may be better to change this value
+  // according to the key length.
+  static constexpr size_t kOnlyFirstSegmentCandidateSize = 3;
+  const size_t single_segment_candidates_size =
+      ((max_candidates_size > kOnlyFirstSegmentCandidateSize)
+           ? max_candidates_size - kOnlyFirstSegmentCandidateSize
+           : 1);
+  InsertCandidates(request, segments, lattice, group,
+                   single_segment_candidates_size, SINGLE_SEGMENT);
+
+  // Even if single_segment_candidates_size + kOnlyFirstSegmentCandidateSize
+  // is greater than max_candidates_size, we cannot skip
+  // InsertFirstSegmentToCandidates().
+  // For example:
+  //   the sum: 11
+  //   max_candidates_size: 10
+  //   current candidate size: 8
+  // In this case, the sum > |max_candidates_size|, but we should not
+  // skip calling InsertFirstSegmentToCandidates, as we want to add
+  // two candidates.
+  const size_t only_first_segment_candidates_size =
+      std::min(max_candidates_size,
+               single_segment_candidates_size + kOnlyFirstSegmentCandidateSize);
+  InsertFirstSegmentToCandidates(request, segments, lattice, group,
+                                 only_first_segment_candidates_size,
+                                 false /* allow_exact */);
+
+  const bool is_prediction =
+      request.request_type() == ConversionRequest::PREDICTION ||
+      request.request_type() == ConversionRequest::PARTIAL_PREDICTION;
+  // TODO(taku): We do not want to refer `is_prediction` here.
+  // This is a temporal workaround to fill all personal names appeared
+  // as exact partial candidates. Expand candidates as many as possible
+  //
+  // Note: This check is for just in case.
+  // For now,
+  // 1. `create_partial_candidates` can be true only for Mobile
+  //    (Request::mixed_conversion == true)
+  // 2. For mobile, the session does not issue SUGGESTION command
+  // So we do not check `is_prediction` here.
+  if (is_prediction) {
+    InsertFirstSegmentToCandidates(request, segments, lattice, group,
+                                   request.max_conversion_candidates_size(),
+                                   true /* allow exact */);
+  }
+}
+
 void ImmutableConverterImpl::MakeGroup(const Segments &segments,
                                        std::vector<uint16_t> *group) const {
   group->clear();
