@@ -30,45 +30,46 @@
 #ifndef MOZC_CONVERTER_IMMUTABLE_CONVERTER_H_
 #define MOZC_CONVERTER_IMMUTABLE_CONVERTER_H_
 
+#include <cstddef>
 #include <cstdint>
 #include <string>
 #include <vector>
 
-#include "base/port.h"
 #include "converter/connector.h"
 #include "converter/immutable_converter_interface.h"
+#include "converter/lattice.h"
+#include "converter/nbest_generator.h"
 #include "converter/node.h"
+#include "converter/segmenter.h"
 #include "converter/segments.h"
 #include "dictionary/dictionary_interface.h"
 #include "dictionary/pos_group.h"
 #include "dictionary/pos_matcher.h"
 #include "dictionary/suppression_dictionary.h"
-//  for FRIEND_TEST()
-#include "testing/base/public/gunit_prod.h"
+#include "prediction/suggestion_filter.h"
+#include "request/conversion_request.h"
+#include "testing/gunit_prod.h"  //  for FRIEND_TEST()
+#include "absl/base/attributes.h"
+#include "absl/types/span.h"
 
 namespace mozc {
 
-struct Node;
-class ImmutableConverterInterface;
-class Lattice;
-class NBestGenerator;
-class Segmenter;
-class SuggestionFilter;
-
 class ImmutableConverterImpl : public ImmutableConverterInterface {
  public:
   ImmutableConverterImpl(
       const dictionary::DictionaryInterface *dictionary,
       const dictionary::DictionaryInterface *suffix_dictionary,
       const dictionary::SuppressionDictionary *suppression_dictionary,
-      const Connector *connector, const Segmenter *segmenter,
+      const Connector &connector, const Segmenter *segmenter,
       const dictionary::PosMatcher *pos_matcher,
       const dictionary::PosGroup *pos_group,
-      const SuggestionFilter *suggestion_filter);
+      const SuggestionFilter &suggestion_filter);
+  ImmutableConverterImpl(const ImmutableConverterImpl &) = delete;
+  ImmutableConverterImpl &operator=(const ImmutableConverterImpl &) = delete;
   ~ImmutableConverterImpl() override = default;
 
-  bool ConvertForRequest(const ConversionRequest &request,
-                         Segments *segments) const override;
+  ABSL_MUST_USE_RESULT bool ConvertForRequest(
+      const ConversionRequest &request, Segments *segments) const override;
 
  private:
   FRIEND_TEST(ImmutableConverterTest, AddPredictiveNodes);
@@ -85,24 +86,19 @@ class ImmutableConverterImpl : public ImmutableConverterInterface {
     MULTI_SEGMENTS,      // Normal conversion ("私の|名前は|中野です")
     SINGLE_SEGMENT,      // Realtime conversion ("私の名前は中野です")
     ONLY_FIRST_SEGMENT,  // Insert only first segment ("私の")
-  };
-
-  enum FilterType {
-    // Suppress exact match for prediction.
-    // Users will use conversion for candidates for exact match.
-    DESKTOP,
-    // Do not suppress exact match for prediction.
-    // Users will mainly use suggestion/prediction for all mach cases.
-    MOBILE,
+    // Insert only first segment for n-best path
+    // ここでは着物を脱ぐ
+    // ここで履物を脱ぐ
+    // → ここでは, ここで
+    FIRST_INNER_SEGMENT,
   };
 
   void ExpandCandidates(const ConversionRequest &request,
                         const std::string &original_key, NBestGenerator *nbest,
                         Segment *segment, size_t expand_size) const;
   void InsertDummyCandidates(Segment *segment, size_t expand_size) const;
-  Node *Lookup(const int begin_pos, const int end_pos,
-               const ConversionRequest &request, bool is_reverse,
-               bool is_prediction, Lattice *lattice) const;
+  Node *Lookup(int begin_pos, int end_pos, const ConversionRequest &request,
+               bool is_reverse, bool is_prediction, Lattice *lattice) const;
   Node *AddCharacterTypeBasedNodes(const char *begin, const char *end,
                                    Lattice *lattice, Node *nodes) const;
 
@@ -147,35 +143,39 @@ class ImmutableConverterImpl : public ImmutableConverterInterface {
   void InsertFirstSegmentToCandidates(const ConversionRequest &request,
                                       Segments *segments,
                                       const Lattice &lattice,
-                                      const std::vector<uint16_t> &group,
+                                      absl::Span<const uint16_t> group,
                                       size_t max_candidates_size,
-                                      FilterType filter_type,
                                       bool allow_exact) const;
 
   void InsertCandidates(const ConversionRequest &request, Segments *segments,
                         const Lattice &lattice,
-                        const std::vector<uint16_t> &group,
-                        size_t max_candidates_size, InsertCandidatesType type,
-                        FilterType filter_type) const;
+                        absl::Span<const uint16_t> group,
+                        size_t max_candidates_size,
+                        InsertCandidatesType type) const;
+
+  void InsertCandidatesForRealtime(const ConversionRequest &request,
+                                   const Lattice &lattice,
+                                   absl::Span<const uint16_t> group,
+                                   Segments *segments) const;
 
   // Helper function for InsertCandidates().
   // Returns true if |node| is valid node for segment end.
   bool IsSegmentEndNode(const ConversionRequest &request,
                         const Segments &segments, const Node *node,
-                        const std::vector<uint16_t> &group,
+                        absl::Span<const uint16_t> group,
                         bool is_single_segment) const;
 
   // Helper function for InsertCandidates().
   // Returns the segment for inserting candidates.
   Segment *GetInsertTargetSegment(const Lattice &lattice,
-                                  const std::vector<uint16_t> &group,
+                                  absl::Span<const uint16_t> group,
                                   InsertCandidatesType type, size_t begin_pos,
                                   const Node *node, Segments *segments) const;
 
   bool MakeSegments(const ConversionRequest &request, const Lattice &lattice,
-                    const std::vector<uint16_t> &group,
-                    Segments *segments) const;
+                    absl::Span<const uint16_t> group, Segments *segments) const;
 
+  // |group| will be used to specify the segmentation.
   void MakeGroup(const Segments &segments, std::vector<uint16_t> *group) const;
 
   inline int GetCost(const Node *lnode, const Node *rnode) const {
@@ -184,17 +184,27 @@ class ImmutableConverterImpl : public ImmutableConverterInterface {
         lnode != rnode->constrained_prev) {
       return kInvalidPenaltyCost;
     }
-    return connector_->GetTransitionCost(lnode->rid, rnode->lid) + rnode->wcost;
+    return connector_.GetTransitionCost(lnode->rid, rnode->lid) + rnode->wcost;
   }
 
+  void InsertCandidatesForConversion(const ConversionRequest &request,
+                                     const Lattice &lattice,
+                                     absl::Span<const uint16_t> group,
+                                     Segments *segments) const;
+
+  void InsertCandidatesForPrediction(const ConversionRequest &request,
+                                     const Lattice &lattice,
+                                     absl::Span<const uint16_t> group,
+                                     Segments *segments) const;
+
   const dictionary::DictionaryInterface *dictionary_;
   const dictionary::DictionaryInterface *suffix_dictionary_;
   const dictionary::SuppressionDictionary *suppression_dictionary_;
-  const Connector *connector_;
+  const Connector &connector_;
   const Segmenter *segmenter_;
   const dictionary::PosMatcher *pos_matcher_;
   const dictionary::PosGroup *pos_group_;
-  const SuggestionFilter *suggestion_filter_;
+  const SuggestionFilter &suggestion_filter_;
 
   // Cache for POS ids.
   const uint16_t first_name_id_;
@@ -204,8 +214,6 @@ class ImmutableConverterImpl : public ImmutableConverterInterface {
 
   // Cache for transition cost.
   const int32_t last_to_first_name_transition_cost_;
-
-  DISALLOW_COPY_AND_ASSIGN(ImmutableConverterImpl);
 };
 
 }  // namespace mozc
