@@ -32,13 +32,14 @@
 #include <cstdint>
 #include <memory>
 #include <string>
+#include <utility>
+#include <vector>
 
 #include "base/logging.h"
-#include "base/port.h"
-#include "base/system_util.h"
-#include "config/config_handler.h"
 #include "converter/connector.h"
 #include "converter/immutable_converter.h"
+#include "converter/lattice.h"
+#include "converter/node.h"
 #include "converter/segmenter.h"
 #include "converter/segments.h"
 #include "data_manager/data_manager_interface.h"
@@ -46,6 +47,7 @@
 #include "dictionary/dictionary_impl.h"
 #include "dictionary/dictionary_interface.h"
 #include "dictionary/pos_group.h"
+#include "dictionary/pos_matcher.h"
 #include "dictionary/suffix_dictionary.h"
 #include "dictionary/suppression_dictionary.h"
 #include "dictionary/system/system_dictionary.h"
@@ -53,8 +55,7 @@
 #include "dictionary/user_dictionary_stub.h"
 #include "prediction/suggestion_filter.h"
 #include "request/conversion_request.h"
-#include "testing/base/public/googletest.h"
-#include "testing/base/public/gunit.h"
+#include "testing/gunit.h"
 #include "absl/strings/string_view.h"
 
 namespace mozc {
@@ -104,23 +105,19 @@ class MockDataAndImmutableConverter {
 
     connector_ = Connector::CreateFromDataManager(*data_manager_).value();
 
-    segmenter_.reset(Segmenter::CreateFromDataManager(*data_manager_));
+    segmenter_ = Segmenter::CreateFromDataManager(*data_manager_);
     CHECK(segmenter_);
 
     pos_group_ = std::make_unique<PosGroup>(data_manager_->GetPosGroupData());
     CHECK(pos_group_);
 
-    {
-      const char *data = nullptr;
-      size_t size = 0;
-      data_manager_->GetSuggestionFilterData(&data, &size);
-      suggestion_filter_ = std::make_unique<SuggestionFilter>(data, size);
-    }
+    suggestion_filter_ =
+        SuggestionFilter::CreateOrDie(data_manager_->GetSuggestionFilterData());
 
     immutable_converter_ = std::make_unique<ImmutableConverterImpl>(
         dictionary_.get(), suffix_dictionary_.get(),
-        suppression_dictionary_.get(), connector_.get(), segmenter_.get(),
-        &pos_matcher_, pos_group_.get(), suggestion_filter_.get());
+        suppression_dictionary_.get(), connector_, segmenter_.get(),
+        &pos_matcher_, pos_group_.get(), suggestion_filter_);
     CHECK(immutable_converter_);
   }
 
@@ -128,19 +125,19 @@ class MockDataAndImmutableConverter {
 
   std::unique_ptr<NBestGenerator> CreateNBestGenerator(const Lattice *lattice) {
     return std::make_unique<NBestGenerator>(
-        suppression_dictionary_.get(), segmenter_.get(), connector_.get(),
-        &pos_matcher_, lattice, suggestion_filter_.get(), true);
+        suppression_dictionary_.get(), segmenter_.get(), connector_,
+        &pos_matcher_, lattice, suggestion_filter_);
   }
 
  private:
   std::unique_ptr<const DataManagerInterface> data_manager_;
   std::unique_ptr<const SuppressionDictionary> suppression_dictionary_;
-  std::unique_ptr<const Connector> connector_;
+  Connector connector_;
   std::unique_ptr<const Segmenter> segmenter_;
   std::unique_ptr<const DictionaryInterface> suffix_dictionary_;
   std::unique_ptr<const DictionaryInterface> dictionary_;
   std::unique_ptr<const PosGroup> pos_group_;
-  std::unique_ptr<const SuggestionFilter> suggestion_filter_;
+  SuggestionFilter suggestion_filter_;
   std::unique_ptr<ImmutableConverterImpl> immutable_converter_;
   UserDictionaryStub user_dictionary_stub_;
   dictionary::PosMatcher pos_matcher_;
@@ -150,19 +147,6 @@ class MockDataAndImmutableConverter {
 
 class NBestGeneratorTest : public ::testing::Test {
  protected:
-  void GatherCandidates(size_t size, const ConversionRequest &request,
-                        NBestGenerator *nbest, Segment *segment) const {
-    while (segment->candidates_size() < size) {
-      Segment::Candidate *candidate = segment->push_back_candidate();
-      candidate->Init();
-
-      if (!nbest->Next(request, segment->key(), candidate)) {
-        segment->pop_back_candidate();
-        break;
-      }
-    }
-  }
-
   const Node *GetEndNode(const ConversionRequest &request,
                          const ImmutableConverterImpl &converter,
                          const Segments &segments, const Node &begin_node,
@@ -214,26 +198,30 @@ TEST_F(NBestGeneratorTest, MultiSegmentConnectionTest) {
   const Node *end_node = GetEndNode(request, *converter, segments, *begin_node,
                                     group, kSingleSegment);
   {
-    nbest_generator->Reset(begin_node, end_node, NBestGenerator::STRICT);
+    nbest_generator->Reset(
+        begin_node, end_node,
+        {NBestGenerator::STRICT, NBestGenerator::CANDIDATE_MODE_NONE});
     Segment result_segment;
-    GatherCandidates(10, request, nbest_generator.get(), &result_segment);
+    nbest_generator->SetCandidates(request, "", 10, &result_segment);
     // The top result is treated exceptionally and has no boundary check
     // in NBestGenerator.
     // The best route is calculated in ImmutalbeConverter with boundary check.
     // So, the top result should be inserted, but other candidates will be cut
     // due to boundary check between "する".
-    ASSERT_EQ(1, result_segment.candidates_size());
-    EXPECT_EQ("進行", result_segment.candidate(0).value);
+    ASSERT_EQ(result_segment.candidates_size(), 1);
+    EXPECT_EQ(result_segment.candidate(0).value, "進行");
   }
 
   {
-    nbest_generator->Reset(begin_node, end_node, NBestGenerator::ONLY_MID);
+    nbest_generator->Reset(
+        begin_node, end_node,
+        {NBestGenerator::ONLY_MID, NBestGenerator::CANDIDATE_MODE_NONE});
     Segment result_segment;
-    GatherCandidates(10, request, nbest_generator.get(), &result_segment);
-    ASSERT_EQ(3, result_segment.candidates_size());
-    EXPECT_EQ("進行", result_segment.candidate(0).value);
-    EXPECT_EQ("信仰", result_segment.candidate(1).value);
-    EXPECT_EQ("深耕", result_segment.candidate(2).value);
+    nbest_generator->SetCandidates(request, "", 10, &result_segment);
+    ASSERT_EQ(result_segment.candidates_size(), 3);
+    EXPECT_EQ(result_segment.candidate(0).value, "進行");
+    EXPECT_EQ(result_segment.candidate(1).value, "信仰");
+    EXPECT_EQ(result_segment.candidate(2).value, "深耕");
   }
 }
 
@@ -267,21 +255,25 @@ TEST_F(NBestGeneratorTest, SingleSegmentConnectionTest) {
   const Node *end_node = GetEndNode(request, *converter, segments, *begin_node,
                                     group, kSingleSegment);
   {
-    nbest_generator->Reset(begin_node, end_node, NBestGenerator::STRICT);
+    nbest_generator->Reset(
+        begin_node, end_node,
+        {NBestGenerator::STRICT, NBestGenerator::CANDIDATE_MODE_NONE});
     Segment result_segment;
-    GatherCandidates(10, request, nbest_generator.get(), &result_segment);
+    nbest_generator->SetCandidates(request, "", 10, &result_segment);
     // Top result should be inserted, but other candidates will be cut
     // due to boundary check.
-    ASSERT_EQ(1, result_segment.candidates_size());
-    EXPECT_EQ("私の名前は中ノです", result_segment.candidate(0).value);
+    ASSERT_EQ(result_segment.candidates_size(), 1);
+    EXPECT_EQ(result_segment.candidate(0).value, "私の名前は中ノです");
   }
   {
-    nbest_generator->Reset(begin_node, end_node, NBestGenerator::ONLY_EDGE);
+    nbest_generator->Reset(
+        begin_node, end_node,
+        {NBestGenerator::ONLY_EDGE, NBestGenerator::FILL_INNER_SEGMENT_INFO});
     Segment result_segment;
-    GatherCandidates(10, request, nbest_generator.get(), &result_segment);
+    nbest_generator->SetCandidates(request, "", 10, &result_segment);
     // We can get several candidates.
     ASSERT_LT(1, result_segment.candidates_size());
-    EXPECT_EQ("私の名前は中ノです", result_segment.candidate(0).value);
+    EXPECT_EQ(result_segment.candidate(0).value, "私の名前は中ノです");
   }
 }
 
@@ -315,14 +307,16 @@ TEST_F(NBestGeneratorTest, InnerSegmentBoundary) {
   const Node *end_node = GetEndNode(request, *converter, segments, *begin_node,
                                     group, kSingleSegment);
 
-  nbest_generator->Reset(begin_node, end_node, NBestGenerator::ONLY_EDGE);
+  nbest_generator->Reset(
+      begin_node, end_node,
+      {NBestGenerator::ONLY_EDGE, NBestGenerator::FILL_INNER_SEGMENT_INFO});
   Segment result_segment;
-  GatherCandidates(10, request, nbest_generator.get(), &result_segment);
+  nbest_generator->SetCandidates(request, "", 10, &result_segment);
   ASSERT_LE(1, result_segment.candidates_size());
 
   const Segment::Candidate &top_cand = result_segment.candidate(0);
-  EXPECT_EQ(kInput, top_cand.key);
-  EXPECT_EQ("東京か名古屋に行きたい", top_cand.value);
+  EXPECT_EQ(top_cand.key, kInput);
+  EXPECT_EQ(top_cand.value, "東京か名古屋に行きたい");
 
   std::vector<absl::string_view> keys, values, content_keys, content_values;
   for (Segment::Candidate::InnerSegmentIterator iter(&top_cand); !iter.Done();
@@ -332,31 +326,31 @@ TEST_F(NBestGeneratorTest, InnerSegmentBoundary) {
     content_keys.push_back(iter.GetContentKey());
     content_values.push_back(iter.GetContentValue());
   }
-  ASSERT_EQ(3, keys.size());
-  ASSERT_EQ(3, values.size());
-  ASSERT_EQ(3, content_keys.size());
-  ASSERT_EQ(3, content_values.size());
+  ASSERT_EQ(keys.size(), 3);
+  ASSERT_EQ(values.size(), 3);
+  ASSERT_EQ(content_keys.size(), 3);
+  ASSERT_EQ(content_values.size(), 3);
 
   // Inner segment 0
-  EXPECT_EQ("とうきょうか", keys[0]);
-  EXPECT_EQ("東京か", values[0]);
-  EXPECT_EQ("とうきょう", content_keys[0]);
-  EXPECT_EQ("東京", content_values[0]);
+  EXPECT_EQ(keys[0], "とうきょうか");
+  EXPECT_EQ(values[0], "東京か");
+  EXPECT_EQ(content_keys[0], "とうきょう");
+  EXPECT_EQ(content_values[0], "東京");
 
   // Inner segment 1
-  EXPECT_EQ("なごやに", keys[1]);
-  EXPECT_EQ("名古屋に", values[1]);
-  EXPECT_EQ("なごや", content_keys[1]);
-  EXPECT_EQ("名古屋", content_values[1]);
+  EXPECT_EQ(keys[1], "なごやに");
+  EXPECT_EQ(values[1], "名古屋に");
+  EXPECT_EQ(content_keys[1], "なごや");
+  EXPECT_EQ(content_values[1], "名古屋");
 
   // Inner segment 2: In the original segment, "行きたい" has the form
   // "行き" (content word) + "たい" (functional).  However, since "行き" is
   // Yougen, our rule for inner segment boundary doesn't handle it as a content
   // value.  Thus, "行きたい" becomes the content value.
-  EXPECT_EQ("いきたい", keys[2]);
-  EXPECT_EQ("行きたい", values[2]);
-  EXPECT_EQ("いきたい", content_keys[2]);
-  EXPECT_EQ("行きたい", content_values[2]);
+  EXPECT_EQ(keys[2], "いきたい");
+  EXPECT_EQ(values[2], "行きたい");
+  EXPECT_EQ(content_keys[2], "いきたい");
+  EXPECT_EQ(content_values[2], "行きたい");
 }
 
 }  // namespace mozc
