@@ -30,18 +30,33 @@
 #ifndef MOZC_CONVERTER_SEGMENTS_H_
 #define MOZC_CONVERTER_SEGMENTS_H_
 
+#include <cstddef>
 #include <cstdint>
 #include <deque>
 #include <memory>
+#include <ostream>
 #include <string>
+#include <utility>
 #include <vector>
 
-#include "base/freelist.h"
+#include "base/container/freelist.h"
+#include "base/logging.h"
 #include "base/number_util.h"
-#include "base/port.h"
+#include "base/strings/assign.h"
 #include "converter/lattice.h"
 #include "absl/strings/string_view.h"
 
+#ifndef NDEBUG
+#define MOZC_CANDIDATE_DEBUG
+#define MOZC_CANDIDATE_LOG(result, message) \
+  (result)->Dlog(__FILE__, __LINE__, message)
+
+#else  // NDEBUG
+#define MOZC_CANDIDATE_LOG(result, message) \
+  {}
+
+#endif  // NDEBUG
+
 namespace mozc {
 
 class Segment final {
@@ -56,6 +71,7 @@ class Segment final {
   };
 
   struct Candidate {
+    // LINT.IfChange
     enum Attribute {
       DEFAULT_ATTRIBUTE = 0,
       // this was the best candidate before learning
@@ -107,7 +123,10 @@ class Segment final {
       USER_HISTORY_PREDICTION = 1 << 14,
       // Contains suffix dictionary.
       SUFFIX_DICTIONARY = 1 << 15,
+      // Disables modification and removal in rewriters.
+      NO_MODIFICATION = 1 << 16,
     };
+    // LINT.ThenChange(//converter/converter_main.cc)
 
     enum Command {
       DEFAULT_COMMAND = 0,
@@ -137,6 +156,13 @@ class Segment final {
       USER_HISTORY_PREDICTOR = 1 << 6,
     };
 
+    enum Category {
+      DEFAULT_CATEGORY,  // Realtime conversion, history prediction, etc
+      SYMBOL,            // Symbol, emoji
+      OTHER,             // Misc candidate
+    };
+
+    // LINT.IfChange
     std::string key;    // reading
     std::string value;  // surface form
     std::string content_key;
@@ -149,6 +175,8 @@ class Segment final {
     std::string suffix;
     // Description including description type and message
     std::string description;
+    // Description for A11y support (e.g. "あ。ヒラガナ あ")
+    std::string a11y_description;
 
     // Usage ID
     int32_t usage_id = 0;
@@ -180,6 +208,8 @@ class Segment final {
     // Candidate's source info which will be used for usage stats.
     uint32_t source_info = SOURCE_INFO_NONE;
 
+    Category category = DEFAULT_CATEGORY;
+
     // Candidate style. This is not a bit-field.
     // The style is defined in enum |Style|.
     NumberUtil::NumberString::Style style =
@@ -189,14 +219,19 @@ class Segment final {
     // The style is defined in enum |Command|.
     Command command = DEFAULT_COMMAND;
 
-    // Boundary information for realtime conversion.  This will be set only for
-    // realtime conversion result candidates.  Each element is the encoded
+    // Boundary information for real time conversion.  This will be set only for
+    // real time conversion result candidates.  Each element is the encoded
     // lengths of key, value, content key and content value.
     std::vector<uint32_t> inner_segment_boundary;
+    // LINT.ThenChange(//converter/segments_matchers.h)
 
-#ifndef NDEBUG
-    std::string log;
-#endif  // NDEBUG
+    // The original cost before rescoring. Used for debugging purpose.
+    int32_t cost_before_rescoring = 0;
+#ifdef MOZC_CANDIDATE_DEBUG
+    void Dlog(absl::string_view filename, int line,
+              absl::string_view message) const;
+    mutable std::string log;
+#endif  // MOZC_CANDIDATE_DEBUG
 
     static bool EncodeLengths(size_t key_len, size_t value_len,
                               size_t content_key_len, size_t content_value_len,
@@ -240,6 +275,8 @@ class Segment final {
       absl::string_view GetValue() const;
       absl::string_view GetContentKey() const;
       absl::string_view GetContentValue() const;
+      absl::string_view GetFunctionalKey() const;
+      absl::string_view GetFunctionalValue() const;
 
      private:
       const Candidate *candidate_;
@@ -248,7 +285,10 @@ class Segment final {
       size_t index_;
     };
 
-    void Init();
+    // Clears the Candidate with default values. Note that the default
+    // constructor already does the same so you don't need to call Clear
+    // explicitly.
+    void Clear();
 
     // Returns functional key.
     // functional_key =
@@ -260,27 +300,46 @@ class Segment final {
     // value.substr(content_value.size(), value.size() - content_value.size());
     absl::string_view functional_value() const;
 
+    // Returns whether the inner_segment_boundary member is consistent with
+    // key and value.
+    // Note: content_key and content_value are not checked here.
+    // We cannot compose candidate's content_key and content_value directly
+    // from the inner segments in the current implementation.
+    // Example:
+    // value: 車のほうがあとだ
+    // content_value: 車のほうがあとだ
+    // inner_segments:
+    // <くるまのほうが, 車のほうが, くるま, 車>
+    // <あとだ, あとだ, あとだ, あとだ>
     bool IsValid() const;
     std::string DebugString() const;
+
+    friend std::ostream &operator<<(std::ostream &os,
+                                    const Candidate &candidate) {
+      return os << candidate.DebugString();
+    }
   };
 
-  Segment();
+  Segment() : segment_type_(FREE), pool_(kCandidatesPoolSize) {}
 
   Segment(const Segment &x);
   Segment &operator=(const Segment &x);
 
-  ~Segment();
+  SegmentType segment_type() const { return segment_type_; }
+  void set_segment_type(const SegmentType segment_type) {
+    segment_type_ = segment_type;
+  }
 
-  SegmentType segment_type() const;
-  void set_segment_type(const SegmentType &segment_type);
-
-  const std::string &key() const;
-  void set_key(absl::string_view key);
+  const std::string &key() const { return key_; }
+  template <typename T>
+  void set_key(T &&key) {
+    strings::Assign(key_, std::forward<T>(key));
+  }
 
   // check if the specified index is valid or not.
   bool is_valid_index(int i) const;
 
-  // Candidate manupluations
+  // Candidate manipulations
   // getter
   const Candidate &candidate(int i) const;
 
@@ -290,14 +349,15 @@ class Segment final {
   // push and insert candidates
   Candidate *push_front_candidate();
   Candidate *push_back_candidate();
-  Candidate *add_candidate();  // alias of push_back_candidate()
+  // alias of push_back_candidate()
+  Candidate *add_candidate() { return push_back_candidate(); }
   Candidate *insert_candidate(int i);
   void insert_candidate(int i, std::unique_ptr<Candidate> candidate);
   void insert_candidates(int i,
-                         std::vector<std::unique_ptr<Candidate>> &&candidates);
+                         std::vector<std::unique_ptr<Candidate>> candidates);
 
   // get size of candidates
-  size_t candidates_size() const;
+  size_t candidates_size() const { return candidates_.size(); }
 
   // erase candidate
   void pop_front_candidate();
@@ -311,10 +371,14 @@ class Segment final {
 
   // meta candidates
   // TODO(toshiyuki): Integrate meta candidates to candidate and delete these
-  size_t meta_candidates_size() const;
-  void clear_meta_candidates();
-  const std::vector<Candidate> &meta_candidates() const;
-  std::vector<Candidate> *mutable_meta_candidates();
+  size_t meta_candidates_size() const { return meta_candidates_.size(); }
+  void clear_meta_candidates() { meta_candidates_.clear(); }
+  const std::vector<Candidate> &meta_candidates() const {
+    return meta_candidates_;
+  }
+  std::vector<Candidate> *mutable_meta_candidates() {
+    return &meta_candidates_;
+  }
   const Candidate &meta_candidate(size_t i) const;
   Candidate *mutable_meta_candidate(size_t i);
   Candidate *add_meta_candidate();
@@ -329,10 +393,21 @@ class Segment final {
 
   std::string DebugString() const;
 
+  friend std::ostream &operator<<(std::ostream &os, const Segment &segment) {
+    return os << segment.DebugString();
+  }
+
+  const std::deque<Candidate *> &candidates() const { return candidates_; }
+
   // For debug. Candidate words removed through conversion process.
   std::vector<Candidate> removed_candidates_for_debug_;
 
  private:
+  void DeepCopyCandidates(const std::deque<Candidate *> &candidates);
+
+  static constexpr int kCandidatesPoolSize = 16;
+
+  // LINT.IfChange
   SegmentType segment_type_;
   // Note that |key_| is shorter than usual when partial suggestion is
   // performed.
@@ -345,6 +420,7 @@ class Segment final {
   std::deque<Candidate *> candidates_;
   std::vector<Candidate> meta_candidates_;
   std::vector<std::unique_ptr<Candidate>> pool_;
+  // LINT.ThenChange(//converter/segments_matchers.h)
 };
 
 // Segments is basically an array of Segment.
@@ -384,33 +460,42 @@ class Segments final {
     std::string key;
   };
 
-  Segments();
+  Segments()
+      : max_history_segments_size_(0),
+        resized_(false),
+        pool_(32),
+        cached_lattice_() {}
 
   Segments(const Segments &x);
   Segments &operator=(const Segments &x);
 
-  ~Segments();
-
   // getter
-  const Segment &segment(size_t i) const;
-  const Segment &conversion_segment(size_t i) const;
-  const Segment &history_segment(size_t i) const;
+  const Segment &segment(size_t i) const { return *segments_[i]; }
+  const Segment &conversion_segment(size_t i) const {
+    return *segments_[i + history_segments_size()];
+  }
+  const Segment &history_segment(size_t i) const { return *segments_[i]; }
 
   // setter
-  Segment *mutable_segment(size_t i);
-  Segment *mutable_conversion_segment(size_t i);
-  Segment *mutable_history_segment(size_t i);
+  Segment *mutable_segment(size_t i) { return segments_[i]; }
+  Segment *mutable_conversion_segment(size_t i) {
+    return segments_[i + history_segments_size()];
+  }
+  Segment *mutable_history_segment(size_t i) { return segments_[i]; }
 
   // push and insert segments
   Segment *push_front_segment();
   Segment *push_back_segment();
-  Segment *add_segment();  // alias of push_back_segment()
+  // alias of push_back_segment()
+  Segment *add_segment() { return push_back_segment(); }
   Segment *insert_segment(size_t i);
 
   // get size of segments
-  size_t segments_size() const;
+  size_t segments_size() const { return segments_.size(); }
   size_t history_segments_size() const;
-  size_t conversion_segments_size() const;
+  size_t conversion_segments_size() const {
+    return (segments_size() - history_segments_size());
+  }
 
   // erase segment
   void pop_front_segment();
@@ -424,10 +509,20 @@ class Segments final {
   void clear_segments();
 
   void set_max_history_segments_size(size_t max_history_segments_size);
-  size_t max_history_segments_size() const;
+  size_t max_history_segments_size() const {
+    return max_history_segments_size_;
+  }
 
-  bool resized() const;
-  void set_resized(bool resized);
+  bool resized() const { return resized_; }
+  void set_resized(bool resized) { resized_ = resized; }
+
+  // Returns history key of `size` segments.
+  // Returns all history key when size == -1.
+  std::string history_key(int size = -1) const;
+
+  // Returns history value of `size` segments.
+  // Returns all history value when size == -1.
+  std::string history_value(int size = -1) const;
 
   // clear segments
   void Clear();
@@ -435,26 +530,73 @@ class Segments final {
   // Dump Segments structure
   std::string DebugString() const;
 
+  friend std::ostream &operator<<(std::ostream &os, const Segments &segments) {
+    return os << segments.DebugString();
+  }
+
   // Revert entries
-  void clear_revert_entries();
-  size_t revert_entries_size() const;
+  void clear_revert_entries() { revert_entries_.clear(); }
+  size_t revert_entries_size() const { return revert_entries_.size(); }
   RevertEntry *push_back_revert_entry();
-  const RevertEntry &revert_entry(size_t i) const;
-  RevertEntry *mutable_revert_entry(size_t i);
+  const RevertEntry &revert_entry(size_t i) const { return revert_entries_[i]; }
+  RevertEntry *mutable_revert_entry(size_t i) { return &revert_entries_[i]; }
 
   // setter
-  Lattice *mutable_cached_lattice();
+  Lattice *mutable_cached_lattice() { return &cached_lattice_; }
 
  private:
+  // LINT.IfChange
   size_t max_history_segments_size_;
   bool resized_;
 
   ObjectPool<Segment> pool_;
   std::deque<Segment *> segments_;
   std::vector<RevertEntry> revert_entries_;
-  std::unique_ptr<Lattice> cached_lattice_;
+  Lattice cached_lattice_;
+  // LINT.ThenChange(//converter/segments_matchers.h)
 };
 
+// Inlining basic accessors here.
+inline absl::string_view Segment::Candidate::functional_key() const {
+  return key.size() <= content_key.size()
+             ? absl::string_view()
+             : absl::string_view(key.data() + content_key.size(),
+                                 key.size() - content_key.size());
+}
+
+inline absl::string_view Segment::Candidate::functional_value() const {
+  return value.size() <= content_value.size()
+             ? absl::string_view()
+             : absl::string_view(value.data() + content_value.size(),
+                                 value.size() - content_value.size());
+}
+
+inline bool Segment::is_valid_index(int i) const {
+  if (i < 0) {
+    return (-i - 1 < meta_candidates_.size());
+  } else {
+    return (i < candidates_.size());
+  }
+}
+
+inline const Segment::Candidate &Segment::candidate(int i) const {
+  if (i < 0) {
+    return meta_candidate(-i - 1);
+  }
+  DCHECK(i < candidates_.size());
+  return *candidates_[i];
+}
+
+inline Segment::Candidate *Segment::mutable_candidate(int i) {
+  if (i < 0) {
+    const size_t meta_index = -i - 1;
+    DCHECK_LT(meta_index, meta_candidates_.size());
+    return &meta_candidates_[meta_index];
+  }
+  DCHECK_LT(i, candidates_.size());
+  return candidates_[i];
+}
+
 }  // namespace mozc
 
 #endif  // MOZC_CONVERTER_SEGMENTS_H_
