@@ -30,17 +30,22 @@
 #ifndef MOZC_DATA_MANAGER_DATA_MANAGER_H_
 #define MOZC_DATA_MANAGER_DATA_MANAGER_H_
 
+#include <cstddef>
 #include <cstdint>
 #include <iosfwd>
+#include <memory>
+#include <optional>
+#include <ostream>
 #include <string>
 #include <utility>
 #include <vector>
 
 #include "base/mmap.h"
-#include "base/port.h"
 #include "data_manager/data_manager_interface.h"
+#include "absl/container/flat_hash_map.h"
 #include "absl/status/statusor.h"
 #include "absl/strings/string_view.h"
+#include "absl/types/span.h"
 
 namespace mozc {
 
@@ -72,13 +77,10 @@ class DataManager : public DataManagerInterface {
   static absl::StatusOr<std::unique_ptr<DataManager>> CreateFromFile(
       const std::string &path, absl::string_view magic);
 
-  DataManager();
-
+  DataManager() = default;
   DataManager(const DataManager &) = delete;
   DataManager &operator=(const DataManager &) = delete;
 
-  ~DataManager() override;
-
   // Parses |array| and extracts byte blocks of data set.  The |array| must
   // outlive this instance.  The second version specifies a custom magic number
   // to expect (e.g., mock data set has a different magic number).
@@ -101,15 +103,15 @@ class DataManager : public DataManagerInterface {
                                         absl::string_view magic);
 
   // Implementation of DataManagerInterface.
+  std::optional<std::string> GetFilename() const override { return filename_; }
   const uint16_t *GetPosMatcherData() const override;
   void GetUserPosData(absl::string_view *token_array_data,
                       absl::string_view *string_array_data) const override;
   void GetConnectorData(const char **data, size_t *size) const override;
   void GetSystemDictionaryData(const char **data, int *size) const override;
-  void GetCollocationData(const char **array, size_t *size) const override;
-  void GetCollocationSuppressionData(const char **array,
-                                     size_t *size) const override;
-  void GetSuggestionFilterData(const char **data, size_t *size) const override;
+  absl::Span<const uint32_t> GetCollocationData() const override;
+  absl::Span<const uint32_t> GetCollocationSuppressionData() const override;
+  absl::Span<const uint32_t> GetSuggestionFilterData() const override;
   const uint8_t *GetPosGroupData() const override;
   void GetSegmenterData(size_t *l_num_elements, size_t *r_num_elements,
                         const uint16_t **l_table, const uint16_t **r_table,
@@ -139,6 +141,9 @@ class DataManager : public DataManagerInterface {
       absl::string_view *variant_string_array_data,
       absl::string_view *noun_prefix_token_array_data,
       absl::string_view *noun_prefix_string_array_data) const override;
+  void GetA11yDescriptionRewriterData(
+      absl::string_view *token_array_data,
+      absl::string_view *string_array_data) const override;
   void GetZeroQueryData(
       absl::string_view *zero_query_token_array_data,
       absl::string_view *zero_query_string_array_data,
@@ -154,12 +159,15 @@ class DataManager : public DataManagerInterface {
       absl::string_view *string_array_data) const override;
 #endif  // NO_USAGE_REWRITER
 
-  absl::string_view GetTypingModel(const std::string &name) const override;
   absl::string_view GetDataVersion() const override;
 
+  std::optional<std::pair<size_t, size_t>> GetOffsetAndSize(
+      absl::string_view name) const override;
+
  private:
   Status InitFromReader(const DataSetReader &reader);
 
+  std::optional<std::string> filename_ = std::nullopt;
   Mmap mmap_;
   absl::string_view pos_matcher_data_;
   absl::string_view user_pos_token_array_data_;
@@ -196,6 +204,8 @@ class DataManager : public DataManagerInterface {
   absl::string_view single_kanji_variant_string_array_data_;
   absl::string_view single_kanji_noun_prefix_token_array_data_;
   absl::string_view single_kanji_noun_prefix_string_array_data_;
+  absl::string_view a11y_description_token_array_data_;
+  absl::string_view a11y_description_string_array_data_;
   absl::string_view zero_query_token_array_data_;
   absl::string_view zero_query_string_array_data_;
   absl::string_view zero_query_number_token_array_data_;
@@ -205,8 +215,8 @@ class DataManager : public DataManagerInterface {
   absl::string_view usage_conjugation_index_data_;
   absl::string_view usage_items_data_;
   absl::string_view usage_string_array_data_;
-  std::vector<std::pair<std::string, absl::string_view>> typing_model_data_;
   absl::string_view data_version_;
+  absl::flat_hash_map<std::string, std::pair<size_t, size_t>> offset_and_size_;
 };
 
 // Print helper for DataManager::Status.  Logging, e.g., CHECK_EQ(), requires
