@@ -29,37 +29,38 @@
 
 #include "data_manager/dataset_reader.h"
 
+#include <cstddef>
 #include <cstdint>
+#include <cstring>
 #include <limits>
+#include <optional>
 #include <sstream>
 #include <string>
 
-#include "base/port.h"
+#include "base/random.h"
 #include "base/util.h"
+#include "data_manager/dataset.pb.h"
 #include "data_manager/dataset_writer.h"
-#include "testing/base/public/gunit.h"
+#include "testing/gmock.h"
+#include "testing/gunit.h"
+#include "absl/random/distributions.h"
+#include "absl/strings/str_cat.h"
 #include "absl/strings/str_format.h"
 #include "absl/strings/string_view.h"
 
 namespace mozc {
 namespace {
 
-std::string GenerateRandomBytes(size_t len) {
-  std::string s;
-  s.resize(len);
-  for (size_t i = 0; i < len; ++i) {
-    s[i] = static_cast<char>(Util::Random(256));
-  }
-  return s;
-}
+using ::testing::Optional;
+using ::testing::Pair;
 
-std::string GetTestMagicNumber() { return std::string("ma\0gic", 6); }
+constexpr absl::string_view kTestMagicNumber = {"ma\0gic", 6};
 
 TEST(DataSetReaderTest, ValidData) {
-  const absl::string_view kGoogle("GOOGLE"), kMozc("m\0zc\xEF", 5);
+  constexpr absl::string_view kGoogle("GOOGLE"), kMozc("m\0zc\xEF", 5);
   std::string image;
   {
-    DataSetWriter w(GetTestMagicNumber());
+    DataSetWriter w(kTestMagicNumber);
     w.Add("google", 16, kGoogle);
     w.Add("mozc", 64, kMozc);
     std::stringstream out;
@@ -69,69 +70,67 @@ TEST(DataSetReaderTest, ValidData) {
 
   DataSetReader r;
   ASSERT_TRUE(DataSetReader::VerifyChecksum(image));
-  ASSERT_TRUE(r.Init(image, GetTestMagicNumber()));
+  ASSERT_TRUE(r.Init(image, kTestMagicNumber));
 
   absl::string_view data;
   EXPECT_TRUE(r.Get("google", &data));
-  EXPECT_EQ(kGoogle, data);
+  EXPECT_EQ(data, kGoogle);
+  EXPECT_THAT(r.GetOffsetAndSize("google"), Optional(Pair(6, kGoogle.size())));
   EXPECT_TRUE(r.Get("mozc", &data));
-  EXPECT_EQ(kMozc, data);
+  EXPECT_EQ(data, kMozc);
+  EXPECT_THAT(r.GetOffsetAndSize("mozc"), Optional(Pair(16, kMozc.size())));
 
   EXPECT_FALSE(r.Get("", &data));
   EXPECT_FALSE(r.Get("foo", &data));
+  EXPECT_EQ(r.GetOffsetAndSize(""), std::nullopt);
+  EXPECT_EQ(r.GetOffsetAndSize("foo"), std::nullopt);
 }
 
 TEST(DataSetReaderTest, InvalidMagicString) {
-  const std::string &magic = GetTestMagicNumber();
   DataSetReader r;
-  EXPECT_FALSE(r.Init("", magic));
-  EXPECT_FALSE(r.Init("abc", magic));
-  EXPECT_FALSE(r.Init("this is a text file", magic));
+  EXPECT_FALSE(r.Init("", kTestMagicNumber));
+  EXPECT_FALSE(r.Init("abc", kTestMagicNumber));
+  EXPECT_FALSE(r.Init("this is a text file", kTestMagicNumber));
 }
 
 TEST(DataSetReaderTest, BrokenMetadata) {
-  const std::string &magic = GetTestMagicNumber();
   DataSetReader r;
+  constexpr absl::string_view kContent = "content and metadata";
 
   // Only magic number, no metadata.
-  EXPECT_FALSE(DataSetReader::VerifyChecksum(magic));
-  EXPECT_FALSE(r.Init(magic, magic));
+  EXPECT_FALSE(DataSetReader::VerifyChecksum(kTestMagicNumber));
+  EXPECT_FALSE(r.Init(kTestMagicNumber, kTestMagicNumber));
 
   // Metadata size is too small.
-  std::string data = magic;
-  data.append("content and metadata");
-  data.append(Util::SerializeUint64(0));
+  std::string data =
+      absl::StrCat(kTestMagicNumber, kContent, Util::SerializeUint64(0));
   EXPECT_FALSE(DataSetReader::VerifyChecksum(data));
-  EXPECT_FALSE(r.Init(data, magic));
+  EXPECT_FALSE(r.Init(data, kTestMagicNumber));
 
   // Metadata size is too small.
-  data = magic;
-  data.append("content and metadata");
-  data.append(Util::SerializeUint64(4));
+  data = absl::StrCat(kTestMagicNumber, kContent, Util::SerializeUint64(4));
   EXPECT_FALSE(DataSetReader::VerifyChecksum(data));
-  EXPECT_FALSE(r.Init(data, magic));
+  EXPECT_FALSE(r.Init(data, kTestMagicNumber));
 
   // Metadata size is too large.
-  data = magic;
-  data.append("content and metadata");
-  data.append(Util::SerializeUint64(std::numeric_limits<uint64_t>::max()));
+  data =
+      absl::StrCat(kTestMagicNumber, kContent,
+                   Util::SerializeUint64(std::numeric_limits<uint64_t>::max()));
   EXPECT_FALSE(DataSetReader::VerifyChecksum(data));
-  EXPECT_FALSE(r.Init(data, magic));
+  EXPECT_FALSE(r.Init(data, kTestMagicNumber));
 
-  // Metadata chunk is not a serialied protobuf.
-  data = magic;
-  data.append("content and metadata");
-  data.append(Util::SerializeUint64(strlen("content and metadata")));
+  // Metadata chunk is not a serialized protobuf.
+  data = absl::StrCat(kTestMagicNumber, kContent,
+                      Util::SerializeUint64(kContent.size()));
   EXPECT_FALSE(DataSetReader::VerifyChecksum(data));
-  EXPECT_FALSE(r.Init(data, magic));
+  EXPECT_FALSE(r.Init(data, kTestMagicNumber));
 }
 
 TEST(DataSetReaderTest, BrokenMetadataFields) {
-  const std::string &magic = GetTestMagicNumber();
-  const absl::string_view kGoogle("GOOGLE"), kMozc("m\0zc\xEF", 5);
+  constexpr absl::string_view kGoogle("GOOGLE"), kMozc("m\0zc\xEF", 5);
   std::string content;
   {
-    DataSetWriter w(magic);
+    DataSetWriter w(kTestMagicNumber);
     w.Add("google", 16, kGoogle);
     w.Add("mozc", 64, kMozc);
     std::stringstream out;
@@ -140,7 +139,7 @@ TEST(DataSetReaderTest, BrokenMetadataFields) {
     // Remove the metadata chunk at the bottom, which will be appended in each
     // test below.
     content = out.str();
-    const size_t metadata_size = w.metadata().SerializeAsString().size();
+    const size_t metadata_size = w.metadata().ByteSizeLong();
     content.erase(content.size() - metadata_size - 8);
   }
   {
@@ -151,13 +150,12 @@ TEST(DataSetReaderTest, BrokenMetadataFields) {
     e->set_offset(content.size() + 3);  // Invalid offset
     e->set_size(kGoogle.size());
     const std::string &md_str = md.SerializeAsString();
-    std::string image = content;
-    image.append(md_str);
-    image.append(Util::SerializeUint64(md_str.size()));
+    std::string image =
+        absl::StrCat(content, md_str, Util::SerializeUint64(md_str.size()));
 
     DataSetReader r;
     EXPECT_FALSE(DataSetReader::VerifyChecksum(image));
-    EXPECT_FALSE(r.Init(image, magic));
+    EXPECT_FALSE(r.Init(image, kTestMagicNumber));
   }
   {
     // Create an image with broken size.
@@ -167,27 +165,28 @@ TEST(DataSetReaderTest, BrokenMetadataFields) {
     e->set_offset(content.size());
     e->set_size(std::numeric_limits<uint64_t>::max());  // Too big size
     const std::string &md_str = md.SerializeAsString();
-    std::string image = content;
-    image.append(md_str);
-    image.append(Util::SerializeUint64(md_str.size()));
+    std::string image =
+        absl::StrCat(content, md_str, Util::SerializeUint64(md_str.size()));
 
     DataSetReader r;
     EXPECT_FALSE(DataSetReader::VerifyChecksum(image));
-    EXPECT_FALSE(r.Init(image, magic));
+    EXPECT_FALSE(r.Init(image, kTestMagicNumber));
   }
 }
 
 TEST(DataSetReaderTest, OneBitError) {
-  const char *kTestMagicNumber = "Dummy magic number\r\n";
+  constexpr absl::string_view kTestMagicNumber = "Dummy magic number\r\n";
 
   // Create data at random.
   std::string image;
   {
     constexpr int kAlignments[] = {8, 16, 32, 64};
     DataSetWriter w(kTestMagicNumber);
+    Random random;
     for (int i = 0; i < 10; ++i) {
-      w.Add(absl::StrFormat("key%d", i), kAlignments[Util::Random(4)],
-            GenerateRandomBytes(1 + Util::Random(1024)));
+      w.Add(absl::StrFormat("key%d", i),
+            kAlignments[absl::Uniform(random, 0u, std::size(kAlignments))],
+            random.ByteString(absl::Uniform(random, 1, 1024)));
     }
     std::stringstream out;
     w.Finish(&out);
