@@ -35,12 +35,13 @@
 #include <vector>
 
 #include "base/logging.h"
+#include "base/random.h"
 #include "base/util.h"
 #include "dictionary/dictionary_token.h"
 #include "dictionary/system/codec_interface.h"
 #include "dictionary/system/words_info.h"
-#include "testing/base/public/googletest.h"
-#include "testing/base/public/gunit.h"
+#include "testing/gunit.h"
+#include "absl/random/distributions.h"
 #include "absl/strings/str_format.h"
 #include "absl/strings/string_view.h"
 
@@ -48,17 +49,17 @@ namespace mozc {
 namespace dictionary {
 namespace {
 
-::testing::AssertionResult MakeAssertResult(bool success, char32 c,
+::testing::AssertionResult MakeAssertResult(bool success, char32_t c,
                                             const char *message) {
   if (success) {
     return ::testing::AssertionSuccess();
   }
   return ::testing::AssertionFailure()
-         << message << " c = " << absl::StrFormat("U+%05X", c);
+         << message << " c = " << absl::StreamFormat("U+%05X", c);
 }
 
-::testing::AssertionResult IsExpectedEncodedSize(char32 c,
-                                                 const std::string &encoded) {
+::testing::AssertionResult IsExpectedEncodedSize(
+    char32_t c, const absl::string_view encoded) {
   const std::string::size_type size = encoded.size();
   if (c == 0x00) {
     return ::testing::AssertionFailure() << "NUL is not supported.";
@@ -112,7 +113,7 @@ namespace {
                             "are encoded into 2 bytes.");
   }
   return MakeAssertResult(size == 3, c,
-                          "Other charaters should be encoded into 3bytes.");
+                          "Other characters should be encoded into 3bytes.");
 }
 
 }  // namespace
@@ -154,16 +155,16 @@ class SystemDictionaryCodecTest : public ::testing::Test {
     CHECK(token_info);
     token_info->pos_type = TokenInfo::DEFAULT_POS;
     // set id randomly
-    const int id = Util::Random(50);
+    const int id = absl::Uniform(random_, 0, 50);
     token_info->token->lid = id;
-    token_info->token->rid = (Util::Random(2) == 0) ? id : id + 1;
+    token_info->token->rid = absl::Bernoulli(random_, 0.5) ? id : id + 1;
   }
 
   void SetFrequentPos(TokenInfo *token_info) const {
     CHECK(token_info);
     token_info->pos_type = TokenInfo::FREQUENT_POS;
     // set id randomly
-    const int id = Util::Random(256);
+    const int id = absl::Uniform(random_, 0, 256);
     token_info->id_in_frequent_pos_map = id;
   }
 
@@ -175,7 +176,7 @@ class SystemDictionaryCodecTest : public ::testing::Test {
   void SetRandPos() {
     for (size_t i = 0; i < source_tokens_.size(); ++i) {
       CHECK(source_tokens_[i].token);
-      int n = Util::Random(TokenInfo::POS_TYPE_SIZE);
+      int n = absl::Uniform(random_, 0, TokenInfo::POS_TYPE_SIZE);
       CHECK_GE(n, 0);
       CHECK_LT(n, TokenInfo::POS_TYPE_SIZE);
       if (i == 0 && n == 2) {
@@ -199,7 +200,7 @@ class SystemDictionaryCodecTest : public ::testing::Test {
     CHECK(token_info);
     token_info->cost_type = TokenInfo::DEFAULT_COST;
     // set cost randomly
-    const int cost = Util::Random(8000);
+    const int cost = absl::Uniform(random_, 0, 8000);
     token_info->token->cost = cost;
   }
 
@@ -207,14 +208,14 @@ class SystemDictionaryCodecTest : public ::testing::Test {
     CHECK(token_info);
     token_info->cost_type = TokenInfo::CAN_USE_SMALL_ENCODING;
     // set cost randomly
-    const int cost = Util::Random(8000);
+    const int cost = absl::Uniform(random_, 0, 8000);
     token_info->token->cost = cost;
   }
 
   void SetRandCost() {
     for (size_t i = 0; i < source_tokens_.size(); ++i) {
       CHECK(source_tokens_[i].token);
-      int n = Util::Random(TokenInfo::COST_TYPE_SIZE);
+      int n = absl::Uniform(random_, 0, TokenInfo::COST_TYPE_SIZE);
       CHECK_GE(n, 0);
       CHECK_LT(n, TokenInfo::POS_TYPE_SIZE);
       if (n == 0) {
@@ -229,7 +230,7 @@ class SystemDictionaryCodecTest : public ::testing::Test {
     CHECK(token_info);
     token_info->value_type = TokenInfo::DEFAULT_VALUE;
     // set id randomly
-    const int id = Util::Random(50000);
+    const int id = absl::Uniform(random_, 0, 50000);
     token_info->id_in_value_trie = id;
   }
 
@@ -241,7 +242,7 @@ class SystemDictionaryCodecTest : public ::testing::Test {
   void SetRandValue() {
     for (size_t i = 0; i < source_tokens_.size(); ++i) {
       CHECK(source_tokens_[i].token);
-      int n = Util::Random(TokenInfo::VALUE_TYPE_SIZE);
+      int n = absl::Uniform(random_, 0, TokenInfo::VALUE_TYPE_SIZE);
       CHECK_GE(n, 0);
       CHECK_LT(n, TokenInfo::VALUE_TYPE_SIZE);
       if (i == 0 && n == 1) {
@@ -263,7 +264,7 @@ class SystemDictionaryCodecTest : public ::testing::Test {
   void SetRandLabel() {
     for (size_t i = 0; i < source_tokens_.size(); ++i) {
       CHECK(source_tokens_[i].token);
-      int n = Util::Random(Token::LABEL_SIZE);
+      int n = absl::Uniform(random_, 0, Token::LABEL_SIZE);
       CHECK_GE(n, 0);
       CHECK_LT(n, Token::LABEL_SIZE);
       if (n == 0) {
@@ -275,7 +276,7 @@ class SystemDictionaryCodecTest : public ::testing::Test {
   }
 
   void CheckDecoded() const {
-    EXPECT_EQ(source_tokens_.size(), decoded_tokens_.size());
+    EXPECT_EQ(decoded_tokens_.size(), source_tokens_.size());
     for (size_t i = 0; i < source_tokens_.size(); ++i) {
       EXPECT_TRUE(source_tokens_[i].token != nullptr);
       EXPECT_TRUE(decoded_tokens_[i].token != nullptr);
@@ -283,41 +284,42 @@ class SystemDictionaryCodecTest : public ::testing::Test {
       EXPECT_EQ(source_tokens_[i].token->attributes,
                 decoded_tokens_[i].token->attributes);
 
-      EXPECT_EQ(source_tokens_[i].pos_type, decoded_tokens_[i].pos_type);
+      EXPECT_EQ(decoded_tokens_[i].pos_type, source_tokens_[i].pos_type);
       if (source_tokens_[i].pos_type == TokenInfo::DEFAULT_POS) {
-        EXPECT_EQ(source_tokens_[i].token->lid, decoded_tokens_[i].token->lid);
-        EXPECT_EQ(source_tokens_[i].token->rid, decoded_tokens_[i].token->rid);
+        EXPECT_EQ(decoded_tokens_[i].token->lid, source_tokens_[i].token->lid);
+        EXPECT_EQ(decoded_tokens_[i].token->rid, source_tokens_[i].token->rid);
       } else if (source_tokens_[i].pos_type == TokenInfo::FREQUENT_POS) {
-        EXPECT_EQ(source_tokens_[i].id_in_frequent_pos_map,
-                  decoded_tokens_[i].id_in_frequent_pos_map);
+        EXPECT_EQ(decoded_tokens_[i].id_in_frequent_pos_map,
+                  source_tokens_[i].id_in_frequent_pos_map);
       }
 
       if (source_tokens_[i].cost_type == TokenInfo::DEFAULT_COST) {
-        EXPECT_EQ(source_tokens_[i].token->cost,
-                  decoded_tokens_[i].token->cost);
+        EXPECT_EQ(decoded_tokens_[i].token->cost,
+                  source_tokens_[i].token->cost);
       } else {  // small cost
         EXPECT_NEAR(source_tokens_[i].token->cost,
                     decoded_tokens_[i].token->cost, 256);
       }
 
-      EXPECT_EQ(source_tokens_[i].value_type, decoded_tokens_[i].value_type);
+      EXPECT_EQ(decoded_tokens_[i].value_type, source_tokens_[i].value_type);
       if (source_tokens_[i].value_type == TokenInfo::DEFAULT_VALUE) {
-        EXPECT_EQ(source_tokens_[i].id_in_value_trie,
-                  decoded_tokens_[i].id_in_value_trie);
+        EXPECT_EQ(decoded_tokens_[i].id_in_value_trie,
+                  source_tokens_[i].id_in_value_trie);
       }
     }
   }
 
   std::vector<TokenInfo> source_tokens_;
   std::vector<TokenInfo> decoded_tokens_;
+  mutable mozc::Random random_;
 };
 
 class SystemDictionaryCodecMock : public SystemDictionaryCodecInterface {
  public:
-  const std::string GetSectionNameForKey() const override { return "Mock"; }
-  const std::string GetSectionNameForValue() const override { return "Mock"; }
-  const std::string GetSectionNameForTokens() const override { return "Mock"; }
-  const std::string GetSectionNameForPos() const override { return "Mock"; }
+  std::string GetSectionNameForKey() const override { return "Mock"; }
+  std::string GetSectionNameForValue() const override { return "Mock"; }
+  std::string GetSectionNameForTokens() const override { return "Mock"; }
+  std::string GetSectionNameForPos() const override { return "Mock"; }
   void EncodeKey(const absl::string_view src, std::string *dst) const override {
   }
   void DecodeKey(const absl::string_view src, std::string *dst) const override {
@@ -354,7 +356,7 @@ TEST_F(SystemDictionaryCodecTest, FactoryTest) {
   SystemDictionaryCodecFactory::SetCodec(mock.get());
   SystemDictionaryCodecInterface *codec =
       SystemDictionaryCodecFactory::GetCodec();
-  EXPECT_EQ("Mock", codec->GetSectionNameForKey());
+  EXPECT_EQ(codec->GetSectionNameForKey(), "Mock");
 }
 
 TEST_F(SystemDictionaryCodecTest, KeyCodecKanaTest) {
@@ -364,12 +366,12 @@ TEST_F(SystemDictionaryCodecTest, KeyCodecKanaTest) {
   std::string encoded;
   codec->EncodeKey(original, &encoded);
   // hiragana should be encoded in 1 byte
-  EXPECT_EQ(2, encoded.size());
-  EXPECT_EQ(encoded.size(), codec->GetEncodedKeyLength(original));
+  EXPECT_EQ(encoded.size(), 2);
+  EXPECT_EQ(codec->GetEncodedKeyLength(original), encoded.size());
   std::string decoded;
   codec->DecodeKey(encoded, &decoded);
-  EXPECT_EQ(original, decoded);
-  EXPECT_EQ(decoded.size(), codec->GetDecodedKeyLength(encoded));
+  EXPECT_EQ(decoded, original);
+  EXPECT_EQ(codec->GetDecodedKeyLength(encoded), decoded.size());
 }
 
 TEST_F(SystemDictionaryCodecTest, KeyCodecSymbolTest) {
@@ -379,19 +381,19 @@ TEST_F(SystemDictionaryCodecTest, KeyCodecSymbolTest) {
   std::string encoded;
   codec->EncodeKey(original, &encoded);
   // middle dot and prolonged sound should be encoded in 1 byte
-  EXPECT_EQ(2, encoded.size());
-  EXPECT_EQ(encoded.size(), codec->GetEncodedKeyLength(original));
+  EXPECT_EQ(encoded.size(), 2);
+  EXPECT_EQ(codec->GetEncodedKeyLength(original), encoded.size());
   std::string decoded;
   codec->DecodeKey(encoded, &decoded);
-  EXPECT_EQ(original, decoded);
-  EXPECT_EQ(decoded.size(), codec->GetDecodedKeyLength(encoded));
+  EXPECT_EQ(decoded, original);
+  EXPECT_EQ(codec->GetDecodedKeyLength(encoded), decoded.size());
 }
 
 TEST_F(SystemDictionaryCodecTest, ValueCodecTest) {
   std::unique_ptr<SystemDictionaryCodec> codec(new SystemDictionaryCodec);
   // TODO(toshiyuki): Use 0x10ffff instead when UCS4 is supported.
-  constexpr char32 kMaxUniChar = 0x10ffff;
-  for (char32 c = 0x01; c <= kMaxUniChar; ++c) {
+  constexpr char32_t kMaxUniChar = 0x10ffff;
+  for (char32_t c = 0x01; c <= kMaxUniChar; ++c) {
     std::string original;
     Util::Ucs4ToUtf8(c, &original);
     std::string encoded;
@@ -399,7 +401,7 @@ TEST_F(SystemDictionaryCodecTest, ValueCodecTest) {
     EXPECT_TRUE(IsExpectedEncodedSize(c, encoded));
     std::string decoded;
     codec->DecodeValue(encoded, &decoded);
-    EXPECT_EQ(original, decoded) << "failed at: " << static_cast<uint32_t>(c);
+    EXPECT_EQ(decoded, original) << "failed at: " << static_cast<uint32_t>(c);
   }
 }
 
@@ -410,10 +412,10 @@ TEST_F(SystemDictionaryCodecTest, ValueCodecKanaTest) {
   std::string encoded;
   codec->EncodeValue(original, &encoded);
   // kana should be encoded in 1 byte
-  EXPECT_EQ(2, encoded.size());
+  EXPECT_EQ(encoded.size(), 2);
   std::string decoded;
   codec->DecodeValue(encoded, &decoded);
-  EXPECT_EQ(original, decoded);
+  EXPECT_EQ(decoded, original);
 }
 
 TEST_F(SystemDictionaryCodecTest, ValueCodecAsciiTest) {
@@ -423,10 +425,10 @@ TEST_F(SystemDictionaryCodecTest, ValueCodecAsciiTest) {
   std::string encoded;
   codec->EncodeValue(original, &encoded);
   // ascii should be encoded in 2 bytes
-  EXPECT_EQ(8, encoded.size());
+  EXPECT_EQ(encoded.size(), 8);
   std::string decoded;
   codec->DecodeValue(encoded, &decoded);
-  EXPECT_EQ(original, decoded);
+  EXPECT_EQ(decoded, original);
 }
 
 TEST_F(SystemDictionaryCodecTest, TokenDefaultPosTest) {
@@ -487,7 +489,6 @@ TEST_F(SystemDictionaryCodecTest, TokenRandomPosTest) {
   SystemDictionaryCodecInterface *codec =
       SystemDictionaryCodecFactory::GetCodec();
   InitTokens(50);
-  Util::SetRandomSeed(0);
   SetRandPos();
   std::string encoded;
   codec->EncodeTokens(source_tokens_, &encoded);
@@ -527,7 +528,6 @@ TEST_F(SystemDictionaryCodecTest, TokenRandomCostTest) {
   SystemDictionaryCodecInterface *codec =
       SystemDictionaryCodecFactory::GetCodec();
   InitTokens(50);
-  Util::SetRandomSeed(0);
   SetRandCost();
   std::string encoded;
   codec->EncodeTokens(source_tokens_, &encoded);
@@ -650,7 +650,7 @@ TEST_F(SystemDictionaryCodecTest, UCS4CharactersTest) {
   EXPECT_GT(encoded.size(), 0);
   std::string decoded;
   codec->DecodeValue(encoded, &decoded);
-  EXPECT_EQ(ucs4_including, decoded);
+  EXPECT_EQ(decoded, ucs4_including);
 }
 
 TEST_F(SystemDictionaryCodecTest, TokenSameValueTest) {
@@ -671,7 +671,6 @@ TEST_F(SystemDictionaryCodecTest, TokenRandomValueTest) {
   SystemDictionaryCodecInterface *codec =
       SystemDictionaryCodecFactory::GetCodec();
   InitTokens(50);
-  Util::SetRandomSeed(0);
   SetRandValue();
   std::string encoded;
   codec->EncodeTokens(source_tokens_, &encoded);
@@ -685,7 +684,6 @@ TEST_F(SystemDictionaryCodecTest, TokenRandomLabelTest) {
   SystemDictionaryCodecInterface *codec =
       SystemDictionaryCodecFactory::GetCodec();
   InitTokens(50);
-  Util::SetRandomSeed(0);
   SetRandLabel();
   std::string encoded;
   codec->EncodeTokens(source_tokens_, &encoded);
@@ -699,7 +697,6 @@ TEST_F(SystemDictionaryCodecTest, TokenRandomTest) {
   SystemDictionaryCodecInterface *codec =
       SystemDictionaryCodecFactory::GetCodec();
   InitTokens(50);
-  Util::SetRandomSeed(0);
   SetRandPos();
   SetRandCost();
   SetRandValue();
@@ -716,7 +713,6 @@ TEST_F(SystemDictionaryCodecTest, ReadTokenRandomTest) {
   SystemDictionaryCodecInterface *codec =
       SystemDictionaryCodecFactory::GetCodec();
   InitTokens(50);
-  Util::SetRandomSeed(0);
   SetRandPos();
   SetRandCost();
   SetRandValue();
@@ -733,9 +729,9 @@ TEST_F(SystemDictionaryCodecTest, ReadTokenRandomTest) {
         reinterpret_cast<const unsigned char *>(encoded.data()) + offset,
         &value_id, &read_byte));
     if (source_tokens_[read_num].value_type == TokenInfo::DEFAULT_VALUE) {
-      EXPECT_EQ(source_tokens_[read_num].id_in_value_trie, value_id);
+      EXPECT_EQ(value_id, source_tokens_[read_num].id_in_value_trie);
     } else {
-      EXPECT_EQ(-1, value_id);
+      EXPECT_EQ(value_id, -1);
     }
     offset += read_byte;
     ++read_num;
@@ -743,7 +739,7 @@ TEST_F(SystemDictionaryCodecTest, ReadTokenRandomTest) {
       break;
     }
   }
-  EXPECT_EQ(source_tokens_.size(), read_num);
+  EXPECT_EQ(read_num, source_tokens_.size());
 }
 
 TEST_F(SystemDictionaryCodecTest, CodecTest) {
@@ -753,7 +749,6 @@ TEST_F(SystemDictionaryCodecTest, CodecTest) {
       SystemDictionaryCodecFactory::GetCodec();
   {  // Token
     InitTokens(50);
-    Util::SetRandomSeed(0);
     SetRandPos();
     SetRandCost();
     SetRandValue();
@@ -775,9 +770,9 @@ TEST_F(SystemDictionaryCodecTest, CodecTest) {
           reinterpret_cast<const unsigned char *>(encoded.data()) + offset,
           &value_id, &read_byte));
       if (source_tokens_[read_num].value_type == TokenInfo::DEFAULT_VALUE) {
-        EXPECT_EQ(source_tokens_[read_num].id_in_value_trie, value_id);
+        EXPECT_EQ(value_id, source_tokens_[read_num].id_in_value_trie);
       } else {
-        EXPECT_EQ(-1, value_id);
+        EXPECT_EQ(value_id, -1);
       }
       offset += read_byte;
       ++read_num;
@@ -785,42 +780,32 @@ TEST_F(SystemDictionaryCodecTest, CodecTest) {
         break;
       }
     }
-    EXPECT_EQ(source_tokens_.size(), read_num);
+    EXPECT_EQ(read_num, source_tokens_.size());
   }
-  {  // Value
-    std::string original;
-    {
-      char32 a_ucs4 = '!';
-      Util::SetRandomSeed(0);
-      for (size_t i = 0; i < 10000; ++i) {
-        // U+4E00-9FFF CJK Unified Ideographs
-        const char32 c = a_ucs4 + static_cast<uint16_t>(Util::Random(0x9f00));
-        Util::Ucs4ToUtf8Append(c, &original);
-      }
-    }
+  {
+    // Value
+    // U+4E00-9FFF CJK Unified Ideographs
+    constexpr char32_t a_ucs4 = '!';
+    const std::string original =
+        random_.Utf8String(10000, a_ucs4, a_ucs4 + 0x9f00);
     std::string encoded;
     codec->EncodeValue(original, &encoded);
     std::string decoded;
     codec->DecodeValue(encoded, &decoded);
-    EXPECT_EQ(original, decoded);
+    EXPECT_EQ(decoded, original);
   }
-  {  // Key
-    std::string original;
-    {
-      char32 a_ucs4 = 0x3041;  // "ぁ"
-      Util::SetRandomSeed(0);
-      for (size_t i = 0; i < 1000; ++i) {
-        const char32 c = a_ucs4 + static_cast<uint16_t>(Util::Random(1000));
-        Util::Ucs4ToUtf8Append(c, &original);
-      }
-    }
+  {
+    // Key
+    constexpr char32_t a_ucs4 = 0x3041;  // "ぁ"
+    const std::string original =
+        random_.Utf8String(1000, a_ucs4, a_ucs4 + 1000);
     std::string encoded;
     codec->EncodeKey(original, &encoded);
-    EXPECT_EQ(encoded.size(), codec->GetEncodedKeyLength(original));
+    EXPECT_EQ(codec->GetEncodedKeyLength(original), encoded.size());
     std::string decoded;
     codec->DecodeKey(encoded, &decoded);
-    EXPECT_EQ(original, decoded);
-    EXPECT_EQ(decoded.size(), codec->GetDecodedKeyLength(encoded));
+    EXPECT_EQ(decoded, original);
+    EXPECT_EQ(codec->GetDecodedKeyLength(encoded), decoded.size());
   }
 }
 
