@@ -32,34 +32,33 @@
 #include <algorithm>
 #include <cstdint>
 #include <cstdlib>
+#include <iterator>
 #include <limits>
 #include <memory>
+#include <set>
 #include <string>
 #include <utility>
 #include <vector>
 
+#include "base/file/temp_dir.h"
 #include "base/file_util.h"
-#include "base/logging.h"
-#include "base/port.h"
-#include "base/system_util.h"
-#include "base/util.h"
 #include "config/config_handler.h"
 #include "data_manager/testing/mock_data_manager.h"
+#include "dictionary/dictionary_interface.h"
 #include "dictionary/dictionary_test_util.h"
 #include "dictionary/dictionary_token.h"
 #include "dictionary/pos_matcher.h"
-#include "dictionary/system/codec_interface.h"
 #include "dictionary/system/system_dictionary_builder.h"
 #include "dictionary/text_dictionary_loader.h"
 #include "protocol/commands.pb.h"
 #include "protocol/config.pb.h"
 #include "request/conversion_request.h"
-#include "testing/base/public/googletest.h"
-#include "testing/base/public/gunit.h"
-#include "testing/base/public/mozctest.h"
+#include "testing/gunit.h"
+#include "testing/mozctest.h"
 #include "absl/container/btree_set.h"
 #include "absl/flags/declare.h"
 #include "absl/flags/flag.h"
+#include "absl/strings/str_cat.h"
 #include "absl/strings/str_format.h"
 #include "absl/strings/string_view.h"
 
@@ -73,15 +72,15 @@ namespace mozc {
 namespace dictionary {
 namespace {
 
-class SystemDictionaryTest : public ::testing::Test {
+class SystemDictionaryTest : public testing::TestWithTempUserProfile {
  protected:
   SystemDictionaryTest()
       : pos_matcher_(mock_data_manager_.GetPosMatcherData()),
         text_dict_(pos_matcher_),
-        dic_fn_(
-            FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir), "mozc.dic")) {
+        temp_dir_(testing::MakeTempDirectoryOrDie()),
+        dic_fn_(FileUtil::JoinPath(temp_dir_.path(), "mozc.dic")) {
     const std::string dic_path = mozc::testing::GetSourceFileOrDie(
-        {"data", "dictionary_oss", "dictionary00.txt"});
+        {MOZC_DICT_DIR_COMPONENTS, "dictionary_oss", "dictionary00.txt"});
     text_dict_.LoadWithLineLimit(dic_path, "",
                                  absl::GetFlag(FLAGS_dictionary_test_size));
 
@@ -120,7 +119,6 @@ class SystemDictionaryTest : public ::testing::Test {
   bool CompareTokensForLookup(const Token &a, const Token &b,
                               bool reverse) const;
 
-  const testing::ScopedTmpUserProfileDirectory scoped_profile_dir_;
   const testing::MockDataManager mock_data_manager_;
   dictionary::PosMatcher pos_matcher_;
   TextDictionaryLoader text_dict_;
@@ -128,6 +126,7 @@ class SystemDictionaryTest : public ::testing::Test {
   ConversionRequest convreq_;
   config::Config config_;
   commands::Request request_;
+  TempDirectory temp_dir_;
   const std::string dic_fn_;
   int original_flags_min_key_length_to_use_small_cost_encoding_;
 };
@@ -454,7 +453,7 @@ TEST_F(SystemDictionaryTest, LookupPrefix) {
           expected.find(kKeyValues[i].key) != expected.end();
       const std::pair<std::string, std::string> entry(kKeyValues[i].key,
                                                       kKeyValues[i].value);
-      EXPECT_EQ(to_be_found, result.find(entry) != result.end());
+      EXPECT_EQ(result.find(entry) != result.end(), to_be_found);
     }
   }
 }
@@ -613,7 +612,7 @@ TEST_F(SystemDictionaryTest, LookupReverse) {
   {
     // test for non exact transliterated index string.
     // append "が"
-    const std::string key = tokens[7].value + "が";
+    const std::string key = absl::StrCat(tokens[7].value, "が");
     CollectTokenCallback callback;
     system_dic->LookupReverse(key, convreq_, &callback);
     bool found = false;
@@ -844,6 +843,32 @@ TEST_F(SystemDictionaryTest, DoNotReturnNoModifierTargetWithLoudsTrie) {
   }
 }
 
+TEST_F(SystemDictionaryTest, ShouldNotUseSmallCostEncodingForHeteronyms) {
+  absl::SetFlag(&FLAGS_min_key_length_to_use_small_cost_encoding,
+                original_flags_min_key_length_to_use_small_cost_encoding_);
+
+  std::vector<Token> tokens = {
+      {"しょうろんぽう", "ショウロンポウ", 5948, 100, 100, Token::NONE},
+      {"しょうろんぽう", "小籠包", 7692, 100, 100, Token::NONE},
+      {"しょーろんぽう", "ショーロンポウ", 6092, 200, 200, Token::NONE},
+      {"しょーろんぽう", "小籠包", 9000, 100, 100, Token::NONE},
+  };
+
+  std::vector<Token *> source_tokens = MakeTokenPointers(&tokens);
+  std::unique_ptr<SystemDictionary> system_dic =
+      BuildSystemDictionary(source_tokens, source_tokens.size());
+  ASSERT_TRUE(system_dic);
+
+  for (size_t i = 0; i < source_tokens.size(); ++i) {
+    CheckTokenExistenceCallback callback(source_tokens[i]);
+    system_dic->LookupPrefix(source_tokens[i]->key, convreq_, &callback);
+    // The original token will not be found when the token is encoded with
+    // small_cost_encoding
+    EXPECT_TRUE(callback.found())
+        << "Token " << i << " was not found: " << PrintToken(*source_tokens[i]);
+  }
+}
+
 }  // namespace
 }  // namespace dictionary
 }  // namespace mozc
