@@ -27,35 +27,36 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-// This dictionary supports system dicitonary that will belookup from
+// This dictionary supports system dicitonary that will be looked up from
 // value, rather than key.
+// Value dictionary is mainly used to lookup the English words from
+// user-typed literal sequences. (English has no 'reading' representation.)
 // Token's key, cost, ids will not be looked up due to performance concern
 
 #ifndef MOZC_DICTIONARY_SYSTEM_VALUE_DICTIONARY_H_
 #define MOZC_DICTIONARY_SYSTEM_VALUE_DICTIONARY_H_
 
-#include "base/port.h"
+#include <cstdint>
+
 #include "dictionary/dictionary_interface.h"
+#include "dictionary/pos_matcher.h"
+#include "dictionary/system/codec_interface.h"
+#include "request/conversion_request.h"
 #include "storage/louds/louds_trie.h"
 #include "absl/strings/string_view.h"
 
 namespace mozc {
 namespace dictionary {
 
-class POSMatcher;
-class SystemDictionaryCodecInterface;
-
 class ValueDictionary : public DictionaryInterface {
  public:
   // This class doesn't take the ownership of |value_trie|.
-  ValueDictionary(const POSMatcher &pos_matcher,
+  ValueDictionary(const PosMatcher &pos_matcher,
                   const storage::louds::LoudsTrie *value_trie);
 
   ValueDictionary(const ValueDictionary &) = delete;
   ValueDictionary &operator=(const ValueDictionary &) = delete;
 
-  ~ValueDictionary() override;
-
   // Implementation of DictionaryInterface
   bool HasKey(absl::string_view key) const override;
   bool HasValue(absl::string_view value) const override;
@@ -75,7 +76,7 @@ class ValueDictionary : public DictionaryInterface {
  private:
   const storage::louds::LoudsTrie *value_trie_;
   const SystemDictionaryCodecInterface *codec_;
-  const uint16 suggestion_only_word_id_;
+  const uint16_t suggestion_only_word_id_;
 };
 
 }  // namespace dictionary
