@@ -33,15 +33,14 @@
 #include <string>
 #include <vector>
 
+#include "base/file/temp_dir.h"
 #include "base/file_util.h"
-#include "base/util.h"
 #include "data_manager/testing/mock_data_manager.h"
 #include "dictionary/dictionary_token.h"
 #include "dictionary/pos_matcher.h"
-#include "testing/base/public/gmock.h"
-#include "testing/base/public/googletest.h"
-#include "testing/base/public/gunit.h"
-#include "absl/flags/flag.h"
+#include "testing/gmock.h"
+#include "testing/gunit.h"
+#include "testing/mozctest.h"
 
 namespace mozc {
 namespace dictionary {
@@ -60,22 +59,17 @@ constexpr char kReadingCorrectionLines[] =
 
 class TextDictionaryLoaderTest : public ::testing::Test {
  protected:
-  // Explicitly define constructor to prevent Visual C++ from
-  // considering this class as POD.
-  TextDictionaryLoaderTest() {}
-
-  void SetUp() override {
-    pos_matcher_.Set(mock_data_manager_.GetPosMatcherData());
-  }
+  TextDictionaryLoaderTest()
+      : pos_matcher_(mock_data_manager_.GetPosMatcherData()),
+        temp_dir_(testing::MakeTempDirectoryOrDie()) {}
 
   std::unique_ptr<TextDictionaryLoader> CreateTextDictionaryLoader() {
     return std::make_unique<TextDictionaryLoader>(pos_matcher_);
   }
 
+  testing::MockDataManager mock_data_manager_;
   PosMatcher pos_matcher_;
-
- private:
-  const testing::MockDataManager mock_data_manager_;
+  TempDirectory temp_dir_;
 };
 
 TEST_F(TextDictionaryLoaderTest, BasicTest) {
@@ -86,8 +80,7 @@ TEST_F(TextDictionaryLoaderTest, BasicTest) {
     EXPECT_TRUE(tokens.empty());
   }
 
-  const std::string filename =
-      FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir), "test.tsv");
+  const std::string filename = FileUtil::JoinPath(temp_dir_.path(), "test.tsv");
   ASSERT_OK(FileUtil::SetContents(filename, kTextLines));
 
   {
@@ -95,25 +88,25 @@ TEST_F(TextDictionaryLoaderTest, BasicTest) {
     loader->Load(filename, "");
     const std::vector<std::unique_ptr<Token>> &tokens = loader->tokens();
 
-    EXPECT_EQ(3, tokens.size());
+    EXPECT_EQ(tokens.size(), 3);
 
-    EXPECT_EQ("key_test1", tokens[0]->key);
-    EXPECT_EQ("value_test1", tokens[0]->value);
-    EXPECT_EQ(0, tokens[0]->lid);
-    EXPECT_EQ(0, tokens[0]->rid);
-    EXPECT_EQ(1, tokens[0]->cost);
+    EXPECT_EQ(tokens[0]->key, "key_test1");
+    EXPECT_EQ(tokens[0]->value, "value_test1");
+    EXPECT_EQ(tokens[0]->lid, 0);
+    EXPECT_EQ(tokens[0]->rid, 0);
+    EXPECT_EQ(tokens[0]->cost, 1);
 
-    EXPECT_EQ("foo", tokens[1]->key);
-    EXPECT_EQ("bar", tokens[1]->value);
-    EXPECT_EQ(1, tokens[1]->lid);
-    EXPECT_EQ(2, tokens[1]->rid);
-    EXPECT_EQ(3, tokens[1]->cost);
+    EXPECT_EQ(tokens[1]->key, "foo");
+    EXPECT_EQ(tokens[1]->value, "bar");
+    EXPECT_EQ(tokens[1]->lid, 1);
+    EXPECT_EQ(tokens[1]->rid, 2);
+    EXPECT_EQ(tokens[1]->cost, 3);
 
-    EXPECT_EQ("buz", tokens[2]->key);
-    EXPECT_EQ("foobar", tokens[2]->value);
-    EXPECT_EQ(10, tokens[2]->lid);
-    EXPECT_EQ(20, tokens[2]->rid);
-    EXPECT_EQ(30, tokens[2]->cost);
+    EXPECT_EQ(tokens[2]->key, "buz");
+    EXPECT_EQ(tokens[2]->value, "foobar");
+    EXPECT_EQ(tokens[2]->lid, 10);
+    EXPECT_EQ(tokens[2]->rid, 20);
+    EXPECT_EQ(tokens[2]->cost, 30);
 
     loader->Clear();
     EXPECT_TRUE(loader->tokens().empty());
@@ -124,19 +117,19 @@ TEST_F(TextDictionaryLoaderTest, BasicTest) {
     loader->LoadWithLineLimit(filename, "", 2);
     const std::vector<std::unique_ptr<Token>> &tokens = loader->tokens();
 
-    EXPECT_EQ(2, tokens.size());
+    EXPECT_EQ(tokens.size(), 2);
 
-    EXPECT_EQ("key_test1", tokens[0]->key);
-    EXPECT_EQ("value_test1", tokens[0]->value);
-    EXPECT_EQ(0, tokens[0]->lid);
-    EXPECT_EQ(0, tokens[0]->rid);
-    EXPECT_EQ(1, tokens[0]->cost);
+    EXPECT_EQ(tokens[0]->key, "key_test1");
+    EXPECT_EQ(tokens[0]->value, "value_test1");
+    EXPECT_EQ(tokens[0]->lid, 0);
+    EXPECT_EQ(tokens[0]->rid, 0);
+    EXPECT_EQ(tokens[0]->cost, 1);
 
-    EXPECT_EQ("foo", tokens[1]->key);
-    EXPECT_EQ("bar", tokens[1]->value);
-    EXPECT_EQ(1, tokens[1]->lid);
-    EXPECT_EQ(2, tokens[1]->rid);
-    EXPECT_EQ(3, tokens[1]->cost);
+    EXPECT_EQ(tokens[1]->key, "foo");
+    EXPECT_EQ(tokens[1]->value, "bar");
+    EXPECT_EQ(tokens[1]->lid, 1);
+    EXPECT_EQ(tokens[1]->rid, 2);
+    EXPECT_EQ(tokens[1]->cost, 3);
 
     loader->Clear();
     EXPECT_TRUE(loader->tokens().empty());
@@ -148,7 +141,7 @@ TEST_F(TextDictionaryLoaderTest, BasicTest) {
     loader->Load(filename, "");
     loader->Load(filename, "");
     const std::vector<std::unique_ptr<Token>> &tokens = loader->tokens();
-    EXPECT_EQ(3, tokens.size());
+    EXPECT_EQ(tokens.size(), 3);
   }
 
   EXPECT_OK(FileUtil::Unlink(filename));
@@ -161,9 +154,9 @@ TEST_F(TextDictionaryLoaderTest, RewriteSpecialTokenTest) {
     token.lid = 100;
     token.rid = 200;
     EXPECT_TRUE(loader->RewriteSpecialToken(&token, ""));
-    EXPECT_EQ(100, token.lid);
-    EXPECT_EQ(200, token.rid);
-    EXPECT_EQ(Token::NONE, token.attributes);
+    EXPECT_EQ(token.lid, 100);
+    EXPECT_EQ(token.rid, 200);
+    EXPECT_EQ(token.attributes, Token::NONE);
   }
 
   {
@@ -171,9 +164,9 @@ TEST_F(TextDictionaryLoaderTest, RewriteSpecialTokenTest) {
     token.lid = 100;
     token.rid = 200;
     EXPECT_TRUE(loader->RewriteSpecialToken(&token, "SPELLING_CORRECTION"));
-    EXPECT_EQ(100, token.lid);
-    EXPECT_EQ(200, token.rid);
-    EXPECT_EQ(Token::SPELLING_CORRECTION, token.attributes);
+    EXPECT_EQ(token.lid, 100);
+    EXPECT_EQ(token.rid, 200);
+    EXPECT_EQ(token.attributes, Token::SPELLING_CORRECTION);
   }
 
   {
@@ -181,9 +174,9 @@ TEST_F(TextDictionaryLoaderTest, RewriteSpecialTokenTest) {
     token.lid = 100;
     token.rid = 200;
     EXPECT_TRUE(loader->RewriteSpecialToken(&token, "ZIP_CODE"));
-    EXPECT_EQ(pos_matcher_.GetZipcodeId(), token.lid);
-    EXPECT_EQ(pos_matcher_.GetZipcodeId(), token.rid);
-    EXPECT_EQ(Token::NONE, token.attributes);
+    EXPECT_EQ(token.lid, pos_matcher_.GetZipcodeId());
+    EXPECT_EQ(token.rid, pos_matcher_.GetZipcodeId());
+    EXPECT_EQ(token.attributes, Token::NONE);
   }
 
   {
@@ -191,9 +184,9 @@ TEST_F(TextDictionaryLoaderTest, RewriteSpecialTokenTest) {
     token.lid = 100;
     token.rid = 200;
     EXPECT_TRUE(loader->RewriteSpecialToken(&token, "ENGLISH:RATED"));
-    EXPECT_EQ(pos_matcher_.GetIsolatedWordId(), token.lid);
-    EXPECT_EQ(pos_matcher_.GetIsolatedWordId(), token.rid);
-    EXPECT_EQ(Token::NONE, token.attributes);
+    EXPECT_EQ(token.lid, pos_matcher_.GetIsolatedWordId());
+    EXPECT_EQ(token.rid, pos_matcher_.GetIsolatedWordId());
+    EXPECT_EQ(token.attributes, Token::NONE);
   }
 
   {
@@ -201,17 +194,17 @@ TEST_F(TextDictionaryLoaderTest, RewriteSpecialTokenTest) {
     token.lid = 100;
     token.rid = 200;
     EXPECT_FALSE(loader->RewriteSpecialToken(&token, "foo"));
-    EXPECT_EQ(100, token.lid);
-    EXPECT_EQ(200, token.rid);
-    EXPECT_EQ(Token::NONE, token.attributes);
+    EXPECT_EQ(token.lid, 100);
+    EXPECT_EQ(token.rid, 200);
+    EXPECT_EQ(token.attributes, Token::NONE);
   }
 }
 
 TEST_F(TextDictionaryLoaderTest, LoadMultipleFilesTest) {
   const std::string filename1 =
-      FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir), "test1.tsv");
+      FileUtil::JoinPath(temp_dir_.path(), "test1.tsv");
   const std::string filename2 =
-      FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir), "test2.tsv");
+      FileUtil::JoinPath(temp_dir_.path(), "test2.tsv");
   const std::string filename = filename1 + "," + filename2;
 
   ASSERT_OK(FileUtil::SetContents(filename1, kTextLines));
@@ -222,7 +215,7 @@ TEST_F(TextDictionaryLoaderTest, LoadMultipleFilesTest) {
   {
     std::unique_ptr<TextDictionaryLoader> loader = CreateTextDictionaryLoader();
     loader->Load(filename, "");
-    EXPECT_EQ(6, loader->tokens().size());
+    EXPECT_EQ(loader->tokens().size(), 6);
   }
 }
 
@@ -230,9 +223,9 @@ TEST_F(TextDictionaryLoaderTest, ReadingCorrectionTest) {
   std::unique_ptr<TextDictionaryLoader> loader = CreateTextDictionaryLoader();
 
   const std::string dic_filename =
-      FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir), "test.tsv");
-  const std::string reading_correction_filename = FileUtil::JoinPath(
-      absl::GetFlag(FLAGS_test_tmpdir), "reading_correction.tsv");
+      FileUtil::JoinPath(temp_dir_.path(), "test.tsv");
+  const std::string reading_correction_filename =
+      FileUtil::JoinPath(temp_dir_.path(), "reading_correction.tsv");
 
   ASSERT_OK(FileUtil::SetContents(dic_filename, kTextLines));
   FileUnlinker dic_unlinker(dic_filename);
@@ -243,11 +236,11 @@ TEST_F(TextDictionaryLoaderTest, ReadingCorrectionTest) {
   loader->Load(dic_filename, reading_correction_filename);
   const std::vector<std::unique_ptr<Token>> &tokens = loader->tokens();
   ASSERT_EQ(tokens.size(), 4);
-  EXPECT_EQ("foobar_error", tokens[3]->key);
-  EXPECT_EQ("foobar", tokens[3]->value);
-  EXPECT_EQ(10, tokens[3]->lid);
-  EXPECT_EQ(20, tokens[3]->rid);
-  EXPECT_EQ(30 + 2302, tokens[3]->cost);
+  EXPECT_EQ(tokens[3]->key, "foobar_error");
+  EXPECT_EQ(tokens[3]->value, "foobar");
+  EXPECT_EQ(tokens[3]->lid, 10);
+  EXPECT_EQ(tokens[3]->rid, 20);
+  EXPECT_EQ(tokens[3]->cost, 30 + 2302);
 }
 
 }  // namespace dictionary
