@@ -29,21 +29,35 @@
 
 #include "engine/engine_builder.h"
 
-#include <filesystem>
+#include <atomic>
+#include <cstdint>
 #include <memory>
+#include <optional>
+#include <string>
 #include <utility>
 
 #include "base/file_util.h"
+#include "base/hash.h"
 #include "base/logging.h"
+#include "base/protobuf/message.h"
 #include "base/thread.h"
 #include "data_manager/data_manager.h"
 #include "engine/engine.h"
+#include "engine/engine_interface.h"
 #include "protocol/engine_builder.pb.h"
 #include "absl/status/status.h"
+#include "absl/status/statusor.h"
 
 namespace mozc {
 namespace {
 
+uint64_t GetRequestHash(const EngineReloadRequest &request) {
+  if (request.file_path().empty() || request.install_location().empty()) {
+    return 0;
+  }
+  return Fingerprint(request.SerializeAsString());
+}
+
 EngineReloadResponse::Status ConvertStatus(DataManager::Status status) {
   switch (status) {
     case DataManager::Status::ENGINE_VERSION_MISMATCH:
@@ -63,108 +77,106 @@ EngineReloadResponse::Status ConvertStatus(DataManager::Status status) {
 }
 }  // namespace
 
-class EngineBuilder::Preparator : public Thread {
- public:
-  explicit Preparator(const EngineReloadRequest &request) {
-    *response_.mutable_request() = request;
+EngineBuilder::~EngineBuilder() { Clear(); }
+
+void EngineBuilder::PrepareAsync(const EngineReloadRequest &request,
+                                 EngineReloadResponse *response) {
+  *response->mutable_request() = request;
+
+  // Skips the sync when loading the model with the same path.
+  const auto model_path_fp = model_path_fp_.load();
+  if (model_path_fp != 0 && model_path_fp == GetRequestHash(request)) {
+    LOG(INFO)
+        << "PrepareAsync is skipped because the same model is already loaded.";
+    response->set_status(EngineReloadResponse::RELOADED);
+    return;
   }
 
-  ~Preparator() override = default;
-
-  void Run() override {
-    const EngineReloadRequest &request = response_.request();
-
-    auto tmp_data_manager = std::make_unique<DataManager>();
-    const DataManager::Status status =
-        InitDataManager(request, tmp_data_manager.get());
-    if (status != DataManager::Status::OK) {
-      LOG(ERROR) << "Failed to load data [" << status << "] "
-                 << request.Utf8DebugString();
-      response_.set_status(ConvertStatus(status));
+  if (prepare_.has_value()) {
+    if (!prepare_->Ready()) {
+      response->set_status(EngineReloadResponse::ALREADY_RUNNING);
       return;
     }
+    VLOG(1) << "Previously loaded data is discarded";
+  }
+
+  prepare_.emplace([request]() -> Prepared {
+    EngineReloadResponse response;
+    *response.mutable_request() = request;
+
+    auto init_data_manager = [](const EngineReloadRequest &request,
+                                DataManager *data_manager) {
+      if (request.has_magic_number()) {
+        return data_manager->InitFromFile(request.file_path(),
+                                          request.magic_number());
+      }
+      return data_manager->InitFromFile(request.file_path());
+    };
+
+    auto data_manager = std::make_unique<DataManager>();
+    if (DataManager::Status status =
+            init_data_manager(request, data_manager.get());
+        status != DataManager::Status::OK) {
+      LOG(ERROR) << "Failed to load data [" << status << "] "
+                 << protobuf::Utf8Format(request);
+      response.set_status(ConvertStatus(status));
+      return {std::move(response), std::move(data_manager)};
+    }
 
     if (request.has_install_location()) {
       if (absl::Status s = FileUtil::LinkOrCopyFile(request.file_path(),
                                                     request.install_location());
           !s.ok()) {
-        LOG(ERROR) << "Copy faild: " << request.Utf8DebugString() << ": " << s;
-        response_.set_status(EngineReloadResponse::INSTALL_FAILURE);
-        return;
+        LOG(ERROR) << "Copy faild: " << protobuf::Utf8Format(request) << ": "
+                   << s;
+        response.set_status(EngineReloadResponse::INSTALL_FAILURE);
+        return {
+            std::move(response),
+            std::move(data_manager),
+        };
       }
     }
 
-    response_.set_status(EngineReloadResponse::RELOAD_READY);
-    data_manager_ = std::move(tmp_data_manager);
-  }
-
- private:
-  DataManager::Status InitDataManager(const EngineReloadRequest &request,
-                                      DataManager *data_manager) {
-    if (request.has_magic_number()) {
-      return data_manager->InitFromFile(request.file_path(),
-                                        request.magic_number());
-    }
-    return data_manager->InitFromFile(request.file_path());
-  }
-
-  friend class EngineBuilder;
-  EngineReloadResponse response_;
-  std::unique_ptr<DataManager> data_manager_;
-};
-
-EngineBuilder::EngineBuilder() = default;
-EngineBuilder::~EngineBuilder() = default;
-
-void EngineBuilder::PrepareAsync(const EngineReloadRequest &request,
-                                 EngineReloadResponse *response) {
-  *response->mutable_request() = request;
-  if (preparator_) {
-    if (preparator_->IsRunning()) {
-      response->set_status(EngineReloadResponse::ALREADY_RUNNING);
-      return;
-    }
-    preparator_->Join();
-    VLOG(1) << "Previously loaded data is discarded";
-  }
-  preparator_ = std::make_unique<Preparator>(request);
-  preparator_->SetJoinable(true);
-  preparator_->Start("EngineBuilder::Preparator");
+    response.set_status(EngineReloadResponse::RELOAD_READY);
+    return {std::move(response), std::move(data_manager)};
+  });
   response->set_status(EngineReloadResponse::ACCEPTED);
 }
 
 void EngineBuilder::Wait() {
-  if (preparator_) {
-    preparator_->Join();
+  if (prepare_.has_value()) {
+    prepare_->Wait();
   }
 }
 
 bool EngineBuilder::HasResponse() const {
-  return preparator_ && !preparator_->IsRunning();
+  return prepare_.has_value() && prepare_->Ready();
 }
 
 void EngineBuilder::GetResponse(EngineReloadResponse *response) const {
   if (!HasResponse()) {
     return;
   }
-  *response = preparator_->response_;
+  *response = prepare_->Get().response;
 }
 
 std::unique_ptr<EngineInterface> EngineBuilder::BuildFromPreparedData() {
-  if (!HasResponse() || !preparator_->data_manager_ ||
-      preparator_->response_.status() != EngineReloadResponse::RELOAD_READY) {
+  if (!HasResponse() || prepare_->Get().data_manager == nullptr ||
+      prepare_->Get().response.status() != EngineReloadResponse::RELOAD_READY) {
     LOG(ERROR) << "Build() is called in invalid state";
     return nullptr;
   }
+  // operator-> doesn't support rvalue delegation :/
+  Prepared prepared = (*std::move(prepare_)).Get();
+  prepare_.reset();
+
   absl::StatusOr<std::unique_ptr<Engine>> engine;
-  switch (preparator_->response_.request().engine_type()) {
+  switch (prepared.response.request().engine_type()) {
     case EngineReloadRequest::DESKTOP:
-      engine =
-          Engine::CreateDesktopEngine(std::move(preparator_->data_manager_));
+      engine = Engine::CreateDesktopEngine(std::move(prepared.data_manager));
       break;
     case EngineReloadRequest::MOBILE:
-      engine =
-          Engine::CreateMobileEngine(std::move(preparator_->data_manager_));
+      engine = Engine::CreateMobileEngine(std::move(prepared.data_manager));
       break;
     default:
       LOG(DFATAL) << "Should not reach here";
@@ -175,15 +187,18 @@ std::unique_ptr<EngineInterface> EngineBuilder::BuildFromPreparedData() {
     LOG(ERROR) << engine.status();
     return nullptr;
   }
+
+  model_path_fp_.store(GetRequestHash(prepared.response.request()));
+
   return *std::move(engine);
 }
 
 void EngineBuilder::Clear() {
-  if (!preparator_) {
+  if (!prepare_.has_value()) {
     return;
   }
-  preparator_->Join();
-  preparator_.reset();
+  prepare_->Wait();
+  prepare_.reset();
 }
 
 }  // namespace mozc
