@@ -30,16 +30,24 @@
 #ifndef MOZC_ENGINE_ENGINE_BUILDER_H_
 #define MOZC_ENGINE_ENGINE_BUILDER_H_
 
+#include <atomic>
+#include <cstdint>
 #include <memory>
+#include <optional>
 
-#include "base/port.h"
+#include "base/thread.h"
+#include "data_manager/data_manager.h"
 #include "engine/engine_builder_interface.h"
+#include "engine/engine_interface.h"
+#include "protocol/engine_builder.pb.h"
 
 namespace mozc {
 
 class EngineBuilder : public EngineBuilderInterface {
  public:
-  EngineBuilder();
+  EngineBuilder() = default;
+  EngineBuilder(const EngineBuilder &) = delete;
+  EngineBuilder &operator=(const EngineBuilder &) = delete;
   ~EngineBuilder() override;
 
   // Implementation of EngineBuilderInterface.  PrepareAsync() is implemented
@@ -55,10 +63,13 @@ class EngineBuilder : public EngineBuilderInterface {
   void Wait();
 
  private:
-  class Preparator;
-  std::unique_ptr<Preparator> preparator_;
+  std::atomic<uint64_t> model_path_fp_ = 0;
 
-  DISALLOW_COPY_AND_ASSIGN(EngineBuilder);
+  struct Prepared {
+    EngineReloadResponse response;
+    std::unique_ptr<DataManager> data_manager;
+  };
+  std::optional<BackgroundFuture<Prepared>> prepare_;
 };
 
 }  // namespace mozc
