@@ -31,9 +31,16 @@
 #define MOZC_ENGINE_MINIMAL_ENGINE_H_
 
 #include <memory>
+#include <string>
+#include <vector>
 
-#include "base/port.h"
+#include "converter/converter_interface.h"
+#include "data_manager/data_manager_interface.h"
+#include "dictionary/suppression_dictionary.h"
 #include "engine/engine_interface.h"
+#include "engine/user_data_manager_interface.h"
+#include "prediction/predictor_interface.h"
+#include "absl/strings/string_view.h"
 
 namespace mozc {
 
@@ -42,12 +49,14 @@ namespace mozc {
 class MinimalEngine : public EngineInterface {
  public:
   MinimalEngine();
-  ~MinimalEngine() override;
+  MinimalEngine(const MinimalEngine &) = delete;
+  MinimalEngine &operator=(const MinimalEngine &) = delete;
 
   ConverterInterface *GetConverter() const override;
-  PredictorInterface *GetPredictor() const override;
+  prediction::PredictorInterface *GetPredictor() const override;
   dictionary::SuppressionDictionary *GetSuppressionDictionary() override;
   bool Reload() override { return true; }
+  bool ReloadAndWait() override { return true; }
   UserDataManagerInterface *GetUserDataManager() override;
   absl::string_view GetDataVersion() const override { return "0.0.0"; }
   const DataManagerInterface *GetDataManager() const override;
@@ -55,12 +64,10 @@ class MinimalEngine : public EngineInterface {
 
  private:
   std::unique_ptr<ConverterInterface> converter_;
-  std::unique_ptr<PredictorInterface> predictor_;
+  std::unique_ptr<prediction::PredictorInterface> predictor_;
   std::unique_ptr<dictionary::SuppressionDictionary> suppression_dictionary_;
   std::unique_ptr<UserDataManagerInterface> user_data_manager_;
   std::unique_ptr<const DataManagerInterface> data_manager_;
-
-  DISALLOW_COPY_AND_ASSIGN(MinimalEngine);
 };
 
 }  // namespace mozc
