@@ -1,4 +1,4 @@
-// Copyright 2010-2020, Google Inc.
+// Copyright 2010-2021, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -29,9 +29,10 @@
 
 #include "gui/base/encoding_util.h"
 
+#include <cstddef>
 #include <cstdint>
+#include <string>
 
-#include "base/port.h"
 #include "base/util.h"
 #include "absl/strings/string_view.h"
 
@@ -99,7 +100,7 @@ size_t ComputeIndex(uint8_t first, uint8_t second) {
   return first_index * width + second_index;
 }
 
-bool SJISToUTF8Internal(absl::string_view input, std::string *output) {
+bool SjisToUtf8Internal(absl::string_view input, std::string *output) {
   bool expect_first_byte = true;
   uint8_t first_byte = 0;
   for (const char c : input) {
@@ -107,9 +108,9 @@ bool SJISToUTF8Internal(absl::string_view input, std::string *output) {
 
     if (expect_first_byte) {
       if (IsInFirstByteRange1(byte)) {
-        Util::UCS4ToUTF8Append(byte, output);
+        Util::Ucs4ToUtf8Append(byte, output);
       } else if (IsInFirstByteRange3(byte)) {
-        Util::UCS4ToUTF8Append(byte + 0xFEC0, output);
+        Util::Ucs4ToUtf8Append(byte + 0xFEC0, output);
       } else if (IsInFirstByteRange2(byte) || IsInFirstByteRange4(byte)) {
         first_byte = byte;
         expect_first_byte = false;
@@ -130,7 +131,7 @@ bool SJISToUTF8Internal(absl::string_view input, std::string *output) {
     if (ucs2 == 0) {
       return false;
     }
-    Util::UCS4ToUTF8Append(ucs2, output);
+    Util::Ucs4ToUtf8Append(ucs2, output);
     expect_first_byte = true;
   }
   return expect_first_byte;
@@ -138,11 +139,12 @@ bool SJISToUTF8Internal(absl::string_view input, std::string *output) {
 
 }  // namespace
 
-void EncodingUtil::SJISToUTF8(const std::string &input, std::string *output) {
-  output->clear();
-  if (!SJISToUTF8Internal(input, output)) {
-    output->clear();
+std::string EncodingUtil::SjisToUtf8(absl::string_view input) {
+  std::string output;
+  if (!SjisToUtf8Internal(input, &output)) {
+    return {};
   }
+  return output;
 }
 
 }  // namespace mozc
