@@ -30,20 +30,19 @@
 #ifndef MOZC_GUI_CONFIG_DIALOG_KEYBINDING_EDITOR_DELEGATE_H_
 #define MOZC_GUI_CONFIG_DIALOG_KEYBINDING_EDITOR_DELEGATE_H_
 
-#include <QtCore/QModelIndex>
-#include <QtCore/QObject>
-#include <QtWidgets/QItemDelegate>
+#include <QItemDelegate>
+#include <QModelIndex>
+#include <QObject>
+
+#include "gui/config_dialog/keybinding_editor.h"
 
 namespace mozc {
 namespace gui {
 
-class KeyBindingEditor;
-
 class KeyBindingEditorDelegate : public QItemDelegate {
   Q_OBJECT
  public:
   explicit KeyBindingEditorDelegate(QObject *parent = nullptr);
-  ~KeyBindingEditorDelegate() override;
 
   QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option,
                         const QModelIndex &index) const override;
