@@ -30,41 +30,31 @@
 #ifndef MOZC_GUI_DICTIONARY_TOOL_DICTIONARY_TOOL_H_
 #define MOZC_GUI_DICTIONARY_TOOL_DICTIONARY_TOOL_H_
 
-#include <QtWidgets/QLabel>
-#include <QtWidgets/QPushButton>
-#include <QtWidgets/QSplitter>
-#include <QtWidgets/QSplitterHandle>
+#include <QLabel>
+#include <QPushButton>
+#include <QSplitter>
+#include <QSplitterHandle>
+#include <cstdint>
 #include <memory>
-#include <string>
 #include <vector>
 
-#include "base/port.h"
 #include "dictionary/user_dictionary_importer.h"
+#include "dictionary/user_dictionary_session.h"
+#include "dictionary/user_pos_interface.h"
+#include "absl/strings/string_view.h"
+#include "client/client_interface.h"
+#include "gui/dictionary_tool/find_dialog.h"
+#include "gui/dictionary_tool/import_dialog.h"
 #include "gui/dictionary_tool/ui_dictionary_tool.h"
 
 namespace mozc {
-
-class POSListProviderInterface;
-
-namespace client {
-class ClientInterface;
-}  // namespace client
-
-namespace user_dictionary {
-class UserDictionarySession;
-}  // namespace user_dictionary
-
 namespace gui {
 
-class ImportDialog;
-class FindDialog;
-
 class DictionaryTool : public QMainWindow, private Ui::DictionaryTool {
   Q_OBJECT
 
  public:
   explicit DictionaryTool(QWidget *parent = nullptr);
-  ~DictionaryTool() override;
 
   // return true DictionaryTool is available.
   bool IsAvailable() const { return is_available_; }
@@ -105,7 +95,7 @@ class DictionaryTool : public QMainWindow, private Ui::DictionaryTool {
   // Data type to provide information on a dictionary.
   struct DictionaryInfo {
     int row;                // Row in the list widget.
-    uint64 id;              // ID of the dictionary.
+    uint64_t id;            // ID of the dictionary.
     QListWidgetItem *item;  // Item object for the dictionary.
   };
 
@@ -144,11 +134,10 @@ class DictionaryTool : public QMainWindow, private Ui::DictionaryTool {
   // Show a special dialog message according to the result
   // of UserDictionaryImporter.
   void ReportImportError(UserDictionaryImporter::ErrorType error,
-                         const std::string &dic_name, int added_entries_size);
+                         const QString &dic_name, int added_entries_size);
 
-  void ImportHelper(uint64 dic_id, const std::string &dic_name,
-                    const std::string &file_name,
-                    UserDictionaryImporter::IMEType,
+  void ImportHelper(uint64_t dic_id, const QString &dic_name,
+                    const QString &file_name, UserDictionaryImporter::IMEType,
                     UserDictionaryImporter::EncodingType encoding_type);
 
   // Save storage contents into the disk and
@@ -160,15 +149,15 @@ class DictionaryTool : public QMainWindow, private Ui::DictionaryTool {
   void EditComment();
 
   // Changes the POS of all selected items to |pos|.
-  void EditPOS(const std::string &pos);
+  void EditPos(absl::string_view pos);
 
   // Moves selected items to the dictionary whose row is |dictionary_row|.
   void MoveTo(int dictionary_row);
 
   // Helper functions to check if a file with given name is readable
   // to import or writable to export without trying to open it.
-  static bool IsWritableToExport(const std::string &file_name);
-  static bool IsReadableToImport(const std::string &file_name);
+  static bool IsWritableToExport(const QString &file_name);
+  static bool IsReadableToImport(const QString &file_name);
 
   // Helper function for DeleteWord and MoveTo.
   // Fills selected word entry rows as a unique sorted sequence.
@@ -186,27 +175,7 @@ class DictionaryTool : public QMainWindow, private Ui::DictionaryTool {
   // separate from selection on the list widget because data is saved
   // on the previous dictionary after the selection on the widget is
   // changed. It takes -1 when no dictionary is selected.
-  uint64 current_dic_id_;
-
-  // Whether any change has been made on the current dictionary and
-  // not been saved.
-  bool modified_;
-
-  // Holds information on whether dictionary entries are sorted, key
-  // column of sort and order of sort.
-  //
-  // Current implementation of sort may not be perfect. It doesn't
-  // check if entries are already sorted when they are loaded nor
-  // whether modification is made keeping sorted entries sorted.
-  struct SortState {
-    bool sorted;
-    int column;
-    Qt::SortOrder order;
-  } sort_state_;
-
-  // See comment above StartMonitoringUserEdit() for role of the
-  // variable.
-  bool monitoring_user_edit_;
+  uint64_t current_dic_id_;
 
   // POS shown in the combo box by default.
   QString default_pos_;
@@ -238,12 +207,32 @@ class DictionaryTool : public QMainWindow, private Ui::DictionaryTool {
 
   std::unique_ptr<client::ClientInterface> client_;
 
-  bool is_available_;
+  // Holds information on whether dictionary entries are sorted, key
+  // column of sort and order of sort.
+  //
+  // Current implementation of sort may not be perfect. It doesn't
+  // check if entries are already sorted when they are loaded nor
+  // whether modification is made keeping sorted entries sorted.
+  struct SortState {
+    bool sorted;
+    int column;
+    Qt::SortOrder order;
+  } sort_state_;
 
   // The maximum number of entries for a dictionary currently selected.
   int max_entry_size_;
 
-  std::unique_ptr<const POSListProviderInterface> pos_list_provider_;
+  std::unique_ptr<const PosListProviderInterface> pos_list_provider_;
+
+  // Whether any change has been made on the current dictionary and
+  // not been saved.
+  bool modified_;
+
+  // See comment above StartMonitoringUserEdit() for role of the
+  // variable.
+  bool monitoring_user_edit_;
+
+  bool is_available_;
 };
 
 }  // namespace gui
