@@ -27,32 +27,40 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-// skip all unless OS_WIN
-#ifdef OS_WIN
+// skip all unless _WIN32
+#ifdef _WIN32
 
-#include <Sddl.h>
-#include <Windows.h>
+// clang-format off
+#include <windows.h>
+#include <sddl.h>  // needs to be after windows.h
+// clang-format on
+#include <wil/resource.h>
 
 #include <algorithm>
+#include <cstdint>
 #include <string>
+#include <utility>
 
 #include "base/const.h"
 #include "base/cpu_stats.h"
 #include "base/logging.h"
-#include "base/scoped_handle.h"
 #include "base/singleton.h"
 #include "base/system_util.h"
-#include "base/thread.h"
 #include "base/util.h"
-#include "base/win_sandbox.h"
-#include "base/win_util.h"
+#include "base/win32/wide_char.h"
+#include "base/win32/win_sandbox.h"
+#include "base/win32/win_util.h"
 #include "ipc/ipc.h"
 #include "ipc/ipc_path_manager.h"
 #include "absl/strings/match.h"
+#include "absl/strings/string_view.h"
+#include "absl/time/time.h"
 
 namespace mozc {
 namespace {
 
+using win32::Utf8ToWide;
+
 constexpr bool kReadTypeACK = true;
 constexpr bool kReadTypeData = false;
 constexpr bool kSendTypeData = false;
@@ -92,7 +100,7 @@ bool InitOverlapped(OVERLAPPED *overlapped, HANDLE wait_handle) {
 
 class IPCClientMutexBase {
  public:
-  explicit IPCClientMutexBase(const std::string &ipc_channel_name) {
+  explicit IPCClientMutexBase(const absl::string_view ipc_channel_name) {
     // Make a kernel mutex object so that multiple ipc connections are
     // serialized here. In Windows, there is no useful way to serialize
     // the multiple connections to the single-thread named pipe server.
@@ -102,13 +110,10 @@ class IPCClientMutexBase {
     // thread. The "available" notification is sent to all waiting ipc
     // clients at the same time and only one client gets the connection.
     // This causes redundant and wasteful CreateFile calles.
-    std::string mutex_name = kMutexPathPrefix;
-    mutex_name += SystemUtil::GetUserSidAsString();
-    mutex_name += ".";
-    mutex_name += ipc_channel_name;
-    mutex_name += ".ipc";
-    std::wstring wmutex_name;
-    Util::Utf8ToWide(mutex_name, &wmutex_name);
+    std::string mutex_name =
+        absl::StrCat(kMutexPathPrefix, SystemUtil::GetUserSidAsString(), ".",
+                     ipc_channel_name, ".ipc");
+    std::wstring wmutex_name = Utf8ToWide(mutex_name);
 
     LPSECURITY_ATTRIBUTES security_attributes_ptr = nullptr;
     SECURITY_ATTRIBUTES security_attributes;
@@ -142,85 +147,63 @@ class IPCClientMutexBase {
     }
   }
 
-  virtual ~IPCClientMutexBase() {}
+  virtual ~IPCClientMutexBase() = default;
 
-  HANDLE get() const { return ipc_mutex_.get(); }
+  const wil::unique_mutex_nothrow &mutex() const { return ipc_mutex_; }
 
  private:
-  ScopedHandle ipc_mutex_;
+  wil::unique_mutex_nothrow ipc_mutex_;
 };
 
 class ConverterClientMutex : public IPCClientMutexBase {
  public:
   ConverterClientMutex() : IPCClientMutexBase("converter") {}
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(ConverterClientMutex);
+  ConverterClientMutex(const ConverterClientMutex &) = delete;
+  ConverterClientMutex &operator=(const ConverterClientMutex &) = delete;
 };
 
 class RendererClientMutex : public IPCClientMutexBase {
  public:
   RendererClientMutex() : IPCClientMutexBase("renderer") {}
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(RendererClientMutex);
+  RendererClientMutex(const RendererClientMutex &) = delete;
+  RendererClientMutex &operator=(const RendererClientMutex &) = delete;
 };
 
 class FallbackClientMutex : public IPCClientMutexBase {
  public:
   FallbackClientMutex() : IPCClientMutexBase("fallback") {}
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(FallbackClientMutex);
+  FallbackClientMutex(const FallbackClientMutex &) = delete;
+  FallbackClientMutex &operator=(const FallbackClientMutex &) = delete;
 };
 
 // In Mozc client, we should support different IPC channels (client-converter
 // and client-renderer) so we need to have different global mutexes to
 // serialize each client. Currently |ipc_name| starts with "session" and
 // "renderer" are expected.
-HANDLE GetClientMutex(const std::string &ipc_name) {
+const wil::unique_mutex_nothrow &GetClientMutex(
+    const absl::string_view ipc_name) {
   if (absl::StartsWith(ipc_name, "session")) {
-    return Singleton<ConverterClientMutex>::get()->get();
+    return Singleton<ConverterClientMutex>::get()->mutex();
   }
   if (absl::StartsWith(ipc_name, "renderer")) {
-    return Singleton<RendererClientMutex>::get()->get();
+    return Singleton<RendererClientMutex>::get()->mutex();
   }
   LOG(WARNING) << "unexpected IPC name: " << ipc_name;
-  return Singleton<FallbackClientMutex>::get()->get();
+  return Singleton<FallbackClientMutex>::get()->mutex();
 }
 
-// RAII class for calling ReleaseMutex in destructor.
-class ScopedReleaseMutex {
- public:
-  explicit ScopedReleaseMutex(HANDLE handle) : pipe_handle_(handle) {}
-
-  virtual ~ScopedReleaseMutex() {
-    if (nullptr != pipe_handle_) {
-      ::ReleaseMutex(pipe_handle_);
-    }
-    pipe_handle_ = nullptr;
-  }
-
-  HANDLE get() const { return pipe_handle_; }
-
- private:
-  HANDLE pipe_handle_;
-
-  DISALLOW_IMPLICIT_CONSTRUCTORS(ScopedReleaseMutex);
-};
-
-uint32 GetServerProcessIdImpl(HANDLE handle) {
+uint32_t GetServerProcessIdImpl(HANDLE handle) {
   ULONG pid = 0;
   if (::GetNamedPipeServerProcessId(handle, &pid) == 0) {
     const DWORD get_named_pipe_server_process_id_error = ::GetLastError();
     LOG(ERROR) << "GetNamedPipeServerProcessId failed: "
                << get_named_pipe_server_process_id_error;
-    return static_cast<uint32>(-1);  // always deny the connection
+    return static_cast<uint32_t>(-1);  // always deny the connection
   }
 
   VLOG(1) << "Got server ProcessID: " << pid;
 
-  return static_cast<uint32>(pid);
+  return static_cast<uint32_t>(pid);
 }
 
 void SafeCancelIO(HANDLE device_handle, OVERLAPPED *overlapped) {
@@ -235,12 +218,12 @@ void SafeCancelIO(HANDLE device_handle, OVERLAPPED *overlapped) {
   ::WaitForSingleObject(GetEventHandleFromOverlapped(overlapped), INFINITE);
 }
 
-bool WaitForQuitOrIOImpl(HANDLE device_handle, HANDLE quit_event, DWORD timeout,
-                         OVERLAPPED *overlapped, IPCErrorType *last_ipc_error) {
+IPCErrorType WaitForQuitOrIOImpl(HANDLE device_handle, HANDLE quit_event,
+                                 DWORD timeout, OVERLAPPED *overlapped) {
   const HANDLE events[] = {quit_event,
                            GetEventHandleFromOverlapped(overlapped)};
   const DWORD wait_result =
-      ::WaitForMultipleObjects(ARRAYSIZE(events), events, FALSE, timeout);
+      ::WaitForMultipleObjects(std::size(events), events, FALSE, timeout);
   const DWORD wait_error = ::GetLastError();
   // Clear the I/O operation if still exists.
   if (!HasOverlappedIoCompleted(overlapped)) {
@@ -250,25 +233,22 @@ bool WaitForQuitOrIOImpl(HANDLE device_handle, HANDLE quit_event, DWORD timeout,
   }
   if (wait_result == WAIT_TIMEOUT) {
     LOG(WARNING) << "Timeout: " << timeout;
-    *last_ipc_error = IPC_TIMEOUT_ERROR;
-    return false;
+    return IPC_TIMEOUT_ERROR;
   }
   if (wait_result == WAIT_OBJECT_0) {
     // Should be quit immediately
-    *last_ipc_error = IPC_QUIT_EVENT_SIGNALED;
-    return false;
+    return IPC_QUIT_EVENT_SIGNALED;
   }
   if (wait_result != (WAIT_OBJECT_0 + 1)) {
     LOG(WARNING) << "Unknown result: " << wait_result
                  << ", Error: " << wait_error;
-    *last_ipc_error = IPC_UNKNOWN_ERROR;
-    return false;
+    return IPC_UNKNOWN_ERROR;
   }
-  return true;
+  return IPC_NO_ERROR;
 }
 
-bool WaitForIOImpl(HANDLE device_handle, DWORD timeout, OVERLAPPED *overlapped,
-                   IPCErrorType *last_ipc_error) {
+IPCErrorType WaitForIOImpl(HANDLE device_handle, DWORD timeout,
+                           OVERLAPPED *overlapped) {
   const DWORD wait_result =
       ::WaitForSingleObject(GetEventHandleFromOverlapped(overlapped), timeout);
   // Clear the I/O operation if still exists.
@@ -279,38 +259,34 @@ bool WaitForIOImpl(HANDLE device_handle, DWORD timeout, OVERLAPPED *overlapped,
   }
   if (wait_result == WAIT_TIMEOUT) {
     LOG(WARNING) << "Timeout: " << timeout;
-    *last_ipc_error = IPC_TIMEOUT_ERROR;
-    return false;
+    return IPC_TIMEOUT_ERROR;
   }
   if (wait_result != WAIT_OBJECT_0) {
     LOG(WARNING) << "Unknown result: " << wait_result;
-    *last_ipc_error = IPC_UNKNOWN_ERROR;
-    return false;
+    return IPC_UNKNOWN_ERROR;
   }
-  return true;
+  return IPC_NO_ERROR;
 }
 
-bool WaitForQuitOrIO(HANDLE device_handle, HANDLE quit_event, DWORD timeout,
-                     OVERLAPPED *overlapped, IPCErrorType *last_ipc_error) {
+IPCErrorType WaitForQuitOrIO(HANDLE device_handle, HANDLE quit_event,
+                             DWORD timeout, OVERLAPPED *overlapped) {
   if (quit_event != nullptr) {
-    return WaitForQuitOrIOImpl(device_handle, quit_event, timeout, overlapped,
-                               last_ipc_error);
+    return WaitForQuitOrIOImpl(device_handle, quit_event, timeout, overlapped);
   }
-  return WaitForIOImpl(device_handle, timeout, overlapped, last_ipc_error);
+  return WaitForIOImpl(device_handle, timeout, overlapped);
 }
 
 // To work around a bug of GetOverlappedResult in Vista
 // http://msdn.microsoft.com/en-us/library/dd371711.aspx
-bool SafeWaitOverlappedResult(HANDLE device_handle, HANDLE quit_event,
-                              DWORD timeout, OVERLAPPED *overlapped,
-                              DWORD *num_bytes_updated,
-                              IPCErrorType *last_ipc_error, bool wait_ack) {
+IPCErrorType SafeWaitOverlappedResult(HANDLE device_handle, HANDLE quit_event,
+                                      DWORD timeout, OVERLAPPED *overlapped,
+                                      DWORD *num_bytes_updated, bool wait_ack) {
   DCHECK(overlapped);
   DCHECK(num_bytes_updated);
-  DCHECK(last_ipc_error);
-  if (!WaitForQuitOrIO(device_handle, quit_event, timeout, overlapped,
-                       last_ipc_error)) {
-    return false;
+  const IPCErrorType result =
+      WaitForQuitOrIO(device_handle, quit_event, timeout, overlapped);
+  if (result != IPC_NO_ERROR) {
+    return result;
   }
 
   *num_bytes_updated = 0;
@@ -321,36 +297,34 @@ bool SafeWaitOverlappedResult(HANDLE device_handle, HANDLE quit_event,
     if (get_overlapped_error == ERROR_BROKEN_PIPE) {
       if (wait_ack) {
         // This is an expected behavior.
-        return true;
+        return IPC_NO_ERROR;
       }
       LOG(ERROR) << "GetOverlappedResult() failed: ERROR_BROKEN_PIPE";
+    } else if (get_overlapped_error == ERROR_MORE_DATA) {
+      return IPC_MORE_DATA;
     } else {
       LOG(ERROR) << "GetOverlappedResult() failed: " << get_overlapped_error;
     }
-    *last_ipc_error = IPC_UNKNOWN_ERROR;
-    return false;
+    return IPC_UNKNOWN_ERROR;
   }
-  return true;
+  return IPC_NO_ERROR;
 }
 
-bool SendIPCMessage(HANDLE device_handle, HANDLE write_wait_handle,
-                    const char *buf, size_t buf_length, int timeout,
-                    IPCErrorType *last_ipc_error) {
-  if (buf_length == 0) {
-    LOG(WARNING) << "buf length is 0";
-    *last_ipc_error = IPC_UNKNOWN_ERROR;
-    return false;
+IPCErrorType SendIpcMessage(HANDLE device_handle, HANDLE write_wait_handle,
+                            const std::string &msg, absl::Duration timeout) {
+  if (msg.empty()) {
+    LOG(WARNING) << "msg is empty.";
+    return IPC_UNKNOWN_ERROR;
   }
 
   DWORD num_bytes_written = 0;
   OVERLAPPED overlapped;
   if (!InitOverlapped(&overlapped, write_wait_handle)) {
-    *last_ipc_error = IPC_WRITE_ERROR;
-    return false;
+    return IPC_WRITE_ERROR;
   }
 
   const bool write_file_result =
-      (::WriteFile(device_handle, buf, static_cast<DWORD>(buf_length),
+      (::WriteFile(device_handle, msg.data(), static_cast<DWORD>(msg.size()),
                    &num_bytes_written, &overlapped) != FALSE);
   const DWORD write_file_error = ::GetLastError();
   if (write_file_result) {
@@ -358,79 +332,102 @@ bool SendIPCMessage(HANDLE device_handle, HANDLE write_wait_handle,
   } else {
     if (write_file_error != ERROR_IO_PENDING) {
       LOG(ERROR) << "WriteFile() failed: " << write_file_error;
-      *last_ipc_error = IPC_WRITE_ERROR;
-      return false;
+      return IPC_WRITE_ERROR;
     }
-    if (!SafeWaitOverlappedResult(device_handle, nullptr, timeout, &overlapped,
-                                  &num_bytes_written, last_ipc_error,
-                                  kSendTypeData)) {
-      return false;
+    const IPCErrorType result = SafeWaitOverlappedResult(
+        device_handle, nullptr, absl::ToInt64Milliseconds(timeout), &overlapped,
+        &num_bytes_written, kSendTypeData);
+    if (result != IPC_NO_ERROR) {
+      return result;
     }
   }
 
   // As we use message-type namedpipe, all the data should be written in one
   // shot. Otherwise, single message will be split into multiple packets.
-  if (num_bytes_written != buf_length) {
-    LOG(ERROR) << "Data truncated. buf_length: " << buf_length
+  if (num_bytes_written != msg.size()) {
+    LOG(ERROR) << "Data truncated. msg.size(): " << msg.size()
                << ", num_bytes_written: " << num_bytes_written;
-    *last_ipc_error = IPC_UNKNOWN_ERROR;
-    return false;
+    return IPC_UNKNOWN_ERROR;
   }
-  return true;
+  return IPC_NO_ERROR;
 }
 
-bool RecvIPCMessage(HANDLE device_handle, HANDLE read_wait_handle, char *buf,
-                    size_t *buf_length, int timeout, bool read_type_ack,
-                    IPCErrorType *last_ipc_error) {
-  if (*buf_length == 0) {
-    LOG(WARNING) << "buf length is 0";
-    *last_ipc_error = IPC_UNKNOWN_ERROR;
-    return false;
+IPCErrorType RecvIpcMessage(HANDLE device_handle, HANDLE read_wait_handle,
+                            std::string *msg, absl::Duration timeout,
+                            bool read_type_ack) {
+  if (!msg) {
+    LOG(WARNING) << "msg is nullptr.";
+    return IPC_UNKNOWN_ERROR;
   }
 
-  OVERLAPPED overlapped;
-  if (!InitOverlapped(&overlapped, read_wait_handle)) {
-    *last_ipc_error = IPC_READ_ERROR;
-    return false;
-  }
-
-  DWORD num_bytes_read = 0;
-  const bool read_file_result =
-      (::ReadFile(device_handle, buf, static_cast<DWORD>(*buf_length),
-                  &num_bytes_read, &overlapped) != FALSE);
-  const DWORD read_file_error = ::GetLastError();
-  if (read_file_result) {
-    // ::ReadFile is done as sync operation.
-  } else {
+  msg->clear();
+  DWORD num_bytes_read_total = 0;
+  while (true) {
+    OVERLAPPED overlapped;
+    if (!InitOverlapped(&overlapped, read_wait_handle)) {
+      msg->clear();
+      return IPC_READ_ERROR;
+    }
+    if (num_bytes_read_total == 0) {
+      msg->resize(IPC_INITIAL_READ_BUFFER_SIZE);
+    } else {
+      msg->resize(msg->size() * 2);
+    }
+    const DWORD num_bytes_writable = msg->size() - num_bytes_read_total;
+    DWORD num_bytes_read = 0;
+    const bool read_file_result =
+        (::ReadFile(device_handle, msg->data() + num_bytes_read_total,
+                    num_bytes_writable, &num_bytes_read, &overlapped) != FALSE);
+    const DWORD read_file_error = ::GetLastError();
+    if (read_file_result) {
+      // ::ReadFile is done as sync operation.
+      num_bytes_read_total += num_bytes_read;
+      break;
+    }
+    if (read_file_error == ERROR_MORE_DATA) {
+      // ::ReadFile is done as sync operation with pending data.
+      num_bytes_read_total += num_bytes_writable;
+      continue;
+    }
     if (read_type_ack && (read_file_error == ERROR_BROKEN_PIPE)) {
       // The client has already disconnected this pipe. This is an expected
       // behavior and do not treat as an error.
-      return true;
+      msg->clear();
+      return IPC_NO_ERROR;
     }
     if (read_file_error != ERROR_IO_PENDING) {
       LOG(ERROR) << "ReadFile() failed: " << read_file_error;
-      *last_ipc_error = IPC_READ_ERROR;
-      return false;
+      msg->clear();
+      return IPC_READ_ERROR;
     }
     // Actually this is an async operation. Let's wait for its completion.
-    if (!SafeWaitOverlappedResult(device_handle, nullptr, timeout, &overlapped,
-                                  &num_bytes_read, last_ipc_error,
-                                  read_type_ack)) {
-      return false;
+    bool has_more_data = false;
+    const IPCErrorType result = SafeWaitOverlappedResult(
+        device_handle, nullptr, absl::ToInt64Milliseconds(timeout), &overlapped,
+        &num_bytes_read, read_type_ack);
+    if (result == IPC_MORE_DATA) {
+      num_bytes_read_total += num_bytes_read;
+      continue;
     }
+    if (result != IPC_NO_ERROR) {
+      msg->clear();
+      return result;
+    }
+    num_bytes_read_total += num_bytes_read;
+    break;
   }
 
-  if (!read_type_ack && (num_bytes_read == 0)) {
+  if (!read_type_ack && (num_bytes_read_total == 0)) {
     LOG(WARNING) << "Received 0 result.";
   }
-
-  *buf_length = num_bytes_read;
-
-  return true;
+  msg->resize(num_bytes_read_total);
+  return IPC_NO_ERROR;
 }
 
-HANDLE CreateManualResetEvent() {
-  return ::CreateEvent(nullptr, TRUE, FALSE, nullptr);
+wil::unique_event_nothrow CreateManualResetEvent() {
+  wil::unique_event_nothrow event;
+  event.create(wil::EventOptions::ManualReset, nullptr);
+  return event;
 }
 
 // We do not care about the signaled state of the device handle itself.
@@ -444,11 +441,11 @@ void MaybeDisableFileCompletionNotification(HANDLE device_handle) {
 
 }  // namespace
 
-IPCServer::IPCServer(const std::string &name, int32 num_connections,
-                     int32 timeout)
+IPCServer::IPCServer(const std::string &name, int32_t num_connections,
+                     absl::Duration timeout)
     : connected_(false),
-      pipe_event_(CreateManualResetEvent()),
       quit_event_(CreateManualResetEvent()),
+      pipe_event_(CreateManualResetEvent()),
       timeout_(timeout) {
   IPCPathManager *manager = IPCPathManager::GetIPCPathManager(name);
   std::string server_address;
@@ -472,14 +469,15 @@ IPCServer::IPCServer(const std::string &name, int32 num_connections,
   }
 
   // Create a named pipe.
-  std::wstring wserver_address;
-  Util::Utf8ToWide(server_address, &wserver_address);
+  std::wstring wserver_address = Utf8ToWide(server_address);
   HANDLE handle = ::CreateNamedPipe(
       wserver_address.c_str(),
       PIPE_ACCESS_DUPLEX | FILE_FLAG_OVERLAPPED | FILE_FLAG_FIRST_PIPE_INSTANCE,
-      PIPE_TYPE_MESSAGE | PIPE_READMODE_MESSAGE | PIPE_WAIT,
+      PIPE_TYPE_MESSAGE | PIPE_READMODE_MESSAGE | PIPE_WAIT |
+          PIPE_REJECT_REMOTE_CLIENTS,
       (num_connections <= 0 ? PIPE_UNLIMITED_INSTANCES : num_connections),
-      sizeof(request_), sizeof(response_), 0, &security_attributes);
+      IPC_INITIAL_READ_BUFFER_SIZE, IPC_INITIAL_READ_BUFFER_SIZE, 0,
+      &security_attributes);
   const DWORD create_named_pipe_error = ::GetLastError();
   ::LocalFree(security_attributes.lpSecurityDescriptor);
 
@@ -509,28 +507,21 @@ void IPCServer::Terminate() {
     return;
   }
 
-  if (!server_thread_->IsRunning()) {
-    return;
-  }
-
   if (!::SetEvent(quit_event_.get())) {
     LOG(ERROR) << "SetEvent failed";
   }
 
   // Close the named pipe.
   // This is a workaround for killing child thread
-  if (server_thread_.get() != nullptr) {
-    server_thread_->Join();
-    server_thread_->Terminate();
-  }
+  server_thread_->Join();
 
   connected_ = false;
 }
 
 void IPCServer::Loop() {
-  IPCErrorType last_ipc_error = IPC_NO_ERROR;
-
   int successive_connection_failure_count = 0;
+  std::string request;
+  std::string response;
   while (connected_) {
     OVERLAPPED overlapped;
     if (!InitOverlapped(&overlapped, pipe_event_.get())) {
@@ -550,15 +541,15 @@ void IPCServer::Loop() {
       } else if (connect_named_pipe_error == ERROR_IO_PENDING) {
         // Actually this is async operation.
         DWORD ignored = 0;
-        IPCErrorType ipc_error = IPC_NO_ERROR;
-        if (!SafeWaitOverlappedResult(pipe_handle_.get(), quit_event_.get(),
-                                      INFINITE, &overlapped, &ignored,
-                                      &ipc_error, kReadTypeData)) {
-          if (ipc_error == IPC_QUIT_EVENT_SIGNALED) {
-            VLOG(1) << "Received Conrol event from other thread";
-            connected_ = false;
-            return;
-          }
+        const IPCErrorType ipc_error = SafeWaitOverlappedResult(
+            pipe_handle_.get(), quit_event_.get(), INFINITE, &overlapped,
+            &ignored, kReadTypeData);
+        if (ipc_error == IPC_QUIT_EVENT_SIGNALED) {
+          VLOG(1) << "Received Control event from other thread";
+          connected_ = false;
+          return;
+        }
+        if (ipc_error != IPC_NO_ERROR) {
           ++successive_connection_failure_count;
           if (successive_connection_failure_count >=
               kMaxSuccessiveConnectionFailureCount) {
@@ -576,26 +567,25 @@ void IPCServer::Loop() {
 
     successive_connection_failure_count = 0;
     // Retrieve an incoming message.
-    size_t request_size = sizeof(request_);
-    if (RecvIPCMessage(pipe_handle_.get(), pipe_event_.get(), &request_[0],
-                       &request_size, timeout_, kReadTypeData,
-                       &last_ipc_error)) {
-      size_t response_size = sizeof(response_);
-      if (!Process(&request_[0], request_size, &response_[0], &response_size)) {
+    if (RecvIpcMessage(pipe_handle_.get(), pipe_event_.get(), &request,
+                       timeout_, kReadTypeData) == IPC_NO_ERROR) {
+      if (!Process(request, &response)) {
         connected_ = false;
       }
 
       // When Process() returns 0 result, force to call DisconnectNamedPipe()
       // instead of checking ACK message
-      if (response_size == 0) {
+      if (response.empty()) {
         LOG(WARNING) << "Process() return 0 result";
         ::DisconnectNamedPipe(pipe_handle_.get());
         continue;
       }
 
       // Send a response
-      SendIPCMessage(pipe_handle_.get(), pipe_event_.get(), &response_[0],
-                     response_size, timeout_, &last_ipc_error);
+      if (SendIpcMessage(pipe_handle_.get(), pipe_event_.get(), response,
+                         timeout_) != IPC_NO_ERROR) {
+        LOG(WARNING) << "SendIpcMessage failed.";
+      }
     }
 
     // Special treatment for Windows per discussion with thatanaka:
@@ -609,12 +599,10 @@ void IPCServer::Loop() {
     // Wait ACK-like signal from client for 0.1 second. If we detect the pipe
     // disconnect event, so far so good. If we receive more data, we assume it
     // is an ACK signal (the IPC client of Mozc 1.5.x or earlier does this).
-    char ack_request[1] = {0};
-    size_t ack_request_size = 1;
-    static constexpr int kAckTimeout = 100;
-    if (!RecvIPCMessage(pipe_handle_.get(), pipe_event_.get(), ack_request,
-                        &ack_request_size, kAckTimeout, kReadTypeACK,
-                        &last_ipc_error)) {
+    std::string ack_request;
+    static constexpr absl::Duration kAckTimeout = absl::Milliseconds(100);
+    if (RecvIpcMessage(pipe_handle_.get(), pipe_event_.get(), &ack_request,
+                       kAckTimeout, kReadTypeACK) != IPC_NO_ERROR) {
       // This case happens when the client did not receive the server's response
       // within timeout. Anyway we will close the connection so that the server
       // will not be blocked.
@@ -628,7 +616,7 @@ void IPCServer::Loop() {
 }
 
 // old interface
-IPCClient::IPCClient(const std::string &name)
+IPCClient::IPCClient(const absl::string_view name)
     : pipe_event_(CreateManualResetEvent()),
       connected_(false),
       ipc_path_manager_(nullptr),
@@ -636,7 +624,8 @@ IPCClient::IPCClient(const std::string &name)
   Init(name, "");
 }
 
-IPCClient::IPCClient(const std::string &name, const std::string &server_path)
+IPCClient::IPCClient(const absl::string_view name,
+                     const absl::string_view server_path)
     : pipe_event_(CreateManualResetEvent()),
       connected_(false),
       ipc_path_manager_(nullptr),
@@ -644,17 +633,21 @@ IPCClient::IPCClient(const std::string &name, const std::string &server_path)
   Init(name, server_path);
 }
 
-void IPCClient::Init(const std::string &name, const std::string &server_path) {
+void IPCClient::Init(const absl::string_view name,
+                     const absl::string_view server_path) {
   last_ipc_error_ = IPC_NO_CONNECTION;
 
   // We should change the mutex based on which IPC server we will talk with.
-  ScopedReleaseMutex ipc_mutex(GetClientMutex(name));
+  const wil::unique_mutex_nothrow &ipc_mutex = GetClientMutex(name);
+  wil::mutex_release_scope_exit mutex_releaser;
 
   if (ipc_mutex.get() == nullptr) {
     LOG(ERROR) << "IPC mutex is not available";
   } else {
     constexpr int kMutexTimeout = 10 * 1000;  // wait at most 10sec.
-    switch (::WaitForSingleObject(ipc_mutex.get(), kMutexTimeout)) {
+    DWORD status;
+    mutex_releaser = ipc_mutex.acquire(&status, kMutexTimeout);
+    switch (status) {
       case WAIT_TIMEOUT:
         // TODO(taku): with suspend/resume, WaitForSingleObject may
         // return WAIT_TIMEOUT. We have to consider the case
@@ -692,8 +685,7 @@ void IPCClient::Init(const std::string &name, const std::string &server_path) {
     if (!manager->LoadPathName() || !manager->GetPathName(&server_address)) {
       continue;
     }
-    std::wstring wserver_address;
-    Util::Utf8ToWide(server_address, &wserver_address);
+    std::wstring wserver_address = Utf8ToWide(server_address);
 
     if (GetNumberOfProcessors() == 1) {
       // When the code is running in single processor environment, sometimes
@@ -706,17 +698,20 @@ void IPCClient::Init(const std::string &name, const std::string &server_path) {
       ::WaitNamedPipe(wserver_address.c_str(), kMinWaitTimeForWaitNamedPipe);
     }
 
-    ScopedHandle new_handle(
+    wil::unique_hfile new_handle(
         ::CreateFile(wserver_address.c_str(), GENERIC_READ | GENERIC_WRITE, 0,
                      nullptr, OPEN_EXISTING,
                      FILE_FLAG_OVERLAPPED | SECURITY_SQOS_PRESENT |
                          SECURITY_IDENTIFICATION | SECURITY_EFFECTIVE_ONLY,
                      nullptr));
     const DWORD create_file_error = ::GetLastError();
-    // ScopedHandle returns nullptr even when it received INVALID_HANDLE_VALUE.
-    if (new_handle.get() != nullptr) {
-      pipe_handle_.reset(new_handle.take());
+    if (new_handle) {
+      pipe_handle_ = std::move(new_handle);
       MaybeDisableFileCompletionNotification(pipe_handle_.get());
+      // Set PIPE_READMODE_MESSAGE so that we can rely on ERROR_MORE_DATA.
+      // https://learn.microsoft.com/en-us/windows/win32/ipc/named-pipe-client
+      DWORD mode = PIPE_READMODE_MESSAGE;
+      ::SetNamedPipeHandleState(pipe_handle_.get(), &mode, nullptr, nullptr);
       if (!manager->IsValidServer(GetServerProcessIdImpl(pipe_handle_.get()),
                                   server_path)) {
         LOG(ERROR) << "Connecting to invalid server";
@@ -760,8 +755,8 @@ IPCClient::~IPCClient() {}
 
 bool IPCClient::Connected() const { return connected_; }
 
-bool IPCClient::Call(const char *request, size_t request_size, char *response,
-                     size_t *response_size, int32 timeout) {
+bool IPCClient::Call(const std::string &request, std::string *response,
+                     absl::Duration timeout) {
   last_ipc_error_ = IPC_NO_ERROR;
   if (!connected_) {
     LOG(ERROR) << "IPCClient is not connected";
@@ -769,16 +764,17 @@ bool IPCClient::Call(const char *request, size_t request_size, char *response,
     return false;
   }
 
-  if (!SendIPCMessage(pipe_handle_.get(), pipe_event_.get(), request,
-                      request_size, timeout, &last_ipc_error_)) {
-    LOG(ERROR) << "SendIPCMessage() failed";
+  last_ipc_error_ =
+      SendIpcMessage(pipe_handle_.get(), pipe_event_.get(), request, timeout);
+  if (last_ipc_error_ != IPC_NO_ERROR) {
+    LOG(ERROR) << "SendIpcMessage() failed";
     return false;
   }
 
-  if (!RecvIPCMessage(pipe_handle_.get(), pipe_event_.get(), response,
-                      response_size, timeout, kReadTypeData,
-                      &last_ipc_error_)) {
-    LOG(ERROR) << "RecvIPCMessage() failed";
+  last_ipc_error_ = RecvIpcMessage(pipe_handle_.get(), pipe_event_.get(),
+                                   response, timeout, kReadTypeData);
+  if (last_ipc_error_ != IPC_NO_ERROR) {
+    LOG(ERROR) << "RecvIpcMessage() failed";
     return false;
   }
 
@@ -792,4 +788,4 @@ bool IPCClient::Call(const char *request, size_t request_size, char *response,
 
 }  // namespace mozc
 
-#endif  // OS_WIN
+#endif  // _WIN32
