@@ -36,8 +36,15 @@
 
 #include <unistd.h>
 
+#include <cstdint>
 #include <cstdlib>
+#include <functional>
+#include <map>
+#include <memory>
+#include <new>
 #include <set>
+#include <string>
+#include <utility>
 
 #import "mac/GoogleJapaneseInputControllerInterface.h"
 #import "mac/GoogleJapaneseInputServer.h"
@@ -45,40 +52,37 @@
 
 #include "base/const.h"
 #include "base/logging.h"
-#include "base/mac_process.h"
-#include "base/mac_util.h"
+#include "base/mac/mac_process.h"
+#include "base/mac/mac_util.h"
 #include "base/process.h"
 #include "base/util.h"
-#include "client/client.h"
 #include "ipc/ipc.h"
 #include "protocol/commands.pb.h"
 #include "protocol/config.pb.h"
+#include "absl/strings/string_view.h"
+#include "client/client.h"
 #include "renderer/renderer_client.h"
-#include "session/ime_switch_util.h"
 
-using mozc::commands::Candidates;
+using mozc::kProductNameInEnglish;
+using mozc::MacProcess;
 using mozc::commands::Capability;
 using mozc::commands::CompositionMode;
-using mozc::commands::Input;
 using mozc::commands::KeyEvent;
 using mozc::commands::Output;
 using mozc::commands::Preedit;
 using mozc::commands::RendererCommand;
 using mozc::commands::SessionCommand;
 using mozc::config::Config;
-using mozc::config::ImeSwitchUtil;
-using mozc::kProductNameInEnglish;
-using mozc::MacProcess;
 
 namespace {
 // set of bundle IDs of applications on which Mozc should not open urls.
-const std::set<std::string> *gNoOpenLinkApps = nullptr;
+const std::set<std::string, std::less<>> *gNoOpenLinkApps = nullptr;
 // The mapping from the CompositionMode enum to the actual id string
 // of composition modes.
 const std::map<CompositionMode, NSString *> *gModeIdMap = nullptr;
-const std::set<std::string> *gNoSelectedRangeApps = nullptr;
-const std::set<std::string> *gNoDisplayModeSwitchApps = nullptr;
-const std::set<std::string> *gNoSurroundingTextApps = nullptr;
+const std::set<std::string, std::less<>> *gNoSelectedRangeApps = nullptr;
+const std::set<std::string, std::less<>> *gNoDisplayModeSwitchApps = nullptr;
+const std::set<std::string, std::less<>> *gNoSurroundingTextApps = nullptr;
 
 // TODO(horo): This value should be get from system configuration.
 //  DoubleClickInterval can be get from NSEvent (MacOSX ver >= 10.6)
@@ -89,7 +93,7 @@ const int kMaxSurroundingLength = 20;
 // surrounding text takes too much time. So we set this limitation.
 const int kGetSurroundingTextClientLengthLimit = 1000;
 
-NSString *GetLabelForSuffix(const std::string &suffix) {
+NSString *GetLabelForSuffix(const absl::string_view suffix) {
   std::string label = mozc::MacUtil::GetLabelForSuffix(suffix);
   return [NSString stringWithUTF8String:label.c_str()];
 }
@@ -141,7 +145,8 @@ CompositionMode GetCompositionMode(NSString *modeID) {
   return mozc::commands::DIRECT;
 }
 
-bool IsBannedApplication(const std::set<std::string> *bundleIdSet, const std::string &bundleId) {
+bool IsBannedApplication(const std::set<std::string, std::less<>> *bundleIdSet,
+                         const absl::string_view bundleId) {
   return bundleIdSet == nullptr || bundleId.empty() ||
          bundleIdSet->find(bundleId) != bundleIdSet->end();
 }
@@ -156,11 +161,10 @@ bool IsBannedApplication(const std::set<std::string> *bundleIdSet, const std::st
 @synthesize replacementRange = replacementRange_;
 @synthesize imkClientForTest = imkClientForTest_;
 - (mozc::client::ClientInterface *)mozcClient {
-  return mozcClient_;
+  return mozcClient_.get();
 }
-- (void)setMozcClient:(mozc::client::ClientInterface *)newMozcClient {
-  delete mozcClient_;
-  mozcClient_ = newMozcClient;
+- (void)setMozcClient:(std::unique_ptr<mozc::client::ClientInterface>)newMozcClient {
+  mozcClient_ = std::move(newMozcClient);
 }
 - (mozc::renderer::RendererInterface *)renderer {
   return candidateController_;
@@ -172,7 +176,7 @@ bool IsBannedApplication(const std::set<std::string> *bundleIdSet, const std::st
 
 #pragma mark object init/dealloc
 // Initializer designated in IMKInputController. see:
-// http://developer.apple.com/documentation/Cocoa/Reference/IMKInputController_Class/
+// https://developer.apple.com/documentation/inputmethodkit/imkinputcontroller?language=objc
 
 - (id)initWithServer:(IMKServer *)server delegate:(id)delegate client:(id)inputClient {
   self = [super initWithServer:server delegate:delegate client:inputClient];
@@ -192,7 +196,7 @@ bool IsBannedApplication(const std::set<std::string> *bundleIdSet, const std::st
   candidateController_ = new (std::nothrow) mozc::renderer::RendererClient;
   rendererCommand_ = new (std::nothrow) RendererCommand;
   mozcClient_ = mozc::client::ClientFactory::NewClient();
-  imkServer_ = reinterpret_cast<id<ServerCallback> >(server);
+  imkServer_ = reinterpret_cast<id<ServerCallback>>(server);
   imkClientForTest_ = nil;
   lastKeyDownTime_ = 0;
   lastKeyCode_ = 0;
@@ -231,7 +235,6 @@ bool IsBannedApplication(const std::set<std::string> *bundleIdSet, const std::st
   imkClientForTest_ = nil;
   delete clientBundle_;
   delete candidateController_;
-  delete mozcClient_;
   delete rendererCommand_;
   DLOG(INFO) << "dealloc server";
 }
@@ -248,7 +251,8 @@ bool IsBannedApplication(const std::set<std::string> *bundleIdSet, const std::st
 }
 
 + (void)initializeConstants {
-  std::set<std::string> *noOpenlinkApps = new (std::nothrow) std::set<std::string>;
+  std::set<std::string, std::less<>> *noOpenlinkApps =
+      new (std::nothrow) std::set<std::string, std::less<>>;
   if (noOpenlinkApps) {
     // should not open links during screensaver.
     noOpenlinkApps->insert("com.apple.securityagent");
@@ -267,7 +271,8 @@ bool IsBannedApplication(const std::set<std::string> *bundleIdSet, const std::st
     gModeIdMap = newMap;
   }
 
-  std::set<std::string> *noSelectedRangeApps = new (std::nothrow) std::set<std::string>;
+  std::set<std::string, std::less<>> *noSelectedRangeApps =
+      new (std::nothrow) std::set<std::string, std::less<>>;
   if (noSelectedRangeApps) {
     // Do not call selectedRange: method for the following
     // applications because it could lead to application crash.
@@ -284,13 +289,15 @@ bool IsBannedApplication(const std::set<std::string> *bundleIdSet, const std::st
   // mode.  When the first composition character is alphanumeric (such
   // like pressing Shift-A at first), that character is directly
   // inserted into application instead of composition starting "A".
-  std::set<std::string> *noDisplayModeSwitchApps = new (std::nothrow) std::set<std::string>;
+  std::set<std::string, std::less<>> *noDisplayModeSwitchApps =
+      new (std::nothrow) std::set<std::string, std::less<>>;
   if (noDisplayModeSwitchApps) {
     noDisplayModeSwitchApps->insert("com.microsoft.Word");
     gNoDisplayModeSwitchApps = noDisplayModeSwitchApps;
   }
 
-  std::set<std::string> *noSurroundingTextApps = new (std::nothrow) std::set<std::string>;
+  std::set<std::string, std::less<>> *noSurroundingTextApps =
+      new (std::nothrow) std::set<std::string, std::less<>>;
   if (noSurroundingTextApps) {
     // Disables the surrounding text feature for the following application
     // because calling attributedSubstringFromRange to it is very heavy.
@@ -303,7 +310,7 @@ bool IsBannedApplication(const std::set<std::string> *bundleIdSet, const std::st
 // Currently it just ignores the following methods:
 //   Modes, showPreferences, valueForTag
 // They are described at
-// http://developer.apple.com/documentation/Cocoa/Reference/IMKStateSetting_Protocol/
+// https://developer.apple.com/documentation/inputmethodkit/imkstatesetting?language=objc
 
 - (void)activateServer:(id)sender {
   [super activateServer:sender];
@@ -513,7 +520,7 @@ bool IsBannedApplication(const std::set<std::string> *bundleIdSet, const std::st
   DLOG(INFO) << selectedRange.location << ", " << selectedRange.length;
   SessionCommand sending_command;
   Output output;
-  sending_command.CopyFrom(*command);
+  sending_command = *command;
 
   if (selectedRange.length == 0) {
     // Currently no range is selected for reconversion.  Tries to
@@ -567,7 +574,7 @@ bool IsBannedApplication(const std::set<std::string> *bundleIdSet, const std::st
     return;
   }
 
-  DLOG(INFO) << output->DebugString();
+  DLOG(INFO) << output->Utf8DebugString();
   if (output->has_url()) {
     NSString *url = [NSString stringWithUTF8String:output->url().c_str()];
     [self openLink:[NSURL URLWithString:url]];
@@ -663,7 +670,7 @@ bool IsBannedApplication(const std::set<std::string> *bundleIdSet, const std::st
 //   candidates
 //
 // The meaning of these methods are described at:
-// http://developer.apple.com/documentation/Cocoa/Reference/IMKServerInput_Additions/
+// https://developer.apple.com/documentation/inputmethodkit/imkserverinput?language=objc
 
 - (id)originalString:(id)sender {
   return originalString_;
@@ -671,7 +678,7 @@ bool IsBannedApplication(const std::set<std::string> *bundleIdSet, const std::st
 
 - (void)updateComposedString:(const Preedit *)preedit {
   // If the last and the current composed string length is 0,
-  // we don't call updateComposition.
+  // we don't update the composition.
   if (([composedString_ length] == 0) && ((preedit == nullptr || preedit->segment_size() == 0))) {
     return;
   }
@@ -699,8 +706,10 @@ bool IsBannedApplication(const std::set<std::string> *bundleIdSet, const std::st
     replacementRange_ = NSMakeRange(NSNotFound, 0);
   }
 
-  // Make composed string visible to the client applications.
-  [self updateComposition];
+  // Update the composed string of the client applications.
+  [[self client] setMarkedText:composedString_
+                selectionRange:[self selectionRange]
+              replacementRange:replacementRange_];
 }
 
 - (void)commitComposition:(id)sender {
@@ -740,6 +749,17 @@ bool IsBannedApplication(const std::set<std::string> *bundleIdSet, const std::st
 }
 
 - (void)delayedUpdateCandidates {
+  if (!candidateController_) {
+    return;
+  }
+
+  // If there is no candidate, the candidate window is closed.
+  if (rendererCommand_->output().candidates().candidate_size() == 0) {
+    rendererCommand_->set_visible(false);
+    candidateController_->ExecCommand(*rendererCommand_);
+    return;
+  }
+
   // The candidate window position is not recalculated if the
   // candidate already appears on the screen.  Therefore, if a user
   // moves client application window by mouse, candidate window won't
@@ -748,41 +768,44 @@ bool IsBannedApplication(const std::set<std::string> *bundleIdSet, const std::st
   //    cursor position correctly.  The candidate window moves
   //    frequently with those application, which irritates users.
   //  - Kotoeri does this too.
-  if ((!rendererCommand_->visible()) &&
-      (rendererCommand_->output().candidates().candidate_size() > 0)) {
-    NSRect preeditRect = NSZeroRect;
-    const int32 position = rendererCommand_->output().candidates().position();
-    // Some applications throws error when we call attributesForCharacterIndex.
-    DLOG(INFO) << "attributesForCharacterIndex: " << position;
-    @try {
-      [[self client] attributesForCharacterIndex:position lineHeightRectangle:&preeditRect];
-    } @catch (NSException *exception) {
-      LOG(ERROR) << "Exception from [" << *clientBundle_ << "] " << [[exception name] UTF8String]
-                 << "," << [[exception reason] UTF8String];
-    }
-    DLOG(INFO) << "  preeditRect: ((" << preeditRect.origin.x << ", " << preeditRect.origin.y
-               << "), (" << preeditRect.size.width << ", " << preeditRect.size.height << "))";
-    NSScreen *baseScreen = nil;
-    NSRect baseFrame = NSZeroRect;
-    for (baseScreen in [NSScreen screens]) {
-      baseFrame = [baseScreen frame];
-      if (baseFrame.origin.x == 0 && baseFrame.origin.y == 0) {
-        break;
-      }
-    }
-    int baseHeight = baseFrame.size.height;
-    rendererCommand_->mutable_preedit_rectangle()->set_left(preeditRect.origin.x);
-    rendererCommand_->mutable_preedit_rectangle()->set_top(baseHeight - preeditRect.origin.y -
-                                                           preeditRect.size.height);
-    rendererCommand_->mutable_preedit_rectangle()->set_right(preeditRect.origin.x +
-                                                             preeditRect.size.width);
-    rendererCommand_->mutable_preedit_rectangle()->set_bottom(baseHeight - preeditRect.origin.y);
+  if (rendererCommand_->visible()) {
+    // Call ExecCommand anyway to update other information like candidate words.
+    candidateController_->ExecCommand(*rendererCommand_);
+    return;
   }
 
-  rendererCommand_->set_visible(rendererCommand_->output().candidates().candidate_size() > 0);
-  if (candidateController_) {
-    candidateController_->ExecCommand(*rendererCommand_);
+  rendererCommand_->set_visible(true);
+
+  NSRect preeditRect = NSZeroRect;
+  const int32_t position = rendererCommand_->output().candidates().position();
+  // Some applications throws error when we call attributesForCharacterIndex.
+  DLOG(INFO) << "attributesForCharacterIndex: " << position;
+  @try {
+    NSDictionary *clientData = [[self client] attributesForCharacterIndex:position
+                                                      lineHeightRectangle:&preeditRect];
+
+    // IMKBaseline: Left-bottom of the composition.
+    NSPoint baseline = [clientData[@"IMKBaseline"] pointValue];
+    // IMKTextOrientation: 0: vertical writing, 1: horizontal writing.
+    // IMKLineHeight: Height of the composition (in horizontal writing).
+    // NSFont: Font information of the composition.
+    // IMKLineAscent: Not sure. A float number. (e.g. 9.240234)
+
+    const int right_offset = preeditRect.size.width;
+    const int top_offset = -preeditRect.size.height;
+
+    RendererCommand::Rectangle *rect = rendererCommand_->mutable_preedit_rectangle();
+    rect->set_left(baseline.x);
+    rect->set_right(baseline.x + right_offset);
+    rect->set_top(baseline.y + top_offset);
+    rect->set_bottom(baseline.y);
+
+  } @catch (NSException *exception) {
+    LOG(ERROR) << "Exception from [" << *clientBundle_ << "] " << [[exception name] UTF8String]
+               << "," << [[exception reason] UTF8String];
   }
+
+  candidateController_->ExecCommand(*rendererCommand_);
 }
 
 - (void)updateCandidates:(const Output *)output {
@@ -792,7 +815,7 @@ bool IsBannedApplication(const std::set<std::string> *bundleIdSet, const std::st
   }
 
   rendererCommand_->set_type(RendererCommand::UPDATE);
-  rendererCommand_->mutable_output()->CopyFrom(*output);
+  *rendererCommand_->mutable_output() = *output;
 
   // Runs delayedUpdateCandidates in the next event loop.
   // This is because some applications like Google Docs with Chrome returns
@@ -836,8 +859,13 @@ bool IsBannedApplication(const std::set<std::string> *bundleIdSet, const std::st
 }
 
 - (BOOL)handleEvent:(NSEvent *)event client:(id)sender {
+  if (event == nullptr || [event isEqual:[NSNull null]]) {
+    return NO;
+  }
   if ([event type] == NSEventTypeCursorUpdate) {
-    [self updateComposition];
+    [[self client] setMarkedText:composedString_
+                  selectionRange:[self selectionRange]
+                replacementRange:replacementRange_];
     return NO;
   }
   if ([event type] != NSEventTypeKeyDown && [event type] != NSEventTypeFlagsChanged) {
@@ -900,7 +928,7 @@ bool IsBannedApplication(const std::set<std::string> *bundleIdSet, const std::st
 
   // If the key event is turn on event, the key event has to be sent
   // to the server anyway.
-  if (mode_ == mozc::commands::DIRECT && !ImeSwitchUtil::IsDirectModeCommand(keyEvent)) {
+  if (mode_ == mozc::commands::DIRECT && !mozcClient_->IsDirectModeCommand(keyEvent)) {
     // Yen sign special hack: although the current mode is DIRECT,
     // backslash is sent instead of yen sign for JIS yen key with no
     // modifiers.  This behavior is based on the configuration.
