@@ -29,15 +29,14 @@
 
 #include "prediction/predictor.h"
 
-#include <cstddef>
 #include <memory>
 #include <string>
+#include <utility>
 
 #include "base/logging.h"
-#include "base/singleton.h"
-#include "base/system_util.h"
 #include "composer/composer.h"
 #include "config/config_handler.h"
+#include "converter/converter_mock.h"
 #include "converter/segments.h"
 #include "data_manager/testing/mock_data_manager.h"
 #include "dictionary/dictionary_mock.h"
@@ -49,18 +48,22 @@
 #include "protocol/config.pb.h"
 #include "request/conversion_request.h"
 #include "session/request_test_util.h"
-#include "testing/base/public/gmock.h"
-#include "testing/base/public/googletest.h"
-#include "testing/base/public/gunit.h"
+#include "testing/gmock.h"
+#include "testing/gunit.h"
+#include "absl/strings/string_view.h"
 
-namespace mozc {
+namespace mozc::prediction {
 namespace {
 
-using ::mozc::dictionary::DictionaryMock;
+using ::mozc::dictionary::MockDictionary;
 using ::mozc::dictionary::SuppressionDictionary;
 using ::testing::_;
 using ::testing::AtMost;
+using ::testing::DoAll;
 using ::testing::Return;
+using ::testing::SetArgPointee;
+using ::testing::StrEq;
+using ::testing::Unused;
 
 class CheckCandSizeDictionaryPredictor : public PredictorInterface {
  public:
@@ -70,8 +73,8 @@ class CheckCandSizeDictionaryPredictor : public PredictorInterface {
 
   bool PredictForRequest(const ConversionRequest &request,
                          Segments *segments) const override {
-    EXPECT_EQ(expected_cand_size_,
-              request.max_dictionary_prediction_candidates_size());
+    EXPECT_EQ(request.max_dictionary_prediction_candidates_size(),
+              expected_cand_size_);
     return true;
   }
 
@@ -94,11 +97,11 @@ class CheckCandSizeUserHistoryPredictor : public PredictorInterface {
 
   bool PredictForRequest(const ConversionRequest &request,
                          Segments *segments) const override {
-    EXPECT_EQ(expected_cand_size_,
-              request.max_user_history_prediction_candidates_size());
+    EXPECT_EQ(request.max_user_history_prediction_candidates_size(),
+              expected_cand_size_);
     EXPECT_EQ(
-        expected_cand_size_for_zero_query_,
-        request.max_user_history_prediction_candidates_size_for_zero_query());
+        request.max_user_history_prediction_candidates_size_for_zero_query(),
+        expected_cand_size_for_zero_query_);
     return true;
   }
 
@@ -173,9 +176,10 @@ class MobilePredictorTest : public ::testing::Test {
 };
 
 TEST_F(MobilePredictorTest, CallPredictorsForMobileSuggestion) {
+  MockConverter converter;
   auto predictor = std::make_unique<MobilePredictor>(
       std::make_unique<CheckCandSizeDictionaryPredictor>(20),
-      std::make_unique<CheckCandSizeUserHistoryPredictor>(3, 4));
+      std::make_unique<CheckCandSizeUserHistoryPredictor>(3, 4), &converter);
   Segments segments;
   {
     Segment *segment = segments.add_segment();
@@ -186,10 +190,11 @@ TEST_F(MobilePredictorTest, CallPredictorsForMobileSuggestion) {
 }
 
 TEST_F(MobilePredictorTest, CallPredictorsForMobilePartialSuggestion) {
+  MockConverter converter;
   auto predictor = std::make_unique<MobilePredictor>(
       std::make_unique<CheckCandSizeDictionaryPredictor>(20),
       // We don't call history predictior
-      std::make_unique<CheckCandSizeUserHistoryPredictor>(-1, -1));
+      std::make_unique<CheckCandSizeUserHistoryPredictor>(-1, -1), &converter);
   Segments segments;
   {
     Segment *segment = segments.add_segment();
@@ -200,9 +205,10 @@ TEST_F(MobilePredictorTest, CallPredictorsForMobilePartialSuggestion) {
 }
 
 TEST_F(MobilePredictorTest, CallPredictorsForMobilePrediction) {
+  MockConverter converter;
   auto predictor = std::make_unique<MobilePredictor>(
       std::make_unique<CheckCandSizeDictionaryPredictor>(200),
-      std::make_unique<CheckCandSizeUserHistoryPredictor>(3, 4));
+      std::make_unique<CheckCandSizeUserHistoryPredictor>(3, 4), &converter);
   Segments segments;
   {
     Segment *segment = segments.add_segment();
@@ -213,14 +219,16 @@ TEST_F(MobilePredictorTest, CallPredictorsForMobilePrediction) {
 }
 
 TEST_F(MobilePredictorTest, CallPredictorsForMobilePartialPrediction) {
-  DictionaryMock dictionary_mock;
+  MockConverter converter;
+  MockDictionary dictionary_mock;
   testing::MockDataManager data_manager;
   const dictionary::PosMatcher pos_matcher(data_manager.GetPosMatcherData());
   const SuppressionDictionary suppression_dictionary;
   auto predictor = std::make_unique<MobilePredictor>(
       std::make_unique<CheckCandSizeDictionaryPredictor>(200),
       std::make_unique<UserHistoryPredictor>(&dictionary_mock, &pos_matcher,
-                                             &suppression_dictionary, true));
+                                             &suppression_dictionary, true),
+      &converter);
   Segments segments;
   {
     Segment *segment = segments.add_segment();
@@ -240,8 +248,9 @@ TEST_F(MobilePredictorTest, CallPredictForRequestMobile) {
       .Times(AtMost(1))
       .WillOnce(Return(true));
 
-  auto predictor = std::make_unique<MobilePredictor>(std::move(predictor1),
-                                                     std::move(predictor2));
+  MockConverter converter;
+  auto predictor = std::make_unique<MobilePredictor>(
+      std::move(predictor1), std::move(predictor2), &converter);
   Segments segments;
   {
     Segment *segment = segments.add_segment();
@@ -271,9 +280,10 @@ class PredictorTest : public ::testing::Test {
 };
 
 TEST_F(PredictorTest, AllPredictorsReturnTrue) {
-  auto predictor =
-      std::make_unique<DefaultPredictor>(std::make_unique<NullPredictor>(true),
-                                         std::make_unique<NullPredictor>(true));
+  MockConverter converter;
+  auto predictor = std::make_unique<DefaultPredictor>(
+      std::make_unique<NullPredictor>(true),
+      std::make_unique<NullPredictor>(true), &converter);
   Segments segments;
   {
     Segment *segment = segments.add_segment();
@@ -284,9 +294,10 @@ TEST_F(PredictorTest, AllPredictorsReturnTrue) {
 }
 
 TEST_F(PredictorTest, MixedReturnValue) {
+  MockConverter converter;
   auto predictor = std::make_unique<DefaultPredictor>(
       std::make_unique<NullPredictor>(true),
-      std::make_unique<NullPredictor>(false));
+      std::make_unique<NullPredictor>(false), &converter);
   Segments segments;
   {
     Segment *segment = segments.add_segment();
@@ -297,9 +308,10 @@ TEST_F(PredictorTest, MixedReturnValue) {
 }
 
 TEST_F(PredictorTest, AllPredictorsReturnFalse) {
+  MockConverter converter;
   auto predictor = std::make_unique<DefaultPredictor>(
       std::make_unique<NullPredictor>(false),
-      std::make_unique<NullPredictor>(false));
+      std::make_unique<NullPredictor>(false), &converter);
   Segments segments;
   {
     Segment *segment = segments.add_segment();
@@ -310,12 +322,14 @@ TEST_F(PredictorTest, AllPredictorsReturnFalse) {
 }
 
 TEST_F(PredictorTest, CallPredictorsForSuggestion) {
+  MockConverter converter;
   const int suggestions_size =
       config::ConfigHandler::DefaultConfig().suggestions_size();
   auto predictor = std::make_unique<DefaultPredictor>(
       std::make_unique<CheckCandSizeDictionaryPredictor>(suggestions_size),
       std::make_unique<CheckCandSizeUserHistoryPredictor>(suggestions_size,
-                                                          suggestions_size));
+                                                          suggestions_size),
+      &converter);
   Segments segments;
   {
     Segment *segment = segments.add_segment();
@@ -326,11 +340,13 @@ TEST_F(PredictorTest, CallPredictorsForSuggestion) {
 }
 
 TEST_F(PredictorTest, CallPredictorsForPrediction) {
+  MockConverter converter;
   constexpr int kPredictionSize = 100;
   auto predictor = std::make_unique<DefaultPredictor>(
       std::make_unique<CheckCandSizeDictionaryPredictor>(kPredictionSize),
       std::make_unique<CheckCandSizeUserHistoryPredictor>(kPredictionSize,
-                                                          kPredictionSize));
+                                                          kPredictionSize),
+      &converter);
   Segments segments;
   {
     Segment *segment = segments.add_segment();
@@ -350,8 +366,9 @@ TEST_F(PredictorTest, CallPredictForRequest) {
       .Times(AtMost(1))
       .WillOnce(Return(true));
 
-  auto predictor = std::make_unique<DefaultPredictor>(std::move(predictor1),
-                                                      std::move(predictor2));
+  MockConverter converter;
+  auto predictor = std::make_unique<DefaultPredictor>(
+      std::move(predictor1), std::move(predictor2), &converter);
   Segments segments;
   {
     Segment *segment = segments.add_segment();
@@ -366,8 +383,9 @@ TEST_F(PredictorTest, DisableAllSuggestion) {
   auto predictor2 = std::make_unique<NullPredictor>(true);
   const auto *pred1 = predictor1.get();  // Keep the reference
   const auto *pred2 = predictor2.get();  // Keep the reference
-  auto predictor = std::make_unique<DefaultPredictor>(std::move(predictor1),
-                                                      std::move(predictor2));
+  MockConverter converter;
+  auto predictor = std::make_unique<DefaultPredictor>(
+      std::move(predictor1), std::move(predictor2), &converter);
   Segments segments;
   {
     Segment *segment = segments.add_segment();
@@ -386,4 +404,150 @@ TEST_F(PredictorTest, DisableAllSuggestion) {
   EXPECT_TRUE(pred2->predict_called());
 }
 
-}  // namespace mozc
+TEST_F(PredictorTest, PopulateReadingOfCommittedCandidateIfMissing) {
+  MockConverter converter;
+  auto predictor = std::make_unique<MobilePredictor>(
+      std::make_unique<NullPredictor>(true),
+      std::make_unique<NullPredictor>(true), &converter);
+
+  // Mock reverse conversion adds reading "とうきょう".
+  EXPECT_CALL(converter, StartReverseConversion(_, StrEq("東京")))
+      .WillRepeatedly([](Segments *segments, Unused) {
+        segments->add_segment()->add_candidate()->value = "とうきょう";
+        return true;
+      });
+
+  // Test the case where value == content_value.
+  {
+    Segments segments;
+    Segment *segment = segments.add_segment();
+
+    Segment::Candidate *cand1 = segment->add_candidate();
+    cand1->value = "東京";
+    cand1->content_value = "東京";
+
+    Segment::Candidate *cand2 = segment->add_candidate();
+    cand2->value = "大阪";
+    cand2->content_value = "大阪";
+
+    Segment::Candidate *cand3 = segment->add_candidate();
+    cand3->value = "群馬";
+    cand3->content_value = "群馬";
+
+    predictor->Finish(*convreq_, &segments);
+    EXPECT_EQ(cand1->key, "とうきょう");
+    EXPECT_EQ(cand1->content_key, "とうきょう");
+    EXPECT_TRUE(cand2->key.empty());
+    EXPECT_TRUE(cand2->content_key.empty());
+    EXPECT_TRUE(cand3->key.empty());
+    EXPECT_TRUE(cand3->content_key.empty());
+  }
+  // Test the case where value != content_value.
+  {
+    Segments segments;
+    Segment *segment = segments.add_segment();
+
+    Segment::Candidate *cand1 = segment->add_candidate();
+    cand1->value = "東京に";
+    cand1->content_value = "東京";
+
+    Segment::Candidate *cand2 = segment->add_candidate();
+    cand2->value = "大阪に";
+    cand2->content_value = "大阪";
+
+    Segment::Candidate *cand3 = segment->add_candidate();
+    cand3->value = "群馬に";
+    cand3->content_value = "群馬";
+
+    predictor->Finish(*convreq_, &segments);
+    EXPECT_EQ(cand1->key, "とうきょうに");
+    EXPECT_EQ(cand1->content_key, "とうきょう");
+    EXPECT_TRUE(cand2->key.empty());
+    EXPECT_TRUE(cand2->content_key.empty());
+    EXPECT_TRUE(cand3->key.empty());
+    EXPECT_TRUE(cand3->content_key.empty());
+  }
+  // Test the case where value != content_value and the functional value is not
+  // Hiragana. We cannot add the reading in this case.
+  {
+    Segments segments;
+    Segment *segment = segments.add_segment();
+
+    Segment::Candidate *cand1 = segment->add_candidate();
+    cand1->value = "東京便";
+    cand1->content_value = "東京";
+
+    predictor->Finish(*convreq_, &segments);
+    EXPECT_TRUE(cand1->key.empty());
+    EXPECT_TRUE(cand1->content_key.empty());
+  }
+  // Test the case where value != content_value and content_value is empty.
+  {
+    Segments segments;
+    Segment *segment = segments.add_segment();
+
+    Segment::Candidate *cand1 = segment->add_candidate();
+    cand1->value = "東京";
+    cand1->content_value.clear();
+
+    predictor->Finish(*convreq_, &segments);
+    EXPECT_TRUE(cand1->key.empty());
+    EXPECT_TRUE(cand1->content_key.empty());
+  }
+}
+
+TEST_F(MobilePredictorTest, FillPos) {
+  auto mock_dictionary_predictor = std::make_unique<MockPredictor>();
+  auto mock_history_predictor = std::make_unique<MockPredictor>();
+  auto add_candidate = [](absl::string_view key, absl::string_view value,
+                          int lid, int rid, int cost, Segment *segment) {
+    Segment::Candidate *candidate = segment->add_candidate();
+    candidate->key = key;
+    candidate->value = value;
+    candidate->lid = lid;
+    candidate->rid = rid;
+    candidate->cost = cost;
+    candidate->wcost = cost;
+  };
+
+  Segments history_segments;
+  {
+    Segment *segment = history_segments.add_segment();
+    add_candidate("key", "value", 0, 0, 1, segment);
+  }
+  // Mock results of dictionary_predictor. This contains the results from
+  // history_predictor as history_predictor is called before.
+  Segments dictionary_segments;
+  {
+    Segment *segment = dictionary_segments.add_segment();
+    add_candidate("key", "value", 0, 0, 1, segment);
+    add_candidate("key", "value", 2, 3, 100, segment);
+  }
+  EXPECT_CALL(*mock_history_predictor, PredictForRequest(_, _))
+      .WillOnce(DoAll(SetArgPointee<1>(history_segments), Return(true)));
+  EXPECT_CALL(*mock_dictionary_predictor, PredictForRequest(_, _))
+      .WillOnce(DoAll(SetArgPointee<1>(dictionary_segments), Return(true)));
+
+  MockConverter converter;
+  auto predictor = std::make_unique<MobilePredictor>(
+      std::move(mock_dictionary_predictor), std::move(mock_history_predictor),
+      &converter);
+
+  convreq_->set_request_type(ConversionRequest::SUGGESTION);
+  Segments segments;
+  EXPECT_TRUE(predictor->PredictForRequest(*convreq_, &segments));
+
+  EXPECT_EQ(segments.conversion_segments_size(), 1);
+  EXPECT_EQ(segments.conversion_segment(0).candidates_size(), 2);
+  const Segment::Candidate &candidate =
+      segments.conversion_segment(0).candidate(0);
+  EXPECT_EQ(candidate.key, "key");
+  EXPECT_EQ(candidate.value, "value");
+  // lid and rid are filled from another candidate.
+  EXPECT_EQ(candidate.lid, 2);
+  EXPECT_EQ(candidate.rid, 3);
+  // cost is not changed.
+  EXPECT_EQ(candidate.cost, 1);
+}
+
+}  // namespace mozc::prediction
