@@ -34,9 +34,9 @@
 
 #include "base/logging.h"
 #include "protocol/candidates.pb.h"
+#include "absl/strings/str_cat.h"
 #include "renderer/renderer_style_handler.h"
 #include "renderer/window_util.h"
-#include "absl/strings/str_cat.h"
 
 namespace mozc {
 namespace renderer {
@@ -416,7 +416,16 @@ bool QtWindowManager::ShouldShowInfolistWindow(
 }
 
 Rect QtWindowManager::GetMonitorRect(int x, int y) {
-  return GetRect(QGuiApplication::primaryScreen()->geometry());
+  QPoint point{x, y};
+  const QScreen *screen = QGuiApplication::screenAt(point);
+  if (screen == nullptr) {
+    // (x, y) does not belong to any screen. Fall back to the primary screen.
+    // TODO: Return the nearest monitor rect instead.
+    // c.f. GetWorkingAreaFromPointImpl in win32_renderer_util.cc.
+    return GetRect(QGuiApplication::primaryScreen()->geometry());
+  }
+
+  return GetRect(screen->geometry());
 }
 
 void QtWindowManager::UpdateInfolistWindow(
