@@ -34,21 +34,22 @@
 #include <memory>
 #include <string>
 
-#include "base/port.h"
 #include "ipc/ipc.h"
+#include "ipc/process_watch_dog.h"
+#include "absl/strings/string_view.h"
 #include "renderer/renderer_interface.h"
 
 namespace mozc {
 namespace renderer {
 
-class RendererInterface;
 class RendererServerSendCommand;
-class ParentApplicationWatchDog;
 
 // RendererServer base class. Implement Async* method.
 class RendererServer : public IPCServer {
  public:
   RendererServer();
+  RendererServer(const RendererServer &) = delete;
+  RendererServer &operator=(const RendererServer &) = delete;
   ~RendererServer() override;
 
   void SetRendererInterface(RendererInterface *renderer_interface);
@@ -59,8 +60,7 @@ class RendererServer : public IPCServer {
   // The return value is suppose to be used for the arg of exit().
   int StartServer();
 
-  bool Process(const char *request, size_t request_size, char *response,
-               size_t *response_size) override;
+  bool Process(absl::string_view request, std::string *response) override;
 
   // DEPRECATED: this functions is never called
   virtual void AsyncHide() {}
@@ -73,8 +73,7 @@ class RendererServer : public IPCServer {
   // protocol buffer received in Process method so that
   // IPC listener thread can reply to the client request as
   // early as possible.
-  // This function takes the owership of proto_message
-  virtual bool AsyncExecCommand(std::string *proto_message) = 0;
+  virtual bool AsyncExecCommand(absl::string_view proto_message) = 0;
 
  protected:
   // implement Message Loop function.
@@ -93,10 +92,8 @@ class RendererServer : public IPCServer {
 
  private:
   uint32_t timeout_;
-  std::unique_ptr<ParentApplicationWatchDog> watch_dog_;
+  std::unique_ptr<ProcessWatchDog> watch_dog_;
   std::unique_ptr<RendererServerSendCommand> send_command_;
-
-  DISALLOW_COPY_AND_ASSIGN(RendererServer);
 };
 
 }  // namespace renderer
