@@ -30,9 +30,8 @@
 #include "renderer/win32/win32_image_util.h"
 
 // clang-format off
-#define _ATL_NO_AUTOMATIC_NAMESPACE
-#define _WTL_NO_AUTOMATIC_NAMESPACE
 #include <atlbase.h>
+#include <atltypes.h>
 #include <atlapp.h>
 #include <atlmisc.h>
 #include <atlgdi.h>
@@ -40,6 +39,7 @@
 
 #include <algorithm>
 #include <bitset>
+#include <cstdint>
 #include <memory>
 #include <vector>
 
@@ -62,9 +62,6 @@ using ::WTL::CDC;
 using ::WTL::CFont;
 using ::WTL::CFontHandle;
 using ::WTL::CLogFont;
-using ::WTL::CPoint;
-using ::WTL::CRect;
-using ::WTL::CSize;
 
 Rect GetBalloonBoundingRect(
     double left, double top, double width, double height,
@@ -119,6 +116,9 @@ class Balloon {
             left, top, width, height, balloon_tail_height, balloon_tail_width,
             balloon_tail)) {}
 
+  Balloon(const Balloon &) = delete;
+  Balloon &operator=(const Balloon &) = delete;
+
   void RenderPixel(int x, int y, SubdivisionalPixel *pixel) const {
     {
       const PixelType fast_check_type = GetPixelTypeInternalFast(x, y);
@@ -128,6 +128,8 @@ class Balloon {
           break;
         case kOutside:
           return;
+        default:
+          break;
       }
     }
     for (SubdivisionalPixel::SubdivisionalPixelIterator it(x, y); !it.Done();
@@ -140,6 +142,8 @@ class Balloon {
         case kInside:
           pixel->SetSubdivisionalPixel(it.GetFraction(), inside_color_);
           break;
+        default:
+          break;
       }
     }
   }
@@ -353,7 +357,6 @@ class Balloon {
   const RGBColor inside_color_;
   const BalloonImage::BalloonImageInfo::TailDirection balloon_tail_;
   const Rect bounding_rect_;
-  DISALLOW_COPY_AND_ASSIGN(Balloon);
 };
 
 Rect GetBoundingRect(double left, double top, double width, double height) {
@@ -403,7 +406,7 @@ std::vector<std::unique_ptr<TextLabel::BinarySubdivisionalPixel>> Get1bitGlyph(
   bitmap_info.color_palette[0] = kBackgroundColor;  // black
   bitmap_info.color_palette[1] = kForegroundColor;  // white
 
-  uint8 *buffer = nullptr;
+  uint8_t *buffer = nullptr;
   CBitmap dib;
   dib.CreateDIBSection(
       nullptr, reinterpret_cast<const BITMAPINFO *>(&bitmap_info),
@@ -614,10 +617,10 @@ HBITMAP BalloonImage::CreateInternal(const BalloonImageInfo &info,
 
   // GDI native alpha image is Premultiplied BGRA.
   struct PBGRA {
-    uint8 b;
-    uint8 g;
-    uint8 r;
-    uint8 a;
+    uint8_t b;
+    uint8_t g;
+    uint8_t r;
+    uint8_t a;
   };
 
   BITMAPINFO bitmap_info = {};
@@ -853,6 +856,8 @@ void SubdivisionalPixel::SetColorToFilledPixels(const ColorType &color) {
       colors_.reset();
       single_color_ = color;
       break;
+    default:
+      break;
   }
 }
 
@@ -952,8 +957,8 @@ size_t SafeFrameBuffer::GetIndex(int x, int y) const {
 TextLabel::TextLabel(double left, double top, double width, double height,
                      const std::string &text, const std::string &font,
                      size_t font_point, const RGBColor text_color)
-    : bounding_rect_(GetBoundingRect(left, top, width, height)),
-      pixels_(Get1bitGlyph(left, top, width, height, text, font, font_point)),
+    : pixels_(Get1bitGlyph(left, top, width, height, text, font, font_point)),
+      bounding_rect_(GetBoundingRect(left, top, width, height)),
       text_color_(text_color) {}
 
 TextLabel::~TextLabel() = default;
