@@ -29,31 +29,22 @@
 
 #include "renderer/win32/win32_server.h"
 
+#include <windows.h>
+
 #include <memory>
 
 #include "base/logging.h"
-#include "base/port.h"
 #include "base/run_level.h"
 #include "base/util.h"
 #include "protocol/renderer_command.pb.h"
-#include "renderer/win32/window_manager.h"
+#include "absl/strings/string_view.h"
 #include "absl/synchronization/mutex.h"
+#include "renderer/win32/window_manager.h"
 
 namespace mozc {
 namespace renderer {
 namespace {
 
-bool IsIMM32Message(const commands::RendererCommand &command) {
-  if (!command.has_application_info()) {
-    return false;
-  }
-  if (!command.application_info().has_input_framework()) {
-    return false;
-  }
-  return (command.application_info().input_framework() ==
-          commands::RendererCommand::ApplicationInfo::IMM32);
-}
-
 bool IsTSFMessage(const commands::RendererCommand &command) {
   if (!command.has_application_info()) {
     return false;
@@ -122,10 +113,8 @@ bool Win32Server::ExecCommand(const commands::RendererCommand &command) {
     case commands::RendererCommand::UPDATE:
       if (!command.visible()) {
         window_manager_->HideAllWindows();
-      } else if (IsIMM32Message(command)) {
-        window_manager_->UpdateLayoutIMM32(command);
       } else if (IsTSFMessage(command)) {
-        window_manager_->UpdateLayoutTSF(command);
+        window_manager_->UpdateLayout(command);
       } else {
         LOG(WARNING) << "output/left/bottom are not set";
       }
@@ -142,18 +131,17 @@ void Win32Server::SetSendCommandInterface(
   window_manager_->SetSendCommandInterface(send_command_interface);
 }
 
-bool Win32Server::AsyncExecCommand(std::string *proto_message) {
+bool Win32Server::AsyncExecCommand(absl::string_view proto_message) {
   // Take the ownership of |proto_message|.
-  std::unique_ptr<std::string> proto_message_owner(proto_message);
   absl::MutexLock l(&mutex_);
-  if (message_ == *proto_message_owner.get()) {
+  if (message_ == proto_message) {
     // This is exactly the same to the previous message. Theoretically it is
     // safe to do nothing here.
     return true;
   }
   // Since mozc rendering protocol is state-less, we can always ignore the
   // previous content of |message_|.
-  message_.swap(*proto_message_owner.get());
+  message_.assign(proto_message.data(), proto_message.size());
   // Set the event signaled to mark we have a message to render.
   ::SetEvent(event_);
   return true;
