@@ -34,40 +34,30 @@
 
 #include <memory>
 
-#include "base/port.h"
+#include "protocol/renderer_command.pb.h"
+#include "client/client_interface.h"
+#include "renderer/win32/candidate_window.h"
+#include "renderer/win32/indicator_window.h"
+#include "renderer/win32/infolist_window.h"
+#include "renderer/win32/win32_renderer_util.h"
 
 namespace mozc {
-
-namespace client {
-class SendCommandInterface;
-}  // namespace client
-
-namespace commands {
-class RendererCommand;
-}  // namespace commands
-
 namespace renderer {
 namespace win32 {
 
-class CandidateWindow;
-class CompositionWindowList;
-class IndicatorWindow;
-class InfolistWindow;
-class LayoutManager;
-class WorkingAreaInterface;
-
 // This class is the controller of the candidate windows.
 class WindowManager {
  public:
   WindowManager();
+  WindowManager(const WindowManager &) = delete;
+  WindowManager &operator=(const WindowManager &) = delete;
   ~WindowManager();
   void Initialize();
   void AsyncHideAllWindows();
   void AsyncQuitAllWindows();
   void DestroyAllWindows();
   void HideAllWindows();
-  void UpdateLayoutIMM32(const commands::RendererCommand &command);
-  void UpdateLayoutTSF(const commands::RendererCommand &command);
+  void UpdateLayout(const commands::RendererCommand &command);
   bool IsAvailable() const;
   void SetSendCommandInterface(
       client::SendCommandInterface *send_command_interface);
@@ -76,17 +66,13 @@ class WindowManager {
  private:
   std::unique_ptr<CandidateWindow> main_window_;
   std::unique_ptr<CandidateWindow> cascading_window_;
-  std::unique_ptr<CompositionWindowList> composition_window_list_;
   std::unique_ptr<IndicatorWindow> indicator_window_;
   std::unique_ptr<InfolistWindow> infolist_window_;
   std::unique_ptr<LayoutManager> layout_manager_;
-  std::unique_ptr<WorkingAreaInterface> working_area_;
   client::SendCommandInterface *send_command_interface_;
   POINT last_position_;
   int candidates_finger_print_;
   DWORD thread_id_;
-
-  DISALLOW_COPY_AND_ASSIGN(WindowManager);
 };
 
 }  // namespace win32
