@@ -31,7 +31,6 @@
 
 #include <algorithm>
 #include <cstddef>
-#include <cstdint>
 #include <cstring>
 #include <memory>
 #include <string>
@@ -40,8 +39,7 @@
 
 #include "base/japanese_util.h"
 #include "base/logging.h"
-#include "base/util.h"
-#include "config/config_handler.h"
+#include "base/strings/assign.h"
 #include "converter/segments.h"
 #include "protocol/commands.pb.h"
 #include "protocol/config.pb.h"
@@ -49,19 +47,17 @@
 #include "rewriter/rewriter_util.h"
 #include "usage_stats/usage_stats.h"
 #include "absl/strings/match.h"
+#include "absl/strings/str_cat.h"
 #include "absl/strings/string_view.h"
 
 // EmojiRewriter:
 // Converts HIRAGANA strings to emoji characters, if they are names of emojis.
 
 namespace mozc {
-
-using commands::Request;
-
 namespace {
 
-constexpr char kEmoji[] = "絵文字";
-constexpr char kEmojiKey[] = "えもじ";
+constexpr absl::string_view kEmoji = "絵文字";
+constexpr absl::string_view kEmojiKey = "えもじ";
 // Where to insert emoji candidate by default.
 constexpr size_t kDefaultInsertPos = 6;
 
@@ -69,26 +65,26 @@ constexpr size_t kDefaultInsertPos = 6;
 using EmojiEntryList =
     std::vector<std::pair<absl::string_view, absl::string_view>>;
 
-std::unique_ptr<Segment::Candidate> CreateCandidate(absl::string_view key,
-                                   absl::string_view value,
-                                   absl::string_view description, int cost) {
+std::unique_ptr<Segment::Candidate> CreateCandidate(
+    absl::string_view key, absl::string_view value,
+    absl::string_view description, int cost) {
   auto candidate = std::make_unique<Segment::Candidate>();
-  candidate->Init();
   // Fill 0 (BOS/EOS) pos code intentionally.
   candidate->lid = 0;
   candidate->rid = 0;
   candidate->cost = cost;
-  candidate->value.assign(value.data(), value.size());
-  candidate->content_value.assign(value.data(), value.size());
-  candidate->key.assign(key.data(), key.size());
-  candidate->content_key.assign(key.data(), key.size());
-  candidate->description.assign(kEmoji);
-  if (!description.empty()) {
-    Util::AppendStringWithDelimiter(" ", description,
-                                    &(candidate->description));
+  strings::Assign(candidate->value, value);
+  strings::Assign(candidate->content_value, value);
+  strings::Assign(candidate->key, key);
+  strings::Assign(candidate->content_key, key);
+  if (description.empty()) {
+    strings::Assign(candidate->description, kEmoji);
+  } else {
+    candidate->description = absl::StrCat(kEmoji, " ", description);
   }
   candidate->attributes |= Segment::Candidate::NO_VARIANTS_EXPANSION;
   candidate->attributes |= Segment::Candidate::CONTEXT_SENSITIVE;
+  candidate->category = Segment::Candidate::SYMBOL;
 
   return candidate;
 }
@@ -98,8 +94,7 @@ int GetEmojiCost(const Segment &segment) {
   return segment.candidates_size() == 0 ? 0 : segment.candidate(0).cost;
 }
 
-void GatherAllEmojiData(EmojiRewriter::EmojiDataIterator begin,
-                        EmojiRewriter::EmojiDataIterator end,
+void GatherAllEmojiData(EmojiDataIterator begin, EmojiDataIterator end,
                         const SerializedStringArray &string_array,
                         EmojiEntryList *utf8_emoji_list) {
   for (; begin != end; ++begin) {
@@ -119,7 +114,7 @@ std::vector<std::unique_ptr<Segment::Candidate>> CreateAllEmojiData(
   std::vector<std::unique_ptr<Segment::Candidate>> candidates;
   candidates.reserve(utf8_emoji_list.size());
   for (const auto &emoji_entry : utf8_emoji_list) {
-    candidates.emplace_back(
+    candidates.push_back(
         CreateCandidate(key, emoji_entry.first, emoji_entry.second, cost));
   }
   return candidates;
@@ -135,7 +130,7 @@ std::vector<std::unique_ptr<Segment::Candidate>> CreateEmojiData(
     if (utf8_emoji.empty()) {
       continue;
     }
-    candidates.emplace_back(CreateCandidate(
+    candidates.push_back(CreateCandidate(
         key, utf8_emoji, string_array[iter.description_utf8_index()], cost));
   }
   return candidates;
@@ -149,8 +144,6 @@ EmojiRewriter::EmojiRewriter(const DataManagerInterface &data_manager) {
   string_array_.Set(string_array_data);
 }
 
-EmojiRewriter::~EmojiRewriter() = default;
-
 int EmojiRewriter::capability(const ConversionRequest &request) const {
   // The capability of the EmojiRewriter is up to the client's request.
   // Note that the bit representation of RewriterInterface::CapabilityType
@@ -166,13 +159,6 @@ bool EmojiRewriter::Rewrite(const ConversionRequest &request,
     return false;
   }
 
-  // TODO(b/135127317): Remove this protobuf field.
-  int32_t available_emoji_carrier = request.request().available_emoji_carrier();
-  if (!(available_emoji_carrier & Request::UNICODE_EMOJI)) {
-    VLOG(2) << "No available emoji carrier.";
-    return false;
-  }
-
   CHECK(segments != nullptr);
   return RewriteCandidates(segments);
 }
@@ -204,8 +190,8 @@ bool EmojiRewriter::IsEmojiCandidate(const Segment::Candidate &candidate) {
   return absl::StrContains(candidate.description, kEmoji);
 }
 
-std::pair<EmojiRewriter::EmojiDataIterator, EmojiRewriter::EmojiDataIterator>
-EmojiRewriter::LookUpToken(absl::string_view key) const {
+std::pair<EmojiDataIterator, EmojiDataIterator> EmojiRewriter::LookUpToken(
+    absl::string_view key) const {
   // Search string array for key.
   auto iter = std::lower_bound(string_array_.begin(), string_array_.end(), key);
   if (iter == string_array_.end() || *iter != key) {
@@ -226,7 +212,7 @@ bool EmojiRewriter::RewriteCandidates(Segments *segments) const {
       return false;
     }
     const size_t insert_position =
-      RewriterUtil::CalculateInsertPosition(*segment, kDefaultInsertPos);
+        RewriterUtil::CalculateInsertPosition(*segment, kDefaultInsertPos);
     segment->insert_candidates(insert_position, std::move(cands));
     return true;
   };
