@@ -29,10 +29,11 @@
 
 #include <cstdint>
 #include <map>
+#include <memory>
 #include <string>
+#include <utility>
 
 #include "base/init_mozc.h"
-#include "base/port.h"
 #include "data_manager/serialized_dictionary.h"
 #include "absl/flags/flag.h"
 
