@@ -39,7 +39,6 @@
 #include "base/logging.h"
 #include "base/number_util.h"
 #include "base/system_util.h"
-#include "base/util.h"
 #include "config/character_form_manager.h"
 #include "config/config_handler.h"
 #include "converter/segments.h"
@@ -50,11 +49,14 @@
 #include "request/conversion_request.h"
 #include "rewriter/number_rewriter.h"
 #include "rewriter/variants_rewriter.h"
-#include "testing/base/public/gmock.h"
-#include "testing/base/public/googletest.h"
-#include "testing/base/public/gunit.h"
-#include "absl/flags/flag.h"
+#include "session/request_test_util.h"
+#include "testing/gmock.h"
+#include "testing/gunit.h"
+#include "testing/mozctest.h"
+#include "absl/random/random.h"
+#include "absl/strings/str_cat.h"
 #include "absl/strings/string_view.h"
+#include "absl/time/time.h"
 
 namespace mozc {
 namespace {
@@ -64,6 +66,7 @@ using config::Config;
 using config::ConfigHandler;
 using dictionary::PosGroup;
 using dictionary::PosMatcher;
+using ::testing::WithParamInterface;
 
 constexpr size_t kCandidatesSize = 20;
 
@@ -94,30 +97,27 @@ void InitSegments(Segments *segments, size_t size) {
 }
 
 void AppendCandidateSuffix(Segment *segment, size_t index,
-                           const std::string &suffix, uint16_t lid,
+                           const absl::string_view suffix, uint16_t lid,
                            uint16_t rid) {
-  segment->set_key(segment->key() + suffix);
-  segment->mutable_candidate(index)->value += suffix;
+  segment->set_key(absl::StrCat(segment->key(), suffix));
+  absl::StrAppend(&segment->mutable_candidate(index)->value, suffix);
   segment->mutable_candidate(index)->lid = lid;
   segment->mutable_candidate(index)->rid = rid;
 }
 
 void AppendCandidateSuffixWithLid(Segment *segment, size_t index,
-                                  const std::string &suffix, uint16_t lid) {
+                                  const absl::string_view suffix,
+                                  uint16_t lid) {
   // if lid == 0 and rid == 0, we assume that candidate is t13n.
   // we set 1 as rid to avoid this.
   AppendCandidateSuffix(segment, index, suffix, lid, 1);
 }
 
-}  // namespace
-
-class UserSegmentHistoryRewriterTest : public ::testing::Test {
+class UserSegmentHistoryRewriterTest : public testing::TestWithTempUserProfile {
  protected:
   UserSegmentHistoryRewriterTest() { request_.set_config(&config_); }
 
   void SetUp() override {
-    SystemUtil::SetUserProfileDirectory(absl::GetFlag(FLAGS_test_tmpdir));
-
     ConfigHandler::GetDefaultConfig(&config_);
     for (int i = 0; i < config_.character_form_rules_size(); ++i) {
       Config::CharacterFormRule *rule = config_.mutable_character_form_rules(i);
@@ -166,8 +166,9 @@ class UserSegmentHistoryRewriterTest : public ::testing::Test {
       }
     }
     CharacterFormManager::GetCharacterFormManager()->ReloadConfig(config_);
-    EXPECT_EQ(form, CharacterFormManager::GetCharacterFormManager()
-                        ->GetConversionCharacterForm("0"));
+    EXPECT_EQ(CharacterFormManager::GetCharacterFormManager()
+                  ->GetConversionCharacterForm("0"),
+              form);
   }
 
   ConversionRequest request_;
@@ -177,14 +178,13 @@ class UserSegmentHistoryRewriterTest : public ::testing::Test {
   const testing::MockDataManager mock_data_manager_;
   PosMatcher pos_matcher_;
   std::unique_ptr<const PosGroup> pos_group_;
-  DISALLOW_COPY_AND_ASSIGN(UserSegmentHistoryRewriterTest);
 };
 
 TEST_F(UserSegmentHistoryRewriterTest, CreateFile) {
   std::unique_ptr<UserSegmentHistoryRewriter> rewriter(
       CreateUserSegmentHistoryRewriter());
   const std::string history_file =
-      FileUtil::JoinPath(absl::GetFlag(FLAGS_test_tmpdir), "/segment.db");
+      FileUtil::JoinPath(SystemUtil::GetUserProfileDirectory(), "/segment.db");
   EXPECT_OK(FileUtil::FileExists(history_file));
 }
 
@@ -212,12 +212,12 @@ TEST_F(UserSegmentHistoryRewriterTest, IncognitoModeTest) {
     rewriter->Finish(request_, &segments);
     InitSegments(&segments, 1);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate2", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2");
 
     config_.set_incognito_mode(true);
     InitSegments(&segments, 1);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate0", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate0");
   }
 
   {
@@ -231,7 +231,7 @@ TEST_F(UserSegmentHistoryRewriterTest, IncognitoModeTest) {
     rewriter->Finish(request_, &segments);
     InitSegments(&segments, 1);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate0", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate0");
   }
 }
 
@@ -250,17 +250,17 @@ TEST_F(UserSegmentHistoryRewriterTest, ConfigTest) {
     rewriter->Finish(request_, &segments);
     InitSegments(&segments, 1);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate2", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2");
 
     config_.set_history_learning_level(Config::NO_HISTORY);
     InitSegments(&segments, 1);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate0", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate0");
 
     config_.set_history_learning_level(Config::READ_ONLY);
     InitSegments(&segments, 1);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate2", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2");
   }
 
   {
@@ -273,7 +273,7 @@ TEST_F(UserSegmentHistoryRewriterTest, ConfigTest) {
     rewriter->Finish(request_, &segments);
     InitSegments(&segments, 1);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate0", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate0");
   }
 }
 
@@ -293,17 +293,17 @@ TEST_F(UserSegmentHistoryRewriterTest, DisableTest) {
     rewriter->Finish(conversion_request, &segments);
     InitSegments(&segments, 1);
     rewriter->Rewrite(conversion_request, &segments);
-    EXPECT_EQ("candidate2", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2");
 
     InitSegments(&segments, 1);
     conversion_request.set_enable_user_history_for_conversion(false);
     rewriter->Rewrite(conversion_request, &segments);
-    EXPECT_EQ("candidate0", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate0");
 
     InitSegments(&segments, 1);
     conversion_request.set_enable_user_history_for_conversion(true);
     rewriter->Rewrite(conversion_request, &segments);
-    EXPECT_EQ("candidate2", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2");
   }
 
   {
@@ -316,7 +316,7 @@ TEST_F(UserSegmentHistoryRewriterTest, DisableTest) {
     rewriter->Finish(conversion_request, &segments);
     InitSegments(&segments, 1);
     rewriter->Rewrite(conversion_request, &segments);
-    EXPECT_EQ("candidate0", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate0");
   }
 }
 
@@ -338,12 +338,12 @@ TEST_F(UserSegmentHistoryRewriterTest, BasicTest) {
     InitSegments(&segments, 2);
     rewriter->Rewrite(request_, &segments);
 
-    EXPECT_EQ("candidate2", segments.segment(0).candidate(0).value);
-    EXPECT_EQ("candidate0", segments.segment(1).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2");
+    EXPECT_EQ(segments.segment(1).candidate(0).value, "candidate0");
 
     InitSegments(&segments, 1);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate2", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2");
     segments.mutable_segment(0)->move_candidate(1, 0);
     segments.mutable_segment(0)->mutable_candidate(0)->attributes |=
         Segment::Candidate::RERANKED;
@@ -353,12 +353,12 @@ TEST_F(UserSegmentHistoryRewriterTest, BasicTest) {
     InitSegments(&segments, 1);
     rewriter->Rewrite(request_, &segments);
 
-    EXPECT_EQ("candidate0", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate0");
 
     InitSegments(&segments, 2);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate2", segments.segment(0).candidate(0).value);
-    EXPECT_EQ("candidate0", segments.segment(1).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2");
+    EXPECT_EQ(segments.segment(1).candidate(0).value, "candidate0");
     segments.mutable_segment(1)->move_candidate(3, 0);
     segments.mutable_segment(1)->mutable_candidate(0)->attributes |=
         Segment::Candidate::RERANKED;
@@ -367,8 +367,8 @@ TEST_F(UserSegmentHistoryRewriterTest, BasicTest) {
 
     InitSegments(&segments, 2);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate2", segments.segment(0).candidate(0).value);
-    EXPECT_EQ("candidate3", segments.segment(1).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2");
+    EXPECT_EQ(segments.segment(1).candidate(0).value, "candidate3");
   }
 
   rewriter->Clear();
@@ -383,12 +383,12 @@ TEST_F(UserSegmentHistoryRewriterTest, BasicTest) {
     InitSegments(&segments, 2);
     rewriter->Rewrite(request_, &segments);
 
-    EXPECT_EQ("candidate2", segments.segment(0).candidate(0).value);
-    EXPECT_EQ("candidate0", segments.segment(1).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2");
+    EXPECT_EQ(segments.segment(1).candidate(0).value, "candidate0");
 
     InitSegments(&segments, 1);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate2", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2");
 
     // back to the original
     segments.mutable_segment(0)->move_candidate(1, 0);
@@ -399,7 +399,7 @@ TEST_F(UserSegmentHistoryRewriterTest, BasicTest) {
 
     InitSegments(&segments, 1);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate0", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate0");
   }
 }
 
@@ -411,9 +411,7 @@ TEST_F(UserSegmentHistoryRewriterTest, SequenceTest) {
 
   rewriter->Clear();
 
-  constexpr uint64_t kSeconds = 0;
-  constexpr uint32_t kMicroSeconds = 0;
-  ClockMock clock(kSeconds, kMicroSeconds);
+  ClockMock clock(absl::UnixEpoch());
   Clock::SetClockForUnitTest(&clock);
 
   {
@@ -423,31 +421,31 @@ TEST_F(UserSegmentHistoryRewriterTest, SequenceTest) {
     segments.mutable_segment(0)->mutable_candidate(0)->attributes |=
         Segment::Candidate::RERANKED;
     segments.mutable_segment(0)->set_segment_type(Segment::FIXED_VALUE);
-    EXPECT_EQ("candidate2", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2");
     rewriter->Finish(request_, &segments);  // learn "candidate2"
 
     // Next timestamp of learning should be newer than previous learning.
-    clock.PutClockForward(1, 0);
+    clock.Advance(absl::Seconds(1));
 
     InitSegments(&segments, 2);
     segments.mutable_segment(0)->move_candidate(2, 0);
     segments.mutable_segment(0)->set_segment_type(Segment::HISTORY);
     segments.mutable_segment(1)->set_key(segments.segment(0).key());
-    EXPECT_EQ(1, segments.history_segments_size());
+    EXPECT_EQ(segments.history_segments_size(), 1);
     rewriter->Rewrite(request_, &segments);
 
-    EXPECT_EQ("candidate2", segments.segment(0).candidate(0).value);
-    EXPECT_EQ("candidate2", segments.segment(1).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2");
+    EXPECT_EQ(segments.segment(1).candidate(0).value, "candidate2");
     // 2 0 1 3 4 ..
 
     segments.mutable_segment(1)->move_candidate(3, 0);
     segments.mutable_segment(1)->mutable_candidate(0)->attributes |=
         Segment::Candidate::RERANKED;
     segments.mutable_segment(1)->set_segment_type(Segment::FIXED_VALUE);
-    EXPECT_EQ("candidate3", segments.segment(1).candidate(0).value);
+    EXPECT_EQ(segments.segment(1).candidate(0).value, "candidate3");
     rewriter->Finish(request_, &segments);  // learn "candidate3"
 
-    clock.PutClockForward(1, 0);
+    clock.Advance(absl::Seconds(1));
 
     InitSegments(&segments, 3);
     segments.mutable_segment(0)->move_candidate(2, 0);
@@ -456,22 +454,22 @@ TEST_F(UserSegmentHistoryRewriterTest, SequenceTest) {
     segments.mutable_segment(1)->set_segment_type(Segment::HISTORY);
     segments.mutable_segment(1)->set_key(segments.segment(0).key());
     segments.mutable_segment(2)->set_key(segments.segment(0).key());
-    EXPECT_EQ(2, segments.history_segments_size());
+    EXPECT_EQ(segments.history_segments_size(), 2);
     rewriter->Rewrite(request_, &segments);
 
-    EXPECT_EQ("candidate2", segments.segment(0).candidate(0).value);
-    EXPECT_EQ("candidate3", segments.segment(1).candidate(0).value);
-    EXPECT_EQ("candidate3", segments.segment(2).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2");
+    EXPECT_EQ(segments.segment(1).candidate(0).value, "candidate3");
+    EXPECT_EQ(segments.segment(2).candidate(0).value, "candidate3");
     // 3 2 0 1 4 ..
 
     segments.mutable_segment(2)->move_candidate(1, 0);
     segments.mutable_segment(2)->mutable_candidate(0)->attributes |=
         Segment::Candidate::RERANKED;
     segments.mutable_segment(2)->set_segment_type(Segment::FIXED_VALUE);
-    EXPECT_EQ("candidate2", segments.segment(2).candidate(0).value);
+    EXPECT_EQ(segments.segment(2).candidate(0).value, "candidate2");
     rewriter->Finish(request_, &segments);  // learn "candidate2"
 
-    clock.PutClockForward(1, 0);
+    clock.Advance(absl::Seconds(1));
 
     InitSegments(&segments, 4);
     segments.mutable_segment(0)->move_candidate(2, 0);
@@ -483,13 +481,13 @@ TEST_F(UserSegmentHistoryRewriterTest, SequenceTest) {
     segments.mutable_segment(2)->set_segment_type(Segment::HISTORY);
     segments.mutable_segment(2)->set_key(segments.segment(0).key());
     segments.mutable_segment(3)->set_key(segments.segment(0).key());
-    EXPECT_EQ(3, segments.history_segments_size());
+    EXPECT_EQ(segments.history_segments_size(), 3);
     rewriter->Rewrite(request_, &segments);
 
-    EXPECT_EQ("candidate2", segments.segment(0).candidate(0).value);
-    EXPECT_EQ("candidate3", segments.segment(1).candidate(0).value);
-    EXPECT_EQ("candidate2", segments.segment(2).candidate(0).value);
-    EXPECT_EQ("candidate2", segments.segment(3).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2");
+    EXPECT_EQ(segments.segment(1).candidate(0).value, "candidate3");
+    EXPECT_EQ(segments.segment(2).candidate(0).value, "candidate2");
+    EXPECT_EQ(segments.segment(3).candidate(0).value, "candidate2");
     // 2 3 0 1 4 ..
   }
 
@@ -503,9 +501,7 @@ TEST_F(UserSegmentHistoryRewriterTest, DupTest) {
 
   rewriter->Clear();
 
-  constexpr uint64_t kSeconds = 0;
-  constexpr uint32_t kMicroSeconds = 0;
-  ClockMock clock(kSeconds, kMicroSeconds);
+  ClockMock clock(absl::UnixEpoch());
   Clock::SetClockForUnitTest(&clock);
 
   {
@@ -520,31 +516,31 @@ TEST_F(UserSegmentHistoryRewriterTest, DupTest) {
 
     // restored
     // 4,0,1,2,3,5,...
-    EXPECT_EQ("candidate4", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate4");
     segments.mutable_segment(0)->move_candidate(4, 0);
     segments.mutable_segment(0)->mutable_candidate(0)->attributes |=
         Segment::Candidate::RERANKED;
     segments.mutable_segment(0)->set_segment_type(Segment::FIXED_VALUE);
-    clock.PutClockForward(1, 0);
+    clock.Advance(absl::Seconds(1));
     rewriter->Finish(request_, &segments);
     InitSegments(&segments, 1);
     rewriter->Rewrite(request_, &segments);
 
     // 3,4,0,1,2,5
-    EXPECT_EQ("candidate3", segments.segment(0).candidate(0).value);
-    EXPECT_EQ("candidate4", segments.segment(0).candidate(1).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate3");
+    EXPECT_EQ(segments.segment(0).candidate(1).value, "candidate4");
     segments.mutable_segment(0)->move_candidate(4, 0);
     segments.mutable_segment(0)->mutable_candidate(0)->attributes |=
         Segment::Candidate::RERANKED;
     segments.mutable_segment(0)->set_segment_type(Segment::FIXED_VALUE);
-    clock.PutClockForward(1, 0);
+    clock.Advance(absl::Seconds(1));
     rewriter->Finish(request_, &segments);
     InitSegments(&segments, 1);
     rewriter->Rewrite(request_, &segments);
 
-    EXPECT_EQ("candidate2", segments.segment(0).candidate(0).value);
-    EXPECT_EQ("candidate3", segments.segment(0).candidate(1).value);
-    EXPECT_EQ("candidate4", segments.segment(0).candidate(2).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2");
+    EXPECT_EQ(segments.segment(0).candidate(1).value, "candidate3");
+    EXPECT_EQ(segments.segment(0).candidate(2).value, "candidate4");
   }
 
   Clock::SetClockForUnitTest(nullptr);
@@ -567,7 +563,7 @@ TEST_F(UserSegmentHistoryRewriterTest, LearningType) {
     rewriter->Finish(request_, &segments);
     InitSegments(&segments, 1);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate0", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate0");
   }
 
   {
@@ -582,7 +578,7 @@ TEST_F(UserSegmentHistoryRewriterTest, LearningType) {
     rewriter->Finish(request_, &segments);
     InitSegments(&segments, 1);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate0", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate0");
   }
 
   {
@@ -597,7 +593,7 @@ TEST_F(UserSegmentHistoryRewriterTest, LearningType) {
     rewriter->Finish(request_, &segments);
     InitSegments(&segments, 1);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate2", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2");
   }
 }
 
@@ -620,11 +616,11 @@ TEST_F(UserSegmentHistoryRewriterTest, ContextSensitive) {
     rewriter->Rewrite(request_, &segments);
 
     // fire if two segments
-    EXPECT_EQ("candidate2", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2");
     // not fire if single segment
     InitSegments(&segments, 1);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate0", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate0");
   }
 
   rewriter->Clear();
@@ -641,12 +637,12 @@ TEST_F(UserSegmentHistoryRewriterTest, ContextSensitive) {
     // fire if even in single segment
     InitSegments(&segments, 1);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate2", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2");
 
     // not fire if two segments
     InitSegments(&segments, 2);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate0", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate0");
   }
 }
 
@@ -674,7 +670,7 @@ TEST_F(UserSegmentHistoryRewriterTest, ContentValueLearning) {
     rewriter->Rewrite(request_, &segments);
 
     // exact match
-    EXPECT_EQ("candidate2:all", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2:all");
 
     InitSegments(&segments, 1);
     rewriter->Rewrite(request_, &segments);
@@ -682,7 +678,7 @@ TEST_F(UserSegmentHistoryRewriterTest, ContentValueLearning) {
     // content value only:
     // in both learning/applying phase, lid and suffix are the same
     // as those of top candidates.
-    EXPECT_EQ("candidate2", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2");
 
     InitSegments(&segments, 1);
     AppendCandidateSuffixWithLid(segments.mutable_segment(0), 0, ":other", 0);
@@ -690,7 +686,7 @@ TEST_F(UserSegmentHistoryRewriterTest, ContentValueLearning) {
 
     rewriter->Rewrite(request_, &segments);
 
-    EXPECT_EQ("candidate2:other", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2:other");
   }
 
   // In learning phase, lid is different
@@ -706,7 +702,7 @@ TEST_F(UserSegmentHistoryRewriterTest, ContentValueLearning) {
     rewriter->Finish(request_, &segments);
     InitSegments(&segments, 1);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate0", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate0");
   }
 
   // In learning phase, suffix (functional value) is different
@@ -722,7 +718,7 @@ TEST_F(UserSegmentHistoryRewriterTest, ContentValueLearning) {
     rewriter->Finish(request_, &segments);
     InitSegments(&segments, 1);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate0", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate0");
   }
 
   // In apply phase, lid is different
@@ -740,7 +736,7 @@ TEST_F(UserSegmentHistoryRewriterTest, ContentValueLearning) {
     AppendCandidateSuffixWithLid(segments.mutable_segment(0), 0, ":other", 0);
     AppendCandidateSuffixWithLid(segments.mutable_segment(0), 2, ":other", 1);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate0:other", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate0:other");
   }
 
   // In apply phase, suffix (functional value) is different
@@ -758,7 +754,7 @@ TEST_F(UserSegmentHistoryRewriterTest, ContentValueLearning) {
     AppendCandidateSuffixWithLid(segments.mutable_segment(0), 0, "", 0);
     AppendCandidateSuffixWithLid(segments.mutable_segment(0), 2, ":other", 0);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate0", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate0");
   }
 }
 
@@ -785,7 +781,7 @@ TEST_F(UserSegmentHistoryRewriterTest, ReplaceableTest) {
 
     rewriter->Rewrite(request_, &segments);
 
-    EXPECT_EQ("candidate2:all", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2:all");
 
     InitSegments(&segments, 1);
     rewriter->Rewrite(request_, &segments);
@@ -809,7 +805,7 @@ TEST_F(UserSegmentHistoryRewriterTest, ReplaceableTest) {
 
     rewriter->Rewrite(request_, &segments);
 
-    EXPECT_EQ("candidate2:all", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2:all");
 
     InitSegments(&segments, 1);
     rewriter->Rewrite(request_, &segments);
@@ -829,7 +825,7 @@ TEST_F(UserSegmentHistoryRewriterTest, ReplaceableTest) {
     AppendCandidateSuffixWithLid(segments.mutable_segment(0), 0, ":all", 0);
     AppendCandidateSuffixWithLid(segments.mutable_segment(0), 2, ":all", 0);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate0:all", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate0:all");
   }
 
   rewriter->Clear();
@@ -846,7 +842,7 @@ TEST_F(UserSegmentHistoryRewriterTest, ReplaceableTest) {
     AppendCandidateSuffixWithLid(segments.mutable_segment(0), 0, ":all", 0);
     AppendCandidateSuffixWithLid(segments.mutable_segment(0), 2, ":all", 1);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate0:all", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate0:all");
   }
 
   rewriter->Clear();
@@ -863,7 +859,7 @@ TEST_F(UserSegmentHistoryRewriterTest, ReplaceableTest) {
     AppendCandidateSuffixWithLid(segments.mutable_segment(0), 0, ":all", 0);
     AppendCandidateSuffixWithLid(segments.mutable_segment(0), 2, ":all", 0);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate0:all", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate0:all");
   }
 
   rewriter->Clear();
@@ -880,7 +876,7 @@ TEST_F(UserSegmentHistoryRewriterTest, ReplaceableTest) {
     AppendCandidateSuffixWithLid(segments.mutable_segment(0), 0, ":all", 0);
     AppendCandidateSuffixWithLid(segments.mutable_segment(0), 2, ":all", 1);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate0:all", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate0:all");
   }
 }
 
@@ -909,7 +905,7 @@ TEST_F(UserSegmentHistoryRewriterTest, NotReplaceableForDifferentId) {
 
     rewriter->Rewrite(request_, &segments);
 
-    EXPECT_NE("candidate2:all", segments.segment(0).candidate(0).value);
+    EXPECT_NE(segments.segment(0).candidate(0).value, "candidate2:all");
   }
 }
 
@@ -938,7 +934,7 @@ TEST_F(UserSegmentHistoryRewriterTest, ReplaceableForSameId) {
 
     rewriter->Rewrite(request_, &segments);
 
-    EXPECT_EQ("candidate2:all", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2:all");
   }
 }
 
@@ -968,7 +964,7 @@ TEST_F(UserSegmentHistoryRewriterTest, ReplaceableT13NTest) {
 
     rewriter->Rewrite(request_, &segments);
 
-    EXPECT_EQ("candidate2:all", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2:all");
   }
 }
 
@@ -990,15 +986,15 @@ TEST_F(UserSegmentHistoryRewriterTest, LeftRightNumber) {
         Segment::Candidate::CONTEXT_SENSITIVE;
     segments.mutable_segment(1)->set_segment_type(Segment::FIXED_VALUE);
     rewriter->Finish(request_, &segments);
-    EXPECT_EQ("1234", segments.segment(0).candidate(0).value);
-    EXPECT_EQ("candidate2", segments.segment(1).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "1234");
+    EXPECT_EQ(segments.segment(1).candidate(0).value, "candidate2");
 
     InitSegments(&segments, 2);
     // different num.
     segments.mutable_segment(0)->mutable_candidate(0)->value = "5678";
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("5678", segments.segment(0).candidate(0).value);
-    EXPECT_EQ("candidate2", segments.segment(1).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "5678");
+    EXPECT_EQ(segments.segment(1).candidate(0).value, "candidate2");
   }
 
   {
@@ -1012,15 +1008,15 @@ TEST_F(UserSegmentHistoryRewriterTest, LeftRightNumber) {
         Segment::Candidate::CONTEXT_SENSITIVE;
     segments.mutable_segment(0)->set_segment_type(Segment::FIXED_VALUE);
     rewriter->Finish(request_, &segments);
-    EXPECT_EQ("candidate2", segments.segment(0).candidate(0).value);
-    EXPECT_EQ("1234", segments.segment(1).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2");
+    EXPECT_EQ(segments.segment(1).candidate(0).value, "1234");
 
     InitSegments(&segments, 2);
     // different num.
     segments.mutable_segment(1)->mutable_candidate(0)->value = "5678";
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("candidate2", segments.segment(0).candidate(0).value);
-    EXPECT_EQ("5678", segments.segment(1).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2");
+    EXPECT_EQ(segments.segment(1).candidate(0).value, "5678");
   }
 }
 
@@ -1055,7 +1051,7 @@ TEST_F(UserSegmentHistoryRewriterTest, BacketMatching) {
 
     rewriter->Rewrite(request_, &segments);
 
-    EXPECT_EQ(")", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, ")");
   }
 }
 
@@ -1112,7 +1108,7 @@ TEST_F(UserSegmentHistoryRewriterTest, MultipleLearning) {
 
     rewriter->Rewrite(request_, &segments);
 
-    EXPECT_EQ("value1", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "value1");
   }
 }
 
@@ -1158,7 +1154,7 @@ TEST_F(UserSegmentHistoryRewriterTest, NumberSpecial) {
     EXPECT_TRUE(number_rewriter->Rewrite(request_, &segments));
     rewriter->Rewrite(request_, &segments);
 
-    EXPECT_EQ("⑭", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "⑭");
   }
 }
 
@@ -1251,7 +1247,76 @@ TEST_F(UserSegmentHistoryRewriterTest, NumberFullWidth) {
     EXPECT_TRUE(number_rewriter->Rewrite(request_, &segments));
     rewriter->Rewrite(request_, &segments);
 
-    EXPECT_EQ("１，２３４", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "１，２３４");
+  }
+}
+
+class UserSegmentHistoryNumberTest
+    : public UserSegmentHistoryRewriterTest,
+      public WithParamInterface<commands::Request> {};
+
+INSTANTIATE_TEST_SUITE_P(
+    NumberStyleLearningTestForRequest, UserSegmentHistoryNumberTest,
+    ::testing::Values(
+        []() {
+          commands::Request request;
+          commands::RequestForUnitTest::FillMobileRequest(&request);
+          return request;
+        }(),
+        []() {
+          commands::Request request;
+          commands::RequestForUnitTest::FillMobileRequestWithHardwareKeyboard(
+              &request);
+          return request;
+        }()));
+
+TEST_P(UserSegmentHistoryNumberTest, UserSegmentHistoryRewriterTest) {
+  const commands::Request request = GetParam();
+  request_.set_request(&request);
+
+  SetNumberForm(Config::FULL_WIDTH);
+  Segments segments;
+  std::unique_ptr<UserSegmentHistoryRewriter> rewriter(
+      CreateUserSegmentHistoryRewriter());
+  std::unique_ptr<NumberRewriter> number_rewriter(CreateNumberRewriter());
+
+  rewriter->Clear();
+
+  {
+    segments.Clear();
+    segments.add_segment();
+    segments.mutable_segment(0)->set_key("1234");
+    Segment::Candidate *candidate =
+        segments.mutable_segment(0)->insert_candidate(0);
+    candidate->value = "1,234";
+    candidate->content_value = "1,2344";
+    candidate->content_key = "1234";
+    candidate->lid = pos_matcher().GetNumberId();
+    candidate->rid = pos_matcher().GetNumberId();
+    candidate->style =
+        NumberUtil::NumberString::NUMBER_SEPARATED_ARABIC_HALFWIDTH;
+    segments.mutable_segment(0)->set_segment_type(Segment::FIXED_VALUE);
+    rewriter->Finish(request_, &segments);  // half-width for separated number
+  }
+
+  {
+    // This rewriter does not handle number candidate
+    segments.Clear();
+    segments.add_segment();
+    {
+      segments.mutable_segment(0)->set_key("1234");
+      Segment::Candidate *candidate =
+          segments.mutable_segment(0)->insert_candidate(0);
+      candidate->value = "1234";
+      candidate->content_value = "1234";
+      candidate->content_key = "1234";
+      candidate->lid = pos_matcher().GetNumberId();
+      candidate->rid = pos_matcher().GetNumberId();
+    }
+    EXPECT_TRUE(number_rewriter->Rewrite(request_, &segments));
+    rewriter->Rewrite(request_, &segments);
+
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "1234");
   }
 }
 
@@ -1312,7 +1377,7 @@ TEST_F(UserSegmentHistoryRewriterTest, NumberNoSeparated) {
     rewriter->Rewrite(request_, &segments);
 
     // 9, not "九"
-    EXPECT_EQ("9", segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, "9");
   }
 }
 
@@ -1323,9 +1388,7 @@ TEST_F(UserSegmentHistoryRewriterTest, Regression2459519) {
 
   rewriter->Clear();
 
-  constexpr uint64_t kSeconds = 0;
-  constexpr uint32_t kMicroSeconds = 0;
-  ClockMock clock(kSeconds, kMicroSeconds);
+  ClockMock clock(absl::UnixEpoch());
   Clock::SetClockForUnitTest(&clock);
 
   InitSegments(&segments, 1);
@@ -1337,32 +1400,32 @@ TEST_F(UserSegmentHistoryRewriterTest, Regression2459519) {
 
   InitSegments(&segments, 1);
   rewriter->Rewrite(request_, &segments);
-  EXPECT_EQ("candidate2", segments.segment(0).candidate(0).value);
-  EXPECT_EQ("candidate0", segments.segment(0).candidate(1).value);
+  EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2");
+  EXPECT_EQ(segments.segment(0).candidate(1).value, "candidate0");
 
   segments.mutable_segment(0)->move_candidate(1, 0);
   segments.mutable_segment(0)->mutable_candidate(0)->attributes |=
       Segment::Candidate::RERANKED;
   segments.mutable_segment(0)->set_segment_type(Segment::FIXED_VALUE);
-  clock.PutClockForward(1, 0);
+  clock.Advance(absl::Seconds(1));
   rewriter->Finish(request_, &segments);
 
   InitSegments(&segments, 1);
   rewriter->Rewrite(request_, &segments);
-  EXPECT_EQ("candidate0", segments.segment(0).candidate(0).value);
-  EXPECT_EQ("candidate2", segments.segment(0).candidate(1).value);
+  EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate0");
+  EXPECT_EQ(segments.segment(0).candidate(1).value, "candidate2");
 
   segments.mutable_segment(0)->move_candidate(1, 0);
   segments.mutable_segment(0)->mutable_candidate(0)->attributes |=
       Segment::Candidate::RERANKED;
   segments.mutable_segment(0)->set_segment_type(Segment::FIXED_VALUE);
-  clock.PutClockForward(1, 0);
+  clock.Advance(absl::Seconds(1));
   rewriter->Finish(request_, &segments);
 
   InitSegments(&segments, 1);
   rewriter->Rewrite(request_, &segments);
-  EXPECT_EQ("candidate2", segments.segment(0).candidate(0).value);
-  EXPECT_EQ("candidate0", segments.segment(0).candidate(1).value);
+  EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2");
+  EXPECT_EQ(segments.segment(0).candidate(1).value, "candidate0");
 
   Clock::SetClockForUnitTest(nullptr);
 }
@@ -1388,8 +1451,8 @@ TEST_F(UserSegmentHistoryRewriterTest, Regression2459520) {
 
   InitSegments(&segments, 2);
   rewriter->Rewrite(request_, &segments);
-  EXPECT_EQ("candidate2", segments.segment(0).candidate(0).value);
-  EXPECT_EQ("candidate3", segments.segment(1).candidate(0).value);
+  EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate2");
+  EXPECT_EQ(segments.segment(1).candidate(0).value, "candidate3");
 }
 
 TEST_F(UserSegmentHistoryRewriterTest, PuntuationsTest) {
@@ -1424,7 +1487,7 @@ TEST_F(UserSegmentHistoryRewriterTest, PuntuationsTest) {
 
   // Punctuation is not remembered
   rewriter->Rewrite(request_, &segments);
-  EXPECT_EQ("candidate0", segments.segment(1).candidate(0).value);
+  EXPECT_EQ(segments.segment(1).candidate(0).value, "candidate0");
 }
 
 TEST_F(UserSegmentHistoryRewriterTest, Regression3264619) {
@@ -1442,8 +1505,8 @@ TEST_F(UserSegmentHistoryRewriterTest, Regression3264619) {
   InitSegments(&segments, 2, 1024);
   rewriter->Rewrite(request_, &segments);
 
-  EXPECT_EQ("candidate512", segments.segment(0).candidate(0).value);
-  EXPECT_EQ("candidate0", segments.segment(1).candidate(0).value);
+  EXPECT_EQ(segments.segment(0).candidate(0).value, "candidate512");
+  EXPECT_EQ(segments.segment(1).candidate(0).value, "candidate0");
 }
 
 TEST_F(UserSegmentHistoryRewriterTest, RandomTest) {
@@ -1451,26 +1514,25 @@ TEST_F(UserSegmentHistoryRewriterTest, RandomTest) {
   std::unique_ptr<UserSegmentHistoryRewriter> rewriter(
       CreateUserSegmentHistoryRewriter());
 
-  constexpr uint64_t kSeconds = 0;
-  constexpr uint32_t kMicroSeconds = 0;
-  ClockMock clock(kSeconds, kMicroSeconds);
+  ClockMock clock(absl::UnixEpoch());
   Clock::SetClockForUnitTest(&clock);
 
   rewriter->Clear();
+  absl::BitGen gen;
   for (int i = 0; i < 5; ++i) {
     InitSegments(&segments, 1);
-    const int n = Util::Random(10);
+    const int n = absl::Uniform(gen, 0, 10);
     const std::string expected = segments.segment(0).candidate(n).value;
     segments.mutable_segment(0)->move_candidate(n, 0);
     segments.mutable_segment(0)->mutable_candidate(0)->attributes |=
         Segment::Candidate::RERANKED;
     segments.mutable_segment(0)->set_segment_type(Segment::FIXED_VALUE);
-    EXPECT_EQ(expected, segments.segment(0).candidate(0).value);
+    EXPECT_EQ(segments.segment(0).candidate(0).value, expected);
     rewriter->Finish(request_, &segments);
     InitSegments(&segments, 1);
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ(expected, segments.segment(0).candidate(0).value);
-    clock.PutClockForward(1, 0);  // update LRU timer
+    EXPECT_EQ(segments.segment(0).candidate(0).value, expected);
+    clock.Advance(absl::Seconds(1));  // update LRU timer
   }
 
   Clock::SetClockForUnitTest(nullptr);
@@ -1510,16 +1572,13 @@ TEST_F(UserSegmentHistoryRewriterTest, AnnotationAfterLearning) {
     candidate->description = "[全]アルファベット";
     candidate->content_key = "abc";
     rewriter->Rewrite(request_, &segments);
-    EXPECT_EQ("abc", segments.segment(0).candidate(0).content_value);
-    // "[半] アルファベット"
-    auto expectation = std::string(VariantsRewriter::kHalfWidth);
-    const absl::string_view alphabet = VariantsRewriter::kAlphabet;
-    if (!alphabet.empty()) {
-      expectation.append(1, ' ').append(alphabet.data(), alphabet.size());
-    }
-    EXPECT_EQ(expectation, segments.segment(0).candidate(0).description);
+    EXPECT_EQ(segments.segment(0).candidate(0).content_value, "abc");
+    // "アルファベット"
+    EXPECT_EQ(VariantsRewriter::kAlphabet,
+              segments.segment(0).candidate(0).description);
     rewriter->Finish(request_, &segments);
   }
 }
 
+}  // namespace
 }  // namespace mozc
