@@ -44,6 +44,7 @@
 #include "protocol/commands.pb.h"
 #include "request/conversion_request.h"
 #include "absl/strings/match.h"
+#include "absl/strings/str_join.h"
 #include "absl/strings/str_replace.h"
 #include "absl/strings/string_view.h"
 
@@ -53,19 +54,8 @@ namespace {
 using ::mozc::config::CharacterFormManager;
 using ::mozc::dictionary::PosMatcher;
 
-// Append |src| to |dst| with a separator ' '.
-void AppendString(absl::string_view src, std::string *dst) {
-  DCHECK(dst);
-  if (!src.empty()) {
-    if (!dst->empty()) {
-      dst->append(1, ' ');
-    }
-    dst->append(src.data(), src.size());
-  }
-}
-
 // Returns true if |full| has the corresponding half width form.
-bool IsConvertibleToHalfWidthForm(const std::string &full) {
+bool IsConvertibleToHalfWidthForm(const absl::string_view full) {
   // TODO(b/209357879): remove this line once FullWidthToHalfWidth() itself will
   // support the conversion.
   const std::string tmp =
@@ -88,14 +78,14 @@ bool IsConvertibleToHalfWidthForm(const std::string &full) {
 // "google" => false (not symbol)
 // "㌫" => false (all symbol, all full but not convertible to the corresponding
 // half)
-bool HasCharacterFormDescription(const std::string &value) {
+bool HasCharacterFormDescription(const absl::string_view value) {
   if (value.empty()) {
     return false;
   }
   Util::FormType prev = Util::UNKNOWN_FORM;
 
   for (ConstChar32Iterator iter(value); !iter.Done(); iter.Next()) {
-    const char32 ucs4 = iter.Get();
+    const char32_t ucs4 = iter.Get();
     const Util::FormType type = Util::GetFormType(ucs4);
     if (prev != Util::UNKNOWN_FORM && prev != type) {
       return false;
@@ -115,16 +105,31 @@ bool HasCharacterFormDescription(const std::string &value) {
   return IsConvertibleToHalfWidthForm(value);
 }
 
+// Returns NumberString::Style corresponding to the given form
+NumberUtil::NumberString::Style GetStyle(
+    const NumberUtil::NumberString::Style original_style,
+    CharacterFormManager::FormType form) {
+  switch (original_style) {
+    case NumberUtil::NumberString::NUMBER_SEPARATED_ARABIC_HALFWIDTH:
+    case NumberUtil::NumberString::NUMBER_SEPARATED_ARABIC_FULLWIDTH:
+      return (form == CharacterFormManager::HALF_WIDTH)
+                 ? NumberUtil::NumberString::NUMBER_SEPARATED_ARABIC_HALFWIDTH
+                 : NumberUtil::NumberString::NUMBER_SEPARATED_ARABIC_FULLWIDTH;
+    case NumberUtil::NumberString::NUMBER_ARABIC_AND_KANJI_HALFWIDTH:
+    case NumberUtil::NumberString::NUMBER_ARABIC_AND_KANJI_FULLWIDTH:
+      return (form == CharacterFormManager::HALF_WIDTH)
+                 ? NumberUtil::NumberString::NUMBER_ARABIC_AND_KANJI_HALFWIDTH
+                 : NumberUtil::NumberString::NUMBER_ARABIC_AND_KANJI_FULLWIDTH;
+    default:
+      return original_style;
+  }
+}
+
 }  // namespace
 
-VariantsRewriter::VariantsRewriter(const PosMatcher pos_matcher)
-    : pos_matcher_(pos_matcher) {}
-
-VariantsRewriter::~VariantsRewriter() = default;
-
 // static
 void VariantsRewriter::SetDescriptionForCandidate(
-    const PosMatcher &pos_matcher, Segment::Candidate *candidate) {
+    const PosMatcher pos_matcher, Segment::Candidate *candidate) {
   SetDescription(
       pos_matcher,
       (FULL_HALF_WIDTH | CHARACTER_FORM | ZIPCODE | SPELLING_CORRECTION),
@@ -133,7 +138,7 @@ void VariantsRewriter::SetDescriptionForCandidate(
 
 // static
 void VariantsRewriter::SetDescriptionForTransliteration(
-    const PosMatcher &pos_matcher, Segment::Candidate *candidate) {
+    const PosMatcher pos_matcher, Segment::Candidate *candidate) {
   SetDescription(pos_matcher,
                  (FULL_HALF_WIDTH | CHARACTER_FORM | SPELLING_CORRECTION),
                  candidate);
@@ -141,15 +146,16 @@ void VariantsRewriter::SetDescriptionForTransliteration(
 
 // static
 void VariantsRewriter::SetDescriptionForPrediction(
-    const PosMatcher &pos_matcher, Segment::Candidate *candidate) {
+    const PosMatcher pos_matcher, Segment::Candidate *candidate) {
   SetDescription(pos_matcher, ZIPCODE | SPELLING_CORRECTION, candidate);
 }
 
 // static
-void VariantsRewriter::SetDescription(const PosMatcher &pos_matcher,
+void VariantsRewriter::SetDescription(const PosMatcher pos_matcher,
                                       int description_type,
                                       Segment::Candidate *candidate) {
   absl::string_view character_form_message;
+  std::vector<absl::string_view> pieces;
 
   // Add Character form.
   if (description_type & CHARACTER_FORM) {
@@ -165,14 +171,26 @@ void VariantsRewriter::SetDescription(const PosMatcher &pos_matcher,
       case Util::KATAKANA:
         // character_form_message = "カタカナ";
         character_form_message = kKatakana;
+        // don't need to proactively set full, because katakana is mostly used
+        // in full form.
+        description_type &= ~FULL_HALF_WIDTH;
+        description_type |= HALF_WIDTH;
         break;
       case Util::NUMBER:
         // character_form_message = "数字";
         character_form_message = kNumber;
+        // don't need to proactively set half, because number is mostly used in
+        // half form.
+        description_type &= ~FULL_HALF_WIDTH;
+        description_type |= FULL_WIDTH;
         break;
       case Util::ALPHABET:
         // character_form_message = "アルファベット";
         character_form_message = kAlphabet;
+        // don't need to proactively set half, because alphabet is mostly used
+        // in half form.
+        description_type &= ~FULL_HALF_WIDTH;
+        description_type |= FULL_WIDTH;
         break;
       case Util::KANJI:
       case Util::EMOJI:
@@ -200,48 +218,49 @@ void VariantsRewriter::SetDescription(const PosMatcher &pos_matcher,
     character_form_message = absl::string_view();
   }
 
-  std::string description;
+  const Util::FormType form = Util::GetFormType(candidate->value);
   // full/half char description
   if (description_type & FULL_HALF_WIDTH) {
-    const Util::FormType form = Util::GetFormType(candidate->value);
     switch (form) {
       case Util::FULL_WIDTH:
         // description = "[全]";
-        description = std::string(kFullWidth);
+        pieces.push_back(kFullWidth);
         break;
       case Util::HALF_WIDTH:
         // description = "[半]";
-        description = std::string(kHalfWidth);
+        pieces.push_back(kHalfWidth);
         break;
       default:
         break;
     }
-  } else if (description_type & FULL_WIDTH) {
+  } else if (description_type & FULL_WIDTH && form == Util::FULL_WIDTH) {
     // description = "[全]";
-    description = std::string(kFullWidth);
-  } else if (description_type & HALF_WIDTH) {
+    pieces.push_back(kFullWidth);
+  } else if (description_type & HALF_WIDTH && form == Util::HALF_WIDTH) {
     // description = "[半]";
-    description = std::string(kHalfWidth);
+    pieces.push_back(kHalfWidth);
   }
 
   // add character_form_message
-  AppendString(character_form_message, &description);
+  if (!character_form_message.empty()) {
+    pieces.push_back(character_form_message);
+  }
 
   // add main message
   if (candidate->value == "\\" ||
       candidate->value == "＼") {  // full-width backslash
     // if "\" (half-width backslash) or "＼" ()
-    AppendString("バックスラッシュ", &description);
+    pieces.push_back("バックスラッシュ");
   } else if (candidate->value == "¥") {
     // if "¥" (half-width Yen sign), append kYenKigou.
-    AppendString(kYenKigou, &description);
+    pieces.push_back(kYenKigou);
   } else if (candidate->value == "￥") {
     // if "￥" (full-width Yen sign), append only kYenKigou
-    AppendString(kYenKigou, &description);
+    pieces.push_back(kYenKigou);
   } else if (candidate->value == "~") {
-    AppendString("チルダ", &description);
-  } else {
-    AppendString(candidate->description, &description);
+    pieces.push_back("チルダ");
+  } else if (!candidate->description.empty()) {
+    pieces.push_back(candidate->description);
   }
 
   // The following description tries to overwrite existing description.
@@ -249,9 +268,15 @@ void VariantsRewriter::SetDescription(const PosMatcher &pos_matcher,
   // Zipcode description
   if ((description_type & ZIPCODE) && pos_matcher.IsZipcode(candidate->lid) &&
       candidate->lid == candidate->rid) {
-    description = candidate->content_key;
+    if (!candidate->content_key.empty()) {
+      pieces = {candidate->content_key};
+    } else {
+      pieces.clear();
+    }
     // Append default description because it may contain extra description.
-    AppendString(candidate->description, &description);
+    if (!candidate->description.empty()) {
+      pieces.push_back(candidate->description);
+    }
   }
 
   // The following description tries to overwrite existing description.
@@ -259,15 +284,18 @@ void VariantsRewriter::SetDescription(const PosMatcher &pos_matcher,
   // Spelling Correction description
   if ((description_type & SPELLING_CORRECTION) &&
       (candidate->attributes & Segment::Candidate::SPELLING_CORRECTION)) {
-    description = std::string(kDidYouMean);
     // Add prefix to distinguish this candidate.
     candidate->prefix = "→ ";
     // Append default description because it may contain extra description.
-    AppendString(candidate->description, &description);
+    if (candidate->description.empty()) {
+      pieces = {kDidYouMean};
+    } else {
+      pieces = {kDidYouMean, candidate->description};
+    }
   }
 
   // set new description
-  candidate->description = std::move(description);
+  candidate->description = absl::StrJoin(pieces, " ");
   candidate->attributes |= Segment::Candidate::NO_EXTRA_DESCRIPTION;
 }
 
@@ -355,35 +383,39 @@ bool VariantsRewriter::RewriteSegment(RewriteType type, Segment *seg) const {
       Segment::Candidate *new_candidate = seg->insert_candidate(i);
       DCHECK(new_candidate);
 
-      new_candidate->Init();
       new_candidate->key = original_candidate->key;
-      new_candidate->value = default_value;
+      new_candidate->value = std::move(default_value);
       new_candidate->content_key = original_candidate->content_key;
-      new_candidate->content_value = default_content_value;
+      new_candidate->content_value = std::move(default_content_value);
       new_candidate->consumed_key_size = original_candidate->consumed_key_size;
       new_candidate->cost = original_candidate->cost;
       new_candidate->structure_cost = original_candidate->structure_cost;
       new_candidate->lid = original_candidate->lid;
       new_candidate->rid = original_candidate->rid;
       new_candidate->description = original_candidate->description;
-      new_candidate->inner_segment_boundary.swap(
-          default_inner_segment_boundary);
+      new_candidate->inner_segment_boundary =
+          std::move(default_inner_segment_boundary);
       new_candidate->attributes = original_candidate->attributes;
+      new_candidate->style = GetStyle(original_candidate->style, default_form);
       SetDescription(pos_matcher_, default_description_type, new_candidate);
 
-      original_candidate->value = alternative_value;
-      original_candidate->content_value = alternative_content_value;
-      original_candidate->inner_segment_boundary.swap(
-          alternative_inner_segment_boundary);
+      original_candidate->value = std::move(alternative_value);
+      original_candidate->content_value = std::move(alternative_content_value);
+      original_candidate->inner_segment_boundary =
+          std::move(alternative_inner_segment_boundary);
+      original_candidate->style =
+          GetStyle(original_candidate->style, alternative_form);
       SetDescription(pos_matcher_, alternative_description_type,
                      original_candidate);
       ++i;  // skip inserted candidate
     } else if (type == SELECT_VARIANT) {
       // Rewrite original to default
-      original_candidate->value = default_value;
-      original_candidate->content_value = default_content_value;
-      original_candidate->inner_segment_boundary.swap(
-          default_inner_segment_boundary);
+      original_candidate->value = std::move(default_value);
+      original_candidate->content_value = std::move(default_content_value);
+      original_candidate->inner_segment_boundary =
+          std::move(default_inner_segment_boundary);
+      original_candidate->style =
+          GetStyle(original_candidate->style, default_form);
       SetDescription(pos_matcher_, default_description_type,
                      original_candidate);
     }
@@ -429,8 +461,8 @@ bool VariantsRewriter::GenerateAlternatives(
           original.content_value, default_content_value,
           alternative_content_value);
     } else {
-      default_content_value->assign(*default_value);
-      alternative_content_value->assign(*alternative_value);
+      *default_content_value = *default_value;
+      *alternative_content_value = *alternative_value;
     }
     return true;
   }
@@ -439,15 +471,14 @@ bool VariantsRewriter::GenerateAlternatives(
   // least one inner segment is rewritten, the whole segment is considered
   // rewritten.
   bool at_least_one_modified = false;
-  std::string tmp, inner_default_value, inner_alternative_value;
+  std::string inner_default_value, inner_alternative_value;
   std::string inner_default_content_value, inner_alternative_content_value;
   for (Segment::Candidate::InnerSegmentIterator iter(&original); !iter.Done();
        iter.Next()) {
-    tmp.assign(iter.GetValue().data(), iter.GetValue().size());
     inner_default_value.clear();
     inner_alternative_value.clear();
     if (!manager->ConvertConversionStringWithAlternative(
-            tmp, &inner_default_value, &inner_alternative_value)) {
+            iter.GetValue(), &inner_default_value, &inner_alternative_value)) {
       inner_default_value.assign(iter.GetValue().data(),
                                  iter.GetValue().size());
       inner_alternative_value.assign(iter.GetValue().data(),
@@ -458,17 +489,17 @@ bool VariantsRewriter::GenerateAlternatives(
     if (iter.GetValue() != iter.GetContentValue()) {
       inner_default_content_value.clear();
       inner_alternative_content_value.clear();
-      tmp.assign(iter.GetContentValue().data(), iter.GetContentValue().size());
       manager->ConvertConversionStringWithAlternative(
-          tmp, &inner_default_content_value, &inner_alternative_content_value);
+          iter.GetContentValue(), &inner_default_content_value,
+          &inner_alternative_content_value);
     } else {
       inner_default_content_value = inner_default_value;
       inner_alternative_content_value = inner_alternative_value;
     }
-    default_value->append(inner_default_value);
-    alternative_value->append(inner_alternative_value);
-    default_content_value->append(inner_default_content_value);
-    alternative_content_value->append(inner_alternative_content_value);
+    absl::StrAppend(default_value, inner_default_value);
+    absl::StrAppend(alternative_value, inner_alternative_value);
+    absl::StrAppend(default_content_value, inner_default_content_value);
+    absl::StrAppend(alternative_content_value, inner_alternative_content_value);
     default_inner_segment_boundary->push_back(Segment::Candidate::EncodeLengths(
         iter.GetKey().size(), inner_default_value.size(),
         iter.GetContentKey().size(), inner_default_content_value.size()));
@@ -483,6 +514,11 @@ bool VariantsRewriter::GenerateAlternatives(
 
 void VariantsRewriter::Finish(const ConversionRequest &request,
                               Segments *segments) {
+  if (request.config().history_learning_level() !=
+      config::Config::DEFAULT_HISTORY) {
+    VLOG(2) << "history_learning_level is not DEFAULT_HISTORY";
+    return;
+  }
   if (!request.request().mixed_conversion() &&
       request.request_type() != ConversionRequest::CONVERSION) {
     return;
