@@ -33,41 +33,45 @@
 #include "session/session_handler.h"
 
 #include <algorithm>
+#include <cstddef>
 #include <cstdint>
-#include <string>
+#include <limits>
+#include <memory>
 #include <utility>
 #include <vector>
 
 #include "base/clock.h"
 #include "base/logging.h"
-#include "base/port.h"
-#include "absl/flags/flag.h"
-#ifndef MOZC_DISABLE_SESSION_WATCHDOG
-#include "base/process.h"
-#endif  // MOZC_DISABLE_SESSION_WATCHDOG
-#include "base/singleton.h"
 #include "base/stopwatch.h"
-#include "base/util.h"
 #include "composer/table.h"
 #include "config/character_form_manager.h"
 #include "config/config_handler.h"
 #include "dictionary/user_dictionary_session_handler.h"
+#include "engine/engine_builder_interface.h"
 #include "engine/engine_interface.h"
 #include "engine/user_data_manager_interface.h"
 #include "protocol/commands.pb.h"
 #include "protocol/config.pb.h"
+#include "protocol/engine_builder.pb.h"
 #include "protocol/user_dictionary_storage.pb.h"
+#include "session/common.h"
+#include "session/internal/keymap.h"
 #include "session/session.h"
+#include "session/session_interface.h"
 #include "session/session_observer_handler.h"
+#include "session/session_observer_interface.h"
+#include "usage_stats/usage_stats.h"
+#include "absl/flags/flag.h"
+#include "absl/random/random.h"
+#include "absl/time/time.h"
+
 #ifndef MOZC_DISABLE_SESSION_WATCHDOG
+#include "base/process.h"
 #include "session/session_watch_dog.h"
 #endif  // MOZC_DISABLE_SESSION_WATCHDOG
-#include <memory>
-
-#include "usage_stats/usage_stats.h"
-
-using mozc::usage_stats::UsageStats;
 
+// TODO(b/275437228): Convert this to use `absl::Duration`. Note that existing
+// clients assume a negative value means we do not timeout at all.
 ABSL_FLAG(int32_t, timeout, -1,
           "server timeout. "
           "if sessions get empty for \"timeout\", "
@@ -78,15 +82,19 @@ ABSL_FLAG(int32_t, max_session_size, 64,
           "if size of sessions reaches to \"max_session_size\", "
           "oldest session is removed");
 
+// TODO(b/275437228): Convert this to `absl::Duration`.
 ABSL_FLAG(int32_t, create_session_min_interval, 0,
           "minimum interval (sec) for create session");
 
+// TODO(b/275437228): Convert this to `absl::Duration`.
 ABSL_FLAG(int32_t, watch_dog_interval, 180, "watch dog timer intaval (sec)");
 
+// TODO(b/275437228): Convert this to `absl::Duration`.
 ABSL_FLAG(int32_t, last_command_timeout, 3600,
           "remove session if it is not accessed for "
           "\"last_command_timeout\" sec");
 
+// TODO(b/275437228): Convert this to `absl::Duration`.
 ABSL_FLAG(int32_t, last_create_session_timeout, 300,
           "remove session if it is not accessed for "
           "\"last_create_session_timeout\" sec "
@@ -95,8 +103,10 @@ ABSL_FLAG(int32_t, last_create_session_timeout, 300,
 ABSL_FLAG(bool, restricted, false, "Launch server with restricted setting");
 
 namespace mozc {
-
 namespace {
+
+using mozc::usage_stats::UsageStats;
+
 bool IsApplicationAlive(const session::SessionInterface *session) {
 #ifndef MOZC_DISABLE_SESSION_WATCHDOG
   const commands::ApplicationInfo &info = session->application_info();
@@ -106,16 +116,16 @@ bool IsApplicationAlive(const session::SessionInterface *session) {
   // Here, we want to kill the session only when the target thread/process
   // are terminated with 100% probability. Otherwise, it's better to do
   // nothing to prevent any side effects.
-#ifdef OS_WIN
+#ifdef _WIN32
   if (info.has_thread_id()) {
     return Process::IsThreadAlive(static_cast<size_t>(info.thread_id()), true);
   }
-#else   // OS_WIN
+#else   // _WIN32
   if (info.has_process_id()) {
     return Process::IsProcessAlive(static_cast<size_t>(info.process_id()),
                                    true);
   }
-#endif  // OS_WIN
+#endif  // _WIN32
 #endif  // MOZC_DISABLE_SESSION_WATCHDOG
   return true;
 }
@@ -136,18 +146,18 @@ void SessionHandler::Init(
     std::unique_ptr<EngineBuilderInterface> engine_builder) {
   is_available_ = false;
   max_session_size_ = 0;
-  last_session_empty_time_ = Clock::GetTime();
-  last_cleanup_time_ = 0;
-  last_create_session_time_ = 0;
+  last_session_empty_time_ = Clock::GetAbslTime();
+  last_cleanup_time_ = absl::InfinitePast();
+  last_create_session_time_ = absl::InfinitePast();
   engine_ = std::move(engine);
   engine_builder_ = std::move(engine_builder);
   observer_handler_ = std::make_unique<session::SessionObserverHandler>();
-  stopwatch_ = std::make_unique<Stopwatch>();
   user_dictionary_session_handler_ =
       std::make_unique<user_dictionary::UserDictionarySessionHandler>();
   table_manager_ = std::make_unique<composer::TableManager>();
   request_ = std::make_unique<commands::Request>();
-  config_ = std::make_unique<config::Config>();
+  config_ = config::ConfigHandler::GetConfig();
+  key_map_manager_ = std::make_unique<keymap::KeyMapManager>(*config_);
 
   if (absl::GetFlag(FLAGS_restricted)) {
     VLOG(1) << "Server starts with restricted mode";
@@ -163,13 +173,6 @@ void SessionHandler::Init(
     absl::SetFlag(&FLAGS_last_command_timeout, 60);
   }
 
-#ifndef MOZC_DISABLE_SESSION_WATCHDOG
-  session_watch_dog_ = std::make_unique<SessionWatchDog>(
-      absl::GetFlag(FLAGS_watch_dog_interval));
-#endif  // MOZC_DISABLE_SESSION_WATCHDOG
-
-  config::ConfigHandler::GetConfig(config_.get());
-
   // allow [2..128] sessions
   max_session_size_ =
       std::max(2, std::min(absl::GetFlag(FLAGS_max_session_size), 128));
@@ -191,50 +194,62 @@ SessionHandler::~SessionHandler() {
     element->value = nullptr;
   }
   session_map_->Clear();
-#ifndef MOZC_DISABLE_SESSION_WATCHDOG
-  if (session_watch_dog_->IsRunning()) {
-    session_watch_dog_->Terminate();
-  }
-#endif  // MOZC_DISABLE_SESSION_WATCHDOG
 }
 
 bool SessionHandler::IsAvailable() const { return is_available_; }
 
-bool SessionHandler::StartWatchDog() {
+void SessionHandler::StartWatchDog() {
 #ifndef MOZC_DISABLE_SESSION_WATCHDOG
-  if (!session_watch_dog_->IsRunning()) {
-    session_watch_dog_->Start("WatchDog");
+  if (!session_watch_dog_.has_value()) {
+    session_watch_dog_.emplace(
+        absl::Seconds(absl::GetFlag(FLAGS_watch_dog_interval)));
   }
-  return session_watch_dog_->IsRunning();
-#else   // MOZC_DISABLE_SESSION_WATCHDOG
-  return false;
 #endif  // MOZC_DISABLE_SESSION_WATCHDOG
 }
 
-void SessionHandler::SetConfig(const config::Config &config) {
-  *config_ = config;
+void SessionHandler::UpdateSessions(const config::Config &config,
+                                    const commands::Request &request) {
+  auto new_config = std::make_unique<config::Config>(config);
+  auto new_request = std::make_unique<commands::Request>(request);
   const auto *data_manager = engine_->GetDataManager();
   const composer::Table *table =
       data_manager != nullptr
-          ? table_manager_->GetTable(*request_, *config_, *data_manager)
+          ? table_manager_->GetTable(*new_request, *new_config, *data_manager)
           : nullptr;
+  auto new_key_map_manager =
+      keymap::KeyMapManager::IsSameKeyMapManagerApplicable(*config_,
+                                                           *new_config)
+          ? nullptr
+          : std::make_unique<keymap::KeyMapManager>(*new_config);
+
   for (SessionElement *element =
            const_cast<SessionElement *>(session_map_->Head());
        element != nullptr; element = element->next) {
     if (element->value != nullptr) {
-      element->value->SetConfig(config_.get());
-      element->value->SetRequest(request_.get());
+      element->value->SetConfig(new_config.get());
+      element->value->SetKeyMapManager(
+          (new_key_map_manager ? new_key_map_manager : key_map_manager_).get());
+      element->value->SetRequest(new_request.get());
       if (table != nullptr) {
         element->value->SetTable(table);
       }
     }
   }
-  config::CharacterFormManager::GetCharacterFormManager()->ReloadConfig(config);
+  config::CharacterFormManager::GetCharacterFormManager()->ReloadConfig(
+      *new_config);
+  // Now no references to the current config/key_map_manager/request
+  // should exist. We can destroy them here.
+  config_ = std::move(new_config);
+  if (new_key_map_manager) {
+    key_map_manager_ = std::move(new_key_map_manager);
+  }
+  request_ = std::move(new_request);
 }
 
 bool SessionHandler::SyncData(commands::Command *command) {
   VLOG(1) << "Syncing user data";
   engine_->GetUserDataManager()->Sync();
+  engine_->GetUserDataManager()->Wait();
   return true;
 }
 
@@ -248,15 +263,18 @@ bool SessionHandler::Shutdown(commands::Command *command) {
 
 bool SessionHandler::Reload(commands::Command *command) {
   VLOG(1) << "Reloading server";
-  {
-    config::Config config;
-    config::ConfigHandler::GetConfig(&config);
-    SetConfig(config);
-  }
+  UpdateSessions(*config::ConfigHandler::GetConfig(), *request_);
   engine_->Reload();
   return true;
 }
 
+bool SessionHandler::ReloadAndWait(commands::Command *command) {
+  VLOG(1) << "Reloading server and wait for reloader";
+  UpdateSessions(*config::ConfigHandler::GetConfig(), *request_);
+  engine_->ReloadAndWait();
+  return true;
+}
+
 bool SessionHandler::ClearUserHistory(commands::Command *command) {
   VLOG(1) << "Clearing user history";
   engine_->GetUserDataManager()->ClearUserHistory();
@@ -278,25 +296,16 @@ bool SessionHandler::ClearUnusedUserPrediction(commands::Command *command) {
   return true;
 }
 
-bool SessionHandler::GetStoredConfig(commands::Command *command) {
-  VLOG(1) << "Getting stored config";
-  // Use GetStoredConfig instead of GetConfig because GET_CONFIG
-  // command should return raw stored config, which is not
-  // affected by imposed config.
-  if (!config::ConfigHandler::GetStoredConfig(
-          command->mutable_output()->mutable_config())) {
-    LOG(WARNING) << "cannot get config";
-    return false;
-  }
-
+bool SessionHandler::GetConfig(commands::Command *command) {
+  VLOG(1) << "Getting config";
+  config::ConfigHandler::GetConfig(command->mutable_output()->mutable_config());
   // Ensure the onmemory config is same as the locally stored one
   // because the local data could be changed by sync.
-  SetConfig(command->output().config());
-
+  UpdateSessions(command->output().config(), *request_);
   return true;
 }
 
-bool SessionHandler::SetStoredConfig(commands::Command *command) {
+bool SessionHandler::SetConfig(commands::Command *command) {
   VLOG(1) << "Setting user config";
   if (!command->input().has_config()) {
     LOG(WARNING) << "config is empty";
@@ -304,38 +313,19 @@ bool SessionHandler::SetStoredConfig(commands::Command *command) {
   }
 
   *command->mutable_output()->mutable_config() = command->input().config();
-  MaybeUpdateStoredConfig(command);
+  MaybeUpdateConfig(command);
 
   UsageStats::IncrementCount("SetConfig");
   return true;
 }
 
-bool SessionHandler::SetImposedConfig(commands::Command *command) {
-  VLOG(1) << "Setting imposed config";
-  if (!command->input().has_config()) {
-    LOG(WARNING) << "config is empty";
-    return false;
-  }
-
-  const mozc::config::Config &config = command->input().config();
-  config::ConfigHandler::SetImposedConfig(config);
-
-  Reload(command);
-
-  return true;
-}
-
 bool SessionHandler::SetRequest(commands::Command *command) {
   VLOG(1) << "Setting client's request";
   if (!command->input().has_request()) {
     LOG(WARNING) << "request is empty";
     return false;
   }
-
-  *request_ = command->input().request();
-
-  Reload(command);
-
+  UpdateSessions(*config_, command->input().request());
   return true;
 }
 
@@ -346,8 +336,8 @@ bool SessionHandler::EvalCommand(commands::Command *command) {
   }
 
   bool eval_succeeded = false;
-  stopwatch_->Reset();
-  stopwatch_->Start();
+  Stopwatch stopwatch;
+  stopwatch.Start();
 
   switch (command->input().type()) {
     case commands::Input::CREATE_SESSION:
@@ -378,13 +368,10 @@ bool SessionHandler::EvalCommand(commands::Command *command) {
       eval_succeeded = ClearUnusedUserPrediction(command);
       break;
     case commands::Input::GET_CONFIG:
-      eval_succeeded = GetStoredConfig(command);
+      eval_succeeded = GetConfig(command);
       break;
     case commands::Input::SET_CONFIG:
-      eval_succeeded = SetStoredConfig(command);
-      break;
-    case commands::Input::SET_IMPOSED_CONFIG:
-      eval_succeeded = SetImposedConfig(command);
+      eval_succeeded = SetConfig(command);
       break;
     case commands::Input::SET_REQUEST:
       eval_succeeded = SetRequest(command);
@@ -395,6 +382,9 @@ bool SessionHandler::EvalCommand(commands::Command *command) {
     case commands::Input::RELOAD:
       eval_succeeded = Reload(command);
       break;
+    case commands::Input::RELOAD_AND_WAIT:
+      eval_succeeded = ReloadAndWait(command);
+      break;
     case commands::Input::CLEANUP:
       eval_succeeded = Cleanup(command);
       break;
@@ -435,9 +425,10 @@ bool SessionHandler::EvalCommand(commands::Command *command) {
     observer_handler_->EvalCommandHandler(*command);
   }
 
-  stopwatch_->Stop();
-  UsageStats::UpdateTiming("ElapsedTimeUSec",
-                           stopwatch_->GetElapsedMicroseconds());
+  stopwatch.Stop();
+  UsageStats::UpdateTiming(
+      "ElapsedTimeUSec",
+      static_cast<uint32_t>(absl::ToInt64Microseconds(stopwatch.GetElapsed())));
 
   return is_available_;
 }
@@ -451,13 +442,12 @@ void SessionHandler::AddObserver(session::SessionObserverInterface *observer) {
   observer_handler_->AddObserver(observer);
 }
 
-void SessionHandler::MaybeUpdateStoredConfig(commands::Command *command) {
+void SessionHandler::MaybeUpdateConfig(commands::Command *command) {
   if (!command->output().has_config()) {
     return;
   }
 
-  *config_ = command->output().config();
-  config::ConfigHandler::SetConfig(*config_);
+  config::ConfigHandler::SetConfig(command->output().config());
   Reload(command);
 }
 
@@ -469,7 +459,7 @@ bool SessionHandler::SendKey(commands::Command *command) {
     return false;
   }
   (*session)->SendKey(command);
-  MaybeUpdateStoredConfig(command);
+  MaybeUpdateConfig(command);
   return true;
 }
 
@@ -493,20 +483,25 @@ bool SessionHandler::SendCommand(commands::Command *command) {
     return false;
   }
   (*session)->SendCommand(command);
-  MaybeUpdateStoredConfig(command);
+  MaybeUpdateConfig(command);
   return true;
 }
 
 bool SessionHandler::CreateSession(commands::Command *command) {
   // prevent DOS attack
   // don't allow CreateSession in very short period.
-  const int create_session_minimum_interval = std::max(
-      0, std::min(absl::GetFlag(FLAGS_create_session_min_interval), 10));
+  const absl::Duration create_session_minimum_interval = std::max(
+      absl::ZeroDuration(),
+      std::min(absl::Seconds(absl::GetFlag(FLAGS_create_session_min_interval)),
+               absl::Seconds(10)));
 
-  uint64_t current_time = Clock::GetTime();
-  if (last_create_session_time_ != 0 &&
-      (current_time - last_create_session_time_) <
-          create_session_minimum_interval) {
+  const absl::Time current_time = Clock::GetAbslTime();
+  const absl::Duration create_session_interval =
+      current_time - last_create_session_time_;
+  // `create_session_interval` can be nagative if a user modifies their system
+  // clock.
+  if (create_session_interval >= absl::Duration() &&
+      create_session_interval < create_session_minimum_interval) {
     return false;
   }
 
@@ -521,7 +516,7 @@ bool SessionHandler::CreateSession(commands::Command *command) {
       return false;
     }
     delete oldest_element->value;
-    oldest_element->value = NULL;
+    oldest_element->value = nullptr;
     session_map_->Erase(oldest_element->key);
     VLOG(1) << "Session is FULL, oldest SessionID " << oldest_element->key
             << " is removed";
@@ -567,16 +562,14 @@ bool SessionHandler::CreateSession(commands::Command *command) {
     session->set_application_info(command->input().application_info());
   }
 
-  // Ensure the onmemory config is same as the locally stored one
-  // because the local data could be changed by sync.
-  {
-    config::Config config;
-    config::ConfigHandler::GetConfig(&config);
-    SetConfig(config);
-  }
+  // The created session has not been fully initialized yet.
+  // SetConfig() will complete the initialization by setting information
+  // (e.g., config, request, keymap, ...) to all the sessions,
+  // including the newly created one.
+  UpdateSessions(*config::ConfigHandler::GetConfig(), *request_);
 
   // session is not empty.
-  last_session_empty_time_ = 0;
+  last_session_empty_time_ = absl::InfinitePast();
 
   UsageStats::IncrementCount("SessionCreated");
 
@@ -599,31 +592,35 @@ bool SessionHandler::DeleteSession(commands::Command *command) {
 // no active session and client doesn't send any conversion
 // request to the server for FLAGS_timeout sec.
 bool SessionHandler::Cleanup(commands::Command *command) {
-  const uint64_t current_time = Clock::GetTime();
+  const absl::Time current_time = Clock::GetAbslTime();
 
   // suspend/hibernation may happen
-  uint64_t suspend_time = 0;
+  absl::Duration suspend_time = absl::ZeroDuration();
 #ifndef MOZC_DISABLE_SESSION_WATCHDOG
-  if (last_cleanup_time_ != 0 && session_watch_dog_->IsRunning() &&
+  if (last_cleanup_time_ != absl::InfinitePast() &&
+      session_watch_dog_.has_value() &&
       (current_time - last_cleanup_time_) >
           2 * session_watch_dog_->interval()) {
     suspend_time =
         current_time - last_cleanup_time_ - session_watch_dog_->interval();
-    LOG(WARNING) << "server went to suspend mode for " << suspend_time
-                 << " sec";
+    LOG(WARNING) << "server went to suspend mode for " << suspend_time;
   }
 #endif  // MOZC_DISABLE_SESSION_WATCHDOG
 
   // allow [1..600] sec. default: 300
-  const uint64_t create_session_timeout =
-      suspend_time +
-      std::max(1,
-               std::min(absl::GetFlag(FLAGS_last_create_session_timeout), 600));
+  const absl::Duration create_session_timeout =
+      suspend_time + std::max(absl::Seconds(1),
+                              std::min(absl::Seconds(absl::GetFlag(
+                                           FLAGS_last_create_session_timeout)),
+                                       absl::Seconds(600)));
 
   // allow [10..7200] sec. default 3600
-  const uint64_t last_command_timeout =
+  const absl::Duration last_command_timeout =
       suspend_time +
-      std::max(10, std::min(absl::GetFlag(FLAGS_last_command_timeout), 7200));
+      std::max(
+          absl::Seconds(10),
+          std::min(absl::Seconds(absl::GetFlag(FLAGS_last_command_timeout)),
+                   absl::Seconds(7200)));
 
   std::vector<SessionID> remove_ids;
   for (SessionElement *element =
@@ -633,7 +630,7 @@ bool SessionHandler::Cleanup(commands::Command *command) {
     if (!IsApplicationAlive(session)) {
       VLOG(2) << "Application is not alive. Removing: " << element->key;
       remove_ids.push_back(element->key);
-    } else if (session->last_command_time() == 0) {
+    } else if (session->last_command_time() == absl::InfinitePast()) {
       // no command is exectuted
       if ((current_time - session->create_session_time()) >=
           create_session_timeout) {
@@ -656,9 +653,9 @@ bool SessionHandler::Cleanup(commands::Command *command) {
   engine_->GetUserDataManager()->Sync();
 
   // timeout is enabled.
-  if (absl::GetFlag(FLAGS_timeout) > 0 && last_session_empty_time_ != 0 &&
+  if (absl::GetFlag(FLAGS_timeout) > 0 &&
       (current_time - last_session_empty_time_) >=
-          suspend_time + absl::GetFlag(FLAGS_timeout)) {
+          suspend_time + absl::Seconds(absl::GetFlag(FLAGS_timeout))) {
     Shutdown(command);
   }
 
@@ -708,19 +705,17 @@ bool SessionHandler::ReloadSpellChecker(commands::Command *command) {
 
 // Create Random Session ID in order to make the session id unpredicable
 SessionID SessionHandler::CreateNewSessionID() {
-  SessionID id = 0;
   while (true) {
-    Util::GetRandomSequence(reinterpret_cast<char *>(&id), sizeof(id));
-    // don't allow id == 0, as it is reserved for
-    // "invalid id"
-    if (id != 0 && !session_map_->HasKey(id)) {
-      break;
+    // don't allow id == 0, as it is reserved for "invalid id"
+    const SessionID id =
+        absl::Uniform<SessionID>(absl::IntervalClosed, bitgen_, 1,
+                                 std::numeric_limits<SessionID>::max());
+    if (!session_map_->HasKey(id)) {
+      return id;
     }
 
     LOG(WARNING) << "Session ID " << id << " is already used. retry";
   }
-
-  return id;
 }
 
 bool SessionHandler::DeleteSessionID(SessionID id) {
@@ -734,8 +729,9 @@ bool SessionHandler::DeleteSessionID(SessionID id) {
   session_map_->Erase(id);  // remove from LRU
 
   // if session gets empty, save the timestamp
-  if (last_session_empty_time_ == 0 && session_map_->Size() == 0) {
-    last_session_empty_time_ = Clock::GetTime();
+  if (last_session_empty_time_ == absl::InfinitePast() &&
+      session_map_->Size() == 0) {
+    last_session_empty_time_ = Clock::GetAbslTime();
   }
 
   return true;
