@@ -33,14 +33,14 @@
 // session_handler_main --logtostderr --input input.txt --profile /tmp/mozc
 //                      --dictionary oss --engine desktop
 //
-/* Example of input.txt
+/* Example of input.txt (tsv format)
 # Enable IME
 SEND_KEY        ON
 
 SET_MOBILE_REQUEST
 
 RESET_CONTEXT
-UPDATE_MOBILE_KEYBOARD  QWERTY_MOBILE_TO_HIRAGANA
+UPDATE_MOBILE_KEYBOARD  QWERTY_MOBILE_TO_HIRAGANA        COMMIT
 SWITCH_INPUT_MODE       HIRAGANA
 
 SEND_KEYS       arigatou
@@ -54,10 +54,15 @@ SHOW_LOG_BY_VALUE       ございました
 
 #include <cstdint>
 #include <iostream>
+#include <memory>
+#include <ostream>
 #include <string>
+#include <utility>
+#include <vector>
 
 #include "base/file_stream.h"
 #include "base/init_mozc.h"
+#include "base/protobuf/message.h"
 #include "base/system_util.h"
 #include "data_manager/oss/oss_data_manager.h"
 #include "engine/engine.h"
@@ -66,8 +71,8 @@ SHOW_LOG_BY_VALUE       ございました
 #include "session/session_handler_tool.h"
 #include "absl/flags/flag.h"
 #include "absl/status/status.h"
-#include "absl/strings/str_join.h"
-#include "absl/strings/str_split.h"
+#include "absl/status/statusor.h"
+#include "absl/strings/numbers.h"
 
 ABSL_FLAG(std::string, input, "", "Input file");
 ABSL_FLAG(std::string, profile, "", "User profile directory");
@@ -112,8 +117,30 @@ void ParseLine(session::SessionHandlerInterpreter &handler, std::string line) {
   }
   const std::string &command = args[0];
 
+  if (command == "SHOW_ALL") {
+    std::cout << protobuf::Utf8Format(handler.LastOutput()) << std::endl;
+    return;
+  }
   if (command == "SHOW_OUTPUT") {
-    std::cout << handler.LastOutput().Utf8DebugString() << std::endl;
+    commands::Output output = handler.LastOutput();
+    output.mutable_removed_candidate_words_for_debug()->Clear();
+    std::cout << protobuf::Utf8Format(output) << std::endl;
+    return;
+  }
+  if (command == "SHOW_RESULT") {
+    commands::Output output = handler.LastOutput();
+    std::cout << protobuf::Utf8Format(output.result()) << std::endl;
+    return;
+  }
+  if (command == "SHOW_CANDIDATES") {
+    std::cout << protobuf::Utf8Format(handler.LastOutput().candidates())
+              << std::endl;
+    return;
+  }
+  if (command == "SHOW_REMOVED_CANDIDATES") {
+    std::cout << protobuf::Utf8Format(
+                     handler.LastOutput().removed_candidate_words_for_debug())
+              << std::endl;
     return;
   }
   if (command == "SHOW") {
@@ -130,17 +157,22 @@ void ParseLine(session::SessionHandlerInterpreter &handler, std::string line) {
     return;
   }
   if (command == "SHOW_LOG_BY_VALUE") {
-    uint32_t id;
-    if (args.size() == 2 && handler.GetCandidateIdByValue(args[1], &id)) {
-      ShowLog(handler.LastOutput(), id);
-    } else {
+    if (args.size() != 2) {
       std::cout << "ERROR: " << line << std::endl;
+      return;
+    }
+    for (const uint32_t id : handler.GetCandidateIdsByValue(args[1])) {
+      ShowLog(handler.LastOutput(), id);
+    }
+    for (const uint32_t id : handler.GetRemovedCandidateIdsByValue(args[1])) {
+      ShowLog(handler.LastOutput(), id);
     }
     return;
   }
 
   const absl::Status status = handler.Eval(args);
   if (!status.ok()) {
+    std::cout << "ERROR: " << line << std::endl;
     std::cout << "ERROR: " << status.message() << std::endl;
   }
 }
@@ -188,7 +220,7 @@ int main(int argc, char **argv) {
 
   std::string line;
   if (!absl::GetFlag(FLAGS_input).empty()) {
-    mozc::InputFileStream input(absl::GetFlag(FLAGS_input).c_str());
+    mozc::InputFileStream input(absl::GetFlag(FLAGS_input));
     while (std::getline(input, line)) {
       mozc::ParseLine(handler, line);
     }
