@@ -32,6 +32,8 @@
 
 #include <cstdint>
 #include <memory>
+#include <string>
+#include <vector>
 
 #include "engine/engine_interface.h"
 #include "engine/user_data_manager_interface.h"
@@ -39,7 +41,10 @@
 #include "protocol/commands.pb.h"
 #include "protocol/config.pb.h"
 #include "session/session_handler_interface.h"
+#include "session/session_observer_interface.h"
 #include "absl/status/status.h"
+#include "absl/strings/string_view.h"
+#include "absl/types/span.h"
 
 namespace mozc {
 namespace session {
@@ -48,7 +53,8 @@ namespace session {
 class SessionHandlerTool {
  public:
   explicit SessionHandlerTool(std::unique_ptr<EngineInterface> engine);
-  ~SessionHandlerTool();
+  SessionHandlerTool(const SessionHandlerTool &) = delete;
+  SessionHandlerTool &operator=(const SessionHandlerTool &) = delete;
 
   bool CreateSession();
   bool DeleteSession();
@@ -69,7 +75,6 @@ class SessionHandlerTool {
                              commands::Output *output);
   bool SelectCandidate(uint32_t id, commands::Output *output);
   bool SubmitCandidate(uint32_t id, commands::Output *output);
-  bool ExpandSuggestion(commands::Output *output);
 
   bool Reload();
   bool ResetContext();
@@ -78,7 +83,7 @@ class SessionHandlerTool {
   bool SetRequest(const commands::Request &request, commands::Output *output);
   bool SetConfig(const config::Config &config, commands::Output *output);
   bool SyncData();
-  void SetCallbackText(const std::string &text);
+  void SetCallbackText(absl::string_view text);
 
  private:
   bool EvalCommand(commands::Input *input, commands::Output *output);
@@ -90,15 +95,13 @@ class SessionHandlerTool {
   UserDataManagerInterface *data_manager_;
   std::unique_ptr<SessionHandlerInterface> handler_;
   std::string callback_text_;
-
-  DISALLOW_COPY_AND_ASSIGN(SessionHandlerTool);
 };
 
-class SessionHandlerInterpreter {
+class SessionHandlerInterpreter final {
  public:
   SessionHandlerInterpreter();
   explicit SessionHandlerInterpreter(std::unique_ptr<EngineInterface> engine);
-  virtual ~SessionHandlerInterpreter();
+  ~SessionHandlerInterpreter();
 
   void ClearState();
   void ClearAll();
@@ -106,15 +109,18 @@ class SessionHandlerInterpreter {
   void SyncDataToStorage();
   void ClearUserPrediction();
   void ClearUsageStats();
-  const commands::Output& LastOutput() const;
-  const commands::CandidateWord& GetCandidateByValue(
-      const absl::string_view value);
-  bool GetCandidateIdByValue(const absl::string_view value, uint32_t *id);
-  std::vector<std::string> Parse(const std::string &line);
-  absl::Status Eval(const std::vector<std::string> &args);
+  const commands::Output &LastOutput() const;
+  const commands::CandidateWord &GetCandidateByValue(
+      absl::string_view value) const;
+  bool GetCandidateIdByValue(absl::string_view value, uint32_t *id) const;
+  std::vector<uint32_t> GetCandidateIdsByValue(absl::string_view value) const;
+  std::vector<uint32_t> GetRemovedCandidateIdsByValue(
+      absl::string_view value) const;
+  std::vector<std::string> Parse(absl::string_view line);
+  absl::Status Eval(absl::Span<const std::string> args);
+  void SetRequest(const commands::Request &request);
 
  private:
-  // std::unique_ptr<EngineInterface> engine_;
   std::unique_ptr<SessionHandlerTool> client_;
   std::unique_ptr<config::Config> config_;
   std::unique_ptr<commands::Output> last_output_;
