@@ -34,29 +34,23 @@
 
 #include <vector>
 
-#include "base/port.h"
+#include "protocol/commands.pb.h"
+#include "session/session_observer_interface.h"
 
 namespace mozc {
-namespace commands {
-class Command;
-}
-
 namespace session {
 
-class SessionObserverInterface;
-
-class SessionObserverHandler {
+class SessionObserverHandler final {
  public:
-  SessionObserverHandler();
-  virtual ~SessionObserverHandler();
+  SessionObserverHandler() = default;
+  SessionObserverHandler(const SessionObserverHandler &) = delete;
+  SessionObserverHandler &operator=(const SessionObserverHandler &) = delete;
 
   void AddObserver(SessionObserverInterface *observer);
   void EvalCommandHandler(const commands::Command &command);
 
  private:
   std::vector<SessionObserverInterface *> observers_;
-
-  DISALLOW_COPY_AND_ASSIGN(SessionObserverHandler);
 };
 
 }  // namespace session
