@@ -29,24 +29,28 @@
 
 #include "session/session_usage_observer.h"
 
-#include <climits>
+#include <cstddef>
 #include <cstdint>
 #include <limits>
 #include <map>
 #include <string>
+#include <utility>
 #include <vector>
 
 #include "base/clock.h"
 #include "base/logging.h"
-#include "base/number_util.h"
-#include "base/port.h"
 #include "config/stats_config_util.h"
 #include "protocol/candidates.pb.h"
 #include "protocol/commands.pb.h"
 #include "protocol/state.pb.h"
 #include "usage_stats/usage_stats.h"
 #include "usage_stats/usage_stats.pb.h"
+#include "absl/base/attributes.h"
+#include "absl/base/const_init.h"
 #include "absl/synchronization/mutex.h"
+#include "absl/time/time.h"
+
+// TODO(tomokinat): Refactor this file to use abseil types for time/duration.
 
 namespace mozc {
 namespace session {
@@ -70,10 +74,7 @@ void AddToDoubleValueStats(double value,
 }
 
 uint64_t GetTimeInMilliSecond() {
-  uint64_t second = 0;
-  uint32_t micro_second = 0;
-  Clock::GetTimeOfDay(&second, &micro_second);
-  return second * 1000 + micro_second / 1000;
+  return absl::ToUnixMillis(Clock::GetAbslTime());
 }
 
 uint32_t GetDuration(uint64_t base_value) {
@@ -86,15 +87,19 @@ uint32_t GetDuration(uint64_t base_value) {
 
 bool IsSessionIndependentCommand(commands::Input::CommandType type) {
   switch (type) {
+    // LINT.IfChange(session_independent_command)
     case commands::Input::NO_OPERATION:
     case commands::Input::SET_CONFIG:
     case commands::Input::GET_CONFIG:
-    case commands::Input::SET_IMPOSED_CONFIG:
     case commands::Input::CLEAR_USER_HISTORY:
     case commands::Input::CLEAR_USER_PREDICTION:
     case commands::Input::CLEAR_UNUSED_USER_PREDICTION:
     case commands::Input::RELOAD:
     case commands::Input::SEND_USER_DICTIONARY_COMMAND:
+    case commands::Input::SYNC_DATA:
+    case commands::Input::CHECK_SPELLING:
+    case commands::Input::SET_REQUEST:
+      // LINT.ThenChange()
       return true;
     default:
       return false;
@@ -457,21 +462,6 @@ void SessionUsageObserver::LogTouchEvent(const commands::Input &input,
     last_touchevents_.clear();
   }
 
-  // When last_touchevents_ is not empty and any kind of commands are send
-  // except for EXPAND_SUGGESTION, save last_touchevents_ as touch_event.
-  // It is because EXPAND_SUGGESTION is automatically send from Java side codes.
-  if (!last_touchevents_.empty() &&
-      !((input.type() == commands::Input::SEND_COMMAND) &&
-        (input.has_command()) && (input.command().has_type()) &&
-        (input.command().type() ==
-         commands::SessionCommand::EXPAND_SUGGESTION))) {
-    for (size_t i = 0; i < last_touchevents_.size(); ++i) {
-      StoreTouchEventStats(last_touchevents_[i],
-                           &usage_cache_.touch_event[keyboard_name]);
-    }
-    last_touchevents_.clear();
-  }
-
   if (input.touch_events_size() > 0) {
     if (input.type() == commands::Input::SEND_KEY) {
       // When the input command contains TouchEvent and the type is SEND_KEY,
@@ -508,7 +498,7 @@ void SessionUsageObserver::EvalCommandHandler(
   if (IsSessionIndependentCommand(input.type())) {
     return;
   } else if (!input.has_id()) {
-    LOG(WARNING) << "no id";
+    LOG(WARNING) << "no id. input type: " << input.type();
     // Should have id
     return;
   }
