@@ -32,62 +32,55 @@
 #ifndef MOZC_SESSION_SESSION_WATCH_DOG_H_
 #define MOZC_SESSION_SESSION_WATCH_DOG_H_
 
-#include <cstdint>
 #include <memory>
 
-#include "base/port.h"
+#include "base/cpu_stats.h"
 #include "base/thread.h"
+#include "absl/synchronization/notification.h"
+#include "absl/time/time.h"
+#include "absl/types/span.h"
+#include "client/client_interface.h"
 
 namespace mozc {
-class CPUStatsInterface;
-class UnnamedEvent;
-namespace client {
-class ClientInterface;
-}
 
 // SessionWatchDog class sends Cleanup command to Sessionhandler
 // for every some specified seconds.
-class SessionWatchDog : public Thread {
+class SessionWatchDog {
  public:
-  // return the interval sec of watch dog timer
-  int32_t interval() const { return interval_sec_; }
+  // Constructs and starts the watch dog by spawning a background thread.
+  explicit SessionWatchDog(absl::Duration interval);
 
-  // Set client interface. This method doesn't take the owership.
-  // mainly for unittesting.
-  void SetClientInterface(client::ClientInterface *client);
+  // DI constructor for testing. Use `Create()` in production code.
+  SessionWatchDog(absl::Duration interval,
+                  std::unique_ptr<client::ClientInterface> client,
+                  std::unique_ptr<CPUStatsInterface> cpu_stats);
 
-  // Set CPUStats interface. This method doesn't take the owership.
-  // mainly for unittesting.
-  void SetCPUStatsInterface(CPUStatsInterface *cpu_stats);
+  SessionWatchDog(const SessionWatchDog &) = delete;
+  SessionWatchDog &operator=(const SessionWatchDog &) = delete;
 
-  explicit SessionWatchDog(int32_t interval_sec);
-  ~SessionWatchDog() override;
+  // Stops the watchdog and joins the background thread.
+  ~SessionWatchDog();
 
-  // inherited from Thread class
-  void Terminate();
+  // Returns the interval of this watch dog.
+  absl::Duration interval() const { return interval_; }
 
-  // inherited from Thread class
-  // start watch dog timer and return immediately
-  // virtual void Start();
-
-  // return true if watch dog can send CleanupCommand:
+  // Returns true if watch dog can send CleanupCommand:
   // |cpu_loads|: An array of cpu loads.
   // |cpu_load_index|: the size of cpu loads
   // |last_cleanup_time|: the last UTC time cleanup is executed
   // TODO(taku): want to define it inside private with FRIEND_TEST
-  bool CanSendCleanupCommand(const volatile float *cpu_loads,
-                             int cpu_loads_index, uint64_t current_cleanup_time,
-                             uint64_t last_cleanup_time) const;
+  bool CanSendCleanupCommand(absl::Span<const float> cpu_loads,
+                             absl::Time current_cleanup_time,
+                             absl::Time last_cleanup_time) const;
 
  private:
-  void Run() override;
+  void ThreadMain();
 
-  int32_t interval_sec_;
-  client::ClientInterface *client_;
-  CPUStatsInterface *cpu_stats_;
-  std::unique_ptr<UnnamedEvent> event_;
-
-  DISALLOW_COPY_AND_ASSIGN(SessionWatchDog);
+  absl::Duration interval_;
+  std::unique_ptr<client::ClientInterface> client_;
+  std::unique_ptr<CPUStatsInterface> cpu_stats_;
+  absl::Notification stop_;
+  Thread thread_;
 };
 
 }  // namespace mozc
