@@ -34,8 +34,6 @@ We don't support --xml command line option in the ibus-engines-mozc command
 since it could make start-up time of ibus-daemon slow when XML cache of the
 daemon in ~/.cache/ibus/bus/ is not ready or is expired.
 """
-
-from __future__ import print_function
 __author__ = "yusukes"
 
 import optparse
@@ -61,13 +59,17 @@ namespace {"""
 CPP_FOOTER = """}  // namespace
 #endif  // %s"""
 
+XDG_CURRENT_DESKTOP_URL = 'https://specifications.freedesktop.org/desktop-entry-spec/desktop-entry-spec-latest.html#recognized-keys'
+
 
 def GetXmlElement(key, value):
+  if key == 'composition_mode':
+    return '  <!-- %s : %s -->' % (key, value)
   return '  <%s>%s</%s>' % (key, value, key)
 
 
 def GetTextProtoElement(key, value):
-  if isinstance(value, int):
+  if isinstance(value, int) or key == 'composition_mode':
     return '  %s : %s' % (key, value)
   return '  %s : "%s"' % (key, value)
 
@@ -113,6 +115,16 @@ def GetIbusConfigTextProto(engines):
       output.append(GetTextProtoElement(key, value))
     output.append('}')
   output.append('active_on_launch: False')
+  output.append('mozc_renderer {')
+  output.append("  # Set 'False' to use IBus' candidate window.")
+  output.append('  enabled : True')
+  output.append("  # For Wayland sessions, 'mozc_renderer' will be used if and "
+                "only if any value")
+  output.append('  # set in this field (e.g. "GNOME", "KDE") is found in '
+                '$XDG_CURRENT_DESKTOP.')
+  output.append(f'  # {XDG_CURRENT_DESKTOP_URL}')
+  output.append('  compatible_wayland_desktop_names : ["GNOME"]')
+  output.append('}')
   return '\n'.join(output)
 
 
@@ -215,7 +227,6 @@ def main():
       # Make sure that the property key 'InputMode' matches to the property name
       # specified to |ibus_property_new| in unix/ibus/property_handler.cc
       'icon_prop_key': 'InputMode',
-      'symbol': '&#x3042;',
       'setup': setup_path + ' --mode=config_dialog',
   }
 
@@ -228,6 +239,25 @@ def main():
       'layout_variant': '',
       'layout_option': '',
       'rank': 80,
+      'symbol': 'あ',
+  }, {
+      'name': 'mozc-on',
+      'longname': product_name + u':あ',
+      'layout': 'default',
+      'layout_variant': '',
+      'layout_option': '',
+      'rank': 99,
+      'symbol': 'あ',
+      'composition_mode': 'HIRAGANA',
+  }, {
+      'name': 'mozc-off',
+      'longname': product_name + ':A_',
+      'layout': 'default',
+      'layout_variant': '',
+      'layout_option': '',
+      'rank': 99,
+      'symbol': 'A',
+      'composition_mode': 'DIRECT',
   }]
 
   if options.output_cpp:
