@@ -33,53 +33,74 @@
 #include <memory>
 #include <vector>
 
-#include "base/port.h"
+#include "protocol/commands.pb.h"
+#include "client/client.h"
 #include "unix/ibus/ibus_header.h"
-#include "unix/ibus/property_handler_interface.h"
+#include "unix/ibus/ibus_wrapper.h"
+#include "unix/ibus/message_translator.h"
 
 namespace mozc {
-
-namespace client {
-class ClientInterface;
-}  // namespace client
 namespace ibus {
 
-class MessageTranslatorInterface;
-
-class PropertyHandler : public PropertyHandlerInterface {
+class PropertyHandler {
  public:
   // This class takes the ownership of translator, but not client.
   PropertyHandler(std::unique_ptr<MessageTranslatorInterface> translator,
                   bool is_active_on_launch,
                   client::ClientInterface *client);
-  virtual ~PropertyHandler();
+  ~PropertyHandler();
 
-  virtual void Register(IBusEngine *engine);
-  virtual void ResetContentType(IBusEngine *engine);
-  virtual void UpdateContentType(IBusEngine *engine);
-  virtual void Update(IBusEngine *engine, const commands::Output &output);
-  virtual void ProcessPropertyActivate(IBusEngine *engine,
-                                       const gchar *property_name,
-                                       guint property_state);
-  virtual bool IsActivated() const;
-  virtual bool IsDisabled() const;
-  virtual commands::CompositionMode GetOriginalCompositionMode() const;
+  // Registers current properties into engine.
+  void Register(IbusEngineWrapper *engine);
+
+  void ResetContentType(IbusEngineWrapper *engine);
+  void UpdateContentType(IbusEngineWrapper *engine);
+
+  // Update properties.
+  void Update(IbusEngineWrapper *engine, const commands::Output &output);
+
+  void ProcessPropertyActivate(IbusEngineWrapper *engine,
+                               const char *property_name, uint property_state);
+
+  // Following two methods represent two aspects of an IME state.
+  // * (activated, disabled) == (false, false)
+  //     This is the state so-called "IME is off". However, an IME is expected
+  //     to monitor keyevents that are assigned to DirectMode. A user should be
+  //     able to turn on the IME by using shortcut or GUI menu.
+  // * (activated, disabled) == (false, true)
+  //     This is a state where an IME is expected to do nothing. A user should
+  //     be unable to turn on the IME by using shortcut nor GUI menu. This state
+  //     is used mainly on the password field. IME becomes to be "turned-off"
+  //     once |disabled| state is flipped to false.
+  // * (activated, disabled) == (true, false)
+  //     This is the state so-called "IME is on". A user should be able to turn
+  //     off the IME by using shortcut or GUI menu.
+  // * (activated, disabled) == (true, true)
+  //     This is the state where an IME is expected to do nothing. A user should
+  //     be unable to turn on the IME by using shortcut nor GUI menu. This state
+  //     is used mainly on the password field. IME becomes to be "turned-on"
+  //     once |disabled| state is flipped to false.
+  bool IsActivated() const;
+  bool IsDisabled() const;
+
+  // Returns original composition mode before.
+  commands::CompositionMode GetOriginalCompositionMode() const;
 
  private:
-  void UpdateContentTypeImpl(IBusEngine *engine, bool disabled);
+  void UpdateContentTypeImpl(IbusEngineWrapper *engine, bool disabled);
   // Appends composition properties into panel
   void AppendCompositionPropertyToPanel();
   // Appends tool properties into panel
   void AppendToolPropertyToPanel();
   // Appends switch properties into panel
   void UpdateCompositionModeIcon(
-      IBusEngine *engine, const commands::CompositionMode new_composition_mode);
-  void SetCompositionMode(IBusEngine *engine,
-                          commands::CompositionMode composition_mode);
+      IbusEngineWrapper *engine,
+      commands::CompositionMode new_composition_mode);
+  void SetCompositionMode(commands::CompositionMode composition_mode);
 
-  IBusPropList *prop_root_;
-  IBusProperty *prop_composition_mode_;
-  IBusProperty *prop_mozc_tool_;
+  IbusPropListWrapper prop_root_;
+  IbusPropertyWrapper prop_composition_mode_;
+  IbusPropertyWrapper prop_mozc_tool_;
   client::ClientInterface *client_;
   std::unique_ptr<MessageTranslatorInterface> translator_;
   commands::CompositionMode original_composition_mode_;
