@@ -29,28 +29,22 @@
 
 #include "usage_stats/usage_stats_uploader.h"
 
-#include <algorithm>
 #include <cstdint>
-#include <map>
 #include <string>
-#include <utility>
-#include <vector>
 
-#include "base/port.h"
-#include "base/system_util.h"
 #include "base/version.h"
 #include "storage/registry.h"
-#include "testing/base/public/googletest.h"
-#include "testing/base/public/gunit.h"
-#include "usage_stats/usage_stats.h"
-#include "absl/flags/flag.h"
+#include "storage/tiny_storage.h"
+#include "testing/gunit.h"
+#include "testing/mozctest.h"
+#include "absl/strings/string_view.h"
 
 namespace mozc {
 namespace usage_stats {
 namespace {
 
 void SetUpMetaDataWithMozcVersion(uint32_t last_upload_time,
-                                  const std::string &mozc_version) {
+                                  const absl::string_view mozc_version) {
   EXPECT_TRUE(
       storage::Registry::Insert("usage_stats.last_upload", last_upload_time));
   EXPECT_TRUE(
@@ -61,10 +55,11 @@ void SetUpMetaData(uint32_t last_upload_time) {
   SetUpMetaDataWithMozcVersion(last_upload_time, Version::GetMozcVersion());
 }
 
-class UsageStatsUploaderTest : public ::testing::Test {
+class UsageStatsUploaderTest : public testing::TestWithTempUserProfile {
  protected:
   void SetUp() override {
-    SystemUtil::SetUserProfileDirectory(absl::GetFlag(FLAGS_test_tmpdir));
+    // Update the registry file path by creating a new storage.
+    storage::Registry::SetStorage(storage::TinyStorage::New());
     EXPECT_TRUE(storage::Registry::Clear());
   }
 
@@ -90,7 +85,7 @@ TEST_F(UsageStatsUploaderTest, SendTestDeleteExistingClientId) {
   storage::Registry::Insert("usage_stats.client_id", store_value);
   std::string client_id;
   EXPECT_TRUE(storage::Registry::Lookup("usage_stats.client_id", &client_id));
-  EXPECT_EQ("some_value", client_id);
+  EXPECT_EQ(client_id, "some_value");
 
   EXPECT_TRUE(UsageStatsUploader::Send(nullptr));
 
