@@ -27,109 +27,110 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-#if defined(OS_WIN)
+#include "win32/base/conversion_mode_util.h"
+
+#if defined(_WIN32)
 // clang-format off
 #include <windows.h>
 #include <imm.h>
 // clang-format on
-#endif  // OS_WIN
+#endif  // _WIN32
+
+#include <cstdint>
 
 #include "protocol/commands.pb.h"
-#include "testing/base/public/googletest.h"
-#include "testing/base/public/gunit.h"
-#include "win32/base/conversion_mode_util.h"
+#include "testing/gunit.h"
 
 namespace mozc {
 namespace win32 {
-#if !defined(OS_WIN)
+#if !defined(_WIN32)
 // Use the same naming convention to emulate imm32.h.
-const uint32 IME_CMODE_ALPHANUMERIC = 0x0;
-const uint32 IME_CMODE_NATIVE = 0x1;
-const uint32 IME_CMODE_KATAKANA = 0x2;
-const uint32 IME_CMODE_LANGUAGE = 0x3;
-const uint32 IME_CMODE_FULLSHAPE = 0x8;
-const uint32 IME_CMODE_ROMAN = 0x10;
-const uint32 IME_CMODE_CHARCODE = 0x20;
-const uint32 IME_CMODE_HANJACONVERT = 0x40;
-const uint32 IME_CMODE_SOFTKBD = 0x80;
-const uint32 IME_CMODE_NOCONVERSION = 0x100;
-const uint32 IME_CMODE_EUDC = 0x200;
-const uint32 IME_CMODE_SYMBOL = 0x400;
-const uint32 IME_CMODE_SYMBOL = 0x800;
-#endif  // !OS_WIN
+const uint32_t IME_CMODE_ALPHANUMERIC = 0x0;
+const uint32_t IME_CMODE_NATIVE = 0x1;
+const uint32_t IME_CMODE_KATAKANA = 0x2;
+const uint32_t IME_CMODE_LANGUAGE = 0x3;
+const uint32_t IME_CMODE_FULLSHAPE = 0x8;
+const uint32_t IME_CMODE_ROMAN = 0x10;
+const uint32_t IME_CMODE_CHARCODE = 0x20;
+const uint32_t IME_CMODE_HANJACONVERT = 0x40;
+const uint32_t IME_CMODE_SOFTKBD = 0x80;
+const uint32_t IME_CMODE_NOCONVERSION = 0x100;
+const uint32_t IME_CMODE_EUDC = 0x200;
+const uint32_t IME_CMODE_SYMBOL = 0x400;
+const uint32_t IME_CMODE_SYMBOL = 0x800;
+#endif  // !_WIN32
 
 TEST(ConversionModeUtilTest, ToNativeMode) {
-  uint32 native_code = 0;
+  uint32_t native_code = 0;
 
   native_code = 0;
   EXPECT_TRUE(ConversionModeUtil::ToNativeMode(mozc::commands::DIRECT, false,
                                                &native_code));
-  EXPECT_EQ(IME_CMODE_ALPHANUMERIC | IME_CMODE_ROMAN, native_code);
+  EXPECT_EQ(native_code, IME_CMODE_ALPHANUMERIC | IME_CMODE_ROMAN);
 
   native_code = 0;
   EXPECT_TRUE(ConversionModeUtil::ToNativeMode(mozc::commands::HIRAGANA, false,
                                                &native_code));
-  EXPECT_EQ(IME_CMODE_NATIVE | IME_CMODE_FULLSHAPE | IME_CMODE_ROMAN,
-            native_code);
+  EXPECT_EQ(native_code,
+            IME_CMODE_NATIVE | IME_CMODE_FULLSHAPE | IME_CMODE_ROMAN);
 
   native_code = 0;
   EXPECT_TRUE(ConversionModeUtil::ToNativeMode(mozc::commands::HALF_KATAKANA,
                                                false, &native_code));
-  EXPECT_EQ(IME_CMODE_NATIVE | IME_CMODE_KATAKANA | IME_CMODE_ROMAN,
-            native_code);
+  EXPECT_EQ(native_code,
+            IME_CMODE_NATIVE | IME_CMODE_KATAKANA | IME_CMODE_ROMAN);
 
   native_code = 0;
   EXPECT_TRUE(ConversionModeUtil::ToNativeMode(mozc::commands::HALF_ASCII,
                                                false, &native_code));
-  EXPECT_EQ(IME_CMODE_ALPHANUMERIC | IME_CMODE_ROMAN, native_code);
+  EXPECT_EQ(native_code, IME_CMODE_ALPHANUMERIC | IME_CMODE_ROMAN);
 
   native_code = 0;
   EXPECT_TRUE(ConversionModeUtil::ToNativeMode(mozc::commands::FULL_ASCII,
                                                false, &native_code));
-  EXPECT_EQ(IME_CMODE_ALPHANUMERIC | IME_CMODE_FULLSHAPE | IME_CMODE_ROMAN,
-            native_code);
+  EXPECT_EQ(native_code,
+            IME_CMODE_ALPHANUMERIC | IME_CMODE_FULLSHAPE | IME_CMODE_ROMAN);
 
   native_code = 0;
   EXPECT_TRUE(ConversionModeUtil::ToNativeMode(mozc::commands::FULL_KATAKANA,
                                                false, &native_code));
-  EXPECT_EQ(IME_CMODE_NATIVE | IME_CMODE_KATAKANA | IME_CMODE_FULLSHAPE |
-                IME_CMODE_ROMAN,
-            native_code);
+  EXPECT_EQ(native_code, IME_CMODE_NATIVE | IME_CMODE_KATAKANA |
+                             IME_CMODE_FULLSHAPE | IME_CMODE_ROMAN);
 }
 
 TEST(ConversionModeUtilTest, ToNativeModeWithKanaLocked) {
-  uint32 native_code = 0;
+  uint32_t native_code = 0;
 
   native_code = 0;
   EXPECT_TRUE(ConversionModeUtil::ToNativeMode(mozc::commands::DIRECT, true,
                                                &native_code));
-  EXPECT_EQ(IME_CMODE_ALPHANUMERIC, native_code);
+  EXPECT_EQ(native_code, IME_CMODE_ALPHANUMERIC);
 
   native_code = 0;
   EXPECT_TRUE(ConversionModeUtil::ToNativeMode(mozc::commands::HIRAGANA, true,
                                                &native_code));
-  EXPECT_EQ(IME_CMODE_NATIVE | IME_CMODE_FULLSHAPE, native_code);
+  EXPECT_EQ(native_code, IME_CMODE_NATIVE | IME_CMODE_FULLSHAPE);
 
   native_code = 0;
   EXPECT_TRUE(ConversionModeUtil::ToNativeMode(mozc::commands::HALF_KATAKANA,
                                                true, &native_code));
-  EXPECT_EQ(IME_CMODE_NATIVE | IME_CMODE_KATAKANA, native_code);
+  EXPECT_EQ(native_code, IME_CMODE_NATIVE | IME_CMODE_KATAKANA);
 
   native_code = 0;
   EXPECT_TRUE(ConversionModeUtil::ToNativeMode(mozc::commands::HALF_ASCII, true,
                                                &native_code));
-  EXPECT_EQ(IME_CMODE_ALPHANUMERIC, native_code);
+  EXPECT_EQ(native_code, IME_CMODE_ALPHANUMERIC);
 
   native_code = 0;
   EXPECT_TRUE(ConversionModeUtil::ToNativeMode(mozc::commands::FULL_ASCII, true,
                                                &native_code));
-  EXPECT_EQ(IME_CMODE_ALPHANUMERIC | IME_CMODE_FULLSHAPE, native_code);
+  EXPECT_EQ(native_code, IME_CMODE_ALPHANUMERIC | IME_CMODE_FULLSHAPE);
 
   native_code = 0;
   EXPECT_TRUE(ConversionModeUtil::ToNativeMode(mozc::commands::FULL_KATAKANA,
                                                true, &native_code));
-  EXPECT_EQ(IME_CMODE_NATIVE | IME_CMODE_KATAKANA | IME_CMODE_FULLSHAPE,
-            native_code);
+  EXPECT_EQ(native_code,
+            IME_CMODE_NATIVE | IME_CMODE_KATAKANA | IME_CMODE_FULLSHAPE);
 }
 
 TEST(ConversionModeUtilTest, ToMozcMode) {
@@ -139,37 +140,37 @@ TEST(ConversionModeUtilTest, ToMozcMode) {
   // NOTE: IME_CMODE_ALPHANUMERIC will be converted to HALF_ASCII, not DIRECT.
   mode = mozc::commands::DIRECT;
   EXPECT_TRUE(ConversionModeUtil::ToMozcMode(IME_CMODE_ALPHANUMERIC, &mode));
-  EXPECT_EQ(mozc::commands::HALF_ASCII, mode);
+  EXPECT_EQ(mode, mozc::commands::HALF_ASCII);
 
   // IME_CMODE_ROMAN has no effect in this conversion.
   mode = mozc::commands::DIRECT;
   EXPECT_TRUE(ConversionModeUtil::ToMozcMode(
       IME_CMODE_ALPHANUMERIC | IME_CMODE_ROMAN, &mode));
-  EXPECT_EQ(mozc::commands::HALF_ASCII, mode);
+  EXPECT_EQ(mode, mozc::commands::HALF_ASCII);
 
   // Check for FULL_ASCII
   mode = mozc::commands::DIRECT;
   EXPECT_TRUE(ConversionModeUtil::ToMozcMode(
       IME_CMODE_ALPHANUMERIC | IME_CMODE_FULLSHAPE, &mode));
-  EXPECT_EQ(mozc::commands::FULL_ASCII, mode);
+  EXPECT_EQ(mode, mozc::commands::FULL_ASCII);
 
   // IME_CMODE_ROMAN has no effect in this conversion.
   mode = mozc::commands::DIRECT;
   EXPECT_TRUE(ConversionModeUtil::ToMozcMode(
       IME_CMODE_ALPHANUMERIC | IME_CMODE_FULLSHAPE | IME_CMODE_ROMAN, &mode));
-  EXPECT_EQ(mozc::commands::FULL_ASCII, mode);
+  EXPECT_EQ(mode, mozc::commands::FULL_ASCII);
 
   // Check for HIRAGANA
   mode = mozc::commands::DIRECT;
   EXPECT_TRUE(ConversionModeUtil::ToMozcMode(
       IME_CMODE_NATIVE | IME_CMODE_FULLSHAPE, &mode));
-  EXPECT_EQ(mozc::commands::HIRAGANA, mode);
+  EXPECT_EQ(mode, mozc::commands::HIRAGANA);
 
   // IME_CMODE_ROMAN has no effect in this conversion.
   mode = mozc::commands::DIRECT;
   EXPECT_TRUE(ConversionModeUtil::ToMozcMode(
       IME_CMODE_NATIVE | IME_CMODE_FULLSHAPE | IME_CMODE_ROMAN, &mode));
-  EXPECT_EQ(mozc::commands::HIRAGANA, mode);
+  EXPECT_EQ(mode, mozc::commands::HIRAGANA);
 
   // There is no "HALF_HIRAGANA"
   mode = mozc::commands::DIRECT;
@@ -184,7 +185,7 @@ TEST(ConversionModeUtilTest, ToMozcMode) {
   mode = mozc::commands::DIRECT;
   EXPECT_TRUE(ConversionModeUtil::ToMozcMode(
       IME_CMODE_NATIVE | IME_CMODE_FULLSHAPE | IME_CMODE_KATAKANA, &mode));
-  EXPECT_EQ(mozc::commands::FULL_KATAKANA, mode);
+  EXPECT_EQ(mode, mozc::commands::FULL_KATAKANA);
 
   // IME_CMODE_ROMAN has no effect in this conversion.
   mode = mozc::commands::DIRECT;
@@ -192,19 +193,19 @@ TEST(ConversionModeUtilTest, ToMozcMode) {
       ConversionModeUtil::ToMozcMode(IME_CMODE_NATIVE | IME_CMODE_FULLSHAPE |
                                          IME_CMODE_KATAKANA | IME_CMODE_ROMAN,
                                      &mode));
-  EXPECT_EQ(mozc::commands::FULL_KATAKANA, mode);
+  EXPECT_EQ(mode, mozc::commands::FULL_KATAKANA);
 
   // Check for HALF_KATAKANA
   mode = mozc::commands::DIRECT;
   EXPECT_TRUE(ConversionModeUtil::ToMozcMode(
       IME_CMODE_NATIVE | IME_CMODE_KATAKANA, &mode));
-  EXPECT_EQ(mozc::commands::HALF_KATAKANA, mode);
+  EXPECT_EQ(mode, mozc::commands::HALF_KATAKANA);
 
   // IME_CMODE_ROMAN has no effect in this conversion.
   mode = mozc::commands::DIRECT;
   EXPECT_TRUE(ConversionModeUtil::ToMozcMode(
       IME_CMODE_NATIVE | IME_CMODE_KATAKANA | IME_CMODE_ROMAN, &mode));
-  EXPECT_EQ(mozc::commands::HALF_KATAKANA, mode);
+  EXPECT_EQ(mode, mozc::commands::HALF_KATAKANA);
 }
 
 TEST(ConversionModeUtilTest, ToMozcModeUnsupportedModes) {
@@ -299,9 +300,9 @@ TEST(ConversionModeUtilTest, ConvertStatusFromMozcToNative) {
   status.set_mode(commands::HALF_ASCII);
   EXPECT_TRUE(ConversionModeUtil::ConvertStatusFromMozcToNative(
       status, false, &is_open, &logical_mode, &visible_mode));
-  EXPECT_EQ(IME_CMODE_NATIVE | IME_CMODE_FULLSHAPE | IME_CMODE_ROMAN,
-            logical_mode);
-  EXPECT_EQ(IME_CMODE_ALPHANUMERIC | IME_CMODE_ROMAN, visible_mode);
+  EXPECT_EQ(logical_mode,
+            IME_CMODE_NATIVE | IME_CMODE_FULLSHAPE | IME_CMODE_ROMAN);
+  EXPECT_EQ(visible_mode, IME_CMODE_ALPHANUMERIC | IME_CMODE_ROMAN);
 
   // The mode conversion should always be done regardless of open/close status.
   is_open = false;
@@ -313,9 +314,9 @@ TEST(ConversionModeUtilTest, ConvertStatusFromMozcToNative) {
   status.set_mode(commands::HALF_ASCII);
   EXPECT_TRUE(ConversionModeUtil::ConvertStatusFromMozcToNative(
       status, false, &is_open, &logical_mode, &visible_mode));
-  EXPECT_EQ(IME_CMODE_NATIVE | IME_CMODE_FULLSHAPE | IME_CMODE_ROMAN,
-            logical_mode);
-  EXPECT_EQ(IME_CMODE_ALPHANUMERIC | IME_CMODE_ROMAN, visible_mode);
+  EXPECT_EQ(logical_mode,
+            IME_CMODE_NATIVE | IME_CMODE_FULLSHAPE | IME_CMODE_ROMAN);
+  EXPECT_EQ(visible_mode, IME_CMODE_ALPHANUMERIC | IME_CMODE_ROMAN);
 }
 
 TEST(ConversionModeUtilTest, GetMozcModeFromNativeMode) {
@@ -325,13 +326,13 @@ TEST(ConversionModeUtilTest, GetMozcModeFromNativeMode) {
   commands::CompositionMode mozc_mode = commands::HIRAGANA;
   EXPECT_TRUE(ConversionModeUtil::GetMozcModeFromNativeMode(
       IME_CMODE_NATIVE | IME_CMODE_FULLSHAPE | IME_CMODE_ROMAN, &mozc_mode));
-  EXPECT_EQ(commands::HIRAGANA, mozc_mode);
+  EXPECT_EQ(mozc_mode, commands::HIRAGANA);
 
   // The mode conversion should always be done regardless of open/close status,
   // that is, we no longer rely on |mozc::commands::DIRECT|.
   EXPECT_TRUE(ConversionModeUtil::GetMozcModeFromNativeMode(
       IME_CMODE_NATIVE | IME_CMODE_FULLSHAPE | IME_CMODE_ROMAN, &mozc_mode));
-  EXPECT_EQ(commands::HIRAGANA, mozc_mode);
+  EXPECT_EQ(mozc_mode, commands::HIRAGANA);
 }
 }  // namespace win32
 }  // namespace mozc
