@@ -32,15 +32,20 @@
 
 #include <windows.h>
 
+#include <array>
+#include <cstddef>
+#include <cstdint>
+#include <memory>
 #include <vector>
 
-#include "base/port.h"
+#include "base/logging.h"
 
 namespace mozc {
 namespace win32 {
+
 class KeyboardStatus {
  public:
-  KeyboardStatus();
+  KeyboardStatus() : status_({}) {}
   explicit KeyboardStatus(const BYTE key_status[256]);
   BYTE GetState(int virtual_key) const;
   void SetState(int virtual_key, BYTE value);
@@ -51,7 +56,7 @@ class KeyboardStatus {
   size_t status_size() const;
 
  private:
-  BYTE status_[256];
+  std::array<BYTE, 256> status_;
 };
 
 class LParamKeyInfo {
@@ -78,30 +83,37 @@ class LParamKeyInfo {
 
 class VirtualKey {
  public:
-  VirtualKey();
+  VirtualKey() = default;
 
   // Construct an instance from a given |virtual_key|.
   // You cannot specify VK_PACKET for |virtual_key|.
-  static VirtualKey FromVirtualKey(BYTE virtual_key);
+  static constexpr VirtualKey FromVirtualKey(BYTE virtual_key) {
+    return VirtualKey(virtual_key, L'\0', L'\0');
+  }
   // Construct an instance from a given |combined_virtual_key|.
   // If the low word of |combined_virtual_key| is VK_PACKET,
   // the high word will be used as |wide_char_|.
   // Otherwise, the lowest byte of |combined_virtual_key| will be
   // used as |virtual_key_|.
-  static VirtualKey FromCombinedVirtualKey(UINT combined_virtual_key);
+  static constexpr VirtualKey FromCombinedVirtualKey(UINT combined_virtual_key);
   // Construct an instance from a given ucs4 character.
   // In this case, |virtual_key_| will be set to VK_PACKET.
-  static VirtualKey FromUnicode(char32 unicode);
+  static constexpr VirtualKey FromUnicode(char32_t unicode);
 
   wchar_t wide_char() const;
-  char32 unicode_char() const;
+  char32_t unicode_char() const;
   BYTE virtual_key() const;
 
  private:
-  VirtualKey(BYTE virtual_key, wchar_t wide_char, char32 unicode_char);
-  char32 unicode_char_;
-  wchar_t wide_char_;
-  BYTE virtual_key_;
+  constexpr VirtualKey(BYTE virtual_key, wchar_t wide_char,
+                       char32_t unicode_char)
+      : unicode_char_(unicode_char),
+        wide_char_(wide_char),
+        virtual_key_(virtual_key) {}
+
+  char32_t unicode_char_ = 0;
+  wchar_t wide_char_ = 0;
+  BYTE virtual_key_ = 0;
 };
 
 // We intentionally wrap some APIs as virthal methods so that unit tests can
@@ -110,7 +122,7 @@ class VirtualKey {
 // a mock which emulates the API predictably for unit tests.
 class Win32KeyboardInterface {
  public:
-  virtual ~Win32KeyboardInterface() {}
+  virtual ~Win32KeyboardInterface() = default;
 
   // Injection point for keyboard_state.IsPressed(VK_KANA).
   virtual bool IsKanaLocked(const KeyboardStatus &keyboard_state) = 0;
@@ -131,13 +143,20 @@ class Win32KeyboardInterface {
                         __in UINT wFlags) = 0;
 
   // Injection point for SendInput API.
-  virtual UINT SendInput(const std::vector<INPUT> &inputs) = 0;
+  // Note that the inputs are passed by value because the SendInput API requires
+  // LPINPUT (NOT const INPUT *).
+  virtual UINT SendInput(std::vector<INPUT> inputs) = 0;
 
-  static Win32KeyboardInterface *CreateDefault();
+  static std::unique_ptr<Win32KeyboardInterface> CreateDefault();
 };
 
 class JapaneseKeyboardLayoutEmulator {
  public:
+  JapaneseKeyboardLayoutEmulator(const JapaneseKeyboardLayoutEmulator &) =
+      delete;
+  JapaneseKeyboardLayoutEmulator &operator=(
+      const JapaneseKeyboardLayoutEmulator &) = delete;
+
   // This methods emulates ToUnicode API as if the current keyboard layout was
   // Japanese keyboard.  Currently this emulation ignores |scan_code|.
   static int ToUnicode(__in UINT virtual_key, __in UINT scan_code,
@@ -154,10 +173,49 @@ class JapaneseKeyboardLayoutEmulator {
   static wchar_t GetCharacterForKeyDown(BYTE virtual_key,
                                         const BYTE keyboard_state[256],
                                         bool is_menu_active);
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(JapaneseKeyboardLayoutEmulator);
 };
+
+namespace keyboard_internal {
+
+constexpr BYTE ParseVirtualKey(UINT combined_virtual_key) {
+  const uint16_t loword = LOWORD(combined_virtual_key);
+  if (loword <= 0xff) {
+    return loword;
+  }
+  DLOG(INFO) << "Unexpected VK found. VK = " << loword;
+  return 0;
+}
+
+constexpr wchar_t ParseWideChar(UINT combined_virtual_key) {
+  if (ParseVirtualKey(combined_virtual_key) == VK_PACKET) {
+    return HIWORD(combined_virtual_key);
+  } else {
+    return 0;
+  }
+}
+
+}  // namespace keyboard_internal
+
+constexpr VirtualKey VirtualKey::FromCombinedVirtualKey(
+    UINT combined_virtual_key) {
+  const BYTE vk = keyboard_internal::ParseVirtualKey(combined_virtual_key);
+  const wchar_t wchar = keyboard_internal::ParseWideChar(combined_virtual_key);
+  char32_t unicode_char = 0;
+  if (!IS_HIGH_SURROGATE(wchar) && !IS_LOW_SURROGATE(wchar)) {
+    unicode_char = wchar;
+  }
+
+  return VirtualKey(vk, wchar, unicode_char);
+}
+
+constexpr VirtualKey VirtualKey::FromUnicode(char32_t unicode_char) {
+  wchar_t wchar = L'\0';
+  if (unicode_char <= 0xffff) {
+    wchar = static_cast<wchar_t>(unicode_char);
+  }
+  return VirtualKey(VK_PACKET, wchar, unicode_char);
+}
+
 }  // namespace win32
 }  // namespace mozc
 #endif  // MOZC_WIN32_BASE_KEYBOARD_H_
