@@ -38,10 +38,10 @@
 #include "base/japanese_util.h"
 #include "base/logging.h"
 #include "base/util.h"
-#include "client/client_interface.h"
 #include "config/config_handler.h"
 #include "protocol/commands.pb.h"
 #include "session/key_info_util.h"
+#include "client/client_interface.h"
 #include "win32/base/conversion_mode_util.h"
 #include "win32/base/input_state.h"
 #include "win32/base/keyboard.h"
@@ -406,7 +406,7 @@ bool ConvertToKeyEventMain(const VirtualKey &virtual_key, BYTE scan_code,
 
   // Support VK_PACKET.
   if (virtual_key.virtual_key() == VK_PACKET) {
-    const char32 character = virtual_key.unicode_char();
+    const char32_t character = virtual_key.unicode_char();
     std::string utf8_characters;
     Util::Ucs4ToUtf8(character, &utf8_characters);
     if (utf8_characters.empty()) {
@@ -638,7 +638,7 @@ KeyEventHandlerResult::KeyEventHandlerResult()
 
 KeyEventHandlerResult KeyEventHandler::HandleKey(
     const VirtualKey &virtual_key, BYTE scan_code, bool is_key_down,
-    const KeyboardStatus &keyboard_status, const InputBehavior &behavior,
+    const KeyboardStatus &initial_status, const InputBehavior &behavior,
     const InputState &ime_state, Win32KeyboardInterface *keyboard,
     mozc::commands::KeyEvent *key) {
   KeyEventHandlerResult result;
@@ -668,7 +668,7 @@ KeyEventHandlerResult KeyEventHandler::HandleKey(
   }
 
   if (!ConvertToKeyEvent(virtual_key, scan_code, is_key_down, false, behavior,
-                         ime_state, keyboard_status, keyboard, key)) {
+                         ime_state, initial_status, keyboard, key)) {
     result.succeeded = true;
     result.should_be_eaten = false;
     result.should_be_sent_to_server = false;
