@@ -30,18 +30,28 @@
 #ifndef MOZC_WIN32_TIP_TIP_LANG_BAR_MENU_H_
 #define MOZC_WIN32_TIP_TIP_LANG_BAR_MENU_H_
 
+#include <ctfutb.h>
 #include <msctf.h>
 #include <rpcsal.h>
+#include <unknwn.h>
+#include <wil/com.h>
 #include <windows.h>
 
+#include <cstddef>
 #include <string>
 #include <vector>
 
-#include "win32/tip/tip_lang_bar.h"
-#include "win32/tip/tip_ref_count.h"
+#include "win32/tip/tip_dll_module.h"
+#include "win32/tip/tip_lang_bar_callback.h"
 
 namespace mozc {
 namespace win32 {
+
+// This declaration is necessary because this header file is included multiple
+// times.
+template <>
+bool IsIIDOf<ITfLangBarItemButton>(REFIID riid);
+
 namespace tsf {
 
 // MIDL_INTERFACE expects a string literal rather than a constant array of
@@ -49,10 +59,10 @@ namespace tsf {
 #ifdef GOOGLE_JAPANESE_INPUT_BUILD
 #define IIDSTR_IMozcLangBarItem "C6057858-8A94-4B40-8C99-D1C4B4A0B9DB"
 #define IIDSTR_IMozcLangBarToggleItem "72B4C4E3-B9F3-478A-B8A8-753AFF37EB94"
-#else
+#else  // GOOGLE_JAPANESE_INPUT_BUILD
 #define IIDSTR_IMozcLangBarItem "75B2153A-504B-48C9-9257-BA8D60E523E6"
 #define IIDSTR_IMozcLangBarToggleItem "9ABF0C3B-4AC6-4DED-9EF6-97E728852CF3"
-#endif
+#endif  // !GOOGLE_JAPANESE_INPUT_BUILD
 
 class TipBarCallback;
 
@@ -74,7 +84,7 @@ struct TipLangBarMenuItem {
   UINT icon_id_for_theme_;
 };
 
-// Reperesents the data possessed internally by a language bar menu item.
+// Represents the data possessed internally by a language bar menu item.
 struct TipLangBarMenuData {
   int flags_;
   UINT item_id_;
@@ -85,7 +95,7 @@ struct TipLangBarMenuData {
   wchar_t text_[TF_LBI_DESC_MAXLEN];
 };
 
-// Reperesents the data possessed by a language bar menu.
+// Represents the data possessed by a language bar menu.
 class TipLangBarMenuDataArray {
  public:
   HRESULT Init(HINSTANCE instance, const TipLangBarMenuItem *menu, int count);
@@ -110,42 +120,33 @@ IMozcLangBarToggleItem : public IUnknown {
 };
 
 // Represents the common operations for a button-menu item in the language bar.
-class TipLangBarButton : public ITfLangBarItemButton,
-                         public ITfSource,
-                         public IMozcLangBarItem {
+class TipLangBarButton : public TipComImplements<ITfLangBarItemButton,
+                                                 ITfSource, IMozcLangBarItem> {
  public:
-  TipLangBarButton(TipLangBarCallback *langbar_callback, const GUID &guid,
-                   bool is_menu, bool show_in_tray);
-  // It is OK to declare the destructor as non-virtual because all the
-  // insbtances will be deleted from and only from the Release() method by
-  // |delete this| where |this| has the true (derived) type of the instance.
-  ~TipLangBarButton();
-
-  // The IUnknown interface methods
-  virtual STDMETHODIMP QueryInterface(REFIID guid, void **object) = 0;
-  virtual STDMETHODIMP_(ULONG) AddRef() = 0;
-  virtual STDMETHODIMP_(ULONG) Release() = 0;
+  TipLangBarButton(wil::com_ptr_nothrow<TipLangBarCallback> lang_bar_callback,
+                   const GUID &guid, bool is_menu, bool show_in_tray);
 
   // The ITfLangBarItem interface methods
-  virtual STDMETHODIMP GetInfo(TF_LANGBARITEMINFO *item_info) = 0;
-  virtual STDMETHODIMP GetStatus(DWORD *status);
-  virtual STDMETHODIMP Show(BOOL show);
-  virtual STDMETHODIMP GetTooltipString(BSTR *tooltip);
+  STDMETHODIMP GetInfo(TF_LANGBARITEMINFO *item_info) override = 0;
+  STDMETHODIMP GetStatus(DWORD *status) override;
+  STDMETHODIMP Show(BOOL show) override;
+  STDMETHODIMP GetTooltipString(BSTR *tooltip) override;
 
   // The ITfLangBarItemButton interface methods
-  virtual STDMETHODIMP OnClick(TfLBIClick clink, POINT point, const RECT *rect);
-  virtual STDMETHODIMP InitMenu(ITfMenu *menu) = 0;
-  virtual STDMETHODIMP OnMenuSelect(UINT menu_id) = 0;
-  virtual STDMETHODIMP GetIcon(HICON *icon) = 0;
-  virtual STDMETHODIMP GetText(BSTR *text);
+  STDMETHODIMP OnClick(TfLBIClick click, POINT point,
+                       const RECT *rect) override;
+  STDMETHODIMP InitMenu(ITfMenu *menu) override = 0;
+  STDMETHODIMP OnMenuSelect(UINT menu_id) override = 0;
+  STDMETHODIMP GetIcon(HICON *icon) override = 0;
+  STDMETHODIMP GetText(BSTR *text) override;
 
   // The ITfSource interface methods
-  virtual STDMETHODIMP AdviseSink(REFIID interface_id, IUnknown *unknown,
-                                  DWORD *cookie);
-  virtual STDMETHODIMP UnadviseSink(DWORD cookie);
+  STDMETHODIMP AdviseSink(REFIID interface_id, IUnknown *unknown,
+                          DWORD *cookie) override;
+  STDMETHODIMP UnadviseSink(DWORD cookie) override;
 
   // The IMozcLangBarItem interface method
-  virtual STDMETHODIMP SetEnabled(bool enabled);
+  STDMETHODIMP SetEnabled(bool enabled) override;
 
   // Initializes an ImeButtonMenu instance.
   // This function allocates resources for an ImeButtonMenu instance.
@@ -161,8 +162,6 @@ class TipLangBarButton : public ITfLangBarItemButton,
   static bool CanContextMenuDisplay32bppIcon();
 
  protected:
-  STDMETHODIMP QueryInterfaceBase(REFIID guid, void **object);
-
   // Update the item description. The caller is also responsible for calling
   // OnUpdate method to notify the change to the system.
   void SetDescription(const std::wstring &description);
@@ -184,8 +183,10 @@ class TipLangBarButton : public ITfLangBarItemButton,
   // IsMenuButton() returns false.
   void SetContextMenuEnabled(bool enabled);
 
-  ITfLangBarItemSink *item_sink_;
-  TipLangBarCallback *langbar_callback_;
+  wil::com_ptr_nothrow<ITfLangBarItemSink> item_sink_;
+  // Save the TipLangBarCallback object that owns this button to prevent the
+  // object from being deleted.
+  wil::com_ptr_nothrow<TipLangBarCallback> lang_bar_callback_;
 
  private:
   // Represents the information of an instance copied to the TSF manager.
@@ -207,22 +208,17 @@ class TipLangBarButton : public ITfLangBarItemButton,
 
 // Represents the common operations for a button-menu item with an icon in the
 // language bar.
-class TipLangBarMenuButton : public TipLangBarButton {
+class TipLangBarMenuButton final : public TipLangBarButton {
  public:
   TipLangBarMenuButton(TipLangBarCallback *langbar_callback, const GUID &guid,
                        bool show_in_tray);
 
-  // The IUnknown interface methods
-  virtual STDMETHODIMP QueryInterface(REFIID guid, void **object);
-  virtual STDMETHODIMP_(ULONG) AddRef();
-  virtual STDMETHODIMP_(ULONG) Release();
-
-  virtual STDMETHODIMP GetInfo(TF_LANGBARITEMINFO *item_info);
+  STDMETHODIMP GetInfo(TF_LANGBARITEMINFO *item_info) override;
 
   // A part of the ITfLangBarItemButton interface methods
-  virtual STDMETHODIMP InitMenu(ITfMenu *menu);
-  virtual STDMETHODIMP OnMenuSelect(UINT menu_id);
-  virtual STDMETHODIMP GetIcon(HICON *icon);
+  STDMETHODIMP InitMenu(ITfMenu *menu) override;
+  STDMETHODIMP OnMenuSelect(UINT menu_id) override;
+  STDMETHODIMP GetIcon(HICON *icon) override;
 
   // Initializes an ImeButtonMenu instance.
   // This function allocates resources for an ImeButtonMenu instance.
@@ -231,8 +227,6 @@ class TipLangBarMenuButton : public TipLangBarButton {
                UINT menu_icon_id_for_non_theme, UINT menu_icon_id_for_theme);
 
  private:
-  TipRefCount ref_count_;
-
   // Represents the icon of the language bar menu.
   UINT menu_icon_id_for_theme_;
   UINT menu_icon_id_for_non_theme_;
@@ -245,25 +239,28 @@ class TipLangBarToggleButton : public TipLangBarButton,
  public:
   TipLangBarToggleButton(TipLangBarCallback *langbar_callback, const GUID &guid,
                          bool is_menu, bool show_in_tray);
-  virtual ~TipLangBarToggleButton();
 
   // The IUnknown interface methods
-  virtual STDMETHODIMP QueryInterface(REFIID guid, void **object);
-  virtual STDMETHODIMP_(ULONG) AddRef();
-  virtual STDMETHODIMP_(ULONG) Release();
+  // TODO(yuryu): Directly derive from TipLangBarButton and remove these
+  // overrides.
+  STDMETHODIMP_(ULONG) AddRef() override { return TipLangBarButton::AddRef(); }
+  STDMETHODIMP_(ULONG) Release() override {
+    return TipLangBarButton::Release();
+  }
+  STDMETHODIMP QueryInterface(REFIID guid, void **object) override;
 
   // The IMozcLangBarToggleItem interface methods
-  virtual STDMETHODIMP SelectMenuItem(UINT menu_id);
+  STDMETHODIMP SelectMenuItem(UINT menu_id) override;
 
   // The IMozcLangBarItem interface method
   // Overridden from TipLangBarButton.
-  virtual STDMETHODIMP SetEnabled(bool enabled);
+  STDMETHODIMP SetEnabled(bool enabled) override;
 
-  virtual STDMETHODIMP GetInfo(TF_LANGBARITEMINFO *item_info);
+  STDMETHODIMP GetInfo(TF_LANGBARITEMINFO *item_info) override;
 
-  virtual STDMETHODIMP InitMenu(ITfMenu *menu);
-  virtual STDMETHODIMP OnMenuSelect(UINT menu_id);
-  virtual STDMETHODIMP GetIcon(HICON *icon);
+  STDMETHODIMP InitMenu(ITfMenu *menu) override;
+  STDMETHODIMP OnMenuSelect(UINT menu_id) override;
+  STDMETHODIMP GetIcon(HICON *icon) override;
 
   // Initializes an ImeButtonMenu instance.
   // This function allocates resources for an ImeButtonMenu instance.
@@ -272,8 +269,6 @@ class TipLangBarToggleButton : public TipLangBarButton,
                const TipLangBarMenuItem &menu_for_disabled);
 
  private:
-  TipRefCount ref_count_;
-
   // Represents the index of the selected menu item.
   UINT menu_selected_;
   bool disabled_;
@@ -283,27 +278,24 @@ class TipLangBarToggleButton : public TipLangBarButton,
 
 // Represents the common operations for a button-menu item in the system
 // language bar.
-class TipSystemLangBarMenu : public ITfSystemLangBarItemSink {
+class TipSystemLangBarMenu : public TipComImplements<ITfSystemLangBarItemSink> {
  public:
-  TipSystemLangBarMenu(TipLangBarCallback *langbar_callback, const GUID &guid);
-  virtual ~TipSystemLangBarMenu();
-
-  // The IUnknown interface methods
-  virtual STDMETHODIMP QueryInterface(REFIID guid, void **object);
-  virtual STDMETHODIMP_(ULONG) AddRef();
-  virtual STDMETHODIMP_(ULONG) Release();
+  TipSystemLangBarMenu(
+      wil::com_ptr_nothrow<TipLangBarCallback> lang_bar_callback,
+      const GUID &guid);
 
   // The ITfLangBarSystemItem interface methods
-  virtual STDMETHODIMP InitMenu(ITfMenu *menu);
-  virtual STDMETHODIMP OnMenuSelect(UINT menu_id);
+  STDMETHODIMP InitMenu(ITfMenu *menu) override;
+  STDMETHODIMP OnMenuSelect(UINT menu_id) override;
 
   // Initializes an ImeButtonMenu instance.
   // This function allocates resources for an ImeButtonMenu instance.
   HRESULT Init(HINSTANCE instance, const TipLangBarMenuItem *menu, int count);
 
  private:
-  TipRefCount ref_count_;
-  TipLangBarCallback *langbar_callback_;
+  // Save the TipLangBarCallback object that owns this menu to prevent the
+  // object from being deleted.
+  wil::com_ptr_nothrow<TipLangBarCallback> lang_bar_callback_;
 
   // Represents the data possessed by the language bar menu.
   TipLangBarMenuDataArray menu_data_;
