@@ -29,57 +29,50 @@
 
 #include "win32/tip/tip_ui_element_manager.h"
 
-#define _ATL_NO_AUTOMATIC_NAMESPACE
-#define _WTL_NO_AUTOMATIC_NAMESPACE
-#include <atlbase.h>
-#include <atlcom.h>
-#include <atlstr.h>
 #include <msctf.h>
+#include <objbase.h>
+#include <wil/com.h>
 
-#include <unordered_map>
+#include <cstdint>
+#include <utility>
 
+#include "base/win32/com.h"
+#include "protocol/candidates.pb.h"
 #include "protocol/commands.pb.h"
 #include "protocol/renderer_command.pb.h"
+#include "absl/container/flat_hash_map.h"
 #include "win32/base/input_state.h"
 #include "win32/tip/tip_input_mode_manager.h"
 #include "win32/tip/tip_private_context.h"
 #include "win32/tip/tip_text_service.h"
 #include "win32/tip/tip_thread_context.h"
-#include "win32/tip/tip_ui_handler.h"
+#include "win32/tip/tip_ui_element_conventional.h"
 
 namespace mozc {
 namespace win32 {
 namespace tsf {
-
 namespace {
 
-using ATL::CComPtr;
-using ATL::CComQIPtr;
 using ::mozc::commands::Output;
-typedef ::mozc::commands::RendererCommand_IndicatorInfo IndicatorInfo;
+using IndicatorInfo = ::mozc::commands::RendererCommand_IndicatorInfo;
 
-struct UIElementInfo {
-  UIElementInfo() : id(TF_INVALID_UIELEMENTID) {}
-  DWORD id;
-  CComPtr<ITfUIElement> element;
-};
-
-HRESULT BeginUI(ITfUIElementMgr *ui_element_manager, ITfUIElement *ui_element,
+HRESULT BeginUI(const wil::com_ptr_nothrow<ITfUIElementMgr> &ui_element_manager,
+                const wil::com_ptr_nothrow<ITfUIElement> &ui_element,
                 DWORD *new_element_id) {
   BOOL show = FALSE;
   *new_element_id = TF_INVALID_UIELEMENTID;
-  CComPtr<ITfUIElement> element(ui_element);
-  const HRESULT result =
-      ui_element_manager->BeginUIElement(element, &show, new_element_id);
+  const HRESULT result = ui_element_manager->BeginUIElement(
+      ui_element.get(), &show, new_element_id);
   if (FAILED(result)) {
     return result;
   }
-  element->Show(show);
+  ui_element->Show(show);
   return S_OK;
 }
 
-HRESULT EndUI(ITfUIElementMgr *ui_element_manager, DWORD element_id) {
-  CComPtr<ITfUIElement> element;
+HRESULT EndUI(const wil::com_ptr_nothrow<ITfUIElementMgr> &ui_element_manager,
+              DWORD element_id) {
+  wil::com_ptr_nothrow<ITfUIElement> element;
   ui_element_manager->GetUIElement(element_id, &element);
   if (element) {
     element->Show(FALSE);
@@ -90,26 +83,17 @@ HRESULT EndUI(ITfUIElementMgr *ui_element_manager, DWORD element_id) {
 
 }  // namespace
 
-class TipUiElementManager::UiElementMap
-    : public std::unordered_map<TipUiElementManager::UIElementFlags,
-                                UIElementInfo> {};
-
-TipUiElementManager::TipUiElementManager()
-    : ui_element_map_(new UiElementMap) {}
-
-TipUiElementManager::~TipUiElementManager() {}
-
 ITfUIElement *TipUiElementManager::GetElement(UIElementFlags element) const {
-  const UiElementMap::const_iterator it = ui_element_map_->find(element);
-  if (it == ui_element_map_->end()) {
+  const UiElementMap::const_iterator it = ui_element_map_.find(element);
+  if (it == ui_element_map_.end()) {
     return nullptr;
   }
-  return it->second.element;
+  return it->second.element.get();
 }
 
 DWORD TipUiElementManager::GetElementId(UIElementFlags element) const {
-  const UiElementMap::const_iterator it = ui_element_map_->find(element);
-  if (it == ui_element_map_->end()) {
+  const UiElementMap::const_iterator it = ui_element_map_.find(element);
+  if (it == ui_element_map_.end()) {
     return TF_INVALID_UIELEMENTID;
   }
   return it->second.id;
@@ -117,8 +101,8 @@ DWORD TipUiElementManager::GetElementId(UIElementFlags element) const {
 
 HRESULT TipUiElementManager::OnUpdate(TipTextService *text_service,
                                       ITfContext *context) {
-  CComQIPtr<ITfUIElementMgr> ui_element_manager =
-      text_service->GetThreadManager();
+  auto ui_element_manager =
+      ComQuery<ITfUIElementMgr>(text_service->GetThreadManager());
   if (!ui_element_manager) {
     return E_FAIL;
   }
@@ -129,7 +113,7 @@ HRESULT TipUiElementManager::OnUpdate(TipTextService *text_service,
 
   const Output &output = private_context->last_output();
 
-  uint32 existence_bits = kNoneWindow;
+  uint32_t existence_bits = kNoneWindow;
   if (output.has_candidates() && output.candidates().has_category()) {
     switch (output.candidates().category()) {
       case commands::SUGGESTION:
@@ -151,31 +135,31 @@ HRESULT TipUiElementManager::OnUpdate(TipTextService *text_service,
   }
 
   DWORD suggest_ui_id = TF_INVALID_UIELEMENTID;
-  CComPtr<ITfUIElement> suggest_ui;
+  wil::com_ptr_nothrow<ITfUIElement> suggest_ui;
   {
     const UiElementMap::const_iterator it =
-        ui_element_map_->find(kSuggestWindow);
-    if (it != ui_element_map_->end()) {
+        ui_element_map_.find(kSuggestWindow);
+    if (it != ui_element_map_.end()) {
       suggest_ui_id = it->second.id;
       suggest_ui = it->second.element;
     }
   }
   DWORD candidate_ui_id = TF_INVALID_UIELEMENTID;
-  CComPtr<ITfUIElement> candidate_ui;
+  wil::com_ptr_nothrow<ITfUIElement> candidate_ui;
   {
     const UiElementMap::const_iterator it =
-        ui_element_map_->find(kCandidateWindow);
-    if (it != ui_element_map_->end()) {
+        ui_element_map_.find(kCandidateWindow);
+    if (it != ui_element_map_.end()) {
       candidate_ui_id = it->second.id;
       candidate_ui = it->second.element;
     }
   }
   DWORD indicator_ui_id = TF_INVALID_UIELEMENTID;
-  CComPtr<ITfUIElement> indicator_ui;
+  wil::com_ptr_nothrow<ITfUIElement> indicator_ui;
   {
     const UiElementMap::const_iterator it =
-        ui_element_map_->find(kIndicatorWindow);
-    if (it != ui_element_map_->end()) {
+        ui_element_map_.find(kIndicatorWindow);
+    if (it != ui_element_map_.end()) {
       indicator_ui_id = it->second.id;
       indicator_ui = it->second.element;
     }
@@ -230,63 +214,58 @@ HRESULT TipUiElementManager::OnUpdate(TipTextService *text_service,
   if (suggest_mode == kUIEnd) {
     EndUI(ui_element_manager, suggest_ui_id);
     suggest_ui_id = TF_INVALID_UIELEMENTID;
-    ui_element_map_->erase(kSuggestWindow);
-    if (suggest_ui) {
-      TipUiHandler::OnDestroyElement(text_service, suggest_ui);
-    }
+    ui_element_map_.erase(kSuggestWindow);
   }
   if (candidate_mode == kUIEnd) {
     EndUI(ui_element_manager, candidate_ui_id);
     candidate_ui_id = TF_INVALID_UIELEMENTID;
-    ui_element_map_->erase(kCandidateWindow);
-    if (candidate_ui) {
-      TipUiHandler::OnDestroyElement(text_service, candidate_ui);
-    }
+    ui_element_map_.erase(kCandidateWindow);
   }
   if (indicator_mode == kUIEnd) {
     EndUI(ui_element_manager, indicator_ui_id);
     indicator_ui_id = TF_INVALID_UIELEMENTID;
-    ui_element_map_->erase(kIndicatorWindow);
-    if (indicator_ui) {
-      TipUiHandler::OnDestroyElement(text_service, indicator_ui);
-    }
+    ui_element_map_.erase(kIndicatorWindow);
   }
 
   if (suggest_mode == kUIBeginAndUpdate) {
-    CComPtr<ITfUIElement> suggest_ui = TipUiHandler::CreateUI(
-        TipUiHandler::kSuggestWindow, text_service, context);
+    wil::com_ptr_nothrow<ITfUIElement> suggest_ui =
+        TipUiElementConventional::New(
+            TipUiElementConventional::kUnobservableSuggestWindow, text_service,
+            context);
     if (suggest_ui) {
       DWORD new_suggest_ui_id = TF_INVALID_UIELEMENTID;
       if (SUCCEEDED(
               BeginUI(ui_element_manager, suggest_ui, &new_suggest_ui_id))) {
-        (*ui_element_map_)[kSuggestWindow].element = suggest_ui;
-        (*ui_element_map_)[kSuggestWindow].id = new_suggest_ui_id;
+        ui_element_map_[kSuggestWindow].element = std::move(suggest_ui);
+        ui_element_map_[kSuggestWindow].id = new_suggest_ui_id;
         suggest_ui_id = new_suggest_ui_id;
       }
     }
   }
   if (candidate_mode == kUIBeginAndUpdate) {
-    CComPtr<ITfUIElement> candidate_ui = TipUiHandler::CreateUI(
-        TipUiHandler::kCandidateWindow, text_service, context);
+    wil::com_ptr_nothrow<ITfUIElement> candidate_ui =
+        TipUiElementConventional::New(
+            TipUiElementConventional::kCandidateWindow, text_service, context);
     if (candidate_ui) {
       DWORD new_candidate_ui_id = TF_INVALID_UIELEMENTID;
       if (SUCCEEDED(BeginUI(ui_element_manager, candidate_ui,
                             &new_candidate_ui_id))) {
-        (*ui_element_map_)[kCandidateWindow].element = candidate_ui;
-        (*ui_element_map_)[kCandidateWindow].id = new_candidate_ui_id;
+        ui_element_map_[kCandidateWindow].element = std::move(candidate_ui);
+        ui_element_map_[kCandidateWindow].id = new_candidate_ui_id;
         candidate_ui_id = new_candidate_ui_id;
       }
     }
   }
   if (indicator_mode == kUIBeginAndUpdate) {
-    CComPtr<ITfUIElement> indicator_ui = TipUiHandler::CreateUI(
-        TipUiHandler::kIndicatorWindow, text_service, context);
+    wil::com_ptr_nothrow<ITfUIElement> indicator_ui =
+        TipUiElementConventional::New(
+            TipUiElementConventional::KIndicatorWindow, text_service, context);
     if (indicator_ui) {
       DWORD new_indicator_ui_id = TF_INVALID_UIELEMENTID;
       if (SUCCEEDED(BeginUI(ui_element_manager, indicator_ui,
                             &new_indicator_ui_id))) {
-        (*ui_element_map_)[kIndicatorWindow].element = indicator_ui;
-        (*ui_element_map_)[kIndicatorWindow].id = new_indicator_ui_id;
+        ui_element_map_[kIndicatorWindow].element = std::move(indicator_ui);
+        ui_element_map_[kIndicatorWindow].id = new_indicator_ui_id;
         candidate_ui_id = new_indicator_ui_id;
       }
     }
@@ -309,8 +288,8 @@ bool TipUiElementManager::IsVisible(ITfUIElementMgr *ui_element_manager,
   if (ui_element_manager == nullptr) {
     return false;
   }
-  const UiElementMap::const_iterator it = ui_element_map_->find(element);
-  if (it == ui_element_map_->end()) {
+  const UiElementMap::const_iterator it = ui_element_map_.find(element);
+  if (it == ui_element_map_.end()) {
     return false;
   }
 
